@@ -166,7 +166,7 @@ install_jdks() {
 
 graalvm_test() {
   cd "$ROOT"/java
-  mvn -T10 -B --no-transfer-progress clean install -DskipTests -pl '!:fory-format,!:fory-testsuite'
+  mvn -T10 -B --no-transfer-progress clean install -DskipTests -pl '!:fory-testsuite'
   echo "Start to build graalvm native image"
   cd "$ROOT"/integration_tests/graalvm_tests
   mvn -DskipTests=true --no-transfer-progress -Pnative package
@@ -180,14 +180,8 @@ integration_tests() {
   cd "$ROOT"/java
   mvn -T10 -B --no-transfer-progress clean install -DskipTests
   echo "benchmark tests"
-  cd "$ROOT"/java/benchmark
+  cd "$ROOT"/benchmarks/java_benchmark
   mvn -T10 -B --no-transfer-progress clean test install -Pjmh
-  echo "Start latest jdk tests"
-  cd "$ROOT"/integration_tests/latest_jdk_tests
-  echo "latest_jdk_tests: JDK 21"
-  export JAVA_HOME="$ROOT/zulu21.28.85-ca-jdk21.0.0-linux_x64"
-  export PATH=$JAVA_HOME/bin:$PATH
-  mvn -T10 -B --no-transfer-progress clean test
   echo "Start JPMS tests"
   cd "$ROOT"/integration_tests/jpms_tests
   mvn -T10 -B --no-transfer-progress clean compile
@@ -220,10 +214,6 @@ jdk17_plus_tests() {
     exit $testcode
   fi
   echo "Executing fory java tests succeeds"
-  echo "Executing latest_jdk_tests"
-  cd "$ROOT"/integration_tests/latest_jdk_tests
-  mvn -T10 -B --no-transfer-progress clean test
-  echo "Executing latest_jdk_tests succeeds"
 }
 
 kotlin_tests() {
@@ -243,7 +233,7 @@ windows_java21_test() {
   echo "Executing fory java tests"
   cd "$ROOT/java"
   set +e
-  mvn -T10 --batch-mode --no-transfer-progress test -Dtest=!org.apache.fory.CrossLanguageTest install -pl '!fory-format,!fory-testsuite'
+  mvn -T10 --batch-mode --no-transfer-progress test install -pl '!fory-testsuite'
   testcode=$?
   if [[ $testcode -ne 0 ]]; then
     exit $testcode
@@ -400,6 +390,9 @@ case $1 in
       mvn -T10 -B --no-transfer-progress checkstyle:check
       echo "Executing format check succeeds"
     ;;
+    install_pyfory)
+      install_pyfory
+    ;;
     *)
       echo "Execute command $*"
       "$@"
