@@ -24,6 +24,8 @@
 // We use a two-step macro expansion process to ensure that arguments
 // passed to the macros are fully expanded before they are concatenated.
 #define FORY_PP_CONCAT(a, b) FORY_PP_CONCAT_IMPL(a, b)
+#define FORY_PP_CONCAT5(a, b, c, d, e)                                         \
+  FORY_PP_CONCAT(FORY_PP_CONCAT(FORY_PP_CONCAT(FORY_PP_CONCAT(a, b), c), d), e)
 
 #define FORY_PP_STRINGIFY_IMPL(...) #__VA_ARGS__
 #define FORY_PP_STRINGIFY(...) FORY_PP_STRINGIFY_IMPL(__VA_ARGS__)
@@ -36,14 +38,65 @@
     _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, \
     _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, \
     _47, _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, \
-    _62, _63, N, ...)                                                          \
+    _62, _63, _64, N, ...)                                                     \
   N
 
 #define FORY_PP_NARG_REV()                                                     \
-  63, 62, 61, 60, 59, 58, 57, 56, 55, 54, 53, 52, 51, 50, 49, 48, 47, 46, 45,  \
-      44, 43, 42, 41, 40, 39, 38, 37, 36, 35, 34, 33, 32, 31, 30, 29, 28, 27,  \
-      26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9,   \
-      8, 7, 6, 5, 4, 3, 2, 1, 0
+  64, 63, 62, 61, 60, 59, 58, 57, 56, 55, 54, 53, 52, 51, 50, 49, 48, 47, 46,  \
+      45, 44, 43, 42, 41, 40, 39, 38, 37, 36, 35, 34, 33, 32, 31, 30, 29, 28,  \
+      27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10,  \
+      9, 8, 7, 6, 5, 4, 3, 2, 1, 0
+
+#define FORY_PP_HAS_COMMA(...)                                                 \
+  FORY_PP_HAS_COMMA_IMPL(__VA_ARGS__, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,   \
+                         1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, \
+                         1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, \
+                         1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0)
+
+#define FORY_PP_HAS_COMMA_IMPL(                                                \
+    _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, \
+    _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, \
+    _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, \
+    _47, _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, \
+    _62, _63, _64, N, ...)                                                     \
+  N
+
+#define FORY_PP_TRIGGER_PARENTHESIS_(...) ,
+
+#define FORY_PP_EMPTY()
+
+#define FORY_PP_PROBE() ~, 1
+#define FORY_PP_CHECK_N(x, n, ...) n
+#define FORY_PP_CHECK(...) FORY_PP_CHECK_N(__VA_ARGS__, 0)
+
+#define FORY_PP_IS_PAREN(x) FORY_PP_CHECK(FORY_PP_IS_PAREN_PROBE x)
+#define FORY_PP_IS_PAREN_PROBE(...) FORY_PP_PROBE()
+
+#define FORY_PP_IS_EMPTY(...)                                                  \
+  FORY_PP_IS_EMPTY_I(                                                          \
+      FORY_PP_HAS_COMMA(__VA_ARGS__),                                          \
+      FORY_PP_HAS_COMMA(FORY_PP_TRIGGER_PARENTHESIS_ __VA_ARGS__),             \
+      FORY_PP_HAS_COMMA(__VA_ARGS__(/*empty*/)),                               \
+      FORY_PP_HAS_COMMA(FORY_PP_TRIGGER_PARENTHESIS_ __VA_ARGS__(/*empty*/)))
+
+#define FORY_PP_IS_EMPTY_I(_0, _1, _2, _3)                                     \
+  FORY_PP_HAS_COMMA(FORY_PP_CONCAT5(FORY_PP_IS_EMPTY_CASE_, _0, _1, _2, _3))
+
+#define FORY_PP_IS_EMPTY_CASE_0001 ,
+
+#define FORY_PP_NOT_0 1
+#define FORY_PP_NOT_1 0
+#define FORY_PP_NOT(x) FORY_PP_CONCAT(FORY_PP_NOT_, x)
+#define FORY_PP_HAS_ARGS(...) FORY_PP_NOT(FORY_PP_IS_EMPTY(__VA_ARGS__))
+
+#define FORY_PP_IF(c) FORY_PP_CONCAT(FORY_PP_IF_, c)
+#define FORY_PP_IF_1(t, f) t
+#define FORY_PP_IF_0(t, f) f
+
+#define FORY_PP_TUPLE_FIRST(tuple) FORY_PP_TUPLE_FIRST_IMPL tuple
+#define FORY_PP_TUPLE_FIRST_IMPL(a, ...) a
+#define FORY_PP_TUPLE_SECOND(tuple) FORY_PP_TUPLE_SECOND_IMPL tuple
+#define FORY_PP_TUPLE_SECOND_IMPL(a, b, ...) b
 
 #define FORY_PP_INVOKE(X, ...) X(__VA_ARGS__)
 
@@ -54,7 +107,7 @@
       __VA_ARGS__)
 
 // HOW TO GENERATE:
-// for n in range(1, 64):
+// for n in range(1, 65):
 //   params = ', '.join(f'_{i}' for i in range(1, n+1))
 //   calls = ' '.join(f'X(_{i})' for i in range(1, n+1))
 //   print(f"#define FORY_PP_FOREACH_IMPL_{n}(X, {params}) {calls}")
@@ -124,6 +177,7 @@
 #define FORY_PP_FOREACH_IMPL_61(X, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47, _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61) X(_1) X(_2) X(_3) X(_4) X(_5) X(_6) X(_7) X(_8) X(_9) X(_10) X(_11) X(_12) X(_13) X(_14) X(_15) X(_16) X(_17) X(_18) X(_19) X(_20) X(_21) X(_22) X(_23) X(_24) X(_25) X(_26) X(_27) X(_28) X(_29) X(_30) X(_31) X(_32) X(_33) X(_34) X(_35) X(_36) X(_37) X(_38) X(_39) X(_40) X(_41) X(_42) X(_43) X(_44) X(_45) X(_46) X(_47) X(_48) X(_49) X(_50) X(_51) X(_52) X(_53) X(_54) X(_55) X(_56) X(_57) X(_58) X(_59) X(_60) X(_61)
 #define FORY_PP_FOREACH_IMPL_62(X, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47, _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, _62) X(_1) X(_2) X(_3) X(_4) X(_5) X(_6) X(_7) X(_8) X(_9) X(_10) X(_11) X(_12) X(_13) X(_14) X(_15) X(_16) X(_17) X(_18) X(_19) X(_20) X(_21) X(_22) X(_23) X(_24) X(_25) X(_26) X(_27) X(_28) X(_29) X(_30) X(_31) X(_32) X(_33) X(_34) X(_35) X(_36) X(_37) X(_38) X(_39) X(_40) X(_41) X(_42) X(_43) X(_44) X(_45) X(_46) X(_47) X(_48) X(_49) X(_50) X(_51) X(_52) X(_53) X(_54) X(_55) X(_56) X(_57) X(_58) X(_59) X(_60) X(_61) X(_62)
 #define FORY_PP_FOREACH_IMPL_63(X, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47, _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, _62, _63) X(_1) X(_2) X(_3) X(_4) X(_5) X(_6) X(_7) X(_8) X(_9) X(_10) X(_11) X(_12) X(_13) X(_14) X(_15) X(_16) X(_17) X(_18) X(_19) X(_20) X(_21) X(_22) X(_23) X(_24) X(_25) X(_26) X(_27) X(_28) X(_29) X(_30) X(_31) X(_32) X(_33) X(_34) X(_35) X(_36) X(_37) X(_38) X(_39) X(_40) X(_41) X(_42) X(_43) X(_44) X(_45) X(_46) X(_47) X(_48) X(_49) X(_50) X(_51) X(_52) X(_53) X(_54) X(_55) X(_56) X(_57) X(_58) X(_59) X(_60) X(_61) X(_62) X(_63)
+#define FORY_PP_FOREACH_IMPL_64(X, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47, _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, _62, _63, _64) X(_1) X(_2) X(_3) X(_4) X(_5) X(_6) X(_7) X(_8) X(_9) X(_10) X(_11) X(_12) X(_13) X(_14) X(_15) X(_16) X(_17) X(_18) X(_19) X(_20) X(_21) X(_22) X(_23) X(_24) X(_25) X(_26) X(_27) X(_28) X(_29) X(_30) X(_31) X(_32) X(_33) X(_34) X(_35) X(_36) X(_37) X(_38) X(_39) X(_40) X(_41) X(_42) X(_43) X(_44) X(_45) X(_46) X(_47) X(_48) X(_49) X(_50) X(_51) X(_52) X(_53) X(_54) X(_55) X(_56) X(_57) X(_58) X(_59) X(_60) X(_61) X(_62) X(_63) X(_64)
 // clang-format on
 
 // different than FORY_PP_FOREACH, this macro can accept a two-argument function
@@ -134,7 +188,7 @@
       C, __VA_ARGS__)
 
 // HOW TO GENERATE:
-// for n in range(1, 64):
+// for n in range(1, 65):
 //   params = ', '.join(f'_{i}' for i in range(1, n+1))
 //   calls = ' '.join(f'X(C, _{i})' for i in range(1, n+1))
 //   print(f"#define FORY_PP_FOREACH_1_IMPL_{n}(X, C, {params}) {calls}")
@@ -204,4 +258,5 @@
 #define FORY_PP_FOREACH_1_IMPL_61(X, C, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47, _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61) X(C, _1) X(C, _2) X(C, _3) X(C, _4) X(C, _5) X(C, _6) X(C, _7) X(C, _8) X(C, _9) X(C, _10) X(C, _11) X(C, _12) X(C, _13) X(C, _14) X(C, _15) X(C, _16) X(C, _17) X(C, _18) X(C, _19) X(C, _20) X(C, _21) X(C, _22) X(C, _23) X(C, _24) X(C, _25) X(C, _26) X(C, _27) X(C, _28) X(C, _29) X(C, _30) X(C, _31) X(C, _32) X(C, _33) X(C, _34) X(C, _35) X(C, _36) X(C, _37) X(C, _38) X(C, _39) X(C, _40) X(C, _41) X(C, _42) X(C, _43) X(C, _44) X(C, _45) X(C, _46) X(C, _47) X(C, _48) X(C, _49) X(C, _50) X(C, _51) X(C, _52) X(C, _53) X(C, _54) X(C, _55) X(C, _56) X(C, _57) X(C, _58) X(C, _59) X(C, _60) X(C, _61)
 #define FORY_PP_FOREACH_1_IMPL_62(X, C, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47, _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, _62) X(C, _1) X(C, _2) X(C, _3) X(C, _4) X(C, _5) X(C, _6) X(C, _7) X(C, _8) X(C, _9) X(C, _10) X(C, _11) X(C, _12) X(C, _13) X(C, _14) X(C, _15) X(C, _16) X(C, _17) X(C, _18) X(C, _19) X(C, _20) X(C, _21) X(C, _22) X(C, _23) X(C, _24) X(C, _25) X(C, _26) X(C, _27) X(C, _28) X(C, _29) X(C, _30) X(C, _31) X(C, _32) X(C, _33) X(C, _34) X(C, _35) X(C, _36) X(C, _37) X(C, _38) X(C, _39) X(C, _40) X(C, _41) X(C, _42) X(C, _43) X(C, _44) X(C, _45) X(C, _46) X(C, _47) X(C, _48) X(C, _49) X(C, _50) X(C, _51) X(C, _52) X(C, _53) X(C, _54) X(C, _55) X(C, _56) X(C, _57) X(C, _58) X(C, _59) X(C, _60) X(C, _61) X(C, _62)
 #define FORY_PP_FOREACH_1_IMPL_63(X, C, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47, _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, _62, _63) X(C, _1) X(C, _2) X(C, _3) X(C, _4) X(C, _5) X(C, _6) X(C, _7) X(C, _8) X(C, _9) X(C, _10) X(C, _11) X(C, _12) X(C, _13) X(C, _14) X(C, _15) X(C, _16) X(C, _17) X(C, _18) X(C, _19) X(C, _20) X(C, _21) X(C, _22) X(C, _23) X(C, _24) X(C, _25) X(C, _26) X(C, _27) X(C, _28) X(C, _29) X(C, _30) X(C, _31) X(C, _32) X(C, _33) X(C, _34) X(C, _35) X(C, _36) X(C, _37) X(C, _38) X(C, _39) X(C, _40) X(C, _41) X(C, _42) X(C, _43) X(C, _44) X(C, _45) X(C, _46) X(C, _47) X(C, _48) X(C, _49) X(C, _50) X(C, _51) X(C, _52) X(C, _53) X(C, _54) X(C, _55) X(C, _56) X(C, _57) X(C, _58) X(C, _59) X(C, _60) X(C, _61) X(C, _62) X(C, _63)
+#define FORY_PP_FOREACH_1_IMPL_64(X, C, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47, _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, _62, _63, _64) X(C, _1) X(C, _2) X(C, _3) X(C, _4) X(C, _5) X(C, _6) X(C, _7) X(C, _8) X(C, _9) X(C, _10) X(C, _11) X(C, _12) X(C, _13) X(C, _14) X(C, _15) X(C, _16) X(C, _17) X(C, _18) X(C, _19) X(C, _20) X(C, _21) X(C, _22) X(C, _23) X(C, _24) X(C, _25) X(C, _26) X(C, _27) X(C, _28) X(C, _29) X(C, _30) X(C, _31) X(C, _32) X(C, _33) X(C, _34) X(C, _35) X(C, _36) X(C, _37) X(C, _38) X(C, _39) X(C, _40) X(C, _41) X(C, _42) X(C, _43) X(C, _44) X(C, _45) X(C, _46) X(C, _47) X(C, _48) X(C, _49) X(C, _50) X(C, _51) X(C, _52) X(C, _53) X(C, _54) X(C, _55) X(C, _56) X(C, _57) X(C, _58) X(C, _59) X(C, _60) X(C, _61) X(C, _62) X(C, _63) X(C, _64)
 // clang-format on
