@@ -31,29 +31,29 @@
 namespace fory {
 namespace row {
 
-int Getter::GetBinary(int i, uint8_t **out) const {
-  if (IsNullAt(i))
+int Getter::get_binary(int i, uint8_t **out) const {
+  if (is_null_at(i))
     return -1;
-  auto offsetAndSize = GetUint64(i);
-  auto relative_offset = static_cast<uint32_t>(offsetAndSize >> 32);
-  auto size = static_cast<uint32_t>(offsetAndSize);
+  auto offset_and_size = get_uint64(i);
+  auto relative_offset = static_cast<uint32_t>(offset_and_size >> 32);
+  auto size = static_cast<uint32_t>(offset_and_size);
   *out = buffer()->data() + base_offset() + relative_offset;
   return size;
 }
 
-std::vector<uint8_t> Getter::GetBinary(int i) const {
-  if (IsNullAt(i))
+std::vector<uint8_t> Getter::get_binary(int i) const {
+  if (is_null_at(i))
     return std::vector<uint8_t>();
-  auto offsetAndSize = GetUint64(i);
-  auto relative_offset = static_cast<uint32_t>(offsetAndSize >> 32);
-  auto size = static_cast<uint32_t>(offsetAndSize);
+  auto offset_and_size = get_uint64(i);
+  auto relative_offset = static_cast<uint32_t>(offset_and_size >> 32);
+  auto size = static_cast<uint32_t>(offset_and_size);
   auto start = buffer()->data() + base_offset() + relative_offset;
   return std::vector<uint8_t>(start, start + size);
 }
 
-std::string Getter::GetString(int i) const {
+std::string Getter::get_string(int i) const {
   uint8_t *binary;
-  int size = GetBinary(i, &binary);
+  int size = get_binary(i, &binary);
   if (size == -1) {
     return std::string("");
   } else {
@@ -62,69 +62,71 @@ std::string Getter::GetString(int i) const {
   }
 }
 
-std::shared_ptr<Row> Getter::GetStruct(int i, StructTypePtr struct_type) const {
-  if (IsNullAt(i))
+std::shared_ptr<Row> Getter::get_struct(int i,
+                                        StructTypePtr struct_type) const {
+  if (is_null_at(i))
     return nullptr;
-  auto offsetAndSize = GetUint64(i);
-  auto relative_offset = static_cast<uint32_t>(offsetAndSize >> 32);
-  auto size = static_cast<uint32_t>(offsetAndSize);
+  auto offset_and_size = get_uint64(i);
+  auto relative_offset = static_cast<uint32_t>(offset_and_size >> 32);
+  auto size = static_cast<uint32_t>(offset_and_size);
   auto row_schema = schema(struct_type->fields());
   std::shared_ptr<Row> row = std::make_shared<Row>(row_schema);
-  row->PointTo(buffer(), base_offset() + relative_offset, size);
+  row->point_to(buffer(), base_offset() + relative_offset, size);
   return row;
 }
 
-std::shared_ptr<ArrayData> Getter::GetArray(int i,
-                                            ListTypePtr array_type) const {
-  if (IsNullAt(i))
+std::shared_ptr<ArrayData> Getter::get_array(int i,
+                                             ListTypePtr array_type) const {
+  if (is_null_at(i))
     return nullptr;
-  auto offsetAndSize = GetUint64(i);
-  auto relative_offset = static_cast<uint32_t>(offsetAndSize >> 32);
-  auto size = static_cast<uint32_t>(offsetAndSize);
+  auto offset_and_size = get_uint64(i);
+  auto relative_offset = static_cast<uint32_t>(offset_and_size >> 32);
+  auto size = static_cast<uint32_t>(offset_and_size);
   auto arr = std::make_shared<ArrayData>(array_type);
-  arr->PointTo(buffer(), base_offset() + relative_offset, size);
+  arr->point_to(buffer(), base_offset() + relative_offset, size);
   return arr;
 }
 
-std::shared_ptr<MapData> Getter::GetMap(int i, MapTypePtr map_type) const {
-  if (IsNullAt(i))
+std::shared_ptr<MapData> Getter::get_map(int i, MapTypePtr map_type) const {
+  if (is_null_at(i))
     return nullptr;
-  auto offsetAndSize = GetUint64(i);
-  auto relative_offset = static_cast<uint32_t>(offsetAndSize >> 32);
-  auto size = static_cast<uint32_t>(offsetAndSize);
+  auto offset_and_size = get_uint64(i);
+  auto relative_offset = static_cast<uint32_t>(offset_and_size >> 32);
+  auto size = static_cast<uint32_t>(offset_and_size);
   auto map_data = std::make_shared<MapData>(map_type);
-  map_data->PointTo(buffer(), base_offset() + relative_offset, size);
+  map_data->point_to(buffer(), base_offset() + relative_offset, size);
   return map_data;
 }
 
-void Getter::AppendValue(std::stringstream &ss, int i, DataTypePtr type) const {
+void Getter::append_value(std::stringstream &ss, int i,
+                          DataTypePtr type) const {
   TypeId type_id = type->id();
   if (type_id == TypeId::INT8) {
-    ss << static_cast<int>(GetInt8(i));
+    ss << static_cast<int>(get_int8(i));
   } else if (type_id == TypeId::BOOL) {
-    ss << GetBoolean(i);
+    ss << get_boolean(i);
   } else if (type_id == TypeId::INT16) {
-    ss << GetInt16(i);
+    ss << get_int16(i);
   } else if (type_id == TypeId::INT32) {
-    ss << GetInt32(i);
+    ss << get_int32(i);
   } else if (type_id == TypeId::INT64) {
-    ss << GetInt64(i);
+    ss << get_int64(i);
   } else if (type_id == TypeId::FLOAT32) {
-    ss << GetFloat(i);
+    ss << get_float(i);
   } else if (type_id == TypeId::FLOAT64) {
-    ss << GetDouble(i);
+    ss << get_double(i);
   } else if (type_id == TypeId::STRING) {
-    ss << GetString(i);
+    ss << get_string(i);
   } else if (type_id == TypeId::LIST) {
-    ss << GetArray(i)->ToString();
+    ss << get_array(i)->to_string();
   } else if (type_id == TypeId::MAP) {
-    ss << GetMap(i)->ToString();
+    ss << get_map(i)->to_string();
   } else if (type_id == TypeId::STRUCT) {
-    ss << GetStruct(i)->ToString();
+    ss << get_struct(i)->to_string();
   } else if (type_id == TypeId::BINARY) {
-    ss << GetString(i);
+    ss << get_string(i);
   } else {
-    ss << "unsupported type " << type->ToString();
+    ss << "unsupported type " << type->to_string();
   }
 }
 
@@ -135,14 +137,14 @@ Row::Row(const SchemaPtr &schema)
   bitmap_width_bytes_ = ((num_fields_ + 63) / 64) * 8;
 }
 
-void Row::PointTo(std::shared_ptr<Buffer> buffer, int offset,
-                  int size_in_bytes) {
+void Row::point_to(std::shared_ptr<Buffer> buffer, int offset,
+                   int size_in_bytes) {
   buffer_ = std::move(buffer);
   base_offset_ = offset;
   size_bytes_ = size_in_bytes;
 }
 
-std::string Row::ToString() const {
+std::string Row::to_string() const {
   if (!buffer_) {
     return std::string("null");
   } else {
@@ -154,11 +156,11 @@ std::string Row::ToString() const {
       }
       auto f = schema_->field(i);
       ss << f->name() << "=";
-      if (IsNullAt(i)) {
+      if (is_null_at(i)) {
         ss << "null";
       } else {
         auto type = f->type();
-        AppendValue(ss, i, type);
+        append_value(ss, i, type);
       }
     }
     ss << "}";
@@ -167,45 +169,45 @@ std::string Row::ToString() const {
 }
 
 std::ostream &operator<<(std::ostream &os, const Row &data) {
-  os << data.ToString();
+  os << data.to_string();
   return os;
 }
 
 template <typename value_type>
-std::shared_ptr<ArrayData> ArrayDataFrom(const value_type *data,
-                                         int num_elements, int element_size,
-                                         const ListTypePtr &type) {
+std::shared_ptr<ArrayData> array_data_from(const value_type *data,
+                                           int num_elements, int element_size,
+                                           const ListTypePtr &type) {
   auto array_data = std::make_shared<ArrayData>(type);
   std::shared_ptr<Buffer> buffer;
-  auto header_bytes = ArrayData::CalculateHeaderInBytes(num_elements);
+  auto header_bytes = ArrayData::calculate_header_in_bytes(num_elements);
   auto size_bytes = header_bytes + num_elements * element_size;
-  AllocateBuffer(static_cast<int32_t>(size_bytes), &buffer);
-  buffer->ZeroPadding();
-  buffer->UnsafePut(0, static_cast<int64_t>(num_elements));
-  buffer->CopyFrom(header_bytes, reinterpret_cast<const uint8_t *>(data), 0,
-                   static_cast<int32_t>(num_elements * element_size));
-  array_data->PointTo(buffer, 0, size_bytes);
+  allocate_buffer(static_cast<int32_t>(size_bytes), &buffer);
+  buffer->zero_padding();
+  buffer->unsafe_put(0, static_cast<int64_t>(num_elements));
+  buffer->copy_from(header_bytes, reinterpret_cast<const uint8_t *>(data), 0,
+                    static_cast<int32_t>(num_elements * element_size));
+  array_data->point_to(buffer, 0, size_bytes);
   return array_data;
 }
 
-std::shared_ptr<ArrayData> ArrayData::From(const std::vector<int32_t> &vec) {
-  return ArrayDataFrom(vec.data(), static_cast<int>(vec.size()), 4,
-                       list(int32()));
+std::shared_ptr<ArrayData> ArrayData::from(const std::vector<int32_t> &vec) {
+  return array_data_from(vec.data(), static_cast<int>(vec.size()), 4,
+                         list(int32()));
 }
 
-std::shared_ptr<ArrayData> ArrayData::From(const std::vector<int64_t> &vec) {
-  return ArrayDataFrom(vec.data(), static_cast<int>(vec.size()), 8,
-                       list(int64()));
+std::shared_ptr<ArrayData> ArrayData::from(const std::vector<int64_t> &vec) {
+  return array_data_from(vec.data(), static_cast<int>(vec.size()), 8,
+                         list(int64()));
 }
 
-std::shared_ptr<ArrayData> ArrayData::From(const std::vector<float> &vec) {
-  return ArrayDataFrom(vec.data(), static_cast<int>(vec.size()), 4,
-                       list(float32()));
+std::shared_ptr<ArrayData> ArrayData::from(const std::vector<float> &vec) {
+  return array_data_from(vec.data(), static_cast<int>(vec.size()), 4,
+                         list(float32()));
 }
 
-std::shared_ptr<ArrayData> ArrayData::From(const std::vector<double> &vec) {
-  return ArrayDataFrom(vec.data(), static_cast<int>(vec.size()), 8,
-                       list(float64()));
+std::shared_ptr<ArrayData> ArrayData::from(const std::vector<double> &vec) {
+  return array_data_from(vec.data(), static_cast<int>(vec.size()), 8,
+                         list(float64()));
 }
 
 ArrayData::ArrayData(ListTypePtr type) : type_(std::move(type)) {
@@ -218,21 +220,22 @@ ArrayData::ArrayData(ListTypePtr type) : type_(std::move(type)) {
   }
 }
 
-void ArrayData::PointTo(std::shared_ptr<Buffer> buffer, uint32_t offset,
-                        uint32_t size_bytes) {
-  num_elements_ = static_cast<int>(buffer->Get<int64_t>(offset));
+void ArrayData::point_to(std::shared_ptr<Buffer> buffer, uint32_t offset,
+                         uint32_t size_bytes) {
+  num_elements_ = static_cast<int>(buffer->get<int64_t>(offset));
   buffer_ = std::move(buffer);
   base_offset_ = offset;
   size_bytes_ = size_bytes;
-  element_offset_ = offset + CalculateHeaderInBytes(num_elements_);
+  element_offset_ = offset + calculate_header_in_bytes(num_elements_);
 }
 
-int ArrayData::CalculateHeaderInBytes(int num_elements) {
+int ArrayData::calculate_header_in_bytes(int num_elements) {
   return 8 + ((num_elements + 63) / 64) * 8;
 }
 
-int *ArrayData::GetDimensions(ArrayData &array, int num_dims) {
-  // use deep-first search to search to numDimensions-1 layer to get dimensions.
+int *ArrayData::get_dimensions(ArrayData &array, int num_dims) {
+  // use deep-first search to search to num_dimensions-1 layer to get
+  // dimensions.
   int depth = 0;
   auto dimensions = new int[num_dims];
   std::vector<int> start_from_lefts(num_dims);
@@ -254,8 +257,8 @@ int *ArrayData::GetDimensions(ArrayData &array, int num_dims) {
       continue;
     }
     for (int i = start_from_lefts[depth]; i < size; i++) {
-      if (!arr.IsNullAt(i)) {
-        arr = *arr.GetArray(i);
+      if (!arr.is_null_at(i)) {
+        arr = *arr.get_array(i);
         all_null = false;
         break;
       }
@@ -276,7 +279,7 @@ int *ArrayData::GetDimensions(ArrayData &array, int num_dims) {
   return dimensions;
 }
 
-std::string ArrayData::ToString() const {
+std::string ArrayData::to_string() const {
   if (!buffer_) {
     return std::string("null");
   } else {
@@ -286,11 +289,11 @@ std::string ArrayData::ToString() const {
       if (i != 0) {
         ss << ", ";
       }
-      if (IsNullAt(i)) {
+      if (is_null_at(i)) {
         ss << "null";
       } else {
         auto type = type_->value_type();
-        AppendValue(ss, i, type);
+        append_value(ss, i, type);
       }
     }
     ss << "]";
@@ -299,7 +302,7 @@ std::string ArrayData::ToString() const {
 }
 
 std::ostream &operator<<(std::ostream &os, const ArrayData &data) {
-  os << data.ToString();
+  os << data.to_string();
   return os;
 }
 
@@ -308,29 +311,29 @@ MapData::MapData(MapTypePtr type) : type_(std::move(type)) {
   values_ = std::make_shared<ArrayData>(list(type_->item_type()));
 }
 
-void MapData::PointTo(std::shared_ptr<Buffer> buffer, uint32_t offset,
-                      uint32_t size_bytes) {
+void MapData::point_to(std::shared_ptr<Buffer> buffer, uint32_t offset,
+                       uint32_t size_bytes) {
   buffer_ = std::move(buffer);
   base_offset_ = offset;
   size_bytes_ = size_bytes;
-  auto key_array_size = static_cast<int32_t>(buffer_->Get<uint64_t>(offset));
+  auto key_array_size = static_cast<int32_t>(buffer_->get<uint64_t>(offset));
   int32_t value_array_size = size_bytes - 8 - key_array_size;
-  keys_->PointTo(buffer_, offset + 8, key_array_size);
-  values_->PointTo(buffer_, offset + 8 + key_array_size, value_array_size);
+  keys_->point_to(buffer_, offset + 8, key_array_size);
+  values_->point_to(buffer_, offset + 8 + key_array_size, value_array_size);
 }
 
-std::string MapData::ToString() const {
+std::string MapData::to_string() const {
   if (!buffer_) {
     return std::string("null");
   } else {
     std::stringstream ss;
-    ss << "Map(" << keys_->ToString() << ", " << values_->ToString() << ")";
+    ss << "Map(" << keys_->to_string() << ", " << values_->to_string() << ")";
     return ss.str();
   }
 }
 
 std::ostream &operator<<(std::ostream &os, const MapData &data) {
-  os << data.ToString();
+  os << data.to_string();
   return os;
 }
 
