@@ -32,7 +32,7 @@ namespace serialization {
 // ============================================================================
 // Primitive Type Serializers
 // All primitive serializers use context-based error accumulation:
-// - Write methods return void and set ctx.error_ on failure
+// - write methods return void and set ctx.error_ on failure
 // - Read methods return T directly and set ctx.error_ on failure
 // - No per-operation error checks for primitives (buffer auto-grows on write,
 //   errors accumulate on read)
@@ -43,11 +43,11 @@ template <> struct Serializer<bool> {
   static constexpr TypeId type_id = TypeId::BOOL;
 
   static inline void write_type_info(WriteContext &ctx) {
-    ctx.write_varuint32(static_cast<uint32_t>(type_id));
+    ctx.write_uint8(static_cast<uint8_t>(type_id));
   }
 
   static inline void read_type_info(ReadContext &ctx) {
-    uint32_t actual = ctx.read_varuint32(ctx.error());
+    uint32_t actual = ctx.read_uint8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -57,11 +57,11 @@ template <> struct Serializer<bool> {
     }
   }
 
-  static inline void write(bool value, WriteContext &ctx, bool write_ref,
-                           bool write_type, bool has_generics = false) {
-    write_not_null_ref_flag(ctx, write_ref);
+  static inline void write(bool value, WriteContext &ctx, RefMode ref_mode,
+                           bool write_type, bool = false) {
+    write_not_null_ref_flag(ctx, ref_mode);
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
     write_data(value, ctx);
   }
@@ -70,18 +70,17 @@ template <> struct Serializer<bool> {
     ctx.write_uint8(value ? 1 : 0);
   }
 
-  static inline void write_data_generic(bool value, WriteContext &ctx,
-                                        bool has_generics) {
+  static inline void write_data_generic(bool value, WriteContext &ctx, bool) {
     write_data(value, ctx);
   }
 
-  static inline bool read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline bool read(ReadContext &ctx, RefMode ref_mode, bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (ctx.has_error() || !has_value) {
       return false;
     }
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return false;
       }
@@ -100,13 +99,13 @@ template <> struct Serializer<bool> {
     return value != 0;
   }
 
-  static inline bool read_data_generic(ReadContext &ctx, bool has_generics) {
+  static inline bool read_data_generic(ReadContext &ctx, bool) {
     return read_data(ctx);
   }
 
-  static inline bool read_with_type_info(ReadContext &ctx, bool read_ref,
-                                         const TypeInfo &type_info) {
-    return read(ctx, read_ref, false);
+  static inline bool read_with_type_info(ReadContext &ctx, RefMode ref_mode,
+                                         const TypeInfo &) {
+    return read(ctx, ref_mode, false);
   }
 };
 
@@ -115,11 +114,11 @@ template <> struct Serializer<int8_t> {
   static constexpr TypeId type_id = TypeId::INT8;
 
   static inline void write_type_info(WriteContext &ctx) {
-    ctx.write_varuint32(static_cast<uint32_t>(type_id));
+    ctx.write_uint8(static_cast<uint8_t>(type_id));
   }
 
   static inline void read_type_info(ReadContext &ctx) {
-    uint32_t actual = ctx.read_varuint32(ctx.error());
+    uint32_t actual = ctx.read_uint8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -129,11 +128,11 @@ template <> struct Serializer<int8_t> {
     }
   }
 
-  static inline void write(int8_t value, WriteContext &ctx, bool write_ref,
-                           bool write_type, bool has_generics = false) {
-    write_not_null_ref_flag(ctx, write_ref);
+  static inline void write(int8_t value, WriteContext &ctx, RefMode ref_mode,
+                           bool write_type, bool = false) {
+    write_not_null_ref_flag(ctx, ref_mode);
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
     write_data(value, ctx);
   }
@@ -142,18 +141,18 @@ template <> struct Serializer<int8_t> {
     ctx.write_int8(value);
   }
 
-  static inline void write_data_generic(int8_t value, WriteContext &ctx,
-                                        bool has_generics) {
+  static inline void write_data_generic(int8_t value, WriteContext &ctx, bool) {
     write_data(value, ctx);
   }
 
-  static inline int8_t read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline int8_t read(ReadContext &ctx, RefMode ref_mode,
+                            bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (ctx.has_error() || !has_value) {
       return 0;
     }
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return 0;
       }
@@ -170,13 +169,13 @@ template <> struct Serializer<int8_t> {
     return ctx.read_int8(ctx.error());
   }
 
-  static inline int8_t read_data_generic(ReadContext &ctx, bool has_generics) {
+  static inline int8_t read_data_generic(ReadContext &ctx, bool) {
     return read_data(ctx);
   }
 
-  static inline int8_t read_with_type_info(ReadContext &ctx, bool read_ref,
-                                           const TypeInfo &type_info) {
-    return read(ctx, read_ref, false);
+  static inline int8_t read_with_type_info(ReadContext &ctx, RefMode ref_mode,
+                                           const TypeInfo &) {
+    return read(ctx, ref_mode, false);
   }
 };
 
@@ -185,11 +184,11 @@ template <> struct Serializer<int16_t> {
   static constexpr TypeId type_id = TypeId::INT16;
 
   static inline void write_type_info(WriteContext &ctx) {
-    ctx.write_varuint32(static_cast<uint32_t>(type_id));
+    ctx.write_uint8(static_cast<uint8_t>(type_id));
   }
 
   static inline void read_type_info(ReadContext &ctx) {
-    uint32_t actual = ctx.read_varuint32(ctx.error());
+    uint32_t actual = ctx.read_uint8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -199,11 +198,11 @@ template <> struct Serializer<int16_t> {
     }
   }
 
-  static inline void write(int16_t value, WriteContext &ctx, bool write_ref,
-                           bool write_type, bool has_generics = false) {
-    write_not_null_ref_flag(ctx, write_ref);
+  static inline void write(int16_t value, WriteContext &ctx, RefMode ref_mode,
+                           bool write_type, bool = false) {
+    write_not_null_ref_flag(ctx, ref_mode);
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
     write_data(value, ctx);
   }
@@ -213,17 +212,18 @@ template <> struct Serializer<int16_t> {
   }
 
   static inline void write_data_generic(int16_t value, WriteContext &ctx,
-                                        bool has_generics) {
+                                        bool) {
     write_data(value, ctx);
   }
 
-  static inline int16_t read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline int16_t read(ReadContext &ctx, RefMode ref_mode,
+                             bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (ctx.has_error() || !has_value) {
       return 0;
     }
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return 0;
       }
@@ -240,26 +240,26 @@ template <> struct Serializer<int16_t> {
     return ctx.read_int16(ctx.error());
   }
 
-  static inline int16_t read_data_generic(ReadContext &ctx, bool has_generics) {
+  static inline int16_t read_data_generic(ReadContext &ctx, bool) {
     return read_data(ctx);
   }
 
-  static inline int16_t read_with_type_info(ReadContext &ctx, bool read_ref,
-                                            const TypeInfo &type_info) {
-    return read(ctx, read_ref, false);
+  static inline int16_t read_with_type_info(ReadContext &ctx, RefMode ref_mode,
+                                            const TypeInfo &) {
+    return read(ctx, ref_mode, false);
   }
 };
 
-/// int32_t serializer
+/// int32_t serializer - uses VARINT32 to match Java xlang mode and Rust
 template <> struct Serializer<int32_t> {
-  static constexpr TypeId type_id = TypeId::INT32;
+  static constexpr TypeId type_id = TypeId::VARINT32;
 
   static inline void write_type_info(WriteContext &ctx) {
-    ctx.write_varuint32(static_cast<uint32_t>(type_id));
+    ctx.write_uint8(static_cast<uint8_t>(type_id));
   }
 
   static inline void read_type_info(ReadContext &ctx) {
-    uint32_t actual = ctx.read_varuint32(ctx.error());
+    uint32_t actual = ctx.read_uint8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -269,11 +269,11 @@ template <> struct Serializer<int32_t> {
     }
   }
 
-  static inline void write(int32_t value, WriteContext &ctx, bool write_ref,
-                           bool write_type, bool has_generics = false) {
-    write_not_null_ref_flag(ctx, write_ref);
+  static inline void write(int32_t value, WriteContext &ctx, RefMode ref_mode,
+                           bool write_type, bool = false) {
+    write_not_null_ref_flag(ctx, ref_mode);
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
     write_data(value, ctx);
   }
@@ -283,17 +283,18 @@ template <> struct Serializer<int32_t> {
   }
 
   static inline void write_data_generic(int32_t value, WriteContext &ctx,
-                                        bool has_generics) {
+                                        bool) {
     write_data(value, ctx);
   }
 
-  static inline int32_t read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline int32_t read(ReadContext &ctx, RefMode ref_mode,
+                             bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (ctx.has_error() || !has_value) {
       return 0;
     }
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return 0;
       }
@@ -310,26 +311,26 @@ template <> struct Serializer<int32_t> {
     return ctx.read_varint32(ctx.error());
   }
 
-  static inline int32_t read_data_generic(ReadContext &ctx, bool has_generics) {
+  static inline int32_t read_data_generic(ReadContext &ctx, bool) {
     return read_data(ctx);
   }
 
-  static inline int32_t read_with_type_info(ReadContext &ctx, bool read_ref,
-                                            const TypeInfo &type_info) {
-    return read(ctx, read_ref, false);
+  static inline int32_t read_with_type_info(ReadContext &ctx, RefMode ref_mode,
+                                            const TypeInfo &) {
+    return read(ctx, ref_mode, false);
   }
 };
 
-/// int64_t serializer
+/// int64_t serializer - uses VARINT64 to match Java xlang mode and Rust
 template <> struct Serializer<int64_t> {
-  static constexpr TypeId type_id = TypeId::INT64;
+  static constexpr TypeId type_id = TypeId::VARINT64;
 
   static inline void write_type_info(WriteContext &ctx) {
-    ctx.write_varuint32(static_cast<uint32_t>(type_id));
+    ctx.write_uint8(static_cast<uint8_t>(type_id));
   }
 
   static inline void read_type_info(ReadContext &ctx) {
-    uint32_t actual = ctx.read_varuint32(ctx.error());
+    uint32_t actual = ctx.read_uint8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -339,11 +340,11 @@ template <> struct Serializer<int64_t> {
     }
   }
 
-  static inline void write(int64_t value, WriteContext &ctx, bool write_ref,
-                           bool write_type, bool has_generics = false) {
-    write_not_null_ref_flag(ctx, write_ref);
+  static inline void write(int64_t value, WriteContext &ctx, RefMode ref_mode,
+                           bool write_type, bool = false) {
+    write_not_null_ref_flag(ctx, ref_mode);
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
     write_data(value, ctx);
   }
@@ -353,17 +354,18 @@ template <> struct Serializer<int64_t> {
   }
 
   static inline void write_data_generic(int64_t value, WriteContext &ctx,
-                                        bool has_generics) {
+                                        bool) {
     write_data(value, ctx);
   }
 
-  static inline int64_t read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline int64_t read(ReadContext &ctx, RefMode ref_mode,
+                             bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (ctx.has_error() || !has_value) {
       return 0;
     }
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return 0;
       }
@@ -380,13 +382,13 @@ template <> struct Serializer<int64_t> {
     return ctx.read_varint64(ctx.error());
   }
 
-  static inline int64_t read_data_generic(ReadContext &ctx, bool has_generics) {
+  static inline int64_t read_data_generic(ReadContext &ctx, bool) {
     return read_data(ctx);
   }
 
-  static inline int64_t read_with_type_info(ReadContext &ctx, bool read_ref,
-                                            const TypeInfo &type_info) {
-    return read(ctx, read_ref, false);
+  static inline int64_t read_with_type_info(ReadContext &ctx, RefMode ref_mode,
+                                            const TypeInfo &) {
+    return read(ctx, ref_mode, false);
   }
 };
 
@@ -395,11 +397,11 @@ template <> struct Serializer<float> {
   static constexpr TypeId type_id = TypeId::FLOAT32;
 
   static inline void write_type_info(WriteContext &ctx) {
-    ctx.write_varuint32(static_cast<uint32_t>(type_id));
+    ctx.write_uint8(static_cast<uint8_t>(type_id));
   }
 
   static inline void read_type_info(ReadContext &ctx) {
-    uint32_t actual = ctx.read_varuint32(ctx.error());
+    uint32_t actual = ctx.read_uint8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -409,11 +411,11 @@ template <> struct Serializer<float> {
     }
   }
 
-  static inline void write(float value, WriteContext &ctx, bool write_ref,
-                           bool write_type, bool has_generics = false) {
-    write_not_null_ref_flag(ctx, write_ref);
+  static inline void write(float value, WriteContext &ctx, RefMode ref_mode,
+                           bool write_type, bool = false) {
+    write_not_null_ref_flag(ctx, ref_mode);
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
     write_data(value, ctx);
   }
@@ -422,18 +424,17 @@ template <> struct Serializer<float> {
     ctx.write_bytes(&value, sizeof(float));
   }
 
-  static inline void write_data_generic(float value, WriteContext &ctx,
-                                        bool has_generics) {
+  static inline void write_data_generic(float value, WriteContext &ctx, bool) {
     write_data(value, ctx);
   }
 
-  static inline float read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline float read(ReadContext &ctx, RefMode ref_mode, bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (ctx.has_error() || !has_value) {
       return 0.0f;
     }
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return 0.0f;
       }
@@ -450,13 +451,13 @@ template <> struct Serializer<float> {
     return ctx.read_float(ctx.error());
   }
 
-  static inline float read_data_generic(ReadContext &ctx, bool has_generics) {
+  static inline float read_data_generic(ReadContext &ctx, bool) {
     return read_data(ctx);
   }
 
-  static inline float read_with_type_info(ReadContext &ctx, bool read_ref,
-                                          const TypeInfo &type_info) {
-    return read(ctx, read_ref, false);
+  static inline float read_with_type_info(ReadContext &ctx, RefMode ref_mode,
+                                          const TypeInfo &) {
+    return read(ctx, ref_mode, false);
   }
 };
 
@@ -465,11 +466,11 @@ template <> struct Serializer<double> {
   static constexpr TypeId type_id = TypeId::FLOAT64;
 
   static inline void write_type_info(WriteContext &ctx) {
-    ctx.write_varuint32(static_cast<uint32_t>(type_id));
+    ctx.write_uint8(static_cast<uint8_t>(type_id));
   }
 
   static inline void read_type_info(ReadContext &ctx) {
-    uint32_t actual = ctx.read_varuint32(ctx.error());
+    uint32_t actual = ctx.read_uint8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -479,11 +480,11 @@ template <> struct Serializer<double> {
     }
   }
 
-  static inline void write(double value, WriteContext &ctx, bool write_ref,
-                           bool write_type, bool has_generics = false) {
-    write_not_null_ref_flag(ctx, write_ref);
+  static inline void write(double value, WriteContext &ctx, RefMode ref_mode,
+                           bool write_type, bool = false) {
+    write_not_null_ref_flag(ctx, ref_mode);
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
     write_data(value, ctx);
   }
@@ -492,18 +493,18 @@ template <> struct Serializer<double> {
     ctx.write_bytes(&value, sizeof(double));
   }
 
-  static inline void write_data_generic(double value, WriteContext &ctx,
-                                        bool has_generics) {
+  static inline void write_data_generic(double value, WriteContext &ctx, bool) {
     write_data(value, ctx);
   }
 
-  static inline double read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline double read(ReadContext &ctx, RefMode ref_mode,
+                            bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (ctx.has_error() || !has_value) {
       return 0.0;
     }
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return 0.0;
       }
@@ -520,13 +521,13 @@ template <> struct Serializer<double> {
     return ctx.read_double(ctx.error());
   }
 
-  static inline double read_data_generic(ReadContext &ctx, bool has_generics) {
+  static inline double read_data_generic(ReadContext &ctx, bool) {
     return read_data(ctx);
   }
 
-  static inline double read_with_type_info(ReadContext &ctx, bool read_ref,
-                                           const TypeInfo &type_info) {
-    return read(ctx, read_ref, false);
+  static inline double read_with_type_info(ReadContext &ctx, RefMode ref_mode,
+                                           const TypeInfo &) {
+    return read(ctx, ref_mode, false);
   }
 };
 
@@ -539,11 +540,11 @@ template <> struct Serializer<char> {
   static constexpr TypeId type_id = TypeId::CHAR;
 
   static inline void write_type_info(WriteContext &ctx) {
-    ctx.write_varuint32(static_cast<uint32_t>(type_id));
+    ctx.write_uint8(static_cast<uint8_t>(type_id));
   }
 
   static inline void read_type_info(ReadContext &ctx) {
-    uint32_t actual = ctx.read_varuint32(ctx.error());
+    uint32_t actual = ctx.read_uint8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -553,11 +554,11 @@ template <> struct Serializer<char> {
     }
   }
 
-  static inline void write(char value, WriteContext &ctx, bool write_ref,
-                           bool write_type, bool has_generics = false) {
-    write_not_null_ref_flag(ctx, write_ref);
+  static inline void write(char value, WriteContext &ctx, RefMode ref_mode,
+                           bool write_type, bool = false) {
+    write_not_null_ref_flag(ctx, ref_mode);
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
     write_data(value, ctx);
   }
@@ -566,18 +567,17 @@ template <> struct Serializer<char> {
     ctx.write_int8(static_cast<int8_t>(value));
   }
 
-  static inline void write_data_generic(char value, WriteContext &ctx,
-                                        bool has_generics) {
+  static inline void write_data_generic(char value, WriteContext &ctx, bool) {
     write_data(value, ctx);
   }
 
-  static inline char read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline char read(ReadContext &ctx, RefMode ref_mode, bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (ctx.has_error() || !has_value) {
       return '\0';
     }
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return '\0';
       }
@@ -594,13 +594,13 @@ template <> struct Serializer<char> {
     return static_cast<char>(ctx.read_int8(ctx.error()));
   }
 
-  static inline char read_data_generic(ReadContext &ctx, bool has_generics) {
+  static inline char read_data_generic(ReadContext &ctx, bool) {
     return read_data(ctx);
   }
 
-  static inline char read_with_type_info(ReadContext &ctx, bool read_ref,
-                                         const TypeInfo &type_info) {
-    return read(ctx, read_ref, false);
+  static inline char read_with_type_info(ReadContext &ctx, RefMode ref_mode,
+                                         const TypeInfo &) {
+    return read(ctx, ref_mode, false);
   }
 };
 
@@ -609,11 +609,11 @@ template <> struct Serializer<char16_t> {
   static constexpr TypeId type_id = TypeId::CHAR16;
 
   static inline void write_type_info(WriteContext &ctx) {
-    ctx.write_varuint32(static_cast<uint32_t>(type_id));
+    ctx.write_uint8(static_cast<uint8_t>(type_id));
   }
 
   static inline void read_type_info(ReadContext &ctx) {
-    uint32_t actual = ctx.read_varuint32(ctx.error());
+    uint32_t actual = ctx.read_uint8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -623,11 +623,11 @@ template <> struct Serializer<char16_t> {
     }
   }
 
-  static inline void write(char16_t value, WriteContext &ctx, bool write_ref,
-                           bool write_type, bool has_generics = false) {
-    write_not_null_ref_flag(ctx, write_ref);
+  static inline void write(char16_t value, WriteContext &ctx, RefMode ref_mode,
+                           bool write_type, bool = false) {
+    write_not_null_ref_flag(ctx, ref_mode);
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
     write_data(value, ctx);
   }
@@ -637,17 +637,18 @@ template <> struct Serializer<char16_t> {
   }
 
   static inline void write_data_generic(char16_t value, WriteContext &ctx,
-                                        bool has_generics) {
+                                        bool) {
     write_data(value, ctx);
   }
 
-  static inline char16_t read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline char16_t read(ReadContext &ctx, RefMode ref_mode,
+                              bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (ctx.has_error() || !has_value) {
       return u'\0';
     }
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return u'\0';
       }
@@ -667,14 +668,13 @@ template <> struct Serializer<char16_t> {
     return value;
   }
 
-  static inline char16_t read_data_generic(ReadContext &ctx,
-                                           bool has_generics) {
+  static inline char16_t read_data_generic(ReadContext &ctx, bool) {
     return read_data(ctx);
   }
 
-  static inline char16_t read_with_type_info(ReadContext &ctx, bool read_ref,
-                                             const TypeInfo &type_info) {
-    return read(ctx, read_ref, false);
+  static inline char16_t read_with_type_info(ReadContext &ctx, RefMode ref_mode,
+                                             const TypeInfo &) {
+    return read(ctx, ref_mode, false);
   }
 };
 
@@ -683,11 +683,11 @@ template <> struct Serializer<char32_t> {
   static constexpr TypeId type_id = TypeId::CHAR32;
 
   static inline void write_type_info(WriteContext &ctx) {
-    ctx.write_varuint32(static_cast<uint32_t>(type_id));
+    ctx.write_uint8(static_cast<uint8_t>(type_id));
   }
 
   static inline void read_type_info(ReadContext &ctx) {
-    uint32_t actual = ctx.read_varuint32(ctx.error());
+    uint32_t actual = ctx.read_uint8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -697,11 +697,11 @@ template <> struct Serializer<char32_t> {
     }
   }
 
-  static inline void write(char32_t value, WriteContext &ctx, bool write_ref,
-                           bool write_type, bool has_generics = false) {
-    write_not_null_ref_flag(ctx, write_ref);
+  static inline void write(char32_t value, WriteContext &ctx, RefMode ref_mode,
+                           bool write_type, bool = false) {
+    write_not_null_ref_flag(ctx, ref_mode);
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
     write_data(value, ctx);
   }
@@ -711,17 +711,18 @@ template <> struct Serializer<char32_t> {
   }
 
   static inline void write_data_generic(char32_t value, WriteContext &ctx,
-                                        bool has_generics) {
+                                        bool) {
     write_data(value, ctx);
   }
 
-  static inline char32_t read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline char32_t read(ReadContext &ctx, RefMode ref_mode,
+                              bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (ctx.has_error() || !has_value) {
       return U'\0';
     }
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return U'\0';
       }
@@ -741,14 +742,13 @@ template <> struct Serializer<char32_t> {
     return value;
   }
 
-  static inline char32_t read_data_generic(ReadContext &ctx,
-                                           bool has_generics) {
+  static inline char32_t read_data_generic(ReadContext &ctx, bool) {
     return read_data(ctx);
   }
 
-  static inline char32_t read_with_type_info(ReadContext &ctx, bool read_ref,
-                                             const TypeInfo &type_info) {
-    return read(ctx, read_ref, false);
+  static inline char32_t read_with_type_info(ReadContext &ctx, RefMode ref_mode,
+                                             const TypeInfo &) {
+    return read(ctx, ref_mode, false);
   }
 };
 
