@@ -19,6 +19,7 @@
 
 #pragma once
 
+#include "fory/serialization/any_serializer.h"
 #include "fory/serialization/array_serializer.h"
 #include "fory/serialization/collection_serializer.h"
 #include "fory/serialization/config.h"
@@ -31,6 +32,7 @@
 #include "fory/serialization/tuple_serializer.h"
 #include "fory/serialization/type_resolver.h"
 #include "fory/serialization/variant_serializer.h"
+#include "fory/serialization/weak_ptr_serializer.h"
 #include "fory/util/buffer.h"
 #include "fory/util/error.h"
 #include "fory/util/pool.h"
@@ -183,7 +185,7 @@ public:
   // Configuration Access
   // ==========================================================================
 
-  /// Get reference to the serialization configuration.
+  /// get reference to the serialization configuration.
   ///
   /// The configuration contains settings like xlang mode, compatible mode,
   /// reference tracking, etc.
@@ -220,8 +222,10 @@ public:
   ///
   /// Example:
   /// ```cpp
-  /// struct MyStruct { int32_t value; };
-  /// FORY_STRUCT(MyStruct, value);
+  /// struct MyStruct {
+  ///   int32_t value;
+  ///   FORY_STRUCT(MyStruct, value);
+  /// };
   ///
   /// fory.register_struct<MyStruct>(1);
   /// ```
@@ -324,6 +328,39 @@ public:
     return type_resolver_->template register_by_name<T>("", type_name);
   }
 
+  /// Register a union type with a numeric type ID.
+  ///
+  /// Use this method to register union types with generated custom serializers.
+  ///
+  /// @tparam T The union type to register (must provide Serializer<T>).
+  /// @param type_id Unique numeric identifier for this union type.
+  /// @return Success or error if registration fails.
+  template <typename T> Result<void, Error> register_union(uint32_t type_id) {
+    return type_resolver_->template register_union_by_id<T>(type_id);
+  }
+
+  /// Register a union type with namespace and type name.
+  ///
+  /// @tparam T The union type to register (must provide Serializer<T>).
+  /// @param ns Namespace for the type (can be empty string).
+  /// @param type_name Name of the type within the namespace.
+  /// @return Success or error if registration fails.
+  template <typename T>
+  Result<void, Error> register_union(const std::string &ns,
+                                     const std::string &type_name) {
+    return type_resolver_->template register_union_by_name<T>(ns, type_name);
+  }
+
+  /// Register a union type with type name only (no namespace).
+  ///
+  /// @tparam T The union type to register (must provide Serializer<T>).
+  /// @param type_name Name of the type.
+  /// @return Success or error if registration fails.
+  template <typename T>
+  Result<void, Error> register_union(const std::string &type_name) {
+    return type_resolver_->template register_union_by_name<T>("", type_name);
+  }
+
   /// Register an extension type with a numeric type ID.
   ///
   /// Extension types allow custom serialization logic for types that
@@ -431,7 +468,13 @@ public:
     if (FORY_PREDICT_FALSE(!finalized_)) {
       ensure_finalized();
     }
-    return serialize_impl(obj, buffer);
+    // Swap in the caller's buffer so all writes go there.
+    std::swap(buffer, write_ctx_->buffer());
+    auto result = serialize_impl(obj, write_ctx_->buffer());
+    std::swap(buffer, write_ctx_->buffer());
+    // reset internal state after use without clobbering caller buffer.
+    write_ctx_->reset();
+    return result;
   }
 
   /// Serialize an object to an existing byte vector (zero-copy).
@@ -484,10 +527,6 @@ public:
     if (header.is_null) {
       return Unexpected(Error::invalid_data("Cannot deserialize null object"));
     }
-    if (header.is_little_endian != is_little_endian_system()) {
-      return Unexpected(
-          Error::unsupported("Cross-endian deserialization not yet supported"));
-    }
 
     read_ctx_->attach(buffer);
     ReadContextGuard guard(*read_ctx_);
@@ -517,18 +556,10 @@ public:
     if (FORY_PREDICT_FALSE(!finalized_)) {
       ensure_finalized();
     }
-    if (buffer.reader_index() >= buffer.writer_index()) {
-      return Unexpected(Error::invalid("No data to read in buffer"));
-    }
-
     FORY_TRY(header, read_header(buffer));
     if (header.is_null) {
       return Unexpected(Error::invalid_data("Cannot deserialize null object"));
     }
-    if (header.is_little_endian != is_little_endian_system()) {
-      return Unexpected(
-          Error::unsupported("Cross-endian deserialization not yet supported"));
-    }
 
     read_ctx_->attach(buffer);
     ReadContextGuard guard(*read_ctx_);
@@ -555,16 +586,14 @@ private:
   /// Constructor for ForyBuilder - resolver will be finalized lazily.
   explicit Fory(const Config &config, std::shared_ptr<TypeResolver> resolver)
       : BaseFory(config, std::move(resolver)), finalized_(false),
-        precomputed_header_(compute_header(config.xlang)),
-        header_length_(config.xlang ? 4 : 3) {}
+        precomputed_header_(compute_header(config.xlang)) {}
 
   /// Constructor for ThreadSafeFory pool - resolver is already finalized.
   struct PreFinalized {};
   explicit Fory(const Config &config, std::shared_ptr<TypeResolver> resolver,
                 PreFinalized)
       : BaseFory(config, std::move(resolver)), finalized_(false),
-        precomputed_header_(compute_header(config.xlang)),
-        header_length_(config.xlang ? 4 : 3) {
+        precomputed_header_(compute_header(config.xlang)) {
     // Pre-finalized, immediately create contexts
     ensure_finalized();
   }
@@ -588,89 +617,59 @@ private:
   }
 
   /// Compute the precomputed header value.
-  static uint32_t compute_header(bool xlang) {
-    uint32_t header = 0;
-    // Magic number (2 bytes, little endian)
-    header |= (MAGIC_NUMBER & 0xFFFF);
-    // Flags byte at position 2
+  static uint8_t compute_header(bool xlang) {
     uint8_t flags = 0;
-    if (is_little_endian_system()) {
-      flags |= (1 << 1); // bit 1: endian flag
-    }
     if (xlang) {
-      flags |= (1 << 2); // bit 2: xlang flag
+      flags |= (1 << 1); // bit 1: xlang flag
     }
-    header |= (static_cast<uint32_t>(flags) << 16);
-    // Language byte at position 3 (only used if xlang)
-    header |= (static_cast<uint32_t>(Language::CPP) << 24);
-    return header;
+    return flags;
   }
 
   /// Core serialization implementation.
+  /// TypeMeta is written inline using streaming protocol (no deferred writing).
   template <typename T>
   Result<size_t, Error> serialize_impl(const T &obj, Buffer &buffer) {
     size_t start_pos = buffer.writer_index();
 
-    // Write precomputed header (4 bytes), then adjust index if not xlang
-    buffer.Grow(4);
-    buffer.UnsafePut<uint32_t>(buffer.writer_index(), precomputed_header_);
-    buffer.IncreaseWriterIndex(header_length_);
+    // write precomputed header (1 byte flags)
+    buffer.grow(1);
+    buffer.unsafe_put_byte(buffer.writer_index(), precomputed_header_);
+    buffer.increase_writer_index(1);
 
-    // Reserve space for meta offset in compatible mode
-    size_t meta_start_offset = 0;
-    if (write_ctx_->is_compatible()) {
-      meta_start_offset = buffer.writer_index();
-      buffer.WriteInt32(-1); // Placeholder for meta offset (fixed 4 bytes)
-    }
-
-    // Top-level serialization: YES ref flags, yes type info
-    Serializer<T>::write(obj, *write_ctx_, true, true);
+    // Top-level serialization: use Tracking if ref tracking is enabled,
+    // otherwise NullOnly for nullable handling
+    // TypeMeta is written inline during serialization (streaming protocol)
+    const RefMode top_level_ref_mode =
+        write_ctx_->track_ref() ? RefMode::Tracking : RefMode::NullOnly;
+    Serializer<T>::write(obj, *write_ctx_, top_level_ref_mode, true);
     // Check for errors at serialization boundary
     if (FORY_PREDICT_FALSE(write_ctx_->has_error())) {
       return Unexpected(write_ctx_->take_error());
     }
 
-    // Write collected TypeMetas at the end in compatible mode
-    if (write_ctx_->is_compatible() && !write_ctx_->meta_empty()) {
-      write_ctx_->write_meta(meta_start_offset);
-    }
-
     return buffer.writer_index() - start_pos;
   }
 
   /// Core deserialization implementation.
+  /// TypeMeta is read inline using streaming protocol.
   template <typename T> Result<T, Error> deserialize_impl(Buffer &buffer) {
-    // Load TypeMetas at the beginning in compatible mode
-    size_t bytes_to_skip = 0;
-    if (read_ctx_->is_compatible()) {
-      Error error;
-      int32_t meta_offset = buffer.ReadInt32(error);
-      if (FORY_PREDICT_FALSE(!error.ok())) {
-        return Unexpected(std::move(error));
-      }
-      if (meta_offset != -1) {
-        FORY_TRY(meta_size, read_ctx_->load_type_meta(meta_offset));
-        bytes_to_skip = meta_size;
-      }
-    }
-
-    // Top-level deserialization: YES ref flags, yes type info
-    T result = Serializer<T>::read(*read_ctx_, true, true);
+    // Top-level deserialization: use Tracking if ref tracking is enabled,
+    // otherwise NullOnly for nullable handling
+    // TypeMeta is read inline during deserialization (streaming protocol)
+    const RefMode top_level_ref_mode =
+        read_ctx_->track_ref() ? RefMode::Tracking : RefMode::NullOnly;
+    T result = Serializer<T>::read(*read_ctx_, top_level_ref_mode, true);
     // Check for errors at deserialization boundary
     if (FORY_PREDICT_FALSE(read_ctx_->has_error())) {
       return Unexpected(read_ctx_->take_error());
     }
 
     read_ctx_->ref_reader().resolve_callbacks();
-    if (bytes_to_skip > 0) {
-      buffer.IncreaseReaderIndex(static_cast<uint32_t>(bytes_to_skip));
-    }
     return result;
   }
 
   bool finalized_;
-  uint32_t precomputed_header_;
-  uint8_t header_length_;
+  uint8_t precomputed_header_;
   std::optional<WriteContext> write_ctx_;
   std::optional<ReadContext> read_ctx_;
 
