@@ -49,7 +49,7 @@ constexpr uint8_t DECL_VALUE_TYPE = 0b100000;
 // Type Info Methods
 // ============================================================================
 
-/// Write type info for a type to buffer.
+/// write type info for a type to buffer.
 template <typename T> inline void write_type_info(WriteContext &ctx) {
   Serializer<T>::write_type_info(ctx);
 }
@@ -61,7 +61,7 @@ template <typename T> inline void read_type_info(ReadContext &ctx) {
 
 /// Read polymorphic type info from buffer. Returns nullptr on error.
 inline const TypeInfo *read_polymorphic_type_info(ReadContext &ctx) {
-  return ctx.read_any_typeinfo(ctx.error());
+  return ctx.read_any_type_info(ctx.error());
 }
 
 // ============================================================================
@@ -81,11 +81,11 @@ struct MapReserver<MapType,
   static void reserve(MapType &map, uint32_t size) { map.reserve(size); }
 };
 
-/// Write chunk size at header offset
+/// write chunk size at header offset
 inline void write_chunk_size(WriteContext &ctx, size_t header_offset,
                              uint8_t size) {
   // header_offset points to the header byte, size is at offset + 1
-  ctx.buffer().UnsafePutByte(header_offset + 1, size);
+  ctx.buffer().unsafe_put_byte(header_offset + 1, size);
 }
 
 /// Check if we need to write type info for a field type
@@ -105,7 +105,7 @@ template <typename T> inline constexpr bool need_to_write_type_for_field() {
 // Map Data Writing - Fast Path (Non-Polymorphic)
 // ============================================================================
 
-/// Write map data for non-polymorphic, non-shared-ref maps
+/// write map data for non-polymorphic, non-shared-ref maps
 /// This is the optimized fast path for common cases like map<string, int>
 template <typename K, typename V, typename MapType>
 inline void write_map_data_fast(const MapType &map, WriteContext &ctx,
@@ -115,8 +115,8 @@ inline void write_map_data_fast(const MapType &map, WriteContext &ctx,
   static_assert(!is_shared_ref_v<K> && !is_shared_ref_v<V>,
                 "Fast path is for non-shared-ref types only");
 
-  // Write total length
-  ctx.write_varuint32(static_cast<uint32_t>(map.size()));
+  // write total length
+  ctx.write_var_uint32(static_cast<uint32_t>(map.size()));
 
   if (map.empty()) {
     return;
@@ -138,7 +138,7 @@ inline void write_map_data_fast(const MapType &map, WriteContext &ctx,
     // If nullability is needed, use the slow path
 
     if (need_write_header) {
-      // Reserve space for header (1 byte) + chunk size (1 byte)
+      // reserve space for header (1 byte) + chunk size (1 byte)
       header_offset = ctx.buffer().writer_index();
       ctx.write_uint16(0); // Placeholder for header and chunk size
       uint8_t chunk_header = 0;
@@ -153,12 +153,12 @@ inline void write_map_data_fast(const MapType &map, WriteContext &ctx,
         write_type_info<V>(ctx);
       }
 
-      // Write chunk header at reserved position
-      ctx.buffer().UnsafePutByte(header_offset, chunk_header);
+      // write chunk header at reserved position
+      ctx.buffer().unsafe_put_byte(header_offset, chunk_header);
       need_write_header = false;
     }
 
-    // Write key and value data
+    // write key and value data
     if (has_generics && is_generic_type_v<K>) {
       Serializer<K>::write_data_generic(key, ctx, true);
     } else {
@@ -179,7 +179,7 @@ inline void write_map_data_fast(const MapType &map, WriteContext &ctx,
     }
   }
 
-  // Write final chunk size
+  // write final chunk size
   if (pair_counter > 0) {
     write_chunk_size(ctx, header_offset, pair_counter);
   }
@@ -189,13 +189,13 @@ inline void write_map_data_fast(const MapType &map, WriteContext &ctx,
 // Map Data Writing - Slow Path (Polymorphic/Shared-Ref)
 // ============================================================================
 
-/// Write map data for polymorphic or shared-ref maps
+/// write map data for polymorphic or shared-ref maps
 /// This is the versatile slow path that handles all edge cases
 template <typename K, typename V, typename MapType>
 inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
                                 bool has_generics) {
-  // Write total length
-  ctx.write_varuint32(static_cast<uint32_t>(map.size()));
+  // write total length
+  ctx.write_var_uint32(static_cast<uint32_t>(map.size()));
 
   if (map.empty()) {
     return;
@@ -206,8 +206,6 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
   constexpr bool val_is_polymorphic = is_polymorphic_v<V>;
   constexpr bool key_is_shared_ref = is_shared_ref_v<K>;
   constexpr bool val_is_shared_ref = is_shared_ref_v<V>;
-  constexpr bool key_needs_ref = requires_ref_metadata_v<K>;
-  constexpr bool val_needs_ref = requires_ref_metadata_v<V>;
 
   const bool is_key_declared =
       has_generics && !need_to_write_type_for_field<K>();
@@ -220,8 +218,8 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
   bool need_write_header = true;
 
   // Track current chunk's types for polymorphic handling
-  uint32_t current_key_type_id = 0;
-  uint32_t current_val_type_id = 0;
+  const TypeInfo *current_key_type_info = nullptr;
+  const TypeInfo *current_val_type_info = nullptr;
 
   for (const auto &[key, value] : map) {
     // Check if key or value is null (for nullable types: optional, shared_ptr,
@@ -251,7 +249,8 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
         // Non-null key, null value
         // Java writes: chunk_header, then ref_flag, then type_info, then data
         uint8_t chunk_header = VALUE_NULL;
-        bool write_ref = key_is_shared_ref || key_needs_ref;
+        // Only track refs for shared_ptr types when global ref tracking enabled
+        bool write_ref = key_is_shared_ref && ctx.track_ref();
         if (write_ref) {
           chunk_header |= TRACKING_KEY_REF;
         }
@@ -260,9 +259,9 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
         }
         ctx.write_uint8(chunk_header);
 
-        // Write ref flag first if tracking refs
+        // write ref flag first if tracking refs
         if (write_ref) {
-          write_not_null_ref_flag(ctx, true);
+          write_not_null_ref_flag(ctx, RefMode::NullOnly);
         }
 
         // Then write type info if not declared
@@ -275,8 +274,14 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
                   "Polymorphic key shared_ptr must not point to void"));
               return;
             }
-            auto res = ctx.write_any_typeinfo(
-                static_cast<uint32_t>(TypeId::UNKNOWN), concrete_type_id);
+            auto type_info_res =
+                ctx.type_resolver().get_type_info(concrete_type_id);
+            if (!type_info_res.ok()) {
+              ctx.set_error(std::move(type_info_res).error());
+              return;
+            }
+            auto res = ctx.write_any_type_info(type_info_res.value()->type_id,
+                                               concrete_type_id);
             if (!res.ok()) {
               ctx.set_error(std::move(res).error());
               return;
@@ -286,7 +291,7 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
           }
         }
 
-        // Write key data (ref flag and type info already written)
+        // write key data (ref flag and type info already written)
         if (has_generics && is_generic_type_v<K>) {
           Serializer<K>::write_data_generic(key, ctx, has_generics);
         } else {
@@ -297,7 +302,8 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
         // key_is_none
         // Java writes: chunk_header, then ref_flag, then type_info, then data
         uint8_t chunk_header = KEY_NULL;
-        bool write_ref = val_is_shared_ref || val_needs_ref;
+        // Only track refs for shared_ptr types when global ref tracking enabled
+        bool write_ref = val_is_shared_ref && ctx.track_ref();
         if (write_ref) {
           chunk_header |= TRACKING_VALUE_REF;
         }
@@ -306,9 +312,9 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
         }
         ctx.write_uint8(chunk_header);
 
-        // Write ref flag first if tracking refs
+        // write ref flag first if tracking refs
         if (write_ref) {
-          write_not_null_ref_flag(ctx, true);
+          write_not_null_ref_flag(ctx, RefMode::NullOnly);
         }
 
         // Then write type info if not declared
@@ -321,8 +327,14 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
                   "Polymorphic value shared_ptr must not point to void"));
               return;
             }
-            auto res = ctx.write_any_typeinfo(
-                static_cast<uint32_t>(TypeId::UNKNOWN), concrete_type_id);
+            auto type_info_res =
+                ctx.type_resolver().get_type_info(concrete_type_id);
+            if (!type_info_res.ok()) {
+              ctx.set_error(std::move(type_info_res).error());
+              return;
+            }
+            auto res = ctx.write_any_type_info(type_info_res.value()->type_id,
+                                               concrete_type_id);
             if (!res.ok()) {
               ctx.set_error(std::move(res).error());
               return;
@@ -332,7 +344,7 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
           }
         }
 
-        // Write value data (ref flag and type info already written)
+        // write value data (ref flag and type info already written)
         if (has_generics && is_generic_type_v<V>) {
           Serializer<V>::write_data_generic(value, ctx, has_generics);
         } else {
@@ -342,9 +354,9 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
       }
     }
 
-    // Get type IDs for polymorphic types
-    uint32_t key_type_id = 0;
-    uint32_t val_type_id = 0;
+    // get type IDs for polymorphic types
+    const TypeInfo *key_type_info = nullptr;
+    const TypeInfo *val_type_info = nullptr;
     if constexpr (key_is_polymorphic) {
       auto concrete_type_id = get_concrete_type_id(key);
       auto key_type_info_res =
@@ -353,7 +365,7 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
         ctx.set_error(std::move(key_type_info_res).error());
         return;
       }
-      key_type_id = key_type_info_res.value()->type_id;
+      key_type_info = key_type_info_res.value();
     }
     if constexpr (val_is_polymorphic) {
       auto concrete_type_id = get_concrete_type_id(value);
@@ -363,14 +375,19 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
         ctx.set_error(std::move(val_type_info_res).error());
         return;
       }
-      val_type_id = val_type_info_res.value()->type_id;
+      val_type_info = val_type_info_res.value();
     }
 
     // Check if we need to start a new chunk due to type changes
     bool types_changed = false;
     if constexpr (key_is_polymorphic || val_is_polymorphic) {
-      types_changed = (key_type_id != current_key_type_id) ||
-                      (val_type_id != current_val_type_id);
+      if constexpr (key_is_polymorphic) {
+        types_changed = key_type_info != current_key_type_info;
+      }
+      if constexpr (val_is_polymorphic) {
+        types_changed =
+            types_changed || (val_type_info != current_val_type_info);
+      }
     }
 
     if (need_write_header || types_changed) {
@@ -380,39 +397,49 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
         pair_counter = 0;
       }
 
-      // Write new chunk header
+      // write new chunk header
       header_offset = ctx.buffer().writer_index();
       ctx.write_uint16(0); // Placeholder for header and chunk size
 
       uint8_t chunk_header = 0;
-      // Set key flags
-      if (key_is_shared_ref || key_needs_ref) {
+      // Set key flags - only track refs for shared_ptr when global ref tracking
+      // enabled
+      if (key_is_shared_ref && ctx.track_ref()) {
         chunk_header |= TRACKING_KEY_REF;
       }
       if (is_key_declared && !key_is_polymorphic) {
         chunk_header |= DECL_KEY_TYPE;
       }
 
-      // Set value flags
-      if (val_is_shared_ref || val_needs_ref) {
+      // Set value flags - only track refs for shared_ptr when global ref
+      // tracking enabled
+      if (val_is_shared_ref && ctx.track_ref()) {
         chunk_header |= TRACKING_VALUE_REF;
       }
       if (is_val_declared && !val_is_polymorphic) {
         chunk_header |= DECL_VALUE_TYPE;
       }
 
-      // Write chunk header at reserved position
-      ctx.buffer().UnsafePutByte(header_offset, chunk_header);
+      // write chunk header at reserved position
+      ctx.buffer().unsafe_put_byte(header_offset, chunk_header);
 
-      // Write type info if needed
+      // write type info if needed
       // Matches Rust: write type info here in map, then call serializer with
       // write_type=false
       if (!is_key_declared || key_is_polymorphic) {
         if constexpr (key_is_polymorphic) {
           auto concrete_type_id = get_concrete_type_id(key);
-          // Use UNKNOWN for polymorphic shared_ptr
-          auto res = ctx.write_any_typeinfo(
-              static_cast<uint32_t>(TypeId::UNKNOWN), concrete_type_id);
+          if (FORY_PREDICT_FALSE(key_type_info == nullptr)) {
+            auto type_info_res =
+                ctx.type_resolver().get_type_info(concrete_type_id);
+            if (!type_info_res.ok()) {
+              ctx.set_error(std::move(type_info_res).error());
+              return;
+            }
+            key_type_info = type_info_res.value();
+          }
+          auto res =
+              ctx.write_any_type_info(key_type_info->type_id, concrete_type_id);
           if (!res.ok()) {
             ctx.set_error(std::move(res).error());
             return;
@@ -425,9 +452,17 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
       if (!is_val_declared || val_is_polymorphic) {
         if constexpr (val_is_polymorphic) {
           auto concrete_type_id = get_concrete_type_id(value);
-          // Use UNKNOWN for polymorphic shared_ptr
-          auto res = ctx.write_any_typeinfo(
-              static_cast<uint32_t>(TypeId::UNKNOWN), concrete_type_id);
+          if (FORY_PREDICT_FALSE(val_type_info == nullptr)) {
+            auto type_info_res =
+                ctx.type_resolver().get_type_info(concrete_type_id);
+            if (!type_info_res.ok()) {
+              ctx.set_error(std::move(type_info_res).error());
+              return;
+            }
+            val_type_info = type_info_res.value();
+          }
+          auto res =
+              ctx.write_any_type_info(val_type_info->type_id, concrete_type_id);
           if (!res.ok()) {
             ctx.set_error(std::move(res).error());
             return;
@@ -438,17 +473,20 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
       }
 
       need_write_header = false;
-      current_key_type_id = key_type_id;
-      current_val_type_id = val_type_id;
+      current_key_type_info = key_type_info;
+      current_val_type_info = val_type_info;
     }
 
-    // Write key-value pair
-    // For polymorphic types, we've already written type info above,
-    // so we write ref flag + data directly using the serializer
+    // write key-value pair
+    // For shared_ptr with ref tracking: write ref flag + data
+    // For other types: null cases already handled via KEY_NULL/VALUE_NULL,
+    // so just write data directly
     if constexpr (key_is_shared_ref) {
-      Serializer<K>::write(key, ctx, true, false, has_generics);
-    } else if constexpr (key_needs_ref) {
-      Serializer<K>::write(key, ctx, true, false);
+      if (ctx.track_ref()) {
+        Serializer<K>::write(key, ctx, RefMode::Tracking, false, has_generics);
+      } else {
+        Serializer<K>::write_data(key, ctx);
+      }
     } else {
       if (has_generics && is_generic_type_v<K>) {
         Serializer<K>::write_data_generic(key, ctx, has_generics);
@@ -458,9 +496,12 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
     }
 
     if constexpr (val_is_shared_ref) {
-      Serializer<V>::write(value, ctx, true, false, has_generics);
-    } else if constexpr (val_needs_ref) {
-      Serializer<V>::write(value, ctx, true, false);
+      if (ctx.track_ref()) {
+        Serializer<V>::write(value, ctx, RefMode::Tracking, false,
+                             has_generics);
+      } else {
+        Serializer<V>::write_data(value, ctx);
+      }
     } else {
       if (has_generics && is_generic_type_v<V>) {
         Serializer<V>::write_data_generic(value, ctx, has_generics);
@@ -474,12 +515,12 @@ inline void write_map_data_slow(const MapType &map, WriteContext &ctx,
       write_chunk_size(ctx, header_offset, pair_counter);
       pair_counter = 0;
       need_write_header = true;
-      current_key_type_id = 0;
-      current_val_type_id = 0;
+      current_key_type_info = nullptr;
+      current_val_type_info = nullptr;
     }
   }
 
-  // Write final chunk size
+  // write final chunk size
   if (pair_counter > 0) {
     write_chunk_size(ctx, header_offset, pair_counter);
   }
@@ -512,27 +553,66 @@ inline MapType read_map_data_fast(ReadContext &ctx, uint32_t length) {
       return MapType{};
     }
 
-    // Handle null entries (shouldn't happen in fast path, but be defensive)
+    // Handle null entries - insert with default-constructed key/value
     if ((header & KEY_NULL) && (header & VALUE_NULL)) {
-      // Both null - skip for now (would need default values)
+      // Both null - insert with default values
+      result.emplace(K{}, V{});
       len_counter++;
       continue;
     }
     if (header & KEY_NULL) {
-      // Null key - read value and skip
-      Serializer<V>::read(ctx, false, false);
+      // Null key, non-null value
+      // Java writes: header, then type info (if not declared), then value data
+      bool value_declared = (header & DECL_VALUE_TYPE) != 0;
+      bool track_value_ref = (header & TRACKING_VALUE_REF) != 0;
+
+      // Read type info if not declared
+      if (!value_declared) {
+        read_type_info<V>(ctx);
+        if (FORY_PREDICT_FALSE(ctx.has_error())) {
+          return MapType{};
+        }
+      }
+
+      // Read value - consume ref flag if tracking, then read data
+      V value;
+      if (track_value_ref) {
+        value = Serializer<V>::read(ctx, RefMode::Tracking, false);
+      } else {
+        value = Serializer<V>::read_data(ctx);
+      }
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return MapType{};
       }
+      result.emplace(K{}, std::move(value));
       len_counter++;
       continue;
     }
     if (header & VALUE_NULL) {
-      // Null value - read key and skip
-      Serializer<K>::read(ctx, false, false);
+      // Non-null key, null value
+      // Java writes: header, then type info (if not declared), then key data
+      bool key_declared = (header & DECL_KEY_TYPE) != 0;
+      bool track_key_ref = (header & TRACKING_KEY_REF) != 0;
+
+      // Read type info if not declared
+      if (!key_declared) {
+        read_type_info<K>(ctx);
+        if (FORY_PREDICT_FALSE(ctx.has_error())) {
+          return MapType{};
+        }
+      }
+
+      // Read key - consume ref flag if tracking, then read data
+      K key;
+      if (track_key_ref) {
+        key = Serializer<K>::read(ctx, RefMode::Tracking, false);
+      } else {
+        key = Serializer<K>::read_data(ctx);
+      }
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return MapType{};
       }
+      result.emplace(std::move(key), V{});
       len_counter++;
       continue;
     }
@@ -626,7 +706,7 @@ inline MapType read_map_data_slow(ReadContext &ctx, uint32_t length) {
       // Consume ref flag first if tracking refs
       bool has_value = true;
       if (track_value_ref || val_is_shared_ref) {
-        has_value = consume_ref_flag(ctx, true);
+        has_value = read_null_only_flag(ctx, RefMode::NullOnly);
         if (FORY_PREDICT_FALSE(ctx.has_error())) {
           return MapType{};
         }
@@ -656,8 +736,8 @@ inline MapType read_map_data_slow(ReadContext &ctx, uint32_t length) {
       V value;
       if constexpr (val_is_polymorphic) {
         // For polymorphic types, use read_with_type_info
-        value =
-            Serializer<V>::read_with_type_info(ctx, false, *value_type_info);
+        value = Serializer<V>::read_with_type_info(ctx, RefMode::None,
+                                                   *value_type_info);
         if (FORY_PREDICT_FALSE(ctx.has_error())) {
           return MapType{};
         }
@@ -683,7 +763,7 @@ inline MapType read_map_data_slow(ReadContext &ctx, uint32_t length) {
       // Consume ref flag first if tracking refs
       bool has_key = true;
       if (track_key_ref || key_is_shared_ref) {
-        has_key = consume_ref_flag(ctx, true);
+        has_key = read_null_only_flag(ctx, RefMode::NullOnly);
         if (FORY_PREDICT_FALSE(ctx.has_error())) {
           return MapType{};
         }
@@ -712,7 +792,8 @@ inline MapType read_map_data_slow(ReadContext &ctx, uint32_t length) {
       // Read key data (ref flag already consumed above)
       K key;
       if constexpr (key_is_polymorphic) {
-        key = Serializer<K>::read_with_type_info(ctx, false, *key_type_info);
+        key = Serializer<K>::read_with_type_info(ctx, RefMode::None,
+                                                 *key_type_info);
         if (FORY_PREDICT_FALSE(ctx.has_error())) {
           return MapType{};
         }
@@ -782,13 +863,16 @@ inline MapType read_map_data_slow(ReadContext &ctx, uint32_t length) {
       // Read key - use type info if available (polymorphic case)
       K key;
       if constexpr (key_is_polymorphic) {
-        key = Serializer<K>::read_with_type_info(ctx, key_read_ref,
-                                                 *key_type_info);
+        // TRACKING_KEY_REF means full ref tracking for shared_ptr
+        key = Serializer<K>::read_with_type_info(
+            ctx, key_read_ref ? RefMode::Tracking : RefMode::None,
+            *key_type_info);
         if (FORY_PREDICT_FALSE(ctx.has_error())) {
           return MapType{};
         }
       } else if (key_read_ref) {
-        key = Serializer<K>::read(ctx, key_read_ref, false);
+        // TRACKING_KEY_REF means full ref tracking for shared_ptr
+        key = Serializer<K>::read(ctx, RefMode::Tracking, false);
         if (FORY_PREDICT_FALSE(ctx.has_error())) {
           return MapType{};
         }
@@ -803,13 +887,16 @@ inline MapType read_map_data_slow(ReadContext &ctx, uint32_t length) {
       // Read value - use type info if available (polymorphic case)
       V value;
       if constexpr (val_is_polymorphic) {
-        value = Serializer<V>::read_with_type_info(ctx, val_read_ref,
-                                                   *value_type_info);
+        // TRACKING_VALUE_REF means full ref tracking for shared_ptr
+        value = Serializer<V>::read_with_type_info(
+            ctx, val_read_ref ? RefMode::Tracking : RefMode::None,
+            *value_type_info);
         if (FORY_PREDICT_FALSE(ctx.has_error())) {
           return MapType{};
         }
       } else if (val_read_ref) {
-        value = Serializer<V>::read(ctx, val_read_ref, false);
+        // TRACKING_VALUE_REF means full ref tracking for shared_ptr
+        value = Serializer<V>::read(ctx, RefMode::Tracking, false);
         if (FORY_PREDICT_FALSE(ctx.has_error())) {
           return MapType{};
         }
@@ -840,11 +927,11 @@ struct Serializer<std::map<K, V, Args...>> {
   using MapType = std::map<K, V, Args...>;
 
   static inline void write_type_info(WriteContext &ctx) {
-    ctx.write_varuint32(static_cast<uint32_t>(type_id));
+    ctx.write_uint8(static_cast<uint8_t>(type_id));
   }
 
   static inline void read_type_info(ReadContext &ctx) {
-    const TypeInfo *type_info = ctx.read_any_typeinfo(ctx.error());
+    const TypeInfo *type_info = ctx.read_any_type_info(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -857,12 +944,12 @@ struct Serializer<std::map<K, V, Args...>> {
   // Match Rust signature: fory_write(&self, context, write_ref_info,
   // write_type_info, has_generics)
   static inline void write(const MapType &map, WriteContext &ctx,
-                           bool write_ref, bool write_type,
+                           RefMode ref_mode, bool write_type,
                            bool has_generics = false) {
-    write_not_null_ref_flag(ctx, write_ref);
+    write_not_null_ref_flag(ctx, ref_mode);
 
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
 
     write_data_generic(map, ctx, has_generics);
@@ -871,8 +958,7 @@ struct Serializer<std::map<K, V, Args...>> {
   static inline void write_data(const MapType &map, WriteContext &ctx) {
     constexpr bool is_fast_path =
         !is_polymorphic_v<K> && !is_polymorphic_v<V> && !is_shared_ref_v<K> &&
-        !is_shared_ref_v<V> && !requires_ref_metadata_v<K> &&
-        !requires_ref_metadata_v<V>;
+        !is_shared_ref_v<V> && !is_nullable_v<K> && !is_nullable_v<V>;
 
     if constexpr (is_fast_path) {
       write_map_data_fast<K, V>(map, ctx, false);
@@ -885,8 +971,7 @@ struct Serializer<std::map<K, V, Args...>> {
                                         bool has_generics) {
     constexpr bool is_fast_path =
         !is_polymorphic_v<K> && !is_polymorphic_v<V> && !is_shared_ref_v<K> &&
-        !is_shared_ref_v<V> && !requires_ref_metadata_v<K> &&
-        !requires_ref_metadata_v<V>;
+        !is_shared_ref_v<V> && !is_nullable_v<K> && !is_nullable_v<V>;
 
     if constexpr (is_fast_path) {
       write_map_data_fast<K, V>(map, ctx, has_generics);
@@ -895,8 +980,9 @@ struct Serializer<std::map<K, V, Args...>> {
     }
   }
 
-  static inline MapType read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline MapType read(ReadContext &ctx, RefMode ref_mode,
+                             bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return MapType{};
     }
@@ -905,7 +991,7 @@ struct Serializer<std::map<K, V, Args...>> {
     }
 
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return MapType{};
       }
@@ -916,15 +1002,14 @@ struct Serializer<std::map<K, V, Args...>> {
       }
     }
 
-    uint32_t length = ctx.read_varuint32(ctx.error());
+    uint32_t length = ctx.read_var_uint32(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return MapType{};
     }
 
     constexpr bool is_fast_path =
         !is_polymorphic_v<K> && !is_polymorphic_v<V> && !is_shared_ref_v<K> &&
-        !is_shared_ref_v<V> && !requires_ref_metadata_v<K> &&
-        !requires_ref_metadata_v<V>;
+        !is_shared_ref_v<V> && !is_nullable_v<K> && !is_nullable_v<V>;
 
     if constexpr (is_fast_path) {
       return read_map_data_fast<K, V, MapType>(ctx, length);
@@ -933,22 +1018,21 @@ struct Serializer<std::map<K, V, Args...>> {
     }
   }
 
-  static inline MapType read_with_type_info(ReadContext &ctx, bool read_ref,
+  static inline MapType read_with_type_info(ReadContext &ctx, RefMode ref_mode,
                                             const TypeInfo &type_info) {
     // Type info already validated, skip redundant type read
-    return read(ctx, read_ref, false); // read_type=false
+    return read(ctx, ref_mode, false); // read_type=false
   }
 
   static inline MapType read_data(ReadContext &ctx) {
-    uint32_t length = ctx.read_varuint32(ctx.error());
+    uint32_t length = ctx.read_var_uint32(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return MapType{};
     }
 
     constexpr bool is_fast_path =
         !is_polymorphic_v<K> && !is_polymorphic_v<V> && !is_shared_ref_v<K> &&
-        !is_shared_ref_v<V> && !requires_ref_metadata_v<K> &&
-        !requires_ref_metadata_v<V>;
+        !is_shared_ref_v<V> && !is_nullable_v<K> && !is_nullable_v<V>;
 
     if constexpr (is_fast_path) {
       return read_map_data_fast<K, V, MapType>(ctx, length);
@@ -968,17 +1052,16 @@ struct Serializer<std::unordered_map<K, V, Args...>> {
   using MapType = std::unordered_map<K, V, Args...>;
 
   static inline void write(const MapType &map, WriteContext &ctx,
-                           bool write_ref, bool write_type) {
-    write_not_null_ref_flag(ctx, write_ref);
+                           RefMode ref_mode, bool write_type) {
+    write_not_null_ref_flag(ctx, ref_mode);
 
     if (write_type) {
-      ctx.write_varuint32(static_cast<uint32_t>(type_id));
+      ctx.write_uint8(static_cast<uint8_t>(type_id));
     }
 
     constexpr bool is_fast_path =
         !is_polymorphic_v<K> && !is_polymorphic_v<V> && !is_shared_ref_v<K> &&
-        !is_shared_ref_v<V> && !requires_ref_metadata_v<K> &&
-        !requires_ref_metadata_v<V>;
+        !is_shared_ref_v<V> && !is_nullable_v<K> && !is_nullable_v<V>;
 
     if constexpr (is_fast_path) {
       write_map_data_fast<K, V>(map, ctx, false);
@@ -990,8 +1073,7 @@ struct Serializer<std::unordered_map<K, V, Args...>> {
   static inline void write_data(const MapType &map, WriteContext &ctx) {
     constexpr bool is_fast_path =
         !is_polymorphic_v<K> && !is_polymorphic_v<V> && !is_shared_ref_v<K> &&
-        !is_shared_ref_v<V> && !requires_ref_metadata_v<K> &&
-        !requires_ref_metadata_v<V>;
+        !is_shared_ref_v<V> && !is_nullable_v<K> && !is_nullable_v<V>;
 
     if constexpr (is_fast_path) {
       write_map_data_fast<K, V>(map, ctx, false);
@@ -1004,8 +1086,7 @@ struct Serializer<std::unordered_map<K, V, Args...>> {
                                         bool has_generics) {
     constexpr bool is_fast_path =
         !is_polymorphic_v<K> && !is_polymorphic_v<V> && !is_shared_ref_v<K> &&
-        !is_shared_ref_v<V> && !requires_ref_metadata_v<K> &&
-        !requires_ref_metadata_v<V>;
+        !is_shared_ref_v<V> && !is_nullable_v<K> && !is_nullable_v<V>;
 
     if constexpr (is_fast_path) {
       write_map_data_fast<K, V>(map, ctx, has_generics);
@@ -1014,8 +1095,9 @@ struct Serializer<std::unordered_map<K, V, Args...>> {
     }
   }
 
-  static inline MapType read(ReadContext &ctx, bool read_ref, bool read_type) {
-    bool has_value = consume_ref_flag(ctx, read_ref);
+  static inline MapType read(ReadContext &ctx, RefMode ref_mode,
+                             bool read_type) {
+    bool has_value = read_null_only_flag(ctx, ref_mode);
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return MapType{};
     }
@@ -1024,7 +1106,7 @@ struct Serializer<std::unordered_map<K, V, Args...>> {
     }
 
     if (read_type) {
-      uint32_t type_id_read = ctx.read_varuint32(ctx.error());
+      uint32_t type_id_read = ctx.read_uint8(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return MapType{};
       }
@@ -1035,15 +1117,14 @@ struct Serializer<std::unordered_map<K, V, Args...>> {
       }
     }
 
-    uint32_t length = ctx.read_varuint32(ctx.error());
+    uint32_t length = ctx.read_var_uint32(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return MapType{};
     }
 
     constexpr bool is_fast_path =
         !is_polymorphic_v<K> && !is_polymorphic_v<V> && !is_shared_ref_v<K> &&
-        !is_shared_ref_v<V> && !requires_ref_metadata_v<K> &&
-        !requires_ref_metadata_v<V>;
+        !is_shared_ref_v<V> && !is_nullable_v<K> && !is_nullable_v<V>;
 
     if constexpr (is_fast_path) {
       return read_map_data_fast<K, V, MapType>(ctx, length);
@@ -1052,22 +1133,21 @@ struct Serializer<std::unordered_map<K, V, Args...>> {
     }
   }
 
-  static inline MapType read_with_type_info(ReadContext &ctx, bool read_ref,
+  static inline MapType read_with_type_info(ReadContext &ctx, RefMode ref_mode,
                                             const TypeInfo &type_info) {
     // Type info already validated, skip redundant type read
-    return read(ctx, read_ref, false); // read_type=false
+    return read(ctx, ref_mode, false); // read_type=false
   }
 
   static inline MapType read_data(ReadContext &ctx) {
-    uint32_t length = ctx.read_varuint32(ctx.error());
+    uint32_t length = ctx.read_var_uint32(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return MapType{};
     }
 
     constexpr bool is_fast_path =
         !is_polymorphic_v<K> && !is_polymorphic_v<V> && !is_shared_ref_v<K> &&
-        !is_shared_ref_v<V> && !requires_ref_metadata_v<K> &&
-        !requires_ref_metadata_v<V>;
+        !is_shared_ref_v<V> && !is_nullable_v<K> && !is_nullable_v<V>;
 
     if constexpr (is_fast_path) {
       return read_map_data_fast<K, V, MapType>(ctx, length);
