@@ -25,6 +25,7 @@
 #include "fory/type/type.h"
 #include <array>
 #include <deque>
+#include <forward_list>
 #include <list>
 #include <map>
 #include <memory>
@@ -78,6 +79,31 @@ struct is_vector<std::vector<T, Alloc>> : std::true_type {};
 
 template <typename T> inline constexpr bool is_vector_v = is_vector<T>::value;
 
+/// Detect std::list
+template <typename T> struct is_list : std::false_type {};
+
+template <typename T, typename Alloc>
+struct is_list<std::list<T, Alloc>> : std::true_type {};
+
+template <typename T> inline constexpr bool is_list_v = is_list<T>::value;
+
+/// Detect std::deque
+template <typename T> struct is_deque : std::false_type {};
+
+template <typename T, typename Alloc>
+struct is_deque<std::deque<T, Alloc>> : std::true_type {};
+
+template <typename T> inline constexpr bool is_deque_v = is_deque<T>::value;
+
+/// Detect std::forward_list
+template <typename T> struct is_forward_list : std::false_type {};
+
+template <typename T, typename Alloc>
+struct is_forward_list<std::forward_list<T, Alloc>> : std::true_type {};
+
+template <typename T>
+inline constexpr bool is_forward_list_v = is_forward_list<T>::value;
+
 /// Detect std::optional
 template <typename T> struct is_optional : std::false_type {};
 
@@ -143,7 +169,7 @@ template <typename T> inline bool is_null_value(const T &value) {
   }
 }
 
-/// Get the inner/element type of a nullable type
+/// get the inner/element type of a nullable type
 template <typename T> struct nullable_element_type {
   using type = T;
 };
@@ -190,9 +216,9 @@ inline const nullable_element_t<T> &deref_nullable(const T &value) {
 // Fory Struct Detection
 // ============================================================================
 
-/// Check if type has FORY_FIELD_INFO defined via ADL
-/// This trait only evaluates to true if ForyFieldInfo is available AND doesn't
-/// trigger static_assert
+/// Check if type has FORY_STRUCT defined via member lookup or ADL.
+/// This trait only evaluates to true if fory_field_info is available AND
+/// doesn't trigger static_assert.
 template <typename T, typename = void>
 struct has_fory_field_info : std::false_type {};
 
@@ -204,7 +230,7 @@ struct has_fory_field_info<
 template <typename T>
 inline constexpr bool has_fory_field_info_v = has_fory_field_info<T>::value;
 
-/// Check if type is serializable (has both FORY_FIELD_INFO and
+/// Check if type is serializable (has both FORY_STRUCT and
 /// SerializationMeta)
 template <typename T, typename = void>
 struct is_fory_serializable : std::false_type {};
@@ -241,6 +267,15 @@ struct is_generic_type<std::set<T, Args...>> : std::true_type {};
 template <typename T, typename... Args>
 struct is_generic_type<std::unordered_set<T, Args...>> : std::true_type {};
 
+template <typename T, typename Alloc>
+struct is_generic_type<std::list<T, Alloc>> : std::true_type {};
+
+template <typename T, typename Alloc>
+struct is_generic_type<std::deque<T, Alloc>> : std::true_type {};
+
+template <typename T, typename Alloc>
+struct is_generic_type<std::forward_list<T, Alloc>> : std::true_type {};
+
 template <typename... Ts>
 struct is_generic_type<std::tuple<Ts...>> : std::true_type {};
 
@@ -291,7 +326,7 @@ struct is_std_unique_ptr<std::unique_ptr<T>> : std::true_type {};
 template <typename T>
 inline constexpr bool is_std_unique_ptr_v = is_std_unique_ptr<T>::value;
 
-/// Get the concrete type_index for a value
+/// get the concrete type_index for a value
 /// For non-polymorphic types, this is just typeid(T)
 /// For polymorphic types, this returns the runtime type using RTTI
 /// For smart pointers, dereferences to get the actual derived type
@@ -334,36 +369,14 @@ struct is_shared_ref<std::shared_ptr<T>> : std::true_type {};
 template <typename T>
 inline constexpr bool is_shared_ref_v = is_shared_ref<T>::value;
 
-// ============================================================================
-// Reference Metadata Requirements
-// ============================================================================
-
-/// Determine if a type requires reference metadata (null/ref flags) even when
-/// nested inside another structure.
-/// Note: std::string does NOT require ref metadata - it's written inline
-/// without ref tracking, matching Java/Rust xlang behavior.
-template <typename T> struct requires_ref_metadata : std::false_type {};
-
-template <typename T>
-struct requires_ref_metadata<std::optional<T>> : std::true_type {};
-
-template <typename T>
-struct requires_ref_metadata<std::shared_ptr<T>> : std::true_type {};
-
-template <typename T>
-struct requires_ref_metadata<std::unique_ptr<T>> : std::true_type {};
-
-template <typename T>
-inline constexpr bool requires_ref_metadata_v = requires_ref_metadata<T>::value;
-
 // ============================================================================
 // Element Type Extraction
 // ============================================================================
 
-/// Get element type for containers (reuse meta::GetValueType)
+/// get element type for containers (reuse meta::GetValueType)
 template <typename T> using element_type_t = typename meta::GetValueType<T>;
 
-/// Get key type for map-like containers
+/// get key type for map-like containers
 template <typename T, typename = void> struct key_type_impl {};
 
 template <typename T>
@@ -373,7 +386,7 @@ struct key_type_impl<T, std::void_t<typename T::key_type>> {
 
 template <typename T> using key_type_t = typename key_type_impl<T>::type;
 
-/// Get mapped type for map-like containers
+/// get mapped type for map-like containers
 template <typename T, typename = void> struct mapped_type_impl {};
 
 template <typename T>
@@ -409,11 +422,11 @@ template <> struct TypeIndex<int16_t> {
 };
 
 template <> struct TypeIndex<int32_t> {
-  static constexpr uint64_t value = static_cast<uint64_t>(TypeId::INT32);
+  static constexpr uint64_t value = static_cast<uint64_t>(TypeId::VARINT32);
 };
 
 template <> struct TypeIndex<int64_t> {
-  static constexpr uint64_t value = static_cast<uint64_t>(TypeId::INT64);
+  static constexpr uint64_t value = static_cast<uint64_t>(TypeId::VARINT64);
 };
 
 // Note: Unsigned types (uint8_t, uint16_t, uint32_t, uint64_t) use the fallback
@@ -456,6 +469,12 @@ template <typename T> struct TypeIndex<std::deque<T>> {
       fnv1a_64_combine(fnv1a_64("std::deque"), type_index<T>());
 };
 
+// forward_list<T>
+template <typename T> struct TypeIndex<std::forward_list<T>> {
+  static constexpr uint64_t value =
+      fnv1a_64_combine(fnv1a_64("std::forward_list"), type_index<T>());
+};
+
 // set<T>
 template <typename T> struct TypeIndex<std::set<T>> {
   static constexpr uint64_t value =
@@ -526,7 +545,7 @@ template <typename... Ts> struct TypeIndex<std::variant<Ts...>> {
 // type_index<T>() function - main entry point
 // ============================================================================
 
-/// Get compile-time type index for a type.
+/// get compile-time type index for a type.
 /// This is much faster than std::type_index for hash map lookups.
 /// Returns a unique uint64_t for each type.
 template <typename T> constexpr uint64_t type_index() {
