@@ -26,50 +26,34 @@ namespace serialization {
 
 namespace {
 
-// Mirror Rust's field_need_write_ref_into(type_id, nullable) for runtime
-// computation of whether a field writes ref/null flags before the value.
-inline bool field_need_write_ref_into_runtime(const FieldType &field_type) {
-  if (field_type.nullable) {
-    return true;
-  }
-  uint32_t internal = field_type.type_id & 0xffu;
-  TypeId tid = static_cast<TypeId>(internal);
-  switch (tid) {
-  case TypeId::BOOL:
-  case TypeId::INT8:
-  case TypeId::INT16:
-  case TypeId::INT32:
-  case TypeId::INT64:
-  case TypeId::FLOAT32:
-  case TypeId::FLOAT64:
-    return false;
-  default:
-    return true;
-  }
-}
-
+// Compute RefMode from field type at runtime.
+//
+// Per xlang protocol and Java's ObjectSerializer.write_other_field_value:
+// - In xlang mode with ref=false (default), fields only write
+//   ref/null flag if they are nullable
+// - Primitives never have ref flags (handled separately)
 } // namespace
 
 void skip_varint(ReadContext &ctx) {
-  // Skip varint by reading it
-  ctx.read_varuint64(ctx.error());
+  // skip varint by reading it
+  ctx.read_var_uint64(ctx.error());
 }
 
 void skip_string(ReadContext &ctx) {
   // Read string length + encoding
-  uint64_t size_encoding = ctx.read_varuint64(ctx.error());
+  uint64_t size_encoding = ctx.read_var_uint64(ctx.error());
   if (FORY_PREDICT_FALSE(ctx.has_error())) {
     return;
   }
   uint64_t size = size_encoding >> 2;
 
-  // Skip string data
-  ctx.buffer().IncreaseReaderIndex(size);
+  // skip string data
+  ctx.buffer().increase_reader_index(size);
 }
 
 void skip_list(ReadContext &ctx, const FieldType &field_type) {
   // Read list length
-  uint64_t length = ctx.read_varuint64(ctx.error());
+  uint64_t length = ctx.read_var_uint64(ctx.error());
   if (FORY_PREDICT_FALSE(ctx.has_error())) {
     return;
   }
@@ -92,7 +76,7 @@ void skip_list(ReadContext &ctx, const FieldType &field_type) {
     }
   }
 
-  // Get element type
+  // get element type
   FieldType elem_type;
   if (!field_type.generics.empty()) {
     elem_type = field_type.generics[0];
@@ -102,7 +86,7 @@ void skip_list(ReadContext &ctx, const FieldType &field_type) {
     elem_type.nullable = false;
   }
 
-  // Skip each element
+  // skip each element
   for (uint64_t i = 0; i < length; ++i) {
     if (track_ref) {
       // Read and check ref flag
@@ -124,8 +108,8 @@ void skip_list(ReadContext &ctx, const FieldType &field_type) {
       }
     }
 
-    // Skip element value
-    skip_field_value(ctx, elem_type, false); // No ref flag for elements
+    // skip element value
+    skip_field_value(ctx, elem_type, RefMode::None); // No ref flag for elements
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -139,12 +123,12 @@ void skip_set(ReadContext &ctx, const FieldType &field_type) {
 
 void skip_map(ReadContext &ctx, const FieldType &field_type) {
   // Read map length
-  uint64_t total_length = ctx.read_varuint64(ctx.error());
+  uint64_t total_length = ctx.read_var_uint64(ctx.error());
   if (FORY_PREDICT_FALSE(ctx.has_error())) {
     return;
   }
 
-  // Get key and value types
+  // get key and value types
   FieldType key_type, value_type;
   if (field_type.generics.size() >= 2) {
     key_type = field_type.generics[0];
@@ -175,9 +159,9 @@ void skip_map(ReadContext &ctx, const FieldType &field_type) {
     bool value_track_ref = (chunk_header & 0b1000) != 0;
     bool value_has_null = (chunk_header & 0b10000) != 0;
 
-    // Skip key-value pairs in this chunk
+    // skip key-value pairs in this chunk
     for (uint8_t i = 0; i < chunk_size; ++i) {
-      // Skip key with ref flag if needed
+      // skip key with ref flag if needed
       if (key_track_ref || key_has_null) {
         int8_t key_ref = ctx.read_int8(ctx.error());
         if (FORY_PREDICT_FALSE(ctx.has_error())) {
@@ -187,12 +171,12 @@ void skip_map(ReadContext &ctx, const FieldType &field_type) {
           continue; // Null or ref, skip value too
         }
       }
-      skip_field_value(ctx, key_type, false);
+      skip_field_value(ctx, key_type, RefMode::None);
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return;
       }
 
-      // Skip value with ref flag if needed
+      // skip value with ref flag if needed
       if (value_track_ref || value_has_null) {
         int8_t val_ref = ctx.read_int8(ctx.error());
         if (FORY_PREDICT_FALSE(ctx.has_error())) {
@@ -202,7 +186,7 @@ void skip_map(ReadContext &ctx, const FieldType &field_type) {
           continue; // Null or ref
         }
       }
-      skip_field_value(ctx, value_type, false);
+      skip_field_value(ctx, value_type, RefMode::None);
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return;
       }
@@ -212,10 +196,15 @@ void skip_map(ReadContext &ctx, const FieldType &field_type) {
   }
 }
 
-void skip_struct(ReadContext &ctx, const FieldType &field_type) {
+void skip_struct(ReadContext &ctx, const FieldType &) {
   // Struct fields in compatible mode are serialized with type_id and
-  // meta_index followed by field values. We use the loaded TypeMeta to
-  // skip all fields for the remote struct.
+  // optionally meta_index, followed by field values. We use the loaded
+  // TypeMeta to skip all fields for the remote struct.
+  //
+  // Type categories:
+  // - COMPATIBLE_STRUCT/NAMED_COMPATIBLE_STRUCT: type_id + meta_index + fields
+  // - NAMED_STRUCT in compatible mode: type_id + meta_index + fields
+  // - STRUCT: type_id + struct_version + fields (no meta_index)
 
   if (!ctx.is_compatible()) {
     ctx.set_error(Error::unsupported(
@@ -223,24 +212,64 @@ void skip_struct(ReadContext &ctx, const FieldType &field_type) {
     return;
   }
 
-  // Read remote type_id (ignored for now) and meta_index
-  uint32_t remote_type_id = ctx.read_varuint32(ctx.error());
-  if (FORY_PREDICT_FALSE(ctx.has_error())) {
-    return;
-  }
-  (void)remote_type_id;
-
-  uint32_t meta_index = ctx.read_varuint32(ctx.error());
+  // Read remote type_id
+  uint32_t remote_type_id = ctx.read_uint8(ctx.error());
   if (FORY_PREDICT_FALSE(ctx.has_error())) {
     return;
   }
 
-  auto type_info_res = ctx.get_type_info_by_index(meta_index);
-  if (FORY_PREDICT_FALSE(!type_info_res.ok())) {
-    ctx.set_error(std::move(type_info_res).error());
-    return;
+  uint32_t user_type_id = 0;
+  switch (static_cast<TypeId>(remote_type_id)) {
+  case TypeId::ENUM:
+  case TypeId::STRUCT:
+  case TypeId::EXT:
+  case TypeId::TYPED_UNION:
+    user_type_id = ctx.read_var_uint32(ctx.error());
+    if (FORY_PREDICT_FALSE(ctx.has_error())) {
+      return;
+    }
+    break;
+  default:
+    break;
+  }
+  TypeId remote_tid = static_cast<TypeId>(remote_type_id);
+  bool has_user_type_id =
+      remote_tid == TypeId::ENUM || remote_tid == TypeId::STRUCT ||
+      remote_tid == TypeId::EXT || remote_tid == TypeId::TYPED_UNION;
+
+  const TypeInfo *type_info = nullptr;
+
+  if (remote_tid == TypeId::COMPATIBLE_STRUCT ||
+      remote_tid == TypeId::NAMED_COMPATIBLE_STRUCT ||
+      remote_tid == TypeId::NAMED_STRUCT) {
+    // These types write TypeMeta inline using streaming protocol
+    auto type_info_res = ctx.read_type_meta();
+    if (FORY_PREDICT_FALSE(!type_info_res.ok())) {
+      ctx.set_error(std::move(type_info_res).error());
+      return;
+    }
+    type_info = type_info_res.value();
+  } else {
+    // Plain STRUCT: look up by type_id, read struct_version if enabled
+    auto type_info_res =
+        has_user_type_id
+            ? ctx.type_resolver().get_user_type_info_by_id(remote_type_id,
+                                                           user_type_id)
+            : ctx.type_resolver().get_type_info_by_id(remote_type_id);
+    if (FORY_PREDICT_FALSE(!type_info_res.ok())) {
+      ctx.set_error(std::move(type_info_res).error());
+      return;
+    }
+    type_info = type_info_res.value();
+
+    // STRUCT writes struct_version if class version checking is enabled
+    // For now we skip 4 bytes for the version hash
+    // TODO: make this conditional based on config
+    (void)ctx.read_int32(ctx.error());
+    if (FORY_PREDICT_FALSE(ctx.has_error())) {
+      return;
+    }
   }
-  const TypeInfo *type_info = type_info_res.value();
 
   if (!type_info || !type_info->type_meta) {
     ctx.set_error(
@@ -251,18 +280,19 @@ void skip_struct(ReadContext &ctx, const FieldType &field_type) {
   const auto &field_infos = type_info->type_meta->get_field_infos();
 
   for (const auto &fi : field_infos) {
-    bool read_ref = field_need_write_ref_into_runtime(fi.field_type);
-    skip_field_value(ctx, fi.field_type, read_ref);
+    // Use precomputed ref_mode from field metadata
+    skip_field_value(ctx, fi.field_type, fi.field_type.ref_mode);
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
   }
 }
 
-void skip_ext(ReadContext &ctx, const FieldType &field_type) {
-  // EXT fields in compatible mode are serialized with type_id and meta_index
-  // (for named ext) or just the user type_id (for id-based ext).
-  // We look up the registered ext harness and call its read_data function.
+void skip_ext(ReadContext &ctx, const FieldType &) {
+  // EXT fields in compatible mode are serialized with type_id followed by
+  // ext data. For named ext, meta_index is also written after type_id.
+  // We read the type_id, look up the registered ext harness, and call its
+  // read_data function to consume the data bytes.
 
   if (!ctx.is_compatible()) {
     ctx.set_error(Error::unsupported(
@@ -270,35 +300,44 @@ void skip_ext(ReadContext &ctx, const FieldType &field_type) {
     return;
   }
 
-  uint32_t full_type_id = field_type.type_id;
-  uint32_t low = full_type_id & 0xffu;
-  TypeId tid = static_cast<TypeId>(low);
+  // Read remote type_id from buffer - Java always writes type_id for ext
+  uint32_t remote_type_id = ctx.read_uint8(ctx.error());
+  if (FORY_PREDICT_FALSE(ctx.has_error())) {
+    return;
+  }
+
+  uint32_t user_type_id = 0;
+  switch (static_cast<TypeId>(remote_type_id)) {
+  case TypeId::ENUM:
+  case TypeId::STRUCT:
+  case TypeId::COMPATIBLE_STRUCT:
+  case TypeId::EXT:
+  case TypeId::TYPED_UNION:
+    user_type_id = ctx.read_var_uint32(ctx.error());
+    if (FORY_PREDICT_FALSE(ctx.has_error())) {
+      return;
+    }
+    break;
+  default:
+    break;
+  }
+  TypeId remote_tid = static_cast<TypeId>(remote_type_id);
 
   const TypeInfo *type_info = nullptr;
 
-  if (tid == TypeId::NAMED_EXT) {
-    // Named ext: read type_id and meta_index
-    uint32_t remote_type_id = ctx.read_varuint32(ctx.error());
-    if (FORY_PREDICT_FALSE(ctx.has_error())) {
-      return;
-    }
-    (void)remote_type_id;
-
-    uint32_t meta_index = ctx.read_varuint32(ctx.error());
-    if (FORY_PREDICT_FALSE(ctx.has_error())) {
-      return;
-    }
-
-    auto type_info_res = ctx.get_type_info_by_index(meta_index);
+  if (remote_tid == TypeId::NAMED_EXT) {
+    // Named ext in compatible mode: read TypeMeta inline using streaming
+    // protocol
+    auto type_info_res = ctx.read_type_meta();
     if (FORY_PREDICT_FALSE(!type_info_res.ok())) {
       ctx.set_error(std::move(type_info_res).error());
       return;
     }
     type_info = type_info_res.value();
   } else {
-    // ID-based ext: look up by full type_id
-    // The ext fields in TypeMeta store the user type_id (high bits | EXT)
-    auto type_info_res = ctx.type_resolver().get_type_info_by_id(full_type_id);
+    // ID-based ext: look up by remote type_id we just read
+    auto type_info_res = ctx.type_resolver().get_user_type_info_by_id(
+        remote_type_id, user_type_id);
     if (FORY_PREDICT_FALSE(!type_info_res.ok())) {
       ctx.set_error(std::move(type_info_res).error());
       return;
@@ -308,14 +347,14 @@ void skip_ext(ReadContext &ctx, const FieldType &field_type) {
 
   if (!type_info) {
     ctx.set_error(Error::type_error("TypeInfo not found for ext type: " +
-                                    std::to_string(full_type_id)));
+                                    std::to_string(remote_type_id)));
     return;
   }
 
   if (!type_info->harness.valid() || !type_info->harness.read_data_fn) {
     ctx.set_error(
         Error::type_error("Ext harness not found or incomplete for type: " +
-                          std::to_string(full_type_id)));
+                          std::to_string(remote_type_id)));
     return;
   }
 
@@ -347,7 +386,7 @@ void skip_unknown(ReadContext &ctx) {
   // UNKNOWN type means the actual type info is written inline.
   // We need to read the type info and then skip based on the actual type.
   // This is used for polymorphic fields like List<Animal>.
-  const TypeInfo *type_info = ctx.read_any_typeinfo(ctx.error());
+  const TypeInfo *type_info = ctx.read_any_type_info(ctx.error());
   if (FORY_PREDICT_FALSE(ctx.has_error())) {
     return;
   }
@@ -357,8 +396,7 @@ void skip_unknown(ReadContext &ctx) {
   }
 
   // Check the actual type and skip accordingly
-  uint32_t low = type_info->type_id & 0xffu;
-  TypeId actual_tid = static_cast<TypeId>(low);
+  TypeId actual_tid = static_cast<TypeId>(type_info->type_id);
 
   switch (actual_tid) {
   case TypeId::STRUCT:
@@ -373,8 +411,8 @@ void skip_unknown(ReadContext &ctx) {
     }
     const auto &field_infos = type_info->type_meta->get_field_infos();
     for (const auto &fi : field_infos) {
-      bool read_ref = field_need_write_ref_into_runtime(fi.field_type);
-      skip_field_value(ctx, fi.field_type, read_ref);
+      // Use precomputed ref_mode from field metadata
+      skip_field_value(ctx, fi.field_type, fi.field_type.ref_mode);
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return;
       }
@@ -387,7 +425,7 @@ void skip_unknown(ReadContext &ctx) {
     FieldType actual_field_type;
     actual_field_type.type_id = type_info->type_id;
     actual_field_type.nullable = false;
-    skip_field_value(ctx, actual_field_type, false);
+    skip_field_value(ctx, actual_field_type, RefMode::None);
     return;
   }
   }
@@ -395,12 +433,31 @@ void skip_unknown(ReadContext &ctx) {
 
 void skip_union(ReadContext &ctx) {
   // Read the variant index
-  (void)ctx.read_varuint32(ctx.error());
+  (void)ctx.read_var_uint32(ctx.error());
   if (FORY_PREDICT_FALSE(ctx.has_error())) {
     return;
   }
+  // Read ref flag for the union value (Any-style)
+  int8_t ref_flag = ctx.read_int8(ctx.error());
+  if (FORY_PREDICT_FALSE(ctx.has_error())) {
+    return;
+  }
+  if (ref_flag == NULL_FLAG) {
+    return;
+  }
+  if (ref_flag == REF_FLAG) {
+    (void)ctx.read_var_uint32(ctx.error());
+    return;
+  }
+  if (ref_flag != NOT_NULL_VALUE_FLAG && ref_flag != REF_VALUE_FLAG) {
+    ctx.set_error(
+        Error::invalid_data("Unknown reference flag: " +
+                            std::to_string(static_cast<int>(ref_flag))));
+    return;
+  }
+
   // Read and skip the alternative's type info
-  const TypeInfo *type_info = ctx.read_any_typeinfo(ctx.error());
+  const TypeInfo *type_info = ctx.read_any_type_info(ctx.error());
   if (FORY_PREDICT_FALSE(ctx.has_error())) {
     return;
   }
@@ -410,17 +467,17 @@ void skip_union(ReadContext &ctx) {
     return;
   }
 
-  // Skip the alternative's value
+  // skip the alternative's value
   FieldType alt_field_type;
   alt_field_type.type_id = type_info->type_id;
   alt_field_type.nullable = false;
-  skip_field_value(ctx, alt_field_type, false);
+  skip_field_value(ctx, alt_field_type, RefMode::None);
 }
 
 void skip_field_value(ReadContext &ctx, const FieldType &field_type,
-                      bool read_ref_flag) {
+                      RefMode ref_mode) {
   // Read ref flag if needed
-  if (read_ref_flag) {
+  if (ref_mode != RefMode::None) {
     int8_t ref_flag = ctx.read_int8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
@@ -430,24 +487,20 @@ void skip_field_value(ReadContext &ctx, const FieldType &field_type,
     }
   }
 
-  // Skip based on low 8 bits of the type ID.
-  //
-  // xlang user types encode the user type id in the high bits and the
-  // logical TypeId in the low 8 bits (see Java XtypeResolver and Rust
-  // TypeId conventions). For skipping we only care about the logical
-  // category (STRUCT/ENUM/EXT/etc.), so mask off the user id portion.
-  uint32_t low = field_type.type_id & 0xffu;
-  TypeId tid = static_cast<TypeId>(low);
+  // skip based on the logical TypeId (already in 0..255)
+  TypeId tid = static_cast<TypeId>(field_type.type_id);
 
   switch (tid) {
   case TypeId::BOOL:
   case TypeId::INT8:
-    ctx.buffer().IncreaseReaderIndex(1);
+  case TypeId::FLOAT8:
+    ctx.buffer().increase_reader_index(1);
     return;
 
   case TypeId::INT16:
+  case TypeId::BFLOAT16:
   case TypeId::FLOAT16:
-    ctx.buffer().IncreaseReaderIndex(2);
+    ctx.buffer().increase_reader_index(2);
     return;
 
   case TypeId::INT32: {
@@ -459,7 +512,7 @@ void skip_field_value(ReadContext &ctx, const FieldType &field_type,
   }
 
   case TypeId::FLOAT32:
-    ctx.buffer().IncreaseReaderIndex(4);
+    ctx.buffer().increase_reader_index(4);
     return;
 
   case TypeId::INT64: {
@@ -471,11 +524,11 @@ void skip_field_value(ReadContext &ctx, const FieldType &field_type,
   }
 
   case TypeId::FLOAT64:
-    ctx.buffer().IncreaseReaderIndex(8);
+    ctx.buffer().increase_reader_index(8);
     return;
 
-  case TypeId::VAR_INT32:
-  case TypeId::VAR_INT64:
+  case TypeId::VARINT32:
+  case TypeId::VARINT64:
     skip_varint(ctx);
     return;
 
@@ -495,29 +548,39 @@ void skip_field_value(ReadContext &ctx, const FieldType &field_type,
     skip_map(ctx, field_type);
     return;
 
-  case TypeId::DURATION:
-  case TypeId::TIMESTAMP: {
-    // Duration/Timestamp are stored as fixed 8-byte
-    // nanosecond counts.
-    constexpr uint32_t kBytes = static_cast<uint32_t>(sizeof(int64_t));
-    if (ctx.buffer().reader_index() + kBytes > ctx.buffer().size()) {
+  case TypeId::DURATION: {
+    // Duration is stored as fixed 8-byte nanosecond count.
+    constexpr uint32_t k_bytes = static_cast<uint32_t>(sizeof(int64_t));
+    if (ctx.buffer().reader_index() + k_bytes > ctx.buffer().size()) {
       ctx.set_error(Error::buffer_out_of_bound(ctx.buffer().reader_index(),
-                                               kBytes, ctx.buffer().size()));
+                                               k_bytes, ctx.buffer().size()));
       return;
     }
-    ctx.buffer().IncreaseReaderIndex(kBytes);
+    ctx.buffer().increase_reader_index(k_bytes);
+    return;
+  }
+  case TypeId::TIMESTAMP: {
+    // Timestamp is stored as int64 seconds + uint32 nanoseconds.
+    constexpr uint32_t k_bytes =
+        static_cast<uint32_t>(sizeof(int64_t) + sizeof(uint32_t));
+    if (ctx.buffer().reader_index() + k_bytes > ctx.buffer().size()) {
+      ctx.set_error(Error::buffer_out_of_bound(ctx.buffer().reader_index(),
+                                               k_bytes, ctx.buffer().size()));
+      return;
+    }
+    ctx.buffer().increase_reader_index(k_bytes);
     return;
   }
 
-  case TypeId::LOCAL_DATE: {
-    // LocalDate is stored as fixed 4-byte day count.
-    constexpr uint32_t kBytes = static_cast<uint32_t>(sizeof(int32_t));
-    if (ctx.buffer().reader_index() + kBytes > ctx.buffer().size()) {
+  case TypeId::DATE: {
+    // Date is stored as fixed 4-byte day count.
+    constexpr uint32_t k_bytes = static_cast<uint32_t>(sizeof(int32_t));
+    if (ctx.buffer().reader_index() + k_bytes > ctx.buffer().size()) {
       ctx.set_error(Error::buffer_out_of_bound(ctx.buffer().reader_index(),
-                                               kBytes, ctx.buffer().size()));
+                                               k_bytes, ctx.buffer().size()));
       return;
     }
-    ctx.buffer().IncreaseReaderIndex(kBytes);
+    ctx.buffer().increase_reader_index(k_bytes);
     return;
   }
 
@@ -534,11 +597,19 @@ void skip_field_value(ReadContext &ctx, const FieldType &field_type,
   case TypeId::INT16_ARRAY:
   case TypeId::INT32_ARRAY:
   case TypeId::INT64_ARRAY:
+  case TypeId::FLOAT8_ARRAY:
   case TypeId::FLOAT16_ARRAY:
+  case TypeId::BFLOAT16_ARRAY:
   case TypeId::FLOAT32_ARRAY:
   case TypeId::FLOAT64_ARRAY: {
+    if (tid == TypeId::FLOAT8_ARRAY) {
+      ctx.buffer().increase_reader_index(1);
+      if (FORY_PREDICT_FALSE(ctx.has_error())) {
+        return;
+      }
+    }
     // Read array length
-    uint32_t len = ctx.read_varuint32(ctx.error());
+    uint32_t len = ctx.read_var_uint32(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
@@ -548,6 +619,7 @@ void skip_field_value(ReadContext &ctx, const FieldType &field_type,
     switch (tid) {
     case TypeId::INT16_ARRAY:
     case TypeId::FLOAT16_ARRAY:
+    case TypeId::BFLOAT16_ARRAY:
       elem_size = 2;
       break;
     case TypeId::INT32_ARRAY:
@@ -562,24 +634,24 @@ void skip_field_value(ReadContext &ctx, const FieldType &field_type,
       break;
     }
 
-    ctx.buffer().IncreaseReaderIndex(len * elem_size);
+    ctx.buffer().increase_reader_index(len * elem_size);
     return;
   }
 
   case TypeId::BINARY: {
     // Read binary length
-    uint32_t len = ctx.read_varuint32(ctx.error());
+    uint32_t len = ctx.read_var_uint32(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return;
     }
-    ctx.buffer().IncreaseReaderIndex(len);
+    ctx.buffer().increase_reader_index(len);
     return;
   }
 
   case TypeId::ENUM:
   case TypeId::NAMED_ENUM: {
     // Enums are serialized as ordinal varuint32 values.
-    ctx.read_varuint32(ctx.error());
+    ctx.read_var_uint32(ctx.error());
     return;
   }
 
@@ -595,6 +667,10 @@ void skip_field_value(ReadContext &ctx, const FieldType &field_type,
   case TypeId::UNION:
     skip_union(ctx);
     return;
+  case TypeId::TYPED_UNION:
+  case TypeId::NAMED_UNION:
+    skip_union(ctx);
+    return;
 
   case TypeId::NONE:
     // NONE (not-applicable/monostate) has no data to skip
