@@ -29,24 +29,24 @@ namespace serialization {
 
 struct OptionalIntHolder {
   std::optional<int32_t> value;
+  FORY_STRUCT(OptionalIntHolder, value);
 };
-FORY_STRUCT(OptionalIntHolder, value);
 
 struct OptionalSharedHolder {
   std::optional<std::shared_ptr<int32_t>> value;
+  FORY_STRUCT(OptionalSharedHolder, value);
 };
-FORY_STRUCT(OptionalSharedHolder, value);
 
 struct SharedPair {
   std::shared_ptr<int32_t> first;
   std::shared_ptr<int32_t> second;
+  FORY_STRUCT(SharedPair, first, second);
 };
-FORY_STRUCT(SharedPair, first, second);
 
 struct UniqueHolder {
   std::unique_ptr<int32_t> value;
+  FORY_STRUCT(UniqueHolder, value);
 };
-FORY_STRUCT(UniqueHolder, value);
 
 namespace {
 
@@ -188,34 +188,35 @@ struct Base {
   virtual ~Base() = default;
   virtual std::string get_type() const = 0;
   int32_t base_value = 0;
+  FORY_STRUCT(Base, base_value);
 };
-FORY_STRUCT(Base, base_value);
 
 struct Derived1 : Base {
   std::string get_type() const override { return "Derived1"; }
   std::string derived1_data;
+  FORY_STRUCT(Derived1, FORY_BASE(Base), derived1_data);
 };
-FORY_STRUCT(Derived1, base_value, derived1_data);
 
 struct Derived2 : Base {
   std::string get_type() const override { return "Derived2"; }
   int32_t derived2_data = 0;
+  FORY_STRUCT(Derived2, FORY_BASE(Base), derived2_data);
 };
-FORY_STRUCT(Derived2, base_value, derived2_data);
 
 struct PolymorphicSharedHolder {
   std::shared_ptr<Base> ptr;
+  FORY_STRUCT(PolymorphicSharedHolder, ptr);
 };
-FORY_STRUCT(PolymorphicSharedHolder, ptr);
 
 struct PolymorphicUniqueHolder {
   std::unique_ptr<Base> ptr;
+  FORY_STRUCT(PolymorphicUniqueHolder, ptr);
 };
-FORY_STRUCT(PolymorphicUniqueHolder, ptr);
 
 TEST(SmartPtrSerializerTest, PolymorphicSharedPtrDerived1) {
   auto fory = create_serializer(true);
   fory.register_struct<PolymorphicSharedHolder>(200);
+  fory.register_struct<Base>("test", "Base");
   auto register_result = fory.register_struct<Derived1>("test", "Derived1");
   ASSERT_TRUE(register_result.ok())
       << "Failed to register Derived1: " << register_result.error().to_string();
@@ -244,6 +245,7 @@ TEST(SmartPtrSerializerTest, PolymorphicSharedPtrDerived1) {
 TEST(SmartPtrSerializerTest, PolymorphicSharedPtrDerived2) {
   auto fory = create_serializer(true);
   fory.register_struct<PolymorphicSharedHolder>(200);
+  fory.register_struct<Base>("test", "Base");
   auto register_result = fory.register_struct<Derived2>("test", "Derived2");
   ASSERT_TRUE(register_result.ok())
       << "Failed to register Derived2: " << register_result.error().to_string();
@@ -272,6 +274,7 @@ TEST(SmartPtrSerializerTest, PolymorphicSharedPtrDerived2) {
 TEST(SmartPtrSerializerTest, PolymorphicUniquePtrDerived1) {
   auto fory = create_serializer(true);
   fory.register_struct<PolymorphicUniqueHolder>(201);
+  fory.register_struct<Base>("test", "Base");
   auto register_result = fory.register_struct<Derived1>("test", "Derived1");
   ASSERT_TRUE(register_result.ok())
       << "Failed to register Derived1: " << register_result.error().to_string();
@@ -300,6 +303,7 @@ TEST(SmartPtrSerializerTest, PolymorphicUniquePtrDerived1) {
 TEST(SmartPtrSerializerTest, PolymorphicUniquePtrDerived2) {
   auto fory = create_serializer(true);
   fory.register_struct<PolymorphicUniqueHolder>(201);
+  fory.register_struct<Base>("test", "Base");
   auto register_result = fory.register_struct<Derived2>("test", "Derived2");
   ASSERT_TRUE(register_result.ok())
       << "Failed to register Derived2: " << register_result.error().to_string();
@@ -334,24 +338,14 @@ struct NestedContainer {
   virtual ~NestedContainer() = default;
   int32_t value = 0;
   std::shared_ptr<NestedContainer> nested;
+  FORY_STRUCT(NestedContainer, value, nested);
 };
-FORY_STRUCT(NestedContainer, value, nested);
-
-inline bool operator==(const NestedContainer &a, const NestedContainer &b) {
-  if (a.value != b.value)
-    return false;
-  if (static_cast<bool>(a.nested) != static_cast<bool>(b.nested))
-    return false;
-  if (a.nested && b.nested)
-    return *a.nested == *b.nested;
-  return true;
-}
 
 // Holder struct to wrap nested container in a polymorphic shared_ptr
 struct NestedContainerHolder {
   std::shared_ptr<NestedContainer> ptr;
+  FORY_STRUCT(NestedContainerHolder, ptr);
 };
-FORY_STRUCT(NestedContainerHolder, ptr);
 
 TEST(SmartPtrSerializerTest, MaxDynDepthExceeded) {
   // Create Fory with max_dyn_depth=2
@@ -441,5 +435,77 @@ TEST(SmartPtrSerializerTest, MaxDynDepthDefault) {
 }
 
 } // namespace
+
+// ============================================================================
+// Monomorphic field tests (fory::field<> style)
+// ============================================================================
+namespace {
+
+// A polymorphic base class (has virtual methods)
+struct PolymorphicBaseForMono {
+  virtual ~PolymorphicBaseForMono() = default;
+  virtual std::string name() const { return "PolymorphicBaseForMono"; }
+  int32_t value = 0;
+  std::string data;
+  FORY_STRUCT(PolymorphicBaseForMono, value, data);
+};
+
+// Holder with non-dynamic field using fory::field<>
+struct NonDynamicFieldHolder {
+  // Field marked as dynamic<false> - no dynamic type dispatch, always
+  // PolymorphicBaseForMono
+  fory::field<std::shared_ptr<PolymorphicBaseForMono>, 0, fory::nullable,
+              fory::dynamic<false>>
+      ptr;
+  FORY_STRUCT(NonDynamicFieldHolder, ptr);
+};
+
+TEST(SmartPtrSerializerTest, NonDynamicFieldWithForyField) {
+  NonDynamicFieldHolder original;
+  original.ptr.value = std::make_shared<PolymorphicBaseForMono>();
+  original.ptr.value->value = 42;
+  original.ptr.value->data = "test data";
+
+  auto fory = Fory::builder().track_ref(false).build();
+  fory.register_struct<NonDynamicFieldHolder>(400);
+  fory.register_struct<PolymorphicBaseForMono>(401);
+
+  auto bytes_result = fory.serialize(original);
+  ASSERT_TRUE(bytes_result.ok()) << bytes_result.error().to_string();
+
+  auto deserialize_result = fory.deserialize<NonDynamicFieldHolder>(
+      bytes_result->data(), bytes_result->size());
+  ASSERT_TRUE(deserialize_result.ok())
+      << deserialize_result.error().to_string();
+
+  auto deserialized = std::move(deserialize_result).value();
+  ASSERT_TRUE(deserialized.ptr.value);
+  EXPECT_EQ(deserialized.ptr.value->value, 42);
+  EXPECT_EQ(deserialized.ptr.value->data, "test data");
+  EXPECT_EQ(deserialized.ptr.value->name(), "PolymorphicBaseForMono");
+}
+
+TEST(SmartPtrSerializerTest, NonDynamicFieldNullValue) {
+  NonDynamicFieldHolder original;
+  original.ptr.value = nullptr;
+
+  auto fory = Fory::builder().track_ref(false).build();
+  fory.register_struct<NonDynamicFieldHolder>(404);
+  fory.register_struct<PolymorphicBaseForMono>(405);
+
+  auto bytes_result = fory.serialize(original);
+  ASSERT_TRUE(bytes_result.ok()) << bytes_result.error().to_string();
+
+  auto deserialize_result = fory.deserialize<NonDynamicFieldHolder>(
+      bytes_result->data(), bytes_result->size());
+  ASSERT_TRUE(deserialize_result.ok())
+      << deserialize_result.error().to_string();
+
+  auto deserialized = std::move(deserialize_result).value();
+  EXPECT_FALSE(deserialized.ptr.value);
+}
+
+} // namespace
+
 } // namespace serialization
-} // namespace fory
\ No newline at end of file
+} // namespace fory
