@@ -81,18 +81,17 @@ template <typename T> struct Serializer<std::optional<T>> {
   }
 
   static inline void write(const std::optional<T> &opt, WriteContext &ctx,
-                           bool write_ref, bool write_type,
+                           RefMode ref_mode, bool write_type,
                            bool has_generics = false) {
-    (void)has_generics;
-    constexpr bool inner_requires_ref = requires_ref_metadata_v<T>;
+    constexpr bool inner_is_nullable = is_nullable_v<T>;
 
-    if (!write_ref) {
+    if (ref_mode == RefMode::None) {
       if (!opt.has_value()) {
-        ctx.set_error(Error::invalid(
-            "std::optional requires write_ref=true to encode null state"));
+        ctx.set_error(Error::invalid("std::optional requires ref_mode != "
+                                     "RefMode::None to encode null state"));
         return;
       }
-      Serializer<T>::write(*opt, ctx, false, write_type);
+      Serializer<T>::write(*opt, ctx, RefMode::None, write_type, has_generics);
       return;
     }
 
@@ -101,11 +100,12 @@ template <typename T> struct Serializer<std::optional<T>> {
       return;
     }
 
-    if constexpr (inner_requires_ref) {
-      Serializer<T>::write(*opt, ctx, true, write_type);
+    if constexpr (inner_is_nullable) {
+      Serializer<T>::write(*opt, ctx, RefMode::NullOnly, write_type,
+                           has_generics);
     } else {
       ctx.write_int8(NOT_NULL_VALUE_FLAG);
-      Serializer<T>::write(*opt, ctx, false, write_type);
+      Serializer<T>::write(*opt, ctx, RefMode::None, write_type, has_generics);
     }
   }
 
@@ -129,12 +129,11 @@ template <typename T> struct Serializer<std::optional<T>> {
     Serializer<T>::write_data_generic(*opt, ctx, has_generics);
   }
 
-  static inline std::optional<T> read(ReadContext &ctx, bool read_ref,
+  static inline std::optional<T> read(ReadContext &ctx, RefMode ref_mode,
                                       bool read_type) {
-    constexpr bool inner_requires_ref = requires_ref_metadata_v<T>;
-
-    if (!read_ref) {
-      T value = Serializer<T>::read(ctx, false, read_type);
+    constexpr bool inner_is_nullable = is_nullable_v<T>;
+    if (ref_mode == RefMode::None) {
+      T value = Serializer<T>::read(ctx, RefMode::None, read_type);
       if (ctx.has_error()) {
         return std::nullopt;
       }
@@ -151,10 +150,11 @@ template <typename T> struct Serializer<std::optional<T>> {
       return std::optional<T>(std::nullopt);
     }
 
-    if constexpr (inner_requires_ref) {
+    if constexpr (inner_is_nullable) {
       // Rewind so the inner serializer can consume the reference metadata.
-      ctx.buffer().ReaderIndex(flag_pos);
-      T value = Serializer<T>::read(ctx, true, read_type);
+      ctx.buffer().reader_index(flag_pos);
+      // Pass ref_mode directly - let inner serializer handle ref tracking
+      T value = Serializer<T>::read(ctx, ref_mode, read_type);
       if (ctx.has_error()) {
         return std::nullopt;
       }
@@ -168,7 +168,7 @@ template <typename T> struct Serializer<std::optional<T>> {
       return std::nullopt;
     }
 
-    T value = Serializer<T>::read(ctx, false, read_type);
+    T value = Serializer<T>::read(ctx, RefMode::None, read_type);
     if (ctx.has_error()) {
       return std::nullopt;
     }
@@ -176,12 +176,13 @@ template <typename T> struct Serializer<std::optional<T>> {
   }
 
   static inline std::optional<T>
-  read_with_type_info(ReadContext &ctx, bool read_ref,
+  read_with_type_info(ReadContext &ctx, RefMode ref_mode,
                       const TypeInfo &type_info) {
-    constexpr bool inner_requires_ref = requires_ref_metadata_v<T>;
+    constexpr bool inner_is_nullable = is_nullable_v<T>;
 
-    if (!read_ref) {
-      T value = Serializer<T>::read_with_type_info(ctx, false, type_info);
+    if (ref_mode == RefMode::None) {
+      T value =
+          Serializer<T>::read_with_type_info(ctx, RefMode::None, type_info);
       if (ctx.has_error()) {
         return std::nullopt;
       }
@@ -198,10 +199,11 @@ template <typename T> struct Serializer<std::optional<T>> {
       return std::optional<T>(std::nullopt);
     }
 
-    if constexpr (inner_requires_ref) {
+    if constexpr (inner_is_nullable) {
       // Rewind so the inner serializer can consume the reference metadata.
-      ctx.buffer().ReaderIndex(flag_pos);
-      T value = Serializer<T>::read_with_type_info(ctx, true, type_info);
+      ctx.buffer().reader_index(flag_pos);
+      // Pass ref_mode directly - let inner serializer handle ref tracking
+      T value = Serializer<T>::read_with_type_info(ctx, ref_mode, type_info);
       if (ctx.has_error()) {
         return std::nullopt;
       }
@@ -215,7 +217,7 @@ template <typename T> struct Serializer<std::optional<T>> {
       return std::nullopt;
     }
 
-    T value = Serializer<T>::read_with_type_info(ctx, false, type_info);
+    T value = Serializer<T>::read_with_type_info(ctx, RefMode::None, type_info);
     if (ctx.has_error()) {
       return std::nullopt;
     }
@@ -250,14 +252,25 @@ template <typename T> struct SharedPtrTypeIdHelper<T, true> {
 /// Supports reference tracking for shared and circular references.
 /// When reference tracking is enabled, identical shared_ptr instances
 /// will serialize only once and use reference IDs for subsequent occurrences.
+///
+/// Note: The element type T must be a value type (struct/class or primitive).
+/// Raw pointers and nullable wrappers are not allowed as they would require
+/// nested ref metadata handling, which complicates the protocol.
 template <typename T> struct Serializer<std::shared_ptr<T>> {
+  static_assert(!std::is_pointer_v<T>,
+                "shared_ptr of raw pointer types is not supported");
+  static_assert(!is_nullable_v<T>,
+                "shared_ptr of nullable types "
+                "(optional/shared_ptr/unique_ptr/weak_ptr) is not supported. "
+                "Use the wrapper type directly instead.");
+
   static constexpr TypeId type_id =
       SharedPtrTypeIdHelper<T, std::is_polymorphic_v<T>>::value;
 
   static inline void write_type_info(WriteContext &ctx) {
     if constexpr (std::is_polymorphic_v<T>) {
       // For polymorphic types, type info must be written dynamically
-      ctx.write_varuint32(static_cast<uint32_t>(TypeId::UNKNOWN));
+      ctx.write_uint8(static_cast<uint8_t>(TypeId::UNKNOWN));
     } else {
       Serializer<T>::write_type_info(ctx);
     }
@@ -266,7 +279,7 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
   static inline void read_type_info(ReadContext &ctx) {
     if constexpr (std::is_polymorphic_v<T>) {
       // For polymorphic types, type info is read dynamically
-      ctx.read_any_typeinfo(ctx.error());
+      ctx.read_any_type_info(ctx.error());
       // Type checking is done at value read time
     } else {
       Serializer<T>::read_type_info(ctx);
@@ -274,16 +287,15 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
   }
 
   static inline void write(const std::shared_ptr<T> &ptr, WriteContext &ctx,
-                           bool write_ref, bool write_type,
+                           RefMode ref_mode, bool write_type,
                            bool has_generics = false) {
-    constexpr bool inner_requires_ref = requires_ref_metadata_v<T>;
     constexpr bool is_polymorphic = std::is_polymorphic_v<T>;
 
-    // Handle write_ref=false case (similar to Rust)
-    if (!write_ref) {
+    // Handle ref_mode == RefMode::None case (similar to Rust)
+    if (ref_mode == RefMode::None) {
       if (!ptr) {
-        ctx.set_error(Error::invalid(
-            "std::shared_ptr requires write_ref=true to encode null state"));
+        ctx.set_error(Error::invalid("std::shared_ptr requires ref_mode != "
+                                     "RefMode::None to encode null state"));
         return;
       }
       // For polymorphic types, serialize the concrete type dynamically
@@ -297,8 +309,8 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
         }
         const TypeInfo *type_info = type_info_res.value();
         if (write_type) {
-          auto write_res = ctx.write_any_typeinfo(
-              static_cast<uint32_t>(TypeId::UNKNOWN), concrete_type_id);
+          auto write_res =
+              ctx.write_any_type_info(type_info->type_id, concrete_type_id);
           if (!write_res.ok()) {
             ctx.set_error(std::move(write_res).error());
             return;
@@ -307,12 +319,13 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
         const void *value_ptr = ptr.get();
         type_info->harness.write_data_fn(value_ptr, ctx, has_generics);
       } else {
-        Serializer<T>::write(*ptr, ctx, inner_requires_ref, write_type);
+        // T is guaranteed to be a value type by static_assert.
+        Serializer<T>::write(*ptr, ctx, RefMode::None, write_type);
       }
       return;
     }
 
-    // Handle write_ref=true case
+    // Handle ref_mode != RefMode::None case
     if (!ptr) {
       ctx.write_int8(NULL_FLAG);
       return;
@@ -328,7 +341,7 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
 
     // For polymorphic types, serialize the concrete type dynamically
     if constexpr (is_polymorphic) {
-      // Get the concrete type_index from the actual object
+      // get the concrete type_index from the actual object
       std::type_index concrete_type_id = std::type_index(typeid(*ptr));
 
       // Look up the TypeInfo for the concrete type
@@ -339,10 +352,10 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
       }
       const TypeInfo *type_info = type_info_res.value();
 
-      // Write type info if requested
+      // write type info if requested
       if (write_type) {
-        auto write_res = ctx.write_any_typeinfo(
-            static_cast<uint32_t>(TypeId::UNKNOWN), concrete_type_id);
+        auto write_res =
+            ctx.write_any_type_info(type_info->type_id, concrete_type_id);
         if (!write_res.ok()) {
           ctx.set_error(std::move(write_res).error());
           return;
@@ -354,8 +367,8 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
       const void *value_ptr = ptr.get();
       type_info->harness.write_data_fn(value_ptr, ctx, has_generics);
     } else {
-      // Non-polymorphic path
-      Serializer<T>::write(*ptr, ctx, inner_requires_ref, write_type);
+      // T is guaranteed to be a value type by static_assert.
+      Serializer<T>::write(*ptr, ctx, RefMode::None, write_type);
     }
   }
 
@@ -407,30 +420,41 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
     }
   }
 
-  static inline std::shared_ptr<T> read(ReadContext &ctx, bool read_ref,
+  static inline std::shared_ptr<T> read(ReadContext &ctx, RefMode ref_mode,
                                         bool read_type) {
-    constexpr bool inner_requires_ref = requires_ref_metadata_v<T>;
     constexpr bool is_polymorphic = std::is_polymorphic_v<T>;
 
-    // Handle read_ref=false case (similar to Rust)
-    if (!read_ref) {
+    // Handle ref_mode == RefMode::None case (similar to Rust)
+    if (ref_mode == RefMode::None) {
       if constexpr (is_polymorphic) {
-        // For polymorphic types, we must read type info when read_type=true
-        if (!read_type) {
-          ctx.set_error(Error::type_error(
-              "Cannot deserialize polymorphic std::shared_ptr<T> "
-              "without type info (read_type=false)"));
-          return nullptr;
+        if (read_type) {
+          // Polymorphic path: read type info from stream to get concrete type
+          const TypeInfo *type_info = ctx.read_any_type_info(ctx.error());
+          if (ctx.has_error()) {
+            return nullptr;
+          }
+          // Now use read_with_type_info with the concrete type info
+          return read_with_type_info(ctx, ref_mode, *type_info);
+        } else {
+          // Monomorphic path: read_type=false means field is marked monomorphic
+          // Use Serializer<T>::read directly without dynamic type dispatch
+          // Note: abstract types cannot use monomorphic path
+          if constexpr (std::is_abstract_v<T>) {
+            ctx.set_error(Error::unsupported(
+                "Cannot use monomorphic deserialization for abstract type"));
+            return nullptr;
+          } else {
+            T value = Serializer<T>::read(ctx, RefMode::None, false);
+            if (ctx.has_error()) {
+              return nullptr;
+            }
+            return std::make_shared<T>(std::move(value));
+          }
         }
-        // Read type info from stream to get the concrete type
-        const TypeInfo *type_info = ctx.read_any_typeinfo(ctx.error());
-        if (ctx.has_error()) {
-          return nullptr;
-        }
-        // Now use read_with_type_info with the concrete type info
-        return read_with_type_info(ctx, read_ref, *type_info);
       } else {
-        T value = Serializer<T>::read(ctx, inner_requires_ref, read_type);
+        // T is guaranteed to be a value type (not pointer or nullable wrapper)
+        // by static_assert, so no inner ref metadata needed.
+        T value = Serializer<T>::read(ctx, RefMode::None, read_type);
         if (ctx.has_error()) {
           return nullptr;
         }
@@ -438,13 +462,13 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
       }
     }
 
-    // Handle read_ref=true case
+    // Handle ref_mode != RefMode::None case
     int8_t flag = ctx.read_int8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return nullptr;
     }
     if (flag == NULL_FLAG) {
-      return std::shared_ptr<T>(nullptr);
+      return nullptr;
     }
     const bool tracking_refs = ctx.track_ref();
     if (flag == REF_FLAG) {
@@ -453,7 +477,7 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
             "Reference flag encountered when reference tracking disabled"));
         return nullptr;
       }
-      uint32_t ref_id = ctx.read_varuint32(ctx.error());
+      uint32_t ref_id = ctx.read_var_uint32(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return nullptr;
       }
@@ -472,7 +496,8 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
     }
 
     uint32_t reserved_ref_id = 0;
-    if (flag == REF_VALUE_FLAG) {
+    const bool is_first_occurrence = flag == REF_VALUE_FLAG;
+    if (is_first_occurrence) {
       if (!tracking_refs) {
         ctx.set_error(Error::invalid_ref(
             "REF_VALUE flag encountered when reference tracking disabled"));
@@ -483,60 +508,98 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
 
     // For polymorphic types, read type info AFTER handling ref flags
     if constexpr (is_polymorphic) {
-      if (!read_type) {
-        ctx.set_error(Error::type_error(
-            "Cannot deserialize polymorphic std::shared_ptr<T> "
-            "without type info (read_type=false)"));
-        return nullptr;
-      }
+      if (read_type) {
+        // Polymorphic path: read type info and use harness for deserialization
+        // Check and increase dynamic depth for polymorphic deserialization
+        auto depth_res = ctx.increase_dyn_depth();
+        if (!depth_res.ok()) {
+          ctx.set_error(std::move(depth_res).error());
+          return nullptr;
+        }
+        DynDepthGuard dyn_depth_guard(ctx);
 
-      // Check and increase dynamic depth for polymorphic deserialization
-      auto depth_res = ctx.increase_dyn_depth();
-      if (!depth_res.ok()) {
-        ctx.set_error(std::move(depth_res).error());
-        return nullptr;
-      }
-      DynDepthGuard dyn_depth_guard(ctx);
+        // Read type info from stream to get the concrete type
+        const TypeInfo *type_info = ctx.read_any_type_info(ctx.error());
+        if (ctx.has_error()) {
+          return nullptr;
+        }
 
-      // Read type info from stream to get the concrete type
-      const TypeInfo *type_info = ctx.read_any_typeinfo(ctx.error());
-      if (ctx.has_error()) {
-        return nullptr;
+        // Use the harness to deserialize the concrete type
+        void *raw_ptr = read_polymorphic_harness_data(ctx, type_info);
+        if (FORY_PREDICT_FALSE(ctx.has_error())) {
+          return nullptr;
+        }
+        T *obj_ptr = static_cast<T *>(raw_ptr);
+        auto result = std::shared_ptr<T>(obj_ptr);
+        if (is_first_occurrence) {
+          ctx.ref_reader().store_shared_ref_at(reserved_ref_id, result);
+        }
+        return result;
+      } else {
+        // Monomorphic path: read_type=false means field is marked monomorphic
+        // Use Serializer<T>::read directly without dynamic type dispatch
+        // Note: abstract types cannot use monomorphic path
+        if constexpr (std::is_abstract_v<T>) {
+          ctx.set_error(Error::unsupported(
+              "Cannot use monomorphic deserialization for abstract type"));
+          return nullptr;
+        } else {
+          // For circular references: pre-allocate and store BEFORE reading
+          if (is_first_occurrence) {
+            auto result = std::make_shared<T>();
+            ctx.ref_reader().store_shared_ref_at(reserved_ref_id, result);
+            T value = Serializer<T>::read(ctx, RefMode::None, false);
+            if (ctx.has_error()) {
+              return nullptr;
+            }
+            *result = std::move(value);
+            return result;
+          } else {
+            T value = Serializer<T>::read(ctx, RefMode::None, false);
+            if (ctx.has_error()) {
+              return nullptr;
+            }
+            return std::make_shared<T>(std::move(value));
+          }
+        }
       }
-
-      // Use the harness to deserialize the concrete type
-      void *raw_ptr = read_polymorphic_harness_data(ctx, type_info);
-      if (FORY_PREDICT_FALSE(ctx.has_error())) {
-        return nullptr;
-      }
-      T *obj_ptr = static_cast<T *>(raw_ptr);
-      auto result = std::shared_ptr<T>(obj_ptr);
-      if (flag == REF_VALUE_FLAG) {
-        ctx.ref_reader().store_shared_ref_at(reserved_ref_id, result);
-      }
-      return result;
     } else {
-      // Non-polymorphic path
-      T value = Serializer<T>::read(ctx, inner_requires_ref, read_type);
-      if (ctx.has_error()) {
-        return nullptr;
-      }
-      auto result = std::make_shared<T>(std::move(value));
-      if (flag == REF_VALUE_FLAG) {
+      // Non-polymorphic path: T is guaranteed to be a value type (not pointer
+      // or nullable wrapper) by static_assert, so no inner ref metadata needed.
+      //
+      // For circular references: we need to pre-allocate the shared_ptr and
+      // store it BEFORE reading the struct fields. This allows forward
+      // references (like self_ref pointing back to the parent) to resolve.
+      if (is_first_occurrence) {
+        // Pre-allocate with default construction and store immediately
+        auto result = std::make_shared<T>();
         ctx.ref_reader().store_shared_ref_at(reserved_ref_id, result);
+        // Read struct data - forward refs can now find this object
+        T value = Serializer<T>::read(ctx, RefMode::None, read_type);
+        if (ctx.has_error()) {
+          return nullptr;
+        }
+        // Move-assign the read value into the pre-allocated object
+        *result = std::move(value);
+        return result;
+      } else {
+        // Not first occurrence, just read and wrap
+        T value = Serializer<T>::read(ctx, RefMode::None, read_type);
+        if (ctx.has_error()) {
+          return nullptr;
+        }
+        return std::make_shared<T>(std::move(value));
       }
-      return result;
     }
   }
 
   static inline std::shared_ptr<T>
-  read_with_type_info(ReadContext &ctx, bool read_ref,
+  read_with_type_info(ReadContext &ctx, RefMode ref_mode,
                       const TypeInfo &type_info) {
-    constexpr bool inner_requires_ref = requires_ref_metadata_v<T>;
     constexpr bool is_polymorphic = std::is_polymorphic_v<T>;
 
-    // Handle read_ref=false case (similar to Rust)
-    if (!read_ref) {
+    // Handle ref_mode == RefMode::None case (similar to Rust)
+    if (ref_mode == RefMode::None) {
       // For polymorphic types, use the harness to deserialize the concrete type
       if constexpr (is_polymorphic) {
         void *raw_ptr = read_polymorphic_harness_data(ctx, type_info);
@@ -546,9 +609,9 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
         T *obj_ptr = static_cast<T *>(raw_ptr);
         return std::shared_ptr<T>(obj_ptr);
       } else {
-        // Non-polymorphic path
-        T value = Serializer<T>::read_with_type_info(ctx, inner_requires_ref,
-                                                     type_info);
+        // T is guaranteed to be a value type by static_assert.
+        T value =
+            Serializer<T>::read_with_type_info(ctx, RefMode::None, type_info);
         if (ctx.has_error()) {
           return nullptr;
         }
@@ -556,14 +619,14 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
       }
     }
 
-    // Handle read_ref=true case
+    // Handle ref_mode != RefMode::None case
     int8_t flag = ctx.read_int8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return nullptr;
     }
 
     if (flag == NULL_FLAG) {
-      return std::shared_ptr<T>(nullptr);
+      return nullptr;
     }
     const bool tracking_refs = ctx.track_ref();
     if (flag == REF_FLAG) {
@@ -572,7 +635,7 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
             "Reference flag encountered when reference tracking disabled"));
         return nullptr;
       }
-      uint32_t ref_id = ctx.read_varuint32(ctx.error());
+      uint32_t ref_id = ctx.read_var_uint32(ctx.error());
       if (FORY_PREDICT_FALSE(ctx.has_error())) {
         return nullptr;
       }
@@ -622,18 +685,27 @@ template <typename T> struct Serializer<std::shared_ptr<T>> {
       }
       return result;
     } else {
-      // Non-polymorphic path
-      T value = Serializer<T>::read_with_type_info(ctx, inner_requires_ref,
-                                                   type_info);
-      if (ctx.has_error()) {
-        return nullptr;
-      }
-      auto result = std::make_shared<T>(std::move(value));
-      if (flag == REF_VALUE_FLAG) {
+      // T is guaranteed to be a value type by static_assert.
+      // For circular references: pre-allocate and store BEFORE reading
+      const bool is_first_occurrence = flag == REF_VALUE_FLAG;
+      if (is_first_occurrence) {
+        auto result = std::make_shared<T>();
         ctx.ref_reader().store_shared_ref_at(reserved_ref_id, result);
+        T value =
+            Serializer<T>::read_with_type_info(ctx, RefMode::None, type_info);
+        if (ctx.has_error()) {
+          return nullptr;
+        }
+        *result = std::move(value);
+        return result;
+      } else {
+        T value =
+            Serializer<T>::read_with_type_info(ctx, RefMode::None, type_info);
+        if (ctx.has_error()) {
+          return nullptr;
+        }
+        return std::make_shared<T>(std::move(value));
       }
-
-      return result;
     }
   }
 
@@ -665,21 +737,31 @@ template <typename T> struct UniquePtrTypeIdHelper<T, true> {
 /// Note: unique_ptr does not support reference tracking since
 /// it represents exclusive ownership. Each unique_ptr is serialized
 /// independently.
+///
+/// Note: The element type T must be a value type (struct/class or primitive).
+/// Raw pointers and nullable wrappers are not allowed as they would require
+/// nested ref metadata handling, which complicates the protocol.
 template <typename T> struct Serializer<std::unique_ptr<T>> {
+  static_assert(!std::is_pointer_v<T>,
+                "unique_ptr of raw pointer types is not supported");
+  static_assert(!is_nullable_v<T>,
+                "unique_ptr of nullable types "
+                "(optional/shared_ptr/unique_ptr/weak_ptr) is not supported. "
+                "Use the wrapper type directly instead.");
+
   static constexpr TypeId type_id =
       UniquePtrTypeIdHelper<T, std::is_polymorphic_v<T>>::value;
 
   static inline void write(const std::unique_ptr<T> &ptr, WriteContext &ctx,
-                           bool write_ref, bool write_type,
+                           RefMode ref_mode, bool write_type,
                            bool has_generics = false) {
-    constexpr bool inner_requires_ref = requires_ref_metadata_v<T>;
     constexpr bool is_polymorphic = std::is_polymorphic_v<T>;
 
-    // Handle write_ref=false case (similar to Rust)
-    if (!write_ref) {
+    // Handle ref_mode == RefMode::None case (similar to Rust)
+    if (ref_mode == RefMode::None) {
       if (!ptr) {
-        ctx.set_error(Error::invalid(
-            "std::unique_ptr requires write_ref=true to encode null state"));
+        ctx.set_error(Error::invalid("std::unique_ptr requires ref_mode != "
+                                     "RefMode::None to encode null state"));
         return;
       }
       // For polymorphic types, serialize the concrete type dynamically
@@ -693,8 +775,8 @@ template <typename T> struct Serializer<std::unique_ptr<T>> {
         }
         const TypeInfo *type_info = type_info_res.value();
         if (write_type) {
-          auto write_res = ctx.write_any_typeinfo(
-              static_cast<uint32_t>(TypeId::UNKNOWN), concrete_type_id);
+          auto write_res =
+              ctx.write_any_type_info(type_info->type_id, concrete_type_id);
           if (!write_res.ok()) {
             ctx.set_error(std::move(write_res).error());
             return;
@@ -703,12 +785,13 @@ template <typename T> struct Serializer<std::unique_ptr<T>> {
         const void *value_ptr = ptr.get();
         type_info->harness.write_data_fn(value_ptr, ctx, has_generics);
       } else {
-        Serializer<T>::write(*ptr, ctx, inner_requires_ref, write_type);
+        // T is guaranteed to be a value type by static_assert.
+        Serializer<T>::write(*ptr, ctx, RefMode::None, write_type);
       }
       return;
     }
 
-    // Handle write_ref=true case
+    // Handle ref_mode != RefMode::None case
     if (!ptr) {
       ctx.write_int8(NULL_FLAG);
       return;
@@ -726,8 +809,8 @@ template <typename T> struct Serializer<std::unique_ptr<T>> {
       }
       const TypeInfo *type_info = type_info_res.value();
       if (write_type) {
-        auto write_res = ctx.write_any_typeinfo(
-            static_cast<uint32_t>(TypeId::UNKNOWN), concrete_type_id);
+        auto write_res =
+            ctx.write_any_type_info(type_info->type_id, concrete_type_id);
         if (!write_res.ok()) {
           ctx.set_error(std::move(write_res).error());
           return;
@@ -736,7 +819,8 @@ template <typename T> struct Serializer<std::unique_ptr<T>> {
       const void *value_ptr = ptr.get();
       type_info->harness.write_data_fn(value_ptr, ctx, has_generics);
     } else {
-      Serializer<T>::write(*ptr, ctx, inner_requires_ref, write_type);
+      // T is guaranteed to be a value type by static_assert.
+      Serializer<T>::write(*ptr, ctx, RefMode::None, write_type);
     }
   }
 
@@ -786,30 +870,40 @@ template <typename T> struct Serializer<std::unique_ptr<T>> {
     }
   }
 
-  static inline std::unique_ptr<T> read(ReadContext &ctx, bool read_ref,
+  static inline std::unique_ptr<T> read(ReadContext &ctx, RefMode ref_mode,
                                         bool read_type) {
-    constexpr bool inner_requires_ref = requires_ref_metadata_v<T>;
     constexpr bool is_polymorphic = std::is_polymorphic_v<T>;
 
-    // Handle read_ref=false case (similar to Rust)
-    if (!read_ref) {
+    // Handle ref_mode == RefMode::None case (similar to Rust)
+    if (ref_mode == RefMode::None) {
       if constexpr (is_polymorphic) {
-        // For polymorphic types, we must read type info when read_type=true
-        if (!read_type) {
-          ctx.set_error(Error::type_error(
-              "Cannot deserialize polymorphic std::unique_ptr<T> "
-              "without type info (read_type=false)"));
-          return nullptr;
+        if (read_type) {
+          // Polymorphic path: read type info from stream to get concrete type
+          const TypeInfo *type_info = ctx.read_any_type_info(ctx.error());
+          if (ctx.has_error()) {
+            return nullptr;
+          }
+          // Now use read_with_type_info with the concrete type info
+          return read_with_type_info(ctx, ref_mode, *type_info);
+        } else {
+          // Monomorphic path: read_type=false means field is marked monomorphic
+          // Use Serializer<T>::read directly without dynamic type dispatch
+          // Note: abstract types cannot use monomorphic path
+          if constexpr (std::is_abstract_v<T>) {
+            ctx.set_error(Error::unsupported(
+                "Cannot use monomorphic deserialization for abstract type"));
+            return nullptr;
+          } else {
+            T value = Serializer<T>::read(ctx, RefMode::None, false);
+            if (ctx.has_error()) {
+              return nullptr;
+            }
+            return std::make_unique<T>(std::move(value));
+          }
         }
-        // Read type info from stream to get the concrete type
-        const TypeInfo *type_info = ctx.read_any_typeinfo(ctx.error());
-        if (ctx.has_error()) {
-          return nullptr;
-        }
-        // Now use read_with_type_info with the concrete type info
-        return read_with_type_info(ctx, read_ref, *type_info);
       } else {
-        T value = Serializer<T>::read(ctx, inner_requires_ref, read_type);
+        // T is guaranteed to be a value type by static_assert.
+        T value = Serializer<T>::read(ctx, RefMode::None, read_type);
         if (ctx.has_error()) {
           return nullptr;
         }
@@ -817,13 +911,13 @@ template <typename T> struct Serializer<std::unique_ptr<T>> {
       }
     }
 
-    // Handle read_ref=true case
+    // Handle ref_mode != RefMode::None case
     int8_t flag = ctx.read_int8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return nullptr;
     }
     if (flag == NULL_FLAG) {
-      return std::unique_ptr<T>(nullptr);
+      return nullptr;
     }
     if (flag != NOT_NULL_VALUE_FLAG) {
       ctx.set_error(
@@ -834,37 +928,48 @@ template <typename T> struct Serializer<std::unique_ptr<T>> {
 
     // For polymorphic types, read type info AFTER handling ref flags
     if constexpr (is_polymorphic) {
-      if (!read_type) {
-        ctx.set_error(Error::type_error(
-            "Cannot deserialize polymorphic std::unique_ptr<T> "
-            "without type info (read_type=false)"));
-        return nullptr;
-      }
+      if (read_type) {
+        // Polymorphic path: read type info and use harness for deserialization
+        // Check and increase dynamic depth for polymorphic deserialization
+        auto depth_res = ctx.increase_dyn_depth();
+        if (!depth_res.ok()) {
+          ctx.set_error(std::move(depth_res).error());
+          return nullptr;
+        }
+        DynDepthGuard dyn_depth_guard(ctx);
 
-      // Check and increase dynamic depth for polymorphic deserialization
-      auto depth_res = ctx.increase_dyn_depth();
-      if (!depth_res.ok()) {
-        ctx.set_error(std::move(depth_res).error());
-        return nullptr;
-      }
-      DynDepthGuard dyn_depth_guard(ctx);
+        // Read type info from stream to get the concrete type
+        const TypeInfo *type_info = ctx.read_any_type_info(ctx.error());
+        if (ctx.has_error()) {
+          return nullptr;
+        }
 
-      // Read type info from stream to get the concrete type
-      const TypeInfo *type_info = ctx.read_any_typeinfo(ctx.error());
-      if (ctx.has_error()) {
-        return nullptr;
+        // Use the harness to deserialize the concrete type
+        void *raw_ptr = read_polymorphic_harness_data(ctx, type_info);
+        if (FORY_PREDICT_FALSE(ctx.has_error())) {
+          return nullptr;
+        }
+        T *obj_ptr = static_cast<T *>(raw_ptr);
+        return std::unique_ptr<T>(obj_ptr);
+      } else {
+        // Monomorphic path: read_type=false means field is marked monomorphic
+        // Use Serializer<T>::read directly without dynamic type dispatch
+        // Note: abstract types cannot use monomorphic path
+        if constexpr (std::is_abstract_v<T>) {
+          ctx.set_error(Error::unsupported(
+              "Cannot use monomorphic deserialization for abstract type"));
+          return nullptr;
+        } else {
+          T value = Serializer<T>::read(ctx, RefMode::None, false);
+          if (ctx.has_error()) {
+            return nullptr;
+          }
+          return std::make_unique<T>(std::move(value));
+        }
       }
-
-      // Use the harness to deserialize the concrete type
-      void *raw_ptr = read_polymorphic_harness_data(ctx, type_info);
-      if (FORY_PREDICT_FALSE(ctx.has_error())) {
-        return nullptr;
-      }
-      T *obj_ptr = static_cast<T *>(raw_ptr);
-      return std::unique_ptr<T>(obj_ptr);
     } else {
-      // Non-polymorphic path
-      T value = Serializer<T>::read(ctx, inner_requires_ref, read_type);
+      // T is guaranteed to be a value type by static_assert.
+      T value = Serializer<T>::read(ctx, RefMode::None, read_type);
       if (ctx.has_error()) {
         return nullptr;
       }
@@ -873,13 +978,12 @@ template <typename T> struct Serializer<std::unique_ptr<T>> {
   }
 
   static inline std::unique_ptr<T>
-  read_with_type_info(ReadContext &ctx, bool read_ref,
+  read_with_type_info(ReadContext &ctx, RefMode ref_mode,
                       const TypeInfo &type_info) {
-    constexpr bool inner_requires_ref = requires_ref_metadata_v<T>;
     constexpr bool is_polymorphic = std::is_polymorphic_v<T>;
 
-    // Handle read_ref=false case (similar to Rust)
-    if (!read_ref) {
+    // Handle ref_mode == RefMode::None case (similar to Rust)
+    if (ref_mode == RefMode::None) {
       // For polymorphic types, use the harness to deserialize the concrete type
       if constexpr (is_polymorphic) {
         void *raw_ptr = read_polymorphic_harness_data(ctx, type_info);
@@ -889,9 +993,9 @@ template <typename T> struct Serializer<std::unique_ptr<T>> {
         T *obj_ptr = static_cast<T *>(raw_ptr);
         return std::unique_ptr<T>(obj_ptr);
       } else {
-        // Non-polymorphic path
-        T value = Serializer<T>::read_with_type_info(ctx, inner_requires_ref,
-                                                     type_info);
+        // T is guaranteed to be a value type by static_assert.
+        T value =
+            Serializer<T>::read_with_type_info(ctx, RefMode::None, type_info);
         if (ctx.has_error()) {
           return nullptr;
         }
@@ -899,13 +1003,13 @@ template <typename T> struct Serializer<std::unique_ptr<T>> {
       }
     }
 
-    // Handle read_ref=true case
+    // Handle ref_mode != RefMode::None case
     int8_t flag = ctx.read_int8(ctx.error());
     if (FORY_PREDICT_FALSE(ctx.has_error())) {
       return nullptr;
     }
     if (flag == NULL_FLAG) {
-      return std::unique_ptr<T>(nullptr);
+      return nullptr;
     }
     if (flag != NOT_NULL_VALUE_FLAG) {
       ctx.set_error(
@@ -932,9 +1036,9 @@ template <typename T> struct Serializer<std::unique_ptr<T>> {
       T *obj_ptr = static_cast<T *>(raw_ptr);
       return std::unique_ptr<T>(obj_ptr);
     } else {
-      // Non-polymorphic path
-      T value = Serializer<T>::read_with_type_info(ctx, inner_requires_ref,
-                                                   type_info);
+      // T is guaranteed to be a value type by static_assert.
+      T value =
+          Serializer<T>::read_with_type_info(ctx, RefMode::None, type_info);
       if (ctx.has_error()) {
         return nullptr;
       }
