@@ -30,6 +30,7 @@
  */
 
 #include "fory/serialization/fory.h"
+#include "fory/type/type.h"
 #include "gtest/gtest.h"
 #include <cfloat>
 #include <climits>
@@ -39,7 +40,8 @@
 #include <vector>
 
 // ============================================================================
-// ALL STRUCT DEFINITIONS MUST BE AT GLOBAL SCOPE (for FORY_STRUCT macro)
+// FORY_STRUCT can be declared inside the struct/class definition or at
+// namespace scope.
 // ============================================================================
 
 // Edge cases
@@ -48,8 +50,8 @@ struct SingleFieldStruct {
   bool operator==(const SingleFieldStruct &other) const {
     return value == other.value;
   }
+  FORY_STRUCT(SingleFieldStruct, value);
 };
-FORY_STRUCT(SingleFieldStruct, value);
 
 struct TwoFieldStruct {
   int32_t x;
@@ -57,8 +59,8 @@ struct TwoFieldStruct {
   bool operator==(const TwoFieldStruct &other) const {
     return x == other.x && y == other.y;
   }
+  FORY_STRUCT(TwoFieldStruct, x, y);
 };
-FORY_STRUCT(TwoFieldStruct, x, y);
 
 struct ManyFieldsStruct {
   bool b1;
@@ -75,8 +77,53 @@ struct ManyFieldsStruct {
            i32 == other.i32 && i64 == other.i64 && f32 == other.f32 &&
            f64 == other.f64 && str == other.str;
   }
+  FORY_STRUCT(ManyFieldsStruct, b1, i8, i16, i32, i64, f32, f64, str);
+};
+
+struct FieldConfigTaggedStruct {
+  int32_t a;
+  int64_t b;
+  std::string c;
+
+  bool operator==(const FieldConfigTaggedStruct &other) const {
+    return a == other.a && b == other.b && c == other.c;
+  }
+  FORY_STRUCT(FieldConfigTaggedStruct, a, b, c);
+};
+
+struct FieldTagsTaggedStruct {
+  int32_t a;
+  int64_t b;
+  std::string c;
+
+  bool operator==(const FieldTagsTaggedStruct &other) const {
+    return a == other.a && b == other.b && c == other.c;
+  }
+  FORY_STRUCT(FieldTagsTaggedStruct, a, b, c);
+};
+
+FORY_FIELD_CONFIG(FieldConfigTaggedStruct, (a, fory::F().id(1)),
+                  (b, fory::F().id(2)), (c, fory::F().id(3)));
+FORY_FIELD_TAGS(FieldTagsTaggedStruct, (a, 1), (b, 2), (c, 3));
+
+class PrivateFieldsStruct {
+public:
+  PrivateFieldsStruct() = default;
+  PrivateFieldsStruct(int32_t id, std::string name, std::vector<int32_t> scores)
+      : id_(id), name_(std::move(name)), scores_(std::move(scores)) {}
+
+  bool operator==(const PrivateFieldsStruct &other) const {
+    return id_ == other.id_ && name_ == other.name_ && scores_ == other.scores_;
+  }
+
+private:
+  int32_t id_ = 0;
+  std::string name_;
+  std::vector<int32_t> scores_;
+
+public:
+  FORY_STRUCT(PrivateFieldsStruct, id_, name_, scores_);
 };
-FORY_STRUCT(ManyFieldsStruct, b1, i8, i16, i32, i64, f32, f64, str);
 
 // All primitives
 struct AllPrimitivesStruct {
@@ -100,10 +147,10 @@ struct AllPrimitivesStruct {
            uint64_val == other.uint64_val && float_val == other.float_val &&
            double_val == other.double_val;
   }
+  FORY_STRUCT(AllPrimitivesStruct, bool_val, int8_val, int16_val, int32_val,
+              int64_val, uint8_val, uint16_val, uint32_val, uint64_val,
+              float_val, double_val);
 };
-FORY_STRUCT(AllPrimitivesStruct, bool_val, int8_val, int16_val, int32_val,
-            int64_val, uint8_val, uint16_val, uint32_val, uint64_val, float_val,
-            double_val);
 
 // String handling
 struct StringTestStruct {
@@ -116,8 +163,8 @@ struct StringTestStruct {
     return empty == other.empty && ascii == other.ascii && utf8 == other.utf8 &&
            long_text == other.long_text;
   }
+  FORY_STRUCT(StringTestStruct, empty, ascii, utf8, long_text);
 };
-FORY_STRUCT(StringTestStruct, empty, ascii, utf8, long_text);
 
 // Nested structs
 struct Point2D {
@@ -126,8 +173,8 @@ struct Point2D {
   bool operator==(const Point2D &other) const {
     return x == other.x && y == other.y;
   }
+  FORY_STRUCT(Point2D, x, y);
 };
-FORY_STRUCT(Point2D, x, y);
 
 struct Point3D {
   int32_t x;
@@ -136,8 +183,8 @@ struct Point3D {
   bool operator==(const Point3D &other) const {
     return x == other.x && y == other.y && z == other.z;
   }
+  FORY_STRUCT(Point3D, x, y, z);
 };
-FORY_STRUCT(Point3D, x, y, z);
 
 struct Rectangle {
   Point2D top_left;
@@ -145,8 +192,8 @@ struct Rectangle {
   bool operator==(const Rectangle &other) const {
     return top_left == other.top_left && bottom_right == other.bottom_right;
   }
+  FORY_STRUCT(Rectangle, top_left, bottom_right);
 };
-FORY_STRUCT(Rectangle, top_left, bottom_right);
 
 struct BoundingBox {
   Rectangle bounds;
@@ -154,8 +201,8 @@ struct BoundingBox {
   bool operator==(const BoundingBox &other) const {
     return bounds == other.bounds && label == other.label;
   }
+  FORY_STRUCT(BoundingBox, bounds, label);
 };
-FORY_STRUCT(BoundingBox, bounds, label);
 
 struct Scene {
   Point3D camera;
@@ -165,8 +212,22 @@ struct Scene {
     return camera == other.camera && light == other.light &&
            viewport == other.viewport;
   }
+  FORY_STRUCT(Scene, camera, light, viewport);
 };
-FORY_STRUCT(Scene, camera, light, viewport);
+
+struct EvolvingStruct {
+  int32_t id;
+
+  FORY_STRUCT(EvolvingStruct, id);
+};
+
+struct FixedStruct {
+  int32_t id;
+
+  FORY_STRUCT(FixedStruct, id);
+};
+
+FORY_STRUCT_EVOLVING(FixedStruct, false);
 
 // Containers
 struct VectorStruct {
@@ -178,8 +239,18 @@ struct VectorStruct {
     return numbers == other.numbers && strings == other.strings &&
            points == other.points;
   }
+  FORY_STRUCT(VectorStruct, numbers, strings, points);
+};
+
+struct NamedItem {
+  int32_t id;
+  std::string name;
+
+  bool operator==(const NamedItem &other) const {
+    return id == other.id && name == other.name;
+  }
+  FORY_STRUCT(NamedItem, id, name);
 };
-FORY_STRUCT(VectorStruct, numbers, strings, points);
 
 struct MapStruct {
   std::map<std::string, int32_t> str_to_int;
@@ -190,8 +261,8 @@ struct MapStruct {
     return str_to_int == other.str_to_int && int_to_str == other.int_to_str &&
            named_points == other.named_points;
   }
+  FORY_STRUCT(MapStruct, str_to_int, int_to_str, named_points);
 };
-FORY_STRUCT(MapStruct, str_to_int, int_to_str, named_points);
 
 struct NestedContainerStruct {
   std::vector<std::vector<int32_t>> matrix;
@@ -200,8 +271,8 @@ struct NestedContainerStruct {
   bool operator==(const NestedContainerStruct &other) const {
     return matrix == other.matrix && grouped_numbers == other.grouped_numbers;
   }
+  FORY_STRUCT(NestedContainerStruct, matrix, grouped_numbers);
 };
-FORY_STRUCT(NestedContainerStruct, matrix, grouped_numbers);
 
 // Optional fields
 struct OptionalFieldsStruct {
@@ -214,8 +285,8 @@ struct OptionalFieldsStruct {
     return name == other.name && age == other.age && email == other.email &&
            location == other.location;
   }
+  FORY_STRUCT(OptionalFieldsStruct, name, age, email, location);
 };
-FORY_STRUCT(OptionalFieldsStruct, name, age, email, location);
 
 // Enums
 enum class Color { RED = 0, GREEN = 1, BLUE = 2 };
@@ -227,8 +298,8 @@ struct EnumStruct {
   bool operator==(const EnumStruct &other) const {
     return color == other.color && status == other.status;
   }
+  FORY_STRUCT(EnumStruct, color, status);
 };
-FORY_STRUCT(EnumStruct, color, status);
 
 // Real-world scenarios
 struct UserProfile {
@@ -248,9 +319,9 @@ struct UserProfile {
            follower_count == other.follower_count &&
            is_verified == other.is_verified;
   }
+  FORY_STRUCT(UserProfile, user_id, username, email, bio, interests, metadata,
+              follower_count, is_verified);
 };
-FORY_STRUCT(UserProfile, user_id, username, email, bio, interests, metadata,
-            follower_count, is_verified);
 
 struct Product {
   int64_t product_id;
@@ -267,9 +338,9 @@ struct Product {
            stock == other.stock && tags == other.tags &&
            attributes == other.attributes;
   }
+  FORY_STRUCT(Product, product_id, name, description, price, stock, tags,
+              attributes);
 };
-FORY_STRUCT(Product, product_id, name, description, price, stock, tags,
-            attributes);
 
 struct OrderItem {
   int64_t product_id;
@@ -280,8 +351,8 @@ struct OrderItem {
     return product_id == other.product_id && quantity == other.quantity &&
            unit_price == other.unit_price;
   }
+  FORY_STRUCT(OrderItem, product_id, quantity, unit_price);
 };
-FORY_STRUCT(OrderItem, product_id, quantity, unit_price);
 
 struct Order {
   int64_t order_id;
@@ -295,8 +366,53 @@ struct Order {
            items == other.items && total_amount == other.total_amount &&
            order_status == other.order_status;
   }
+  FORY_STRUCT(Order, order_id, customer_id, items, total_amount, order_status);
 };
-FORY_STRUCT(Order, order_id, customer_id, items, total_amount, order_status);
+
+namespace nested_test {
+namespace inner {
+
+struct InClassStruct {
+  int32_t id;
+  std::string name;
+  bool operator==(const InClassStruct &other) const {
+    return id == other.id && name == other.name;
+  }
+  FORY_STRUCT(InClassStruct, id, name);
+};
+
+struct OutClassStruct {
+  int32_t id;
+  std::string name;
+  bool operator==(const OutClassStruct &other) const {
+    return id == other.id && name == other.name;
+  }
+};
+
+FORY_STRUCT(OutClassStruct, id, name);
+
+} // namespace inner
+} // namespace nested_test
+
+namespace external_test {
+
+struct ExternalStruct {
+  int32_t id;
+  std::string name;
+  bool operator==(const ExternalStruct &other) const {
+    return id == other.id && name == other.name;
+  }
+};
+
+FORY_STRUCT(ExternalStruct, id, name);
+
+struct ExternalEmpty {
+  bool operator==(const ExternalEmpty & /*other*/) const { return true; }
+};
+
+FORY_STRUCT(ExternalEmpty);
+
+} // namespace external_test
 
 // ============================================================================
 // TEST IMPLEMENTATION (Inside namespace)
@@ -314,6 +430,7 @@ inline void register_all_test_types(Fory &fory) {
   fory.register_struct<SingleFieldStruct>(type_id++);
   fory.register_struct<TwoFieldStruct>(type_id++);
   fory.register_struct<ManyFieldsStruct>(type_id++);
+  fory.register_struct<PrivateFieldsStruct>(type_id++);
   fory.register_struct<AllPrimitivesStruct>(type_id++);
   fory.register_struct<StringTestStruct>(type_id++);
   fory.register_struct<Point2D>(type_id++);
@@ -330,6 +447,12 @@ inline void register_all_test_types(Fory &fory) {
   fory.register_struct<Product>(type_id++);
   fory.register_struct<OrderItem>(type_id++);
   fory.register_struct<Order>(type_id++);
+  fory.register_struct<nested_test::inner::InClassStruct>(type_id++);
+  fory.register_struct<nested_test::inner::OutClassStruct>(type_id++);
+  fory.register_struct<external_test::ExternalStruct>(type_id++);
+  fory.register_struct<external_test::ExternalEmpty>(type_id++);
+  fory.register_enum<Color>(type_id++);
+  fory.register_enum<Status>(type_id++);
 }
 
 template <typename T> void test_roundtrip(const T &original) {
@@ -377,6 +500,32 @@ TEST(StructComprehensiveTest, ManyFieldsStruct) {
                                   -9223372036854775807LL - 1, -1.0f, -1.0, ""});
 }
 
+TEST(StructComprehensiveTest, FieldTagsMatchFieldConfigSize) {
+  FieldConfigTaggedStruct config_obj{1, 2, "config"};
+  FieldTagsTaggedStruct tags_obj{1, 2, "config"};
+
+  auto fory_config =
+      Fory::builder().xlang(true).compatible(true).track_ref(false).build();
+  auto fory_tags =
+      Fory::builder().xlang(true).compatible(true).track_ref(false).build();
+
+  ASSERT_TRUE(fory_config.register_struct<FieldConfigTaggedStruct>(101).ok());
+  ASSERT_TRUE(fory_tags.register_struct<FieldTagsTaggedStruct>(101).ok());
+
+  auto config_bytes = fory_config.serialize(config_obj);
+  ASSERT_TRUE(config_bytes.ok())
+      << "Serialization failed: " << config_bytes.error().to_string();
+  auto tags_bytes = fory_tags.serialize(tags_obj);
+  ASSERT_TRUE(tags_bytes.ok())
+      << "Serialization failed: " << tags_bytes.error().to_string();
+
+  EXPECT_EQ(config_bytes->size(), tags_bytes->size());
+}
+
+TEST(StructComprehensiveTest, PrivateFieldsStruct) {
+  test_roundtrip(PrivateFieldsStruct{42, "secret", {1, 2, 3}});
+}
+
 TEST(StructComprehensiveTest, AllPrimitivesZero) {
   test_roundtrip(AllPrimitivesStruct{false, 0, 0, 0, 0, 0, 0, 0, 0, 0.0f, 0.0});
 }
@@ -433,6 +582,25 @@ TEST(StructComprehensiveTest, VectorStructMultiple) {
   test_roundtrip(VectorStruct{{1, 2, 3}, {"foo", "bar"}, {{0, 0}, {10, 10}}});
 }
 
+TEST(StructComprehensiveTest, NamedStructElementTypeInfo) {
+  std::vector<NamedItem> items{{1, "alpha"}, {2, "beta"}};
+
+  auto fory = Fory::builder().xlang(true).track_ref(false).build();
+  ASSERT_TRUE(fory.register_struct<NamedItem>("test", "NamedItem").ok());
+
+  auto ser_result = fory.serialize(items);
+  ASSERT_TRUE(ser_result.ok())
+      << "Serialization failed: " << ser_result.error().to_string();
+
+  std::vector<uint8_t> bytes = std::move(ser_result).value();
+  auto deser_result =
+      fory.deserialize<std::vector<NamedItem>>(bytes.data(), bytes.size());
+  ASSERT_TRUE(deser_result.ok())
+      << "Deserialization failed: " << deser_result.error().to_string();
+
+  EXPECT_EQ(items, deser_result.value());
+}
+
 TEST(StructComprehensiveTest, MapStructEmpty) {
   test_roundtrip(MapStruct{{}, {}, {}});
 }
@@ -525,6 +693,39 @@ TEST(StructComprehensiveTest, LargeVectorOfStructs) {
   EXPECT_EQ(points, deser_result.value());
 }
 
+TEST(StructComprehensiveTest, NestedNamespaceInClassStruct) {
+  test_roundtrip(nested_test::inner::InClassStruct{7, "in"});
+}
+
+TEST(StructComprehensiveTest, NestedNamespaceOutClassStruct) {
+  test_roundtrip(nested_test::inner::OutClassStruct{8, "out"});
+}
+
+TEST(StructComprehensiveTest, ExternalStruct) {
+  test_roundtrip(external_test::ExternalStruct{1, "external"});
+  test_roundtrip(external_test::ExternalStruct{42, ""});
+}
+
+TEST(StructComprehensiveTest, ExternalEmptyStruct) {
+  test_roundtrip(external_test::ExternalEmpty{});
+}
+
+TEST(StructComprehensiveTest, StructEvolvingOverride) {
+  auto fory =
+      Fory::builder().xlang(true).compatible(true).track_ref(false).build();
+  ASSERT_TRUE(fory.register_struct<EvolvingStruct>(1).ok());
+  ASSERT_TRUE(fory.register_struct<FixedStruct>(2).ok());
+
+  auto evolving_info = fory.type_resolver().get_type_info<EvolvingStruct>();
+  ASSERT_TRUE(evolving_info.ok());
+  EXPECT_EQ(evolving_info.value()->type_id,
+            static_cast<uint32_t>(TypeId::COMPATIBLE_STRUCT));
+
+  auto fixed_info = fory.type_resolver().get_type_info<FixedStruct>();
+  ASSERT_TRUE(fixed_info.ok());
+  EXPECT_EQ(fixed_info.value()->type_id, static_cast<uint32_t>(TypeId::STRUCT));
+}
+
 } // namespace test
 } // namespace serialization
 } // namespace fory
