@@ -71,57 +71,57 @@ Buffer::~Buffer() {
   }
 }
 
-bool Buffer::Equals(const Buffer &other, int64_t nbytes) const {
+bool Buffer::equals(const Buffer &other, int64_t nbytes) const {
   return this == &other ||
          (size_ >= nbytes && other.size_ >= nbytes &&
           (data_ == other.data_ ||
            !memcmp(data_, other.data_, static_cast<size_t>(nbytes))));
 }
 
-bool Buffer::Equals(const Buffer &other) const {
+bool Buffer::equals(const Buffer &other) const {
   return this == &other ||
          (size_ == other.size_ &&
           (data_ == other.data_ ||
            !memcmp(data_, other.data_, static_cast<size_t>(size_))));
 }
 
-void Buffer::Copy(const uint32_t start, const uint32_t nbytes,
+void Buffer::copy(const uint32_t start, const uint32_t nbytes,
                   std::shared_ptr<Buffer> &out) const {
   std::memcpy(out->data(), data_ + start, static_cast<size_t>(nbytes));
 }
 
-void Buffer::Copy(uint32_t start, uint32_t nbytes, Buffer &out) const {
+void Buffer::copy(uint32_t start, uint32_t nbytes, Buffer &out) const {
   std::memcpy(out.data(), data_ + start, static_cast<size_t>(nbytes));
 }
 
-void Buffer::Copy(uint32_t start, uint32_t nbytes, uint8_t *out) const {
-  Copy(start, nbytes, out, 0);
+void Buffer::copy(uint32_t start, uint32_t nbytes, uint8_t *out) const {
+  copy(start, nbytes, out, 0);
 }
 
-void Buffer::Copy(uint32_t start, uint32_t nbytes, uint8_t *out,
+void Buffer::copy(uint32_t start, uint32_t nbytes, uint8_t *out,
                   uint32_t offset) const {
   std::memcpy(out + offset, data_ + start, static_cast<size_t>(nbytes));
 }
 
-void Buffer::CopyFrom(uint32_t offset, const uint8_t *src, uint32_t src_offset,
-                      uint32_t nbytes) {
+void Buffer::copy_from(uint32_t offset, const uint8_t *src, uint32_t src_offset,
+                       uint32_t nbytes) {
   auto new_size = offset + nbytes;
   if (new_size > size_) {
-    Reserve(new_size * 2);
+    reserve(new_size * 2);
   }
   std::memcpy(data_ + offset, src + src_offset, static_cast<size_t>(nbytes));
 }
 
-std::string Buffer::ToString() const {
+std::string Buffer::to_string() const {
   return std::string(reinterpret_cast<const char *>(data_),
                      static_cast<size_t>(size_));
 }
 
-std::string Buffer::Hex() const {
+std::string Buffer::hex() const {
   return util::hex(data(), static_cast<int32_t>(size_));
 }
 
-bool AllocateBuffer(uint32_t size, std::shared_ptr<Buffer> *out) {
+bool allocate_buffer(uint32_t size, std::shared_ptr<Buffer> *out) {
   auto *data = static_cast<uint8_t *>(malloc(static_cast<size_t>(size)));
   if (data) {
     *out = std::make_shared<Buffer>(data, size);
@@ -131,7 +131,7 @@ bool AllocateBuffer(uint32_t size, std::shared_ptr<Buffer> *out) {
   }
 }
 
-bool AllocateBuffer(uint32_t size, Buffer **out) {
+bool allocate_buffer(uint32_t size, Buffer **out) {
   auto *data = static_cast<uint8_t *>(malloc(static_cast<size_t>(size)));
   if (data) {
     *out = new Buffer(data, size);
@@ -141,7 +141,7 @@ bool AllocateBuffer(uint32_t size, Buffer **out) {
   }
 }
 
-Buffer *AllocateBuffer(uint32_t size) {
+Buffer *allocate_buffer(uint32_t size) {
   auto data = static_cast<uint8_t *>(malloc(static_cast<size_t>(size)));
   if (data) {
     return new Buffer(data, size);
