@@ -30,7 +30,7 @@
 namespace fory {
 
 // Generate ASCII string
-std::string generateAscii(size_t length) {
+std::string generate_ascii(size_t length) {
   const char charset[] =
       "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
   std::default_random_engine rng(std::random_device{}());
@@ -46,55 +46,55 @@ std::string generateAscii(size_t length) {
 
 TEST(StringUtilTest, TestisAsciiLogic) {
   // Test strings with only Ascii characters
-  EXPECT_TRUE(isAscii("Fory"));
-  EXPECT_TRUE(isAscii(generateAscii(80)));
+  EXPECT_TRUE(is_ascii("Fory"));
+  EXPECT_TRUE(is_ascii(generate_ascii(80)));
 
   // Test unaligned strings with only Ascii characters
-  EXPECT_TRUE(isAscii(generateAscii(80) + "1"));
-  EXPECT_TRUE(isAscii(generateAscii(80) + "12"));
-  EXPECT_TRUE(isAscii(generateAscii(80) + "123"));
+  EXPECT_TRUE(is_ascii(generate_ascii(80) + "1"));
+  EXPECT_TRUE(is_ascii(generate_ascii(80) + "12"));
+  EXPECT_TRUE(is_ascii(generate_ascii(80) + "123"));
 
   // Test strings with non-Ascii characters
-  EXPECT_FALSE(isAscii("你好, Fory"));
-  EXPECT_FALSE(isAscii(generateAscii(80) + "你好"));
-  EXPECT_FALSE(isAscii(generateAscii(80) + "1你好"));
-  EXPECT_FALSE(isAscii(generateAscii(11) + "你"));
-  EXPECT_FALSE(isAscii(generateAscii(10) + "你好"));
-  EXPECT_FALSE(isAscii(generateAscii(9) + "性能好"));
-  EXPECT_FALSE(isAscii("\u1234"));
-  EXPECT_FALSE(isAscii("a\u1234"));
-  EXPECT_FALSE(isAscii("ab\u1234"));
-  EXPECT_FALSE(isAscii("abc\u1234"));
-  EXPECT_FALSE(isAscii("abcd\u1234"));
-  EXPECT_FALSE(isAscii("Javaone Keynote\u1234"));
+  EXPECT_FALSE(is_ascii("你好, Fory"));
+  EXPECT_FALSE(is_ascii(generate_ascii(80) + "你好"));
+  EXPECT_FALSE(is_ascii(generate_ascii(80) + "1你好"));
+  EXPECT_FALSE(is_ascii(generate_ascii(11) + "你"));
+  EXPECT_FALSE(is_ascii(generate_ascii(10) + "你好"));
+  EXPECT_FALSE(is_ascii(generate_ascii(9) + "性能好"));
+  EXPECT_FALSE(is_ascii("\u1234"));
+  EXPECT_FALSE(is_ascii("a\u1234"));
+  EXPECT_FALSE(is_ascii("ab\u1234"));
+  EXPECT_FALSE(is_ascii("abc\u1234"));
+  EXPECT_FALSE(is_ascii("abcd\u1234"));
+  EXPECT_FALSE(is_ascii("Javaone Keynote\u1234"));
 
   for (size_t i = 1; i < 256; i++) {
-    EXPECT_TRUE(isAscii(std::string(i, '.') + "Fory"));
-    EXPECT_FALSE(isAscii(std::string(i, '.') + "序列化"));
+    EXPECT_TRUE(is_ascii(std::string(i, '.') + "Fory"));
+    EXPECT_FALSE(is_ascii(std::string(i, '.') + "序列化"));
   }
 }
 
 TEST(StringUtilTest, TestisLatin1) {
   // Test strings with only Latin characters
-  EXPECT_TRUE(isLatin1(u"Fory"));
-  EXPECT_TRUE(isLatin1(u"\xE9")); // é in Latin-1
-  EXPECT_TRUE(isLatin1(u"\xF1")); // ñ in Latin-1
+  EXPECT_TRUE(is_latin1(u"Fory"));
+  EXPECT_TRUE(is_latin1(u"\xE9")); // é in Latin-1
+  EXPECT_TRUE(is_latin1(u"\xF1")); // ñ in Latin-1
   // Test strings with non-Latin characters
-  EXPECT_FALSE(isLatin1(u"你好, Fory"));
-  EXPECT_FALSE(isLatin1(u"a\u1234"));
-  EXPECT_FALSE(isLatin1(u"ab\u1234"));
-  EXPECT_FALSE(isLatin1(u"abc\u1234"));
-  EXPECT_FALSE(isLatin1(u"abcd\u1234"));
-  EXPECT_FALSE(isLatin1(u"Javaone Keynote\u1234"));
-  EXPECT_TRUE(isLatin1(u"a\xFF")); // ÿ in Latin-1
-  EXPECT_TRUE(isLatin1(u"\x80"));  //  in Latin-1
+  EXPECT_FALSE(is_latin1(u"你好, Fory"));
+  EXPECT_FALSE(is_latin1(u"a\u1234"));
+  EXPECT_FALSE(is_latin1(u"ab\u1234"));
+  EXPECT_FALSE(is_latin1(u"abc\u1234"));
+  EXPECT_FALSE(is_latin1(u"abcd\u1234"));
+  EXPECT_FALSE(is_latin1(u"Javaone Keynote\u1234"));
+  EXPECT_TRUE(is_latin1(u"a\xFF")); // ÿ in Latin-1
+  EXPECT_TRUE(is_latin1(u"\x80"));  //  in Latin-1
   for (size_t i = 1; i < 256; i++) {
-    EXPECT_TRUE(isLatin1(std::u16string(i, '.') + u"Fory"));
-    EXPECT_FALSE(isLatin1(std::u16string(i, '.') + u"序列化"));
-    EXPECT_TRUE(isLatin1(std::u16string(i, '.') + u"a\xFF")); // ÿ in Latin-1
-    EXPECT_TRUE(isLatin1(std::u16string(i, '.') + u"\x80"));  //  in Latin-1
-    EXPECT_FALSE(isLatin1(std::u16string(i, '.') +
-                          std::u16string({256}))); // Ā (not in Latin-1)
+    EXPECT_TRUE(is_latin1(std::u16string(i, '.') + u"Fory"));
+    EXPECT_FALSE(is_latin1(std::u16string(i, '.') + u"序列化"));
+    EXPECT_TRUE(is_latin1(std::u16string(i, '.') + u"a\xFF")); // ÿ in Latin-1
+    EXPECT_TRUE(is_latin1(std::u16string(i, '.') + u"\x80"));  //  in Latin-1
+    EXPECT_FALSE(is_latin1(std::u16string(i, '.') +
+                           std::u16string({256}))); // Ā (not in Latin-1)
   }
 }
 
@@ -146,7 +146,7 @@ TEST(StringUtilTest, ToSnakeCaseHandlesDelimitersAndDigits) {
 }
 
 // Generate random UTF-16 string ensuring valid surrogate pairs
-std::u16string generateRandomUTF16String(size_t length) {
+std::u16string generate_random_utf16_string(size_t length) {
   std::u16string str;
   std::mt19937 generator(std::random_device{}());
   std::uniform_int_distribution<uint32_t> distribution(0, 0x10FFFF);
@@ -168,49 +168,50 @@ std::u16string generateRandomUTF16String(size_t length) {
 }
 
 TEST(StringUtilTest, TestUtf16HasSurrogatePairs) {
-  EXPECT_FALSE(utf16HasSurrogatePairs(std::u16string({0x99, 0x100})));
+  EXPECT_FALSE(utf16_has_surrogate_pairs(std::u16string({0x99, 0x100})));
   std::u16string utf16 = {0xD83D, 0xDE00}; // 😀 emoji
-  EXPECT_TRUE(utf16HasSurrogatePairs(utf16));
-  EXPECT_TRUE(utf16HasSurrogatePairs(generateRandomUTF16String(3) + u"性能好"));
+  EXPECT_TRUE(utf16_has_surrogate_pairs(utf16));
   EXPECT_TRUE(
-      utf16HasSurrogatePairs(generateRandomUTF16String(10) + u"性能好"));
+      utf16_has_surrogate_pairs(generate_random_utf16_string(3) + u"性能好"));
   EXPECT_TRUE(
-      utf16HasSurrogatePairs(generateRandomUTF16String(30) + u"性能好"));
+      utf16_has_surrogate_pairs(generate_random_utf16_string(10) + u"性能好"));
   EXPECT_TRUE(
-      utf16HasSurrogatePairs(generateRandomUTF16String(60) + u"性能好"));
+      utf16_has_surrogate_pairs(generate_random_utf16_string(30) + u"性能好"));
   EXPECT_TRUE(
-      utf16HasSurrogatePairs(generateRandomUTF16String(120) + u"性能好"));
+      utf16_has_surrogate_pairs(generate_random_utf16_string(60) + u"性能好"));
   EXPECT_TRUE(
-      utf16HasSurrogatePairs(generateRandomUTF16String(200) + u"性能好"));
+      utf16_has_surrogate_pairs(generate_random_utf16_string(120) + u"性能好"));
   EXPECT_TRUE(
-      utf16HasSurrogatePairs(generateRandomUTF16String(300) + u"性能好"));
+      utf16_has_surrogate_pairs(generate_random_utf16_string(200) + u"性能好"));
+  EXPECT_TRUE(
+      utf16_has_surrogate_pairs(generate_random_utf16_string(300) + u"性能好"));
 }
 
 // Testing Basic Logic
 TEST(UTF16ToUTF8Test, BasicConversion) {
   std::u16string utf16 = u"Hello, 世界!";
-  std::string utf8 = fory::utf16ToUtf8(utf16, true);
+  std::string utf8 = fory::utf16_to_utf8(utf16, true);
   ASSERT_EQ(utf8, u8"Hello, 世界!");
 }
 
 // Testing Empty String
 TEST(UTF16ToUTF8Test, EmptyString) {
   std::u16string utf16 = u"";
-  std::string utf8 = fory::utf16ToUtf8(utf16, true);
+  std::string utf8 = fory::utf16_to_utf8(utf16, true);
   ASSERT_EQ(utf8, "");
 }
 
 // Testing emoji
 TEST(UTF16ToUTF8Test, SurrogatePairs) {
   std::u16string utf16 = {0xD83D, 0xDE00}; // 😀 emoji
-  std::string utf8 = fory::utf16ToUtf8(utf16, true);
+  std::string utf8 = fory::utf16_to_utf8(utf16, true);
   ASSERT_EQ(utf8, "\xF0\x9F\x98\x80");
 }
 
 // Testing Boundary
 TEST(UTF16ToUTF8Test, BoundaryValues) {
   std::u16string utf16 = {0x0000, 0xFFFF};
-  std::string utf8 = fory::utf16ToUtf8(utf16, true);
+  std::string utf8 = fory::utf16_to_utf8(utf16, true);
   std::string expected_utf8 = std::string("\x00", 1) + "\xEF\xBF\xBF";
   ASSERT_EQ(utf8, expected_utf8);
 }
@@ -218,26 +219,26 @@ TEST(UTF16ToUTF8Test, BoundaryValues) {
 // Testing Special Characters
 TEST(UTF16ToUTF8Test, SpecialCharacters) {
   std::u16string utf16 = u" \n\t";
-  std::string utf8 = fory::utf16ToUtf8(utf16, true);
+  std::string utf8 = fory::utf16_to_utf8(utf16, true);
   ASSERT_EQ(utf8, " \n\t");
 }
 
 // Testing LittleEndian
 TEST(UTF16ToUTF8Test, LittleEndian) {
   std::u16string utf16 = {0x61, 0x62}; // "ab"
-  std::string utf8 = fory::utf16ToUtf8(utf16, true);
+  std::string utf8 = fory::utf16_to_utf8(utf16, true);
   ASSERT_EQ(utf8, "ab");
 }
 
 // Testing BigEndian
 TEST(UTF16ToUTF8Test, BigEndian) {
   std::u16string utf16 = {0xFFFE, 0xFFFE};
-  std::string utf8 = fory::utf16ToUtf8(utf16, false);
+  std::string utf8 = fory::utf16_to_utf8(utf16, false);
   ASSERT_EQ(utf8, "\xEF\xBF\xBE\xEF\xBF\xBE");
 }
 
 // Generate random UTF-8 string
-std::string generateRandomUTF8String(size_t length) {
+std::string generate_random_utf8_string(size_t length) {
   std::string str;
   std::mt19937 generator(std::random_device{}());
   std::uniform_int_distribution<uint32_t> distribution(0, 0x10FFFF);
@@ -245,7 +246,7 @@ std::string generateRandomUTF8String(size_t length) {
   while (str.size() < length) {
     uint32_t code_point = distribution(generator);
 
-    // Skip surrogate pairs (0xD800 to 0xDFFF) and other invalid Unicode code
+    // skip surrogate pairs (0xD800 to 0xDFFF) and other invalid Unicode code
     // points
     if ((code_point >= 0xD800 && code_point <= 0xDFFF) ||
         code_point > 0x10FFFF) {
@@ -275,21 +276,21 @@ std::string generateRandomUTF8String(size_t length) {
 // Testing Basic Logic
 TEST(UTF8ToUTF16Test, BasicConversion) {
   std::string utf8 = u8"Hello, 世界!";
-  std::u16string utf16 = fory::utf8ToUtf16(utf8, true);
+  std::u16string utf16 = fory::utf8_to_utf16(utf8, true);
   ASSERT_EQ(utf16, u"Hello, 世界!");
 }
 
 // Testing Empty String
 TEST(UTF8ToUTF16Test, EmptyString) {
   std::string utf8 = "";
-  std::u16string utf16 = fory::utf8ToUtf16(utf8, true);
+  std::u16string utf16 = fory::utf8_to_utf16(utf8, true);
   ASSERT_EQ(utf16, u"");
 }
 
 // Testing emoji
 TEST(UTF8ToUTF16Test, SurrogatePairs) {
   std::string utf8 = "\xF0\x9F\x98\x80"; // 😀 emoji
-  std::u16string utf16 = fory::utf8ToUtf16(utf8, true);
+  std::u16string utf16 = fory::utf8_to_utf16(utf8, true);
   std::u16string expected_utf16 = {0xD83D, 0xDE00}; // Surrogate pair for emoji
   ASSERT_EQ(utf16, expected_utf16);
 }
@@ -298,7 +299,7 @@ TEST(UTF8ToUTF16Test, SurrogatePairs) {
 TEST(UTF8ToUTF16Test, BoundaryValues) {
   // "\xEF\xBF\xBD" is the UTF-8 encoding for U+FFFD (replacement character)
   std::string utf8 = "\xEF\xBF\xBD"; // U+FFFD in UTF-8
-  std::u16string utf16 = fory::utf8ToUtf16(utf8, true);
+  std::u16string utf16 = fory::utf8_to_utf16(utf8, true);
   std::u16string expected_utf16 = {
       0xFFFD}; // Expected UTF-16 representation of U+FFFD
   ASSERT_EQ(utf16, expected_utf16);
@@ -307,14 +308,14 @@ TEST(UTF8ToUTF16Test, BoundaryValues) {
 // Testing Special Characters
 TEST(UTF8ToUTF16Test, SpecialCharacters) {
   std::string utf8 = " \n\t";
-  std::u16string utf16 = fory::utf8ToUtf16(utf8, true);
+  std::u16string utf16 = fory::utf8_to_utf16(utf8, true);
   ASSERT_EQ(utf16, u" \n\t");
 }
 
 // Testing LittleEndian
 TEST(UTF8ToUTF16Test, LittleEndian) {
   std::string utf8 = "ab";
-  std::u16string utf16 = fory::utf8ToUtf16(utf8, true);
+  std::u16string utf16 = fory::utf8_to_utf16(utf8, true);
   std::u16string expected_utf16 = {
       0x61, 0x62}; // Little-endian UTF-16 representation of "ab"
   ASSERT_EQ(utf16, expected_utf16);
@@ -323,7 +324,7 @@ TEST(UTF8ToUTF16Test, LittleEndian) {
 // Correct BigEndian testing for BOM (Byte Order Mark)
 TEST(UTF8ToUTF16Test, BigEndian) {
   std::string utf8 = "\xEF\xBB\xBF"; // BOM in UTF-8 (0xFEFF)
-  std::u16string utf16 = fory::utf8ToUtf16(utf8, false); // Big-endian
+  std::u16string utf16 = fory::utf8_to_utf16(utf8, false); // Big-endian
   std::u16string expected_utf16 = {0xFFFE}; // Expected BOM in UTF-16
   ASSERT_EQ(utf16, expected_utf16);
 }
@@ -331,8 +332,8 @@ TEST(UTF8ToUTF16Test, BigEndian) {
 // Testing round-trip conversion (UTF-8 -> UTF-16 -> UTF-8)
 TEST(UTF8ToUTF16Test, RoundTripConversion) {
   std::string original_utf8 = u8"Hello, 世界!";
-  std::u16string utf16 = fory::utf8ToUtf16(original_utf8, true);
-  std::string utf8_converted_back = fory::utf16ToUtf8(utf16, true);
+  std::u16string utf16 = fory::utf8_to_utf16(original_utf8, true);
+  std::string utf8_converted_back = fory::utf16_to_utf8(utf16, true);
   ASSERT_EQ(original_utf8, utf8_converted_back);
 }
 
