@@ -25,8 +25,11 @@ import 'package:fory/src/datatype/float32.dart';
 import 'package:fory/src/datatype/int16.dart';
 import 'package:fory/src/datatype/int32.dart';
 import 'package:fory/src/datatype/int8.dart';
+import 'package:fory/src/datatype/uint8.dart';
+import 'package:fory/src/datatype/uint16.dart';
+import 'package:fory/src/datatype/uint32.dart';
 
-import 'package:fory/src/const/obj_type.dart';
+import 'package:fory/src/const/types.dart';
 import 'package:fory/src/datatype/local_date.dart';
 import 'package:fory/src/datatype/timestamp.dart';
 
@@ -41,12 +44,15 @@ enum DartTypeEnum{
   INT8(Int8, true, 'Int8', 'package', 'fory/src/datatype/int8.dart', ObjType.INT8, true, 'dart:core@Int8'),
   INT16(Int16, true, 'Int16', 'package', 'fory/src/datatype/int16.dart', ObjType.INT16, true, 'dart:core@Int16'),
   INT32(Int32, true, 'Int32', 'package', 'fory/src/datatype/int32.dart', ObjType.INT32, true, 'dart:core@Int32'),
+  UINT8(UInt8, true, 'UInt8', 'package', 'fory/src/datatype/uint8.dart', ObjType.UINT8, true, 'dart:core@UInt8'),
+  UINT16(UInt16, true, 'UInt16', 'package', 'fory/src/datatype/uint16.dart', ObjType.UINT16, true, 'dart:core@UInt16'),
+  UINT32(UInt32, true, 'UInt32', 'package', 'fory/src/datatype/uint32.dart', ObjType.UINT32, true, 'dart:core@UInt32'),
   INT(int,true, 'int', 'dart', 'core', ObjType.INT64, true, 'dart:core@int'),
   FLOAT32(Float32, true, 'Float32', 'package', 'fory/src/datatype/float32.dart', ObjType.FLOAT32, true, 'dart:core@Float32'),
   DOUBLE(double,true, 'double', 'dart', 'core', ObjType.FLOAT64, true, 'dart:core@double'),
   STRING(String,true, 'String', 'dart', 'core', ObjType.STRING, true, 'dart:core@String'),
 
-  LOCALDATE(LocalDate, true, 'LocalDate', 'package', 'fory/src/datatype/local_date.dart', ObjType.LOCAL_DATE, true, 'dart:core@LocalDate'),
+  LOCALDATE(LocalDate, true, 'LocalDate', 'package', 'fory/src/datatype/local_date.dart', ObjType.DATE, true, 'dart:core@LocalDate'),
   TIMESTAMP(TimeStamp, false, 'TimeStamp', 'package', 'fory/src/datatype/timestamp.dart', ObjType.TIMESTAMP, true, 'dart:core@DateTime'),
 
   BOOLLIST(BoolList, true, 'BoolList', 'package', 'collection/src/boollist.dart', ObjType.BOOL_ARRAY, true, 'dart:typed_data@BoolList'),
@@ -110,27 +116,6 @@ enum DartTypeEnum{
   /// Returning null indicates it is definitely a Dart built-in type, which is unsupported
   /// Returning UNKNOWN means uncertain
   // TODO: Attempt to record the Dart analyzer's ID to achieve a numerical comparison
-  static DartTypeEnum? _findDeprecated(String name, String scheme, String path){
-    int l = 0;
-    int r = DartTypeEnum.values.length - 1;
-    int mid;
-    while(l<=r){
-      mid = (l+r) ~/ 2;
-      int comp = name.compareTo(values[mid].typeName);
-      if (comp < 0){
-        r = mid -1;
-      }else if (comp > 0){
-        l = mid + 1;
-      }else{
-        if (values[mid].scheme == scheme && values[mid].path == path){
-          return values[mid];
-        }else{
-          return null;
-        }
-      }
-    }
-    return null;
-  }
 
   static DartTypeEnum? find(String name, String scheme, String path){
     DartTypeEnum? e = _typeName2Enum[name];
