@@ -283,6 +283,7 @@ func generateCodeForFile(pkg *packages.Package, structs []*StructInfo, sourceFil
 	// Determine which imports are needed
 	needsTime := false
 	needsReflect := false
+	needsOptional := false
 
 	for _, s := range structs {
 		for _, field := range s.Fields {
@@ -290,14 +291,17 @@ func generateCodeForFile(pkg *packages.Package, structs []*StructInfo, sourceFil
 			if typeStr == "time.Time" || typeStr == "github.com/apache/fory/go/fory.Date" {
 				needsTime = true
 			}
+			if field.IsOptional {
+				needsOptional = true
+			}
 			// We need reflect for the interface compatibility methods
 			needsReflect = true
 		}
 	}
 
 	// Generate imports
+	// Note: "fmt" is not imported by default. Add it only if the generated code uses fmt.
 	fmt.Fprintf(&buf, "import (\n")
-	fmt.Fprintf(&buf, "\t\"fmt\"\n")
 	if needsReflect {
 		fmt.Fprintf(&buf, "\t\"reflect\"\n")
 	}
@@ -305,6 +309,9 @@ func generateCodeForFile(pkg *packages.Package, structs []*StructInfo, sourceFil
 		fmt.Fprintf(&buf, "\t\"time\"\n")
 	}
 	fmt.Fprintf(&buf, "\t\"github.com/apache/fory/go/fory\"\n")
+	if needsOptional {
+		fmt.Fprintf(&buf, "\t\"github.com/apache/fory/go/fory/optional\"\n")
+	}
 	fmt.Fprintf(&buf, ")\n\n")
 
 	// Generate init function to register serializer factories
@@ -390,31 +397,30 @@ func cleanupGeneratedFiles(opts *GeneratorOptions) error {
 
 // generateStructSerializer generates a complete serializer for a struct
 func generateStructSerializer(buf *bytes.Buffer, s *StructInfo) error {
-	// Generate struct serializer type
-	fmt.Fprintf(buf, "type %s_ForyGenSerializer struct {}\n\n", s.Name)
+	// Generate struct serializer type with lazy hash computation
+	fmt.Fprintf(buf, "type %s_ForyGenSerializer struct {\n", s.Name)
+	fmt.Fprintf(buf, "\tstructHash int32\n")
+	fmt.Fprintf(buf, "}\n\n")
 
 	// Generate factory function
 	fmt.Fprintf(buf, "func NewSerializerFor_%s() fory.Serializer {\n", s.Name)
-	fmt.Fprintf(buf, "\treturn %s_ForyGenSerializer{}\n", s.Name)
+	fmt.Fprintf(buf, "\treturn &%s_ForyGenSerializer{}\n", s.Name)
 	fmt.Fprintf(buf, "}\n\n")
 
-	// Generate TypeId method
-	fmt.Fprintf(buf, "func (%s_ForyGenSerializer) TypeId() fory.TypeId {\n", s.Name)
-	fmt.Fprintf(buf, "\treturn fory.NAMED_STRUCT\n")
+	// Generate hash initialization method
+	fmt.Fprintf(buf, "func (g *%s_ForyGenSerializer) initHash(resolver *fory.TypeResolver) {\n", s.Name)
+	fmt.Fprintf(buf, "\tif g.structHash == 0 {\n")
+	fmt.Fprintf(buf, "\t\tg.structHash = fory.GetStructHash(reflect.TypeOf(%s{}), resolver)\n", s.Name)
+	fmt.Fprintf(buf, "\t}\n")
 	fmt.Fprintf(buf, "}\n\n")
 
-	// Generate NeedToWriteRef method (note: method name is NeedToWriteRef, not NeedWriteRef)
-	fmt.Fprintf(buf, "func (%s_ForyGenSerializer) NeedToWriteRef() bool {\n", s.Name)
-	fmt.Fprintf(buf, "\treturn true\n")
-	fmt.Fprintf(buf, "}\n\n")
-
-	// Generate strongly-typed Write method (delegate to encoder)
-	if err := generateWriteTyped(buf, s); err != nil {
+	// Generate Write method (entry point with ref/type handling)
+	if err := generateWriteMethod(buf, s); err != nil {
 		return err
 	}
 
-	// Generate strongly-typed Read method (delegate to decoder)
-	if err := generateReadTyped(buf, s); err != nil {
+	// Generate strongly-typed WriteData method (delegate to encoder)
+	if err := generateWriteTyped(buf, s); err != nil {
 		return err
 	}
 
@@ -423,10 +429,109 @@ func generateStructSerializer(buf *bytes.Buffer, s *StructInfo) error {
 		return err
 	}
 
+	// Generate Read method (entry point with ref/type handling)
+	if err := generateReadMethod(buf, s); err != nil {
+		return err
+	}
+
+	// Generate strongly-typed ReadData method (delegate to decoder)
+	if err := generateReadTyped(buf, s); err != nil {
+		return err
+	}
+
 	if err := generateReadInterface(buf, s); err != nil {
 		return err
 	}
 
+	// Generate ReadWithTypeInfo method
+	if err := generateReadWithTypeInfoMethod(buf, s); err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// generateWriteMethod generates the Write method that handles ref/type flags
+func generateWriteMethod(buf *bytes.Buffer, s *StructInfo) error {
+	fmt.Fprintf(buf, "// Write is the entry point for serialization with ref/type handling\n")
+	fmt.Fprintf(buf, "func (g *%s_ForyGenSerializer) Write(ctx *fory.WriteContext, refMode fory.RefMode, writeType bool, hasGenerics bool, value reflect.Value) {\n", s.Name)
+	fmt.Fprintf(buf, "\tg.initHash(ctx.TypeResolver())\n")
+	fmt.Fprintf(buf, "\t_ = hasGenerics // not used for struct serializers\n")
+	fmt.Fprintf(buf, "\tswitch refMode {\n")
+	fmt.Fprintf(buf, "\tcase fory.RefModeTracking:\n")
+	fmt.Fprintf(buf, "\t\tif !value.IsValid() || (value.Kind() == reflect.Ptr && value.IsNil()) {\n")
+	fmt.Fprintf(buf, "\t\t\tctx.Buffer().WriteInt8(-3) // NullFlag\n")
+	fmt.Fprintf(buf, "\t\t\treturn\n")
+	fmt.Fprintf(buf, "\t\t}\n")
+	fmt.Fprintf(buf, "\t\trefWritten, err := ctx.RefResolver().WriteRefOrNull(ctx.Buffer(), value)\n")
+	fmt.Fprintf(buf, "\t\tif err != nil {\n")
+	fmt.Fprintf(buf, "\t\t\tctx.SetError(fory.FromError(err))\n")
+	fmt.Fprintf(buf, "\t\t\treturn\n")
+	fmt.Fprintf(buf, "\t\t}\n")
+	fmt.Fprintf(buf, "\t\tif refWritten {\n")
+	fmt.Fprintf(buf, "\t\t\treturn\n")
+	fmt.Fprintf(buf, "\t\t}\n")
+	fmt.Fprintf(buf, "\tcase fory.RefModeNullOnly:\n")
+	fmt.Fprintf(buf, "\t\tif !value.IsValid() || (value.Kind() == reflect.Ptr && value.IsNil()) {\n")
+	fmt.Fprintf(buf, "\t\t\tctx.Buffer().WriteInt8(-3) // NullFlag\n")
+	fmt.Fprintf(buf, "\t\t\treturn\n")
+	fmt.Fprintf(buf, "\t\t}\n")
+	fmt.Fprintf(buf, "\t\tctx.Buffer().WriteInt8(-1) // NotNullValueFlag\n")
+	fmt.Fprintf(buf, "\t}\n")
+	fmt.Fprintf(buf, "\tif writeType {\n")
+	fmt.Fprintf(buf, "\t\ttypeInfo, err := ctx.TypeResolver().GetTypeInfo(value, true)\n")
+	fmt.Fprintf(buf, "\t\tif err != nil {\n")
+	fmt.Fprintf(buf, "\t\t\tctx.SetError(fory.FromError(err))\n")
+	fmt.Fprintf(buf, "\t\t\treturn\n")
+	fmt.Fprintf(buf, "\t\t}\n")
+	fmt.Fprintf(buf, "\t\tctx.TypeResolver().WriteTypeInfo(ctx.Buffer(), typeInfo, ctx.Err())\n")
+	fmt.Fprintf(buf, "\t}\n")
+	fmt.Fprintf(buf, "\tg.WriteData(ctx, value)\n")
+	fmt.Fprintf(buf, "}\n\n")
+	return nil
+}
+
+// generateReadMethod generates the Read method that handles ref/type flags
+func generateReadMethod(buf *bytes.Buffer, s *StructInfo) error {
+	fmt.Fprintf(buf, "// Read is the entry point for deserialization with ref/type handling\n")
+	fmt.Fprintf(buf, "func (g *%s_ForyGenSerializer) Read(ctx *fory.ReadContext, refMode fory.RefMode, readType bool, hasGenerics bool, value reflect.Value) {\n", s.Name)
+	fmt.Fprintf(buf, "\tg.initHash(ctx.TypeResolver())\n")
+	fmt.Fprintf(buf, "\t_ = hasGenerics // not used for struct serializers\n")
+	fmt.Fprintf(buf, "\terr := ctx.Err() // Get error pointer for deferred error checking\n")
+	fmt.Fprintf(buf, "\tswitch refMode {\n")
+	fmt.Fprintf(buf, "\tcase fory.RefModeTracking:\n")
+	fmt.Fprintf(buf, "\t\trefID, refErr := ctx.RefResolver().TryPreserveRefId(ctx.Buffer())\n")
+	fmt.Fprintf(buf, "\t\tif refErr != nil {\n")
+	fmt.Fprintf(buf, "\t\t\tctx.SetError(fory.FromError(refErr))\n")
+	fmt.Fprintf(buf, "\t\t\treturn\n")
+	fmt.Fprintf(buf, "\t\t}\n")
+	fmt.Fprintf(buf, "\t\tif int8(refID) < -1 { // NotNullValueFlag\n")
+	fmt.Fprintf(buf, "\t\t\tobj := ctx.RefResolver().GetReadObject(refID)\n")
+	fmt.Fprintf(buf, "\t\t\tif obj.IsValid() {\n")
+	fmt.Fprintf(buf, "\t\t\t\tvalue.Set(obj)\n")
+	fmt.Fprintf(buf, "\t\t\t}\n")
+	fmt.Fprintf(buf, "\t\t\treturn\n")
+	fmt.Fprintf(buf, "\t\t}\n")
+	fmt.Fprintf(buf, "\tcase fory.RefModeNullOnly:\n")
+	fmt.Fprintf(buf, "\t\tflag := ctx.Buffer().ReadInt8(err)\n")
+	fmt.Fprintf(buf, "\t\tif flag == -3 { // NullFlag\n")
+	fmt.Fprintf(buf, "\t\t\treturn\n")
+	fmt.Fprintf(buf, "\t\t}\n")
+	fmt.Fprintf(buf, "\t}\n")
+	fmt.Fprintf(buf, "\tif readType {\n")
+	fmt.Fprintf(buf, "\t\tctx.TypeResolver().ReadTypeInfo(ctx.Buffer(), err)\n")
+	fmt.Fprintf(buf, "\t}\n")
+	fmt.Fprintf(buf, "\tg.ReadData(ctx, value)\n")
+	fmt.Fprintf(buf, "}\n\n")
+	return nil
+}
+
+// generateReadWithTypeInfoMethod generates the ReadWithTypeInfo method
+func generateReadWithTypeInfoMethod(buf *bytes.Buffer, s *StructInfo) error {
+	fmt.Fprintf(buf, "// ReadWithTypeInfo deserializes with pre-read type information\n")
+	fmt.Fprintf(buf, "func (g *%s_ForyGenSerializer) ReadWithTypeInfo(ctx *fory.ReadContext, refMode fory.RefMode, typeInfo *fory.TypeInfo, value reflect.Value) {\n", s.Name)
+	fmt.Fprintf(buf, "\tg.Read(ctx, refMode, false, false, value)\n")
+	fmt.Fprintf(buf, "}\n\n")
 	return nil
 }
 
@@ -443,6 +548,7 @@ func generateCode(pkg *packages.Package, structs []*StructInfo) error {
 	// Determine which imports are needed
 	needsTime := false
 	needsReflect := false
+	needsOptional := false
 
 	for _, s := range structs {
 		for _, field := range s.Fields {
@@ -450,14 +556,17 @@ func generateCode(pkg *packages.Package, structs []*StructInfo) error {
 			if typeStr == "time.Time" || typeStr == "github.com/apache/fory/go/fory.Date" {
 				needsTime = true
 			}
+			if field.IsOptional {
+				needsOptional = true
+			}
 			// We need reflect for the interface compatibility methods
 			needsReflect = true
 		}
 	}
 
 	// Generate imports
+	// Note: "fmt" is not imported by default. Add it only if the generated code uses fmt.
 	fmt.Fprintf(&buf, "import (\n")
-	fmt.Fprintf(&buf, "\t\"fmt\"\n")
 	if needsReflect {
 		fmt.Fprintf(&buf, "\t\"reflect\"\n")
 	}
@@ -465,6 +574,9 @@ func generateCode(pkg *packages.Package, structs []*StructInfo) error {
 		fmt.Fprintf(&buf, "\t\"time\"\n")
 	}
 	fmt.Fprintf(&buf, "\t\"github.com/apache/fory/go/fory\"\n")
+	if needsOptional {
+		fmt.Fprintf(&buf, "\t\"github.com/apache/fory/go/fory/optional\"\n")
+	}
 	fmt.Fprintf(&buf, ")\n\n")
 
 	// Generate init function to register serializer factories
@@ -494,7 +606,7 @@ func generateCode(pkg *packages.Package, structs []*StructInfo) error {
 		return fmt.Errorf("formatting generated code: %w", err)
 	}
 
-	// Write to output file (legacy package-based naming)
+	// WriteData to output file (legacy package-based naming)
 	outputFile := filepath.Join(filepath.Dir(pkg.GoFiles[0]), fmt.Sprintf("%s_fory_gen.go", pkg.Name))
 	return ioutil.WriteFile(outputFile, formatted, 0644)
 }
