@@ -60,10 +60,9 @@ import org.apache.fory.reflect.ObjectCreator;
 import org.apache.fory.reflect.ObjectCreators;
 import org.apache.fory.reflect.ReflectionUtils;
 import org.apache.fory.reflect.TypeRef;
-import org.apache.fory.resolver.ClassInfo;
-import org.apache.fory.resolver.ClassInfoHolder;
+import org.apache.fory.resolver.TypeInfo;
+import org.apache.fory.resolver.TypeInfoHolder;
 import org.apache.fory.type.Descriptor;
-import org.apache.fory.type.FinalObjectTypeStub;
 import org.apache.fory.util.GraalvmSupport;
 import org.apache.fory.util.Preconditions;
 import org.apache.fory.util.StringUtils;
@@ -88,8 +87,8 @@ public abstract class CodecBuilder {
   protected static final String FORY_NAME = "_f_fory";
   static TypeRef<Object[]> objectArrayTypeRef = TypeRef.of(Object[].class);
   static TypeRef<MemoryBuffer> bufferTypeRef = TypeRef.of(MemoryBuffer.class);
-  static TypeRef<ClassInfo> classInfoTypeRef = TypeRef.of(ClassInfo.class);
-  static TypeRef<ClassInfoHolder> classInfoHolderTypeRef = TypeRef.of(ClassInfoHolder.class);
+  static TypeRef<TypeInfo> classInfoTypeRef = TypeRef.of(TypeInfo.class);
+  static TypeRef<TypeInfoHolder> classInfoHolderTypeRef = TypeRef.of(TypeInfoHolder.class);
 
   protected final CodegenContext ctx;
   protected final TypeRef<?> beanType;
@@ -154,10 +153,6 @@ public abstract class CodecBuilder {
   protected Expression tryCastIfPublic(
       Expression expression, TypeRef<?> targetType, boolean inline) {
     Class<?> rawType = getRawType(targetType);
-    if (rawType == FinalObjectTypeStub.class) {
-      // final field doesn't exist in this class, skip cast.
-      return expression;
-    }
     if (inline) {
       if (sourcePublicAccessible(rawType)) {
         return new Cast(expression, targetType);
@@ -309,7 +304,7 @@ public abstract class CodecBuilder {
     return new Cast(getObj, descriptor.getTypeRef(), descriptor.getName());
   }
 
-  /** Returns an expression that get field value> from <code>bean</code> using {@link Unsafe}. */
+  /** Returns an expression that get field value> from <code>bean</code> using `Unsafe`. */
   private Expression unsafeAccessField(
       Expression inputObject, Class<?> cls, Descriptor descriptor) {
     String fieldName = descriptor.getName();
@@ -378,15 +373,21 @@ public abstract class CodecBuilder {
     if (!d.isFinalField()
         && Modifier.isPublic(d.getModifiers())
         && Modifier.isPublic(d.getRawType().getModifiers())) {
+      if (!d.getRawType().isAssignableFrom(value.type().getRawType())) {
+        value = tryInlineCast(value, d.getTypeRef());
+      }
       return new Expression.SetField(bean, fieldName, value);
     } else if (d.getWriteMethod() != null && Modifier.isPublic(d.getWriteMethod().getModifiers())) {
+      if (!d.getRawType().isAssignableFrom(value.type().getRawType())) {
+        value = tryInlineCast(value, d.getTypeRef());
+      }
       return new Invoke(bean, d.getWriteMethod().getName(), value);
     } else {
       if (!d.isFinalField() && !Modifier.isPrivate(d.getModifiers())) {
         if (AccessorHelper.defineSetter(d.getField())) {
           Class<?> accessorClass = AccessorHelper.getAccessorClass(d.getField());
-          if (!value.type().equals(d.getTypeRef())) {
-            value = new Cast(value, d.getTypeRef());
+          if (!d.getRawType().isAssignableFrom(value.type().getRawType())) {
+            value = tryInlineCast(value, d.getTypeRef());
           }
           return new StaticInvoke(
               accessorClass, d.getName(), PRIMITIVE_VOID_TYPE, false, bean, value);
@@ -395,8 +396,8 @@ public abstract class CodecBuilder {
       if (d.getWriteMethod() != null && !Modifier.isPrivate(d.getWriteMethod().getModifiers())) {
         if (AccessorHelper.defineSetter(d.getWriteMethod())) {
           Class<?> accessorClass = AccessorHelper.getAccessorClass(d.getWriteMethod());
-          if (!value.type().equals(d.getTypeRef())) {
-            value = new Cast(value, d.getTypeRef());
+          if (!d.getRawType().isAssignableFrom(value.type().getRawType())) {
+            value = tryInlineCast(value, d.getTypeRef());
           }
           return new StaticInvoke(
               accessorClass, d.getWriteMethod().getName(), PRIMITIVE_VOID_TYPE, false, bean, value);
@@ -418,8 +419,7 @@ public abstract class CodecBuilder {
   }
 
   /**
-   * Returns an expression that set field <code>value</code> to <code>bean</code> using {@link
-   * Unsafe}.
+   * Returns an expression that set field <code>value</code> to <code>bean</code> using `Unsafe`.
    */
   private Expression unsafeSetField(Expression bean, Descriptor descriptor, Expression value) {
     TypeRef<?> fieldType = descriptor.getTypeRef();
@@ -479,6 +479,7 @@ public abstract class CodecBuilder {
       boolean isStatic, Class<?> type, String fieldName, Supplier<Expression> value) {
     Reference fieldRef = fieldMap.get(fieldName);
     if (fieldRef == null) {
+      fieldName = ctx.newName(fieldName);
       ctx.addField(isStatic, true, ctx.type(type), fieldName, value.get());
       fieldRef = new Reference(fieldName, TypeRef.of(type));
       fieldMap.put(fieldName, fieldRef);
@@ -574,8 +575,9 @@ public abstract class CodecBuilder {
   }
 
   protected Expression staticClassFieldExpr(Class<?> cls, String fieldName) {
-    Preconditions.checkArgument(
-        !sourcePublicAccessible(cls), "Public class %s should use class literal instead", cls);
+    if (sourcePublicAccessible(cls)) {
+      return Literal.ofClass(cls);
+    }
     return getOrCreateField(
         true,
         Class.class,
@@ -709,13 +711,27 @@ public abstract class CodecBuilder {
     return Platform.IS_LITTLE_ENDIAN ? "_readInt64OnLE" : "_readInt64OnBE";
   }
 
+  public static String readInt16Func() {
+    return Platform.IS_LITTLE_ENDIAN ? "_readInt16OnLE" : "_readInt16OnBE";
+  }
+
+  public static String readVarInt32Func() {
+    return Platform.IS_LITTLE_ENDIAN ? "_readVarInt32OnLE" : "_readVarInt32OnBE";
+  }
+
+  public static String readFloat32Func() {
+    return Platform.IS_LITTLE_ENDIAN ? "_readFloat32OnLE" : "_readFloat32OnBE";
+  }
+
+  public static String readFloat64Func() {
+    return Platform.IS_LITTLE_ENDIAN ? "_readFloat64OnLE" : "_readFloat64OnBE";
+  }
+
   protected Expression readFloat32(Expression buffer) {
-    String func = Platform.IS_LITTLE_ENDIAN ? "_readFloat32OnLE" : "_readFloat32OnBE";
-    return new Invoke(buffer, func, PRIMITIVE_FLOAT_TYPE);
+    return new Invoke(buffer, readFloat32Func(), PRIMITIVE_FLOAT_TYPE);
   }
 
   protected Expression readFloat64(Expression buffer) {
-    String func = Platform.IS_LITTLE_ENDIAN ? "_readFloat64OnLE" : "_readFloat64OnBE";
-    return new Invoke(buffer, func, PRIMITIVE_DOUBLE_TYPE);
+    return new Invoke(buffer, readFloat64Func(), PRIMITIVE_DOUBLE_TYPE);
   }
 }
