@@ -25,8 +25,8 @@ import java.util.concurrent.ConcurrentHashMap;
 import org.apache.fory.Fory;
 import org.apache.fory.codegen.CodeGenerator;
 import org.apache.fory.codegen.CompileUnit;
-import org.apache.fory.collection.Tuple2;
-import org.apache.fory.meta.ClassDef;
+import org.apache.fory.collection.Tuple3;
+import org.apache.fory.meta.TypeDef;
 import org.apache.fory.reflect.TypeRef;
 import org.apache.fory.resolver.ClassResolver;
 import org.apache.fory.serializer.Serializer;
@@ -35,10 +35,11 @@ import org.apache.fory.util.GraalvmSupport;
 import org.apache.fory.util.Preconditions;
 
 /** Codec util to create and load jit serializer class. */
-@SuppressWarnings("rawtypes")
+@SuppressWarnings({"rawtypes", "unchecked"})
 public class CodecUtils {
-  private static ConcurrentHashMap<Tuple2<String, Class<?>>, Class> graalvmSerializers =
-      new ConcurrentHashMap<>();
+  // Cache key includes configHash to distinguish between xlang and non-xlang modes
+  private static final ConcurrentHashMap<Tuple3<String, Class<?>, Integer>, Class>
+      graalvmSerializers = new ConcurrentHashMap<>();
 
   // TODO(chaokunyang) how to uninstall org.apache.fory.codegen/builder classes for graalvm build
   // time
@@ -49,18 +50,20 @@ public class CodecUtils {
     return loadSerializer(
         "loadOrGenObjectCodecClass",
         cls,
+        fory,
         () -> loadOrGenCodecClass(cls, fory, new ObjectCodecBuilder(cls, fory)));
   }
 
   public static <T> Class<? extends Serializer<T>> loadOrGenMetaSharedCodecClass(
-      Fory fory, Class<T> cls, ClassDef classDef) {
+      Fory fory, Class<T> cls, TypeDef typeDef) {
     Preconditions.checkNotNull(fory);
     return loadSerializer(
         "loadOrGenMetaSharedCodecClass",
         cls,
+        fory,
         () ->
             loadOrGenCodecClass(
-                cls, fory, new MetaSharedCodecBuilder(TypeRef.of(cls), fory, classDef)));
+                cls, fory, new MetaSharedCodecBuilder(TypeRef.of(cls), fory, typeDef)));
   }
 
   /**
@@ -68,22 +71,23 @@ public class CodecUtils {
    *
    * @param cls the target class
    * @param fory the Fory instance
-   * @param layerClassDef the ClassDef for this layer only
+   * @param layerTypeDef the TypeDef for this layer only
    * @param layerMarkerClass the marker class for this layer
    * @return the generated serializer class
    */
   public static <T> Class<? extends Serializer<T>> loadOrGenMetaSharedLayerCodecClass(
-      Class<T> cls, Fory fory, ClassDef layerClassDef, Class<?> layerMarkerClass) {
+      Class<T> cls, Fory fory, TypeDef layerTypeDef, Class<?> layerMarkerClass) {
     Preconditions.checkNotNull(fory);
     return loadSerializer(
         "loadOrGenMetaSharedLayerCodecClass",
         cls,
+        fory,
         () ->
             loadOrGenCodecClass(
                 cls,
                 fory,
                 new MetaSharedLayerCodecBuilder(
-                    TypeRef.of(cls), fory, layerClassDef, layerMarkerClass)));
+                    TypeRef.of(cls), fory, layerTypeDef, layerMarkerClass)));
   }
 
   @SuppressWarnings("unchecked")
@@ -148,9 +152,10 @@ public class CodecUtils {
   }
 
   private static <T> Class<? extends Serializer<T>> loadSerializer(
-      String name, Class<?> cls, Callable<Class<? extends Serializer<T>>> func) {
+      String name, Class<?> cls, Fory fory, Callable<Class<? extends Serializer<T>>> func) {
+    int configHash = fory.getConfig().getConfigHash();
     if (GraalvmSupport.IN_GRAALVM_NATIVE_IMAGE) {
-      Tuple2<String, Class<?>> key = Tuple2.of(name, cls);
+      Tuple3<String, Class<?>, Integer> key = Tuple3.of(name, cls, configHash);
       Class serializerClass = graalvmSerializers.get(key);
       if (serializerClass != null) {
         return serializerClass;
@@ -159,7 +164,7 @@ public class CodecUtils {
     try {
       Class serializerClass = func.call();
       if (GraalvmSupport.IN_GRAALVM_NATIVE_IMAGE) {
-        graalvmSerializers.putIfAbsent(Tuple2.of(name, cls), serializerClass);
+        graalvmSerializers.putIfAbsent(Tuple3.of(name, cls, configHash), serializerClass);
       }
       return serializerClass;
     } catch (Exception e) {
