@@ -41,15 +41,15 @@ import org.apache.fory.util.function.SerializableSupplier;
  * <ul>
  *   <li>compiled code for writing every primitive is 22/21 bytes.
  *   <li>compiled code for write/read every boxed primitive is 38/35 bytes.
- *   <li>compiled code for write/read every boxed primitive with ref_tracking is 40/81 bytes.
+ *   <li>compiled code for write/read every boxed primitive with track_ref is 40/81 bytes.
  *   <li>compiled code for write/read every final field on average is 41/82 bytes.
  *   <li>compiled code for write/read every {@literal List<primitive>} on average is 191/201 bytes.
- *   <li>compiled code for write/read every {@literal List<primitive>} with boxed ref_tracking on
+ *   <li>compiled code for write/read every {@literal List<primitive>} with boxed track_ref on
  *       average is 194/240 bytes.
  *   <li>compiled code for write/read every {@literal List<final>} on average is 196/238 bytes.
  *   <li>compiled code for write/read every {@literal Map<boxed, boxed>} on average is 270/196
  *       bytes.
- *   <li>compiled code for write/read every {@literal Map<boxed, boxed>} with boxed ref_tracking on
+ *   <li>compiled code for write/read every {@literal Map<boxed, boxed>} with boxed track_ref on
  *       average is 266/296 bytes.
  *   <li>compiled code for write/read every {@literal Map<final, final>} on average is 268/292
  *       bytes.
@@ -70,8 +70,8 @@ public class ObjectCodecOptimizer extends ExpressionOptimizer {
   final List<List<Descriptor>> primitiveGroups = new ArrayList<>();
   final List<List<Descriptor>> boxedWriteGroups = new ArrayList<>();
   final List<List<Descriptor>> boxedReadGroups = new ArrayList<>();
-  final List<List<Descriptor>> finalWriteGroups = new ArrayList<>();
-  final List<List<Descriptor>> finalReadGroups = new ArrayList<>();
+  final List<List<Descriptor>> buildInWriteGroups = new ArrayList<>();
+  final List<List<Descriptor>> buildInReadGroups = new ArrayList<>();
   final List<List<Descriptor>> otherWriteGroups = new ArrayList<>();
   final List<List<Descriptor>> otherReadGroups = new ArrayList<>();
 
@@ -117,9 +117,9 @@ public class ObjectCodecOptimizer extends ExpressionOptimizer {
                 boxedReadWeight,
                 boxedReadGroups),
             MutableTuple3.of(
-                new ArrayList<>(descriptorGrouper.getFinalDescriptors()), 9, finalWriteGroups),
+                new ArrayList<>(descriptorGrouper.getBuildInDescriptors()), 9, buildInWriteGroups),
             MutableTuple3.of(
-                new ArrayList<>(descriptorGrouper.getFinalDescriptors()), 5, finalReadGroups),
+                new ArrayList<>(descriptorGrouper.getBuildInDescriptors()), 5, buildInReadGroups),
             MutableTuple3.of(
                 new ArrayList<>(descriptorGrouper.getOtherDescriptors()), 4, otherReadGroups),
             MutableTuple3.of(
