@@ -486,6 +486,43 @@ public interface Expression {
     }
   }
 
+  class EnumExpression extends AbstractExpression {
+    private final Enum<?> value;
+    private final TypeRef<?> type;
+
+    public EnumExpression(Enum<?> value) {
+      super(new Expression[0]);
+      Preconditions.checkNotNull(value, "Enum value must not be null");
+      this.value = value;
+      this.type = TypeRef.of(value.getDeclaringClass());
+    }
+
+    public Enum<?> getEnumValue() {
+      return value;
+    }
+
+    @Override
+    public TypeRef<?> type() {
+      return type;
+    }
+
+    @Override
+    public ExprCode doGenCode(CodegenContext ctx) {
+      String enumClassName = ReflectionUtils.getLiteralName(value.getDeclaringClass());
+      String enumValue = enumClassName + "." + value.name();
+      return new ExprCode(FalseLiteral, Code.exprValue(getRawType(type), enumValue));
+    }
+
+    public Enum<?> getValue() {
+      return value;
+    }
+
+    @Override
+    public String toString() {
+      return value.getDeclaringClass().getName() + "." + value.name();
+    }
+  }
+
   class Null extends AbstractExpression {
     private TypeRef<?> type;
     private final boolean typedNull;
@@ -975,6 +1012,9 @@ public interface Expression {
       this.functionName = functionName;
       this.type = type;
       this.arguments = arguments;
+      for (Expression argument : arguments) {
+        Preconditions.checkNotNull(argument);
+      }
       this.returnNamePrefix = returnNamePrefix;
       this.returnNullable = returnNullable;
       inlineCall = inline;
@@ -1866,8 +1906,13 @@ public interface Expression {
                   falseEval.value());
         }
         codeBuilder.append(StringUtils.stripBlankLines(ifCode));
-        return new ExprCode(
-            codeBuilder.toString(), Code.isNullVariable(isNull), Code.variable(rawType, value));
+        if (nullable) {
+          return new ExprCode(
+              codeBuilder.toString(), Code.isNullVariable(isNull), Code.variable(rawType, value));
+        } else {
+          // When not nullable, return FalseLiteral instead of a variable that was never declared
+          return new ExprCode(codeBuilder.toString(), FalseLiteral, Code.variable(rawType, value));
+        }
       } else {
         String ifCode;
         if (falseExpr != null) {
@@ -2216,6 +2261,9 @@ public interface Expression {
       this.predicate = predicate;
       this.action = action;
       this.cutPoints = cutPoints;
+      for (Expression cutPoint : cutPoints) {
+        Preconditions.checkNotNull(cutPoint);
+      }
     }
 
     @Override
@@ -2762,4 +2810,40 @@ public interface Expression {
       return String.format("%s = %s", from, to);
     }
   }
+
+  /** Expression for Java instanceof check. */
+  class InstanceOf extends ValueExpression {
+    private Expression target;
+    private final TypeRef<?> checkType;
+
+    public InstanceOf(Expression target, TypeRef<?> checkType) {
+      super(new Expression[] {target});
+      this.target = target;
+      this.checkType = checkType;
+      this.inlineCall = true;
+    }
+
+    @Override
+    public TypeRef<?> type() {
+      return PRIMITIVE_BOOLEAN_TYPE;
+    }
+
+    @Override
+    public ExprCode doGenCode(CodegenContext ctx) {
+      ExprCode targetCode = target.genCode(ctx);
+      String value = String.format("(%s instanceof %s)", targetCode.value(), ctx.type(checkType));
+      String code = StringUtils.isBlank(targetCode.code()) ? null : targetCode.code();
+      return new ExprCode(code, FalseLiteral, Code.variable(boolean.class, value));
+    }
+
+    @Override
+    public boolean nullable() {
+      return false;
+    }
+
+    @Override
+    public String toString() {
+      return String.format("%s instanceof %s", target, checkType);
+    }
+  }
 }
