@@ -28,6 +28,10 @@ import static org.apache.fory.codegen.Expression.Literal;
 import static org.apache.fory.codegen.Expression.NewArray;
 import static org.apache.fory.codegen.Expression.Not;
 import static org.apache.fory.codegen.Expression.StaticInvoke;
+import static org.apache.fory.type.TypeUtils.PRIMITIVE_CHAR_TYPE;
+import static org.apache.fory.type.TypeUtils.PRIMITIVE_DOUBLE_TYPE;
+import static org.apache.fory.type.TypeUtils.PRIMITIVE_FLOAT_TYPE;
+import static org.apache.fory.type.TypeUtils.PRIMITIVE_SHORT_TYPE;
 import static org.apache.fory.type.TypeUtils.getRawType;
 
 import java.io.Serializable;
@@ -39,6 +43,7 @@ import org.apache.fory.codegen.Expression.BitAnd;
 import org.apache.fory.codegen.Expression.BitOr;
 import org.apache.fory.codegen.Expression.BitShift;
 import org.apache.fory.codegen.Expression.Cast;
+import org.apache.fory.codegen.Expression.EnumExpression;
 import org.apache.fory.codegen.Expression.Invoke;
 import org.apache.fory.codegen.Expression.ListExpression;
 import org.apache.fory.codegen.Expression.LogicalAnd;
@@ -62,6 +67,10 @@ public class ExpressionUtils {
     return new NewArray(TypeRef.of(Object[].class), expressions);
   }
 
+  public static Expression ofEnum(Enum<?> value) {
+    return new EnumExpression(value);
+  }
+
   public static boolean isReturn(Expression expr) {
     if (expr instanceof ListExpression) {
       expr = ((ListExpression) expr).last();
@@ -147,10 +156,45 @@ public class ExpressionUtils {
     return new Not(target);
   }
 
+  public static Literal nullValue(Class<?> type) {
+    if (type.isPrimitive()) {
+      return defaultValue(type);
+    }
+    return new Literal(null, TypeRef.of(type));
+  }
+
   public static Literal nullValue(TypeRef<?> type) {
+    if (type.isPrimitive()) {
+      return defaultValue(type.getRawType());
+    }
     return new Literal(null, type);
   }
 
+  /**
+   * Returns the default value for a type. For primitives returns 0/false, for objects returns null.
+   */
+  public static Literal defaultValue(Class<?> type) {
+    if (type == boolean.class) {
+      return Literal.False;
+    } else if (type == byte.class) {
+      return Literal.ofByte((short) 0);
+    } else if (type == char.class) {
+      return new Literal((char) 0, PRIMITIVE_CHAR_TYPE);
+    } else if (type == short.class) {
+      return new Literal((short) 0, PRIMITIVE_SHORT_TYPE);
+    } else if (type == int.class) {
+      return Literal.ofInt(0);
+    } else if (type == long.class) {
+      return Literal.ofLong(0L);
+    } else if (type == float.class) {
+      return new Literal(0.0f, PRIMITIVE_FLOAT_TYPE);
+    } else if (type == double.class) {
+      return new Literal(0.0, PRIMITIVE_DOUBLE_TYPE);
+    } else {
+      return new Literal(null, TypeRef.of(type));
+    }
+  }
+
   public static Comparator eq(Expression left, Expression right) {
     return eq(left, right, true);
   }
