@@ -34,9 +34,8 @@ import org.apache.fory.util.Preconditions;
 @SuppressWarnings({"rawtypes"})
 public class Config implements Serializable {
   private final String name;
-  private final Language language;
+  private final boolean xlang;
   private final boolean trackingRef;
-  private final boolean basicTypesRefIgnored;
   private final boolean stringRefIgnored;
   private final boolean timeRefIgnored;
   private final boolean copyRef;
@@ -59,22 +58,19 @@ public class Config implements Serializable {
   private final boolean scopedMetaShareEnabled;
   private final MetaCompressor metaCompressor;
   private final boolean asyncCompilationEnabled;
-  private final boolean deserializeNonexistentClass;
+  private final boolean deserializeUnknownClass;
   private final boolean scalaOptimizationEnabled;
   private transient int configHash;
   private final UnknownEnumValueStrategy unknownEnumValueStrategy;
   private final boolean serializeEnumByName;
   private final int bufferSizeLimitBytes;
   private final int maxDepth;
-  boolean foryDebugOutputEnabled =
-      "1".equals(System.getenv("ENABLE_FORY_DEBUG_OUTPUT"))
-          || "true".equals(System.getenv("ENABLE_FORY_DEBUG_OUTPUT"));
+  private final float mapRefLoadFactor;
 
   public Config(ForyBuilder builder) {
     name = builder.name;
-    language = builder.language;
+    xlang = builder.xlang;
     trackingRef = builder.trackingRef;
-    basicTypesRefIgnored = !trackingRef || builder.basicTypesRefIgnored;
     stringRefIgnored = !trackingRef || builder.stringRefIgnored;
     timeRefIgnored = !trackingRef || builder.timeRefIgnored;
     copyRef = builder.copyRef;
@@ -82,7 +78,7 @@ public class Config implements Serializable {
     writeNumUtf16BytesForUtf8Encoding = builder.writeNumUtf16BytesForUtf8Encoding;
     compressInt = builder.compressInt;
     longEncoding = builder.longEncoding;
-    compressLong = longEncoding != LongEncoding.LE_RAW_BYTES;
+    compressLong = longEncoding != LongEncoding.FIXED;
     compressIntArray = builder.compressIntArray;
     compressLongArray = builder.compressLongArray;
     requireClassRegistration = builder.requireClassRegistration;
@@ -96,11 +92,11 @@ public class Config implements Serializable {
     metaShareEnabled = builder.metaShareEnabled;
     scopedMetaShareEnabled = builder.scopedMetaShareEnabled;
     metaCompressor = builder.metaCompressor;
-    deserializeNonexistentClass = builder.deserializeNonexistentClass;
-    if (deserializeNonexistentClass) {
+    deserializeUnknownClass = builder.deserializeUnknownClass;
+    if (deserializeUnknownClass) {
       Preconditions.checkArgument(
           metaShareEnabled || compatibleMode == CompatibleMode.COMPATIBLE,
-          "Configuration error: deserializeNonexistentClass=true requires either "
+          "Configuration error: deserializeUnknownClass=true requires either "
               + "metaShareEnabled=true to access type information OR compatibleMode=COMPATIBLE "
               + "to automatically resolve class schemas.");
     }
@@ -110,6 +106,7 @@ public class Config implements Serializable {
     serializeEnumByName = builder.serializeEnumByName;
     bufferSizeLimitBytes = builder.bufferSizeLimitBytes;
     maxDepth = builder.maxDepth;
+    mapRefLoadFactor = builder.mapRefLoadFactor;
   }
 
   /** Returns the name for Fory serialization. */
@@ -117,8 +114,8 @@ public class Config implements Serializable {
     return name;
   }
 
-  public Language getLanguage() {
-    return language;
+  public boolean isXlang() {
+    return xlang;
   }
 
   public boolean trackingRef() {
@@ -137,16 +134,12 @@ public class Config implements Serializable {
     return copyRef;
   }
 
-  public boolean isBasicTypesRefIgnored() {
-    return basicTypesRefIgnored;
-  }
-
   public boolean isStringRefIgnored() {
     return stringRefIgnored;
   }
 
   /** ignore Enum Deserialize array out of bounds return null. */
-  public boolean deserializeNonexistentEnumValueAsNull() {
+  public boolean deserializeUnknownEnumValueAsNull() {
     return unknownEnumValueStrategy == UnknownEnumValueStrategy.RETURN_NULL;
   }
 
@@ -270,8 +263,8 @@ public class Config implements Serializable {
    * Whether deserialize/skip data of un-existed class. If not enabled, an exception will be thrown
    * if class not exist.
    */
-  public boolean deserializeNonexistentClass() {
-    return deserializeNonexistentClass;
+  public boolean deserializeUnknownClass() {
+    return deserializeUnknownClass;
   }
 
   /**
@@ -297,10 +290,6 @@ public class Config implements Serializable {
     return scalaOptimizationEnabled;
   }
 
-  public boolean isForyDebugOutputEnabled() {
-    return foryDebugOutputEnabled;
-  }
-
   @Override
   public boolean equals(Object o) {
     if (this == o) {
@@ -312,7 +301,7 @@ public class Config implements Serializable {
     Config config = (Config) o;
     return name == config.name
         && trackingRef == config.trackingRef
-        && basicTypesRefIgnored == config.basicTypesRefIgnored
+        && mapRefLoadFactor == config.mapRefLoadFactor
         && stringRefIgnored == config.stringRefIgnored
         && timeRefIgnored == config.timeRefIgnored
         && copyRef == config.copyRef
@@ -333,9 +322,9 @@ public class Config implements Serializable {
         && scopedMetaShareEnabled == config.scopedMetaShareEnabled
         && Objects.equals(metaCompressor, config.metaCompressor)
         && asyncCompilationEnabled == config.asyncCompilationEnabled
-        && deserializeNonexistentClass == config.deserializeNonexistentClass
+        && deserializeUnknownClass == config.deserializeUnknownClass
         && scalaOptimizationEnabled == config.scalaOptimizationEnabled
-        && language == config.language
+        && xlang == config.xlang
         && compatibleMode == config.compatibleMode
         && Objects.equals(defaultJDKStreamSerializerType, config.defaultJDKStreamSerializerType)
         && longEncoding == config.longEncoding;
@@ -345,9 +334,9 @@ public class Config implements Serializable {
   public int hashCode() {
     return Objects.hash(
         name,
-        language,
+        xlang,
+        mapRefLoadFactor,
         trackingRef,
-        basicTypesRefIgnored,
         stringRefIgnored,
         timeRefIgnored,
         copyRef,
@@ -371,7 +360,7 @@ public class Config implements Serializable {
         scopedMetaShareEnabled,
         metaCompressor,
         asyncCompilationEnabled,
-        deserializeNonexistentClass,
+        deserializeUnknownClass,
         scalaOptimizationEnabled);
   }
 
@@ -391,4 +380,9 @@ public class Config implements Serializable {
   public int maxDepth() {
     return maxDepth;
   }
+
+  /** Returns loadFactor of MacRef's writtenObjects. */
+  public float mapRefLoadFactor() {
+    return mapRefLoadFactor;
+  }
 }
