@@ -59,15 +59,14 @@ public final class ForyBuilder {
 
   String name;
   boolean checkClassVersion = false;
-  Language language = Language.JAVA;
+  boolean xlang = false;
   boolean trackingRef = false;
   boolean copyRef = false;
-  boolean basicTypesRefIgnored = true;
   boolean stringRefIgnored = true;
   boolean timeRefIgnored = true;
   ClassLoader classLoader;
   boolean compressInt = true;
-  public LongEncoding longEncoding = LongEncoding.SLI;
+  public LongEncoding longEncoding = LongEncoding.TAGGED;
   boolean compressIntArray = false;
   boolean compressLongArray = false;
   boolean compressString = false;
@@ -79,7 +78,7 @@ public final class ForyBuilder {
   Boolean metaShareEnabled;
   Boolean scopedMetaShareEnabled;
   boolean codeGenEnabled = true;
-  Boolean deserializeNonexistentClass;
+  Boolean deserializeUnknownClass;
   boolean asyncCompilationEnabled = false;
   boolean registerGuavaTypes = true;
   boolean scalaOptimizationEnabled = false;
@@ -89,15 +88,21 @@ public final class ForyBuilder {
   int bufferSizeLimitBytes = 128 * 1024;
   MetaCompressor metaCompressor = new DeflaterMetaCompressor();
   int maxDepth = 50;
+  float mapRefLoadFactor = 0.51f;
 
   public ForyBuilder() {}
 
   /**
-   * Whether cross-language serialize the object. If you used fory for java only, please set
-   * language to {@link Language#JAVA}, which will have much better performance.
+   * Whether cross-language serialize the object. If you used fory for java only, please keep it in
+   * java mode, which will have much better performance.
    */
   public ForyBuilder withLanguage(Language language) {
-    this.language = language;
+    this.xlang = language == Language.XLANG;
+    return this;
+  }
+
+  public ForyBuilder withXlang(boolean xlang) {
+    this.xlang = xlang;
     return this;
   }
 
@@ -120,12 +125,6 @@ public final class ForyBuilder {
     return this;
   }
 
-  /** Whether ignore basic types shared reference. */
-  public ForyBuilder ignoreBasicTypesRef(boolean ignoreBasicTypesRef) {
-    this.basicTypesRefIgnored = ignoreBasicTypesRef;
-    return this;
-  }
-
   /** Whether ignore string shared reference. */
   public ForyBuilder ignoreStringRef(boolean ignoreStringRef) {
     this.stringRefIgnored = ignoreStringRef;
@@ -133,9 +132,12 @@ public final class ForyBuilder {
   }
 
   /** ignore Enum Deserialize array out of bounds. */
-  public ForyBuilder deserializeNonexistentEnumValueAsNull(
-      boolean deserializeNonexistentEnumValueAsNull) {
-    this.unknownEnumValueStrategy = UnknownEnumValueStrategy.RETURN_NULL;
+  public ForyBuilder deserializeUnknownEnumValueAsNull(boolean deserializeUnknownEnumValueAsNull) {
+    if (deserializeUnknownEnumValueAsNull) {
+      this.unknownEnumValueStrategy = UnknownEnumValueStrategy.RETURN_NULL;
+    } else {
+      this.unknownEnumValueStrategy = UnknownEnumValueStrategy.NOT_ALLOWED;
+    }
     return this;
   }
 
@@ -182,11 +184,11 @@ public final class ForyBuilder {
   }
 
   /**
-   * Use variable length encoding for long. Enabled by default, use {@link LongEncoding#SLI} (Small
-   * long as int) for long encoding.
+   * Use variable length encoding for long. Enabled by default, use {@link LongEncoding#TAGGED}
+   * (Small long as int) for long encoding.
    */
   public ForyBuilder withLongCompressed(boolean longCompressed) {
-    return withLongCompressed(longCompressed ? LongEncoding.SLI : LongEncoding.LE_RAW_BYTES);
+    return withLongCompressed(longCompressed ? LongEncoding.TAGGED : LongEncoding.FIXED);
   }
 
   /** Use variable length encoding for long. */
@@ -258,15 +260,18 @@ public final class ForyBuilder {
     return this;
   }
 
+  public ForyBuilder withCompatible(boolean compatible) {
+    return withCompatibleMode(
+        compatible ? CompatibleMode.COMPATIBLE : CompatibleMode.SCHEMA_CONSISTENT);
+  }
+
   /**
    * Whether check class schema consistency, will be disabled automatically when {@link
    * CompatibleMode#COMPATIBLE} is enabled. Do not disable this option unless you can ensure the
    * class won't evolve.
    */
   public ForyBuilder withClassVersionCheck(boolean checkClassVersion) {
-    if (language == Language.XLANG
-        && compatibleMode == CompatibleMode.SCHEMA_CONSISTENT
-        && !checkClassVersion) {
+    if (xlang && compatibleMode == CompatibleMode.SCHEMA_CONSISTENT && !checkClassVersion) {
       throw new IllegalArgumentException(
           "XLANG Schema consistent mode must enable class version check");
     }
@@ -348,10 +353,10 @@ public final class ForyBuilder {
   /**
    * Whether deserialize/skip data of un-existed class.
    *
-   * @see Config#deserializeNonexistentClass()
+   * @see Config#deserializeUnknownClass()
    */
-  public ForyBuilder withDeserializeNonexistentClass(boolean deserializeNonexistentClass) {
-    this.deserializeNonexistentClass = deserializeNonexistentClass;
+  public ForyBuilder withDeserializeUnknownClass(boolean deserializeUnknownClass) {
+    this.deserializeUnknownClass = deserializeUnknownClass;
     return this;
   }
 
@@ -389,6 +394,14 @@ public final class ForyBuilder {
     return this;
   }
 
+  /** Set loadFactor of MapRefResolver writtenObjects. Default value is 0.51 */
+  public ForyBuilder withMapRefLoadFactor(float loadFactor) {
+    Preconditions.checkArgument(
+        loadFactor > 0 && loadFactor < 1, "loadFactor must > 0 and < 1 but got %s", loadFactor);
+    this.mapRefLoadFactor = loadFactor;
+    return this;
+  }
+
   /** Whether enable scala-specific serialization optimization. */
   public ForyBuilder withScalaOptimizationEnabled(boolean enableScalaOptimization) {
     this.scalaOptimizationEnabled = enableScalaOptimization;
@@ -418,9 +431,9 @@ public final class ForyBuilder {
         classLoader = Fory.class.getClassLoader();
       }
     }
-    if (language != Language.JAVA) {
+    if (xlang) {
       stringRefIgnored = true;
-      longEncoding = LongEncoding.PVL;
+      longEncoding = LongEncoding.VARINT;
       compressInt = true;
       compressString = true;
     }
@@ -439,12 +452,12 @@ public final class ForyBuilder {
           Serializer.class);
     }
     if (writeNumUtf16BytesForUtf8Encoding == null) {
-      writeNumUtf16BytesForUtf8Encoding = language == Language.JAVA;
+      writeNumUtf16BytesForUtf8Encoding = !xlang;
     }
     if (compatibleMode == CompatibleMode.COMPATIBLE) {
       checkClassVersion = false;
-      if (deserializeNonexistentClass == null) {
-        deserializeNonexistentClass = true;
+      if (deserializeUnknownClass == null) {
+        deserializeUnknownClass = true;
       }
       if (scopedMetaShareEnabled == null) {
         if (metaShareEnabled == null) {
@@ -459,8 +472,8 @@ public final class ForyBuilder {
         }
       }
     } else {
-      if (deserializeNonexistentClass == null) {
-        deserializeNonexistentClass = false;
+      if (deserializeUnknownClass == null) {
+        deserializeUnknownClass = false;
       }
       if (scopedMetaShareEnabled != null && scopedMetaShareEnabled) {
         LOG.warn("Scoped meta share is for CompatibleMode only, disable it for {}", compatibleMode);
@@ -469,7 +482,7 @@ public final class ForyBuilder {
       if (metaShareEnabled == null) {
         metaShareEnabled = false;
       }
-      if (language != Language.JAVA) {
+      if (xlang) {
         checkClassVersion = true;
       }
     }
@@ -488,7 +501,7 @@ public final class ForyBuilder {
 
   /**
    * Create Fory and print exception when failed. Many application will create fory as a static
-   * variable, Fory creation exception will be swallowed by {@link NoClassDefFoundError}. We print
+   * variable, Fory creation exception will be swallowed by {@link NoTypeDefFoundError}. We print
    * exception explicitly for better debugging.
    */
   private static Fory newFory(ForyBuilder builder, ClassLoader classLoader) {
