@@ -19,49 +19,58 @@
 
 package org.apache.fory.meta;
 
-import static org.apache.fory.meta.ClassDefEncoder.prependHeader;
-import static org.apache.fory.meta.ClassDefEncoder.writePkgName;
-import static org.apache.fory.meta.ClassDefEncoder.writeTypeName;
 import static org.apache.fory.meta.Encoders.fieldNameEncodingsList;
+import static org.apache.fory.meta.NativeTypeDefEncoder.prependHeader;
+import static org.apache.fory.meta.NativeTypeDefEncoder.writePkgName;
+import static org.apache.fory.meta.NativeTypeDefEncoder.writeTypeName;
 
 import java.lang.reflect.Field;
 import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 import org.apache.fory.Fory;
+import org.apache.fory.annotation.ForyField;
+import org.apache.fory.logging.Logger;
+import org.apache.fory.logging.LoggerFactory;
 import org.apache.fory.memory.MemoryBuffer;
-import org.apache.fory.meta.ClassDef.FieldInfo;
-import org.apache.fory.meta.ClassDef.FieldType;
+import org.apache.fory.meta.FieldTypes.FieldType;
 import org.apache.fory.reflect.ReflectionUtils;
-import org.apache.fory.resolver.ClassInfo;
+import org.apache.fory.resolver.TypeInfo;
 import org.apache.fory.resolver.TypeResolver;
 import org.apache.fory.resolver.XtypeResolver;
 import org.apache.fory.type.Descriptor;
 import org.apache.fory.type.DescriptorGrouper;
 import org.apache.fory.type.Types;
 import org.apache.fory.util.Preconditions;
+import org.apache.fory.util.StringUtils;
+import org.apache.fory.util.Utils;
 
 /**
- * An encoder which encode {@link ClassDef} into binary. See spec documentation:
+ * An encoder which encode {@link TypeDef} into binary. Global header layout follows the xlang spec
+ * with an 8-bit meta size and flags at bits 8/9. See spec documentation:
  * docs/specification/fory_xlang_serialization_spec.md <a
  * href="https://fory.apache.org/docs/specification/fory_xlang_serialization_spec">...</a>
  */
 class TypeDefEncoder {
+  private static final Logger LOG = LoggerFactory.getLogger(TypeDefEncoder.class);
+
   /** Build class definition from fields of class. */
-  static ClassDef buildTypeDef(Fory fory, Class<?> type) {
+  static TypeDef buildTypeDef(Fory fory, Class<?> type) {
     DescriptorGrouper descriptorGrouper =
-        fory.getClassResolver()
+        fory.getXtypeResolver()
             .createDescriptorGrouper(
-                fory.getClassResolver().getFieldDescriptors(type, true),
+                fory.getXtypeResolver().getFieldDescriptors(type, true),
                 false,
                 Function.identity());
-    ClassInfo classInfo = fory._getTypeResolver().getClassInfo(type);
+    TypeInfo typeInfo = fory.getTypeResolver().getTypeInfo(type);
     List<Field> fields;
-    int xtypeId = classInfo.getXtypeId();
-    if (Types.isStructType(xtypeId & 0xff)) {
+    int typeId = typeInfo.getTypeId();
+    if (Types.isStructType(typeId)) {
       fields =
           descriptorGrouper.getSortedDescriptors().stream()
               .map(Descriptor::getField)
@@ -69,26 +78,55 @@ class TypeDefEncoder {
     } else {
       fields = new ArrayList<>();
     }
-    return buildClassDefWithFieldInfos(
+    return buildTypeDefWithFieldInfos(
         fory.getXtypeResolver(), type, buildFieldsInfo(fory.getXtypeResolver(), type, fields));
   }
 
   static List<FieldInfo> buildFieldsInfo(TypeResolver resolver, Class<?> type, List<Field> fields) {
+    Set<Integer> usedTagIds = new HashSet<>();
     return fields.stream()
         .map(
-            field ->
-                new FieldInfo(
-                    type.getName(), field.getName(), ClassDef.buildFieldType(resolver, field)))
+            field -> {
+              ForyField foryField = field.getAnnotation(ForyField.class);
+              FieldType fieldType = FieldTypes.buildFieldType(resolver, field);
+              if (foryField != null) {
+                int tagId = foryField.id();
+                if (tagId >= 0) {
+                  if (!usedTagIds.add(tagId)) {
+                    throw new IllegalArgumentException(
+                        "Duplicate tag id "
+                            + tagId
+                            + " for field "
+                            + field.getName()
+                            + " in class "
+                            + type.getName());
+                  }
+                  return new FieldInfo(type.getName(), field.getName(), fieldType, (short) tagId);
+                }
+                // tagId == -1 means use field name, fall through to create regular FieldInfo
+              }
+              return new FieldInfo(type.getName(), field.getName(), fieldType);
+            })
         .collect(Collectors.toList());
   }
 
-  static ClassDef buildClassDefWithFieldInfos(
+  static TypeDef buildTypeDefWithFieldInfos(
       XtypeResolver resolver, Class<?> type, List<FieldInfo> fieldInfos) {
     fieldInfos = new ArrayList<>(getClassFields(type, fieldInfos).values());
-    MemoryBuffer encodeClassDef = encodeClassDef(resolver, type, fieldInfos);
-    byte[] classDefBytes = encodeClassDef.getBytes(0, encodeClassDef.writerIndex());
-    return new ClassDef(
-        Encoders.buildClassSpec(type), fieldInfos, true, encodeClassDef.getInt64(0), classDefBytes);
+    TypeInfo typeInfo = resolver.getTypeInfo(type);
+    MemoryBuffer encodeTypeDef = encodeTypeDef(resolver, type, fieldInfos);
+    byte[] typeDefBytes = encodeTypeDef.getBytes(0, encodeTypeDef.writerIndex());
+    TypeDef typeDef =
+        new TypeDef(
+            new ClassSpec(type, typeInfo.getTypeId(), typeInfo.getUserTypeId()),
+            fieldInfos,
+            true,
+            encodeTypeDef.getInt64(0),
+            typeDefBytes);
+    if (Utils.DEBUG_OUTPUT_ENABLED) {
+      LOG.info("[Java TypeDef BUILT] " + typeDef);
+    }
+    return typeDef;
   }
 
   static final int SMALL_NUM_FIELDS_THRESHOLD = 0b11111;
@@ -97,9 +135,8 @@ class TypeDefEncoder {
 
   // see spec documentation: docs/specification/xlang_serialization_spec.md
   // https://fory.apache.org/docs/specification/fory_xlang_serialization_spec
-  static MemoryBuffer encodeClassDef(
-      XtypeResolver resolver, Class<?> type, List<FieldInfo> fields) {
-    ClassInfo classInfo = resolver.getClassInfo(type);
+  static MemoryBuffer encodeTypeDef(XtypeResolver resolver, Class<?> type, List<FieldInfo> fields) {
+    TypeInfo typeInfo = resolver.getTypeInfo(type);
     MemoryBuffer buffer = MemoryBuffer.newHeapBuffer(128);
     buffer.writeByte(-1); // placeholder for header, update later
     int currentClassHeader = fields.size();
@@ -108,12 +145,17 @@ class TypeDefEncoder {
       buffer.writeVarUint32(fields.size() - SMALL_NUM_FIELDS_THRESHOLD);
     }
     if (resolver.isRegisteredById(type)) {
-      buffer.writeVarUint32(classInfo.getXtypeId());
+      buffer.writeUint8(typeInfo.getTypeId());
+      Preconditions.checkArgument(
+          typeInfo.getUserTypeId() != -1,
+          "User type id is required for typeId %s",
+          typeInfo.getTypeId());
+      buffer.writeVarUint32(typeInfo.getUserTypeId());
     } else {
       Preconditions.checkArgument(resolver.isRegisteredByName(type));
       currentClassHeader |= REGISTER_BY_NAME_FLAG;
-      String ns = classInfo.decodeNamespace();
-      String typename = classInfo.decodeTypeName();
+      String ns = typeInfo.decodeNamespace();
+      String typename = typeInfo.decodeTypeName();
       writePkgName(buffer, ns);
       writeTypeName(buffer, typename);
     }
@@ -136,7 +178,7 @@ class TypeDefEncoder {
 
   static Map<String, FieldInfo> getClassFields(Class<?> type, List<FieldInfo> fieldsInfo) {
     Map<String, FieldInfo> sortedClassFields = new LinkedHashMap<>();
-    Map<String, List<FieldInfo>> classFields = ClassDefEncoder.groupClassFields(fieldsInfo);
+    Map<String, List<FieldInfo>> classFields = NativeTypeDefEncoder.groupClassFields(fieldsInfo);
     for (Class<?> clz : ReflectionUtils.getAllClasses(type, true)) {
       List<FieldInfo> fieldInfos = classFields.get(clz.getName());
       if (fieldInfos != null) {
@@ -157,12 +199,14 @@ class TypeDefEncoder {
       header |= fieldType.nullable() ? 0b10 : 0b00;
       int size, encodingFlags;
       byte[] encoded = null;
-      if (fieldInfo.hasTag()) {
-        size = fieldInfo.getTag();
+      if (fieldInfo.hasFieldId()) {
+        size = fieldInfo.getFieldId();
         encodingFlags = 3;
       } else {
-        MetaString metaString = Encoders.encodeFieldName(fieldInfo.getFieldName());
-        // Encoding `UTF8/ALL_TO_LOWER_SPECIAL/LOWER_UPPER_DIGIT_SPECIAL/TAG_ID`
+        // Convert camelCase field names to snake_case for xlang interoperability
+        String fieldName = StringUtils.lowerCamelToLowerUnderscore(fieldInfo.getFieldName());
+        MetaString metaString = Encoders.encodeFieldName(fieldName);
+        // Encoding `UTF_8/ALL_TO_LOWER_SPECIAL/LOWER_UPPER_DIGIT_SPECIAL/TAG_ID`
         encodingFlags = fieldNameEncodingsList.indexOf(metaString.getEncoding());
         encoded = metaString.getBytes();
         size = (encoded.length - 1);
@@ -179,7 +223,7 @@ class TypeDefEncoder {
       }
       fieldType.xwrite(buffer, false);
       // write field name
-      if (!fieldInfo.hasTag()) {
+      if (!fieldInfo.hasFieldId()) {
         buffer.writeBytes(encoded);
       }
     }
