@@ -15,6 +15,8 @@
 package org.apache.fory.reflect;
 
 import java.lang.annotation.Annotation;
+import java.lang.reflect.AnnotatedArrayType;
+import java.lang.reflect.AnnotatedParameterizedType;
 import java.lang.reflect.AnnotatedType;
 import java.lang.reflect.Array;
 import java.lang.reflect.GenericArrayType;
@@ -34,13 +36,19 @@ import java.util.Objects;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 import javax.annotation.CheckForNull;
+import org.apache.fory.annotation.Ref;
+import org.apache.fory.meta.TypeExtMeta;
 import org.apache.fory.type.TypeUtils;
+import org.apache.fory.type.Types;
 
 // Mostly derived from Guava 32.1.2 com.google.common.reflect.TypeToken
 // https://github.com/google/guava/blob/9f6a3840/guava/src/com/google/common/reflect/TypeToken.java
 public class TypeRef<T> {
   private final Type type;
-  private final Object extInfo;
+  private final TypeExtMeta typeExtMeta;
+  private final List<TypeRef<?>> typeArguments;
+  private final TypeRef<?> componentType;
+  private final boolean hasTypeExtMeta;
   private transient Class<? super T> rawType;
   private transient Map<TypeVariableKey, Type> typeMappings;
 
@@ -58,27 +66,54 @@ public class TypeRef<T> {
    */
   protected TypeRef() {
     this.type = capture();
-    this.extInfo = null;
+    this.typeExtMeta = null;
+    this.typeArguments = null;
+    this.componentType = null;
+    this.hasTypeExtMeta = false;
   }
 
-  protected TypeRef(Object extInfo) {
+  protected TypeRef(TypeExtMeta typeExtMeta) {
     this.type = capture();
-    this.extInfo = extInfo;
+    this.typeExtMeta = typeExtMeta;
+    this.typeArguments = null;
+    this.componentType = null;
+    this.hasTypeExtMeta = typeExtMeta != null;
   }
 
   private TypeRef(Class<T> declaringClass) {
-    this.type = declaringClass;
-    this.extInfo = null;
+    this(declaringClass, null, null, null);
   }
 
-  private TypeRef(Class<T> declaringClass, Object extInfo) {
-    this.type = declaringClass;
-    this.extInfo = extInfo;
+  private TypeRef(Class<T> declaringClass, TypeExtMeta typeExtMeta) {
+    this(declaringClass, typeExtMeta, null, null);
   }
 
   private TypeRef(Type type) {
+    this(type, null, null, null);
+  }
+
+  private TypeRef(
+      Type type,
+      TypeExtMeta typeExtMeta,
+      List<TypeRef<?>> typeArguments,
+      TypeRef<?> componentType) {
     this.type = type;
-    this.extInfo = null;
+    this.typeExtMeta = typeExtMeta;
+    this.typeArguments = typeArguments;
+    this.componentType = componentType;
+    boolean hasMeta = typeExtMeta != null;
+    if (!hasMeta && typeArguments != null) {
+      for (TypeRef<?> typeArg : typeArguments) {
+        if (typeArg != null && typeArg.hasTypeExtMeta()) {
+          hasMeta = true;
+          break;
+        }
+      }
+    }
+    if (!hasMeta && componentType != null) {
+      hasMeta = componentType.hasTypeExtMeta();
+    }
+    this.hasTypeExtMeta = hasMeta;
   }
 
   /** Returns an instance of type token that wraps {@code type}. */
@@ -86,7 +121,7 @@ public class TypeRef<T> {
     return new TypeRef<>(clazz);
   }
 
-  public static <T> TypeRef<T> of(Class<T> clazz, Object extInfo) {
+  public static <T> TypeRef<T> of(Class<T> clazz, TypeExtMeta extInfo) {
     return new TypeRef<>(clazz, extInfo);
   }
 
@@ -95,6 +130,55 @@ public class TypeRef<T> {
     return new TypeRef<>(type);
   }
 
+  public static <T> TypeRef<T> of(AnnotatedType annotatedType) {
+    @SuppressWarnings("unchecked")
+    TypeRef<T> ref = (TypeRef<T>) ofAnnotatedType(annotatedType, false);
+    return ref;
+  }
+
+  private static TypeRef<?> ofAnnotatedType(AnnotatedType annotatedType, boolean includeRefMeta) {
+    if (annotatedType == null) {
+      return null;
+    }
+    Type type = annotatedType.getType();
+    List<TypeRef<?>> typeArguments = null;
+    TypeRef<?> componentType = null;
+    if (annotatedType instanceof AnnotatedParameterizedType) {
+      AnnotatedType[] annotatedArgs =
+          ((AnnotatedParameterizedType) annotatedType).getAnnotatedActualTypeArguments();
+      Type[] args = ((ParameterizedType) type).getActualTypeArguments();
+      List<TypeRef<?>> argRefs = new ArrayList<>(args.length);
+      boolean hasChildMeta = false;
+      for (int i = 0; i < args.length; i++) {
+        AnnotatedType annotatedArg = i < annotatedArgs.length ? annotatedArgs[i] : null;
+        TypeRef<?> argRef =
+            annotatedArg != null ? ofAnnotatedType(annotatedArg, true) : TypeRef.of(args[i]);
+        if (argRef != null && argRef.hasTypeExtMeta()) {
+          hasChildMeta = true;
+        }
+        argRefs.add(argRef);
+      }
+      if (hasChildMeta) {
+        typeArguments = Collections.unmodifiableList(argRefs);
+      }
+    } else if (annotatedType instanceof AnnotatedArrayType) {
+      AnnotatedType annotatedComponent =
+          ((AnnotatedArrayType) annotatedType).getAnnotatedGenericComponentType();
+      TypeRef<?> component = ofAnnotatedType(annotatedComponent, true);
+      if (component != null && component.hasTypeExtMeta()) {
+        componentType = component;
+      }
+    }
+    TypeExtMeta meta = null;
+    if (includeRefMeta) {
+      Ref ref = annotatedType.getAnnotation(Ref.class);
+      if (ref != null) {
+        meta = TypeExtMeta.of(Types.UNKNOWN, true, ref.enable());
+      }
+    }
+    return new TypeRef<>(type, meta, typeArguments, componentType);
+  }
+
   /** Returns the captured type. */
   private Type capture() {
     final Type superclass = getClass().getGenericSuperclass();
@@ -158,8 +242,29 @@ public class TypeRef<T> {
             });
   }
 
-  public Object getExtInfo() {
-    return extInfo;
+  public TypeExtMeta getTypeExtMeta() {
+    return typeExtMeta;
+  }
+
+  public boolean hasTypeExtMeta() {
+    return hasTypeExtMeta;
+  }
+
+  public boolean hasExplicitTypeArguments() {
+    return typeArguments != null;
+  }
+
+  public List<TypeRef<?>> getTypeArguments() {
+    if (typeArguments != null) {
+      return typeArguments;
+    }
+    if (type instanceof ParameterizedType) {
+      ParameterizedType parameterizedType = (ParameterizedType) type;
+      return Arrays.stream(parameterizedType.getActualTypeArguments())
+          .map(TypeRef::of)
+          .collect(Collectors.toList());
+    }
+    return new ArrayList<>();
   }
 
   /** Returns true if this type is one of the primitive types (including {@code void}). */
@@ -171,6 +276,165 @@ public class TypeRef<T> {
     return getRawType().isInterface();
   }
 
+  /**
+   * Returns true if this type is a wildcard type (e.g., {@code ?}, {@code ? extends Foo}, or {@code
+   * ? super Foo}), including captured wildcards.
+   */
+  public boolean isWildcard() {
+    if (type instanceof WildcardType) {
+      return true;
+    }
+    // Check for captured wildcards (TypeVariable created by WildcardCapturer)
+    if (type instanceof TypeVariable) {
+      return ((TypeVariable<?>) type).getGenericDeclaration() == WildcardCapturer.class;
+    }
+    return false;
+  }
+
+  /**
+   * Returns true if this type contains any wildcard bounds. This checks if the type itself is a
+   * wildcard, or if any of its type arguments contain wildcards (for parameterized types).
+   *
+   * <p>Examples:
+   *
+   * <ul>
+   *   <li>{@code ?} → true
+   *   <li>{@code ? extends Foo} → true
+   *   <li>{@code List<?>} → true
+   *   <li>{@code Map<String, ? extends Number>} → true
+   *   <li>{@code List<String>} → false
+   *   <li>{@code String} → false
+   * </ul>
+   */
+  public boolean hasWildcard() {
+    return containsWildcard(type);
+  }
+
+  private static boolean containsWildcard(Type t) {
+    if (t instanceof WildcardType) {
+      return true;
+    }
+    // Check for captured wildcards (TypeVariable created by WildcardCapturer)
+    if (t instanceof TypeVariable) {
+      TypeVariable<?> tv = (TypeVariable<?>) t;
+      if (tv.getGenericDeclaration() == WildcardCapturer.class) {
+        return true;
+      }
+    }
+    if (t instanceof ParameterizedType) {
+      for (Type arg : ((ParameterizedType) t).getActualTypeArguments()) {
+        if (containsWildcard(arg)) {
+          return true;
+        }
+      }
+    }
+    if (t instanceof GenericArrayType) {
+      return containsWildcard(((GenericArrayType) t).getGenericComponentType());
+    }
+    return false;
+  }
+
+  /**
+   * Resolves a wildcard type to its upper bound. This "lowers" the wildcard to a concrete type.
+   *
+   * <ul>
+   *   <li>For {@code ? extends Foo}, returns {@code TypeRef<Foo>}
+   *   <li>For {@code ? super Foo}, returns {@code TypeRef<Object>} (the implicit upper bound)
+   *   <li>For {@code ?}, returns {@code TypeRef<Object>}
+   *   <li>For captured wildcards, returns the upper bound
+   *   <li>For non-wildcard types, returns {@code this} unchanged
+   * </ul>
+   *
+   * <p>This method only resolves the top-level type. To resolve wildcards in type arguments as
+   * well, use {@link #resolveAllWildcards()}.
+   */
+  public TypeRef<?> resolveWildcard() {
+    if (type instanceof WildcardType) {
+      Type[] upperBounds = ((WildcardType) type).getUpperBounds();
+      if (upperBounds.length > 0) {
+        return of(upperBounds[0]);
+      }
+      return of(Object.class);
+    }
+    // Handle captured wildcards (TypeVariable created by WildcardCapturer)
+    if (type instanceof TypeVariable) {
+      TypeVariable<?> tv = (TypeVariable<?>) type;
+      if (tv.getGenericDeclaration() == WildcardCapturer.class) {
+        Type[] bounds = tv.getBounds();
+        if (bounds.length > 0) {
+          return of(bounds[0]);
+        }
+        return of(Object.class);
+      }
+    }
+    return this;
+  }
+
+  /**
+   * Resolves all wildcard types to their upper bounds, including wildcards in type arguments.
+   *
+   * <p>Examples:
+   *
+   * <ul>
+   *   <li>{@code ? extends String} → {@code String}
+   *   <li>{@code List<? extends String>} → {@code List<String>}
+   *   <li>{@code Map<? extends K, ? super V>} → {@code Map<K, Object>}
+   * </ul>
+   */
+  public TypeRef<?> resolveAllWildcards() {
+    Type resolved = resolveWildcardsInType(type);
+    if (resolved == type) {
+      return this;
+    }
+    return of(resolved);
+  }
+
+  private static Type resolveWildcardsInType(Type t) {
+    if (t instanceof WildcardType) {
+      Type[] upperBounds = ((WildcardType) t).getUpperBounds();
+      if (upperBounds.length > 0) {
+        return resolveWildcardsInType(upperBounds[0]);
+      }
+      return Object.class;
+    }
+    // Handle captured wildcards (TypeVariable created by WildcardCapturer)
+    if (t instanceof TypeVariable) {
+      TypeVariable<?> tv = (TypeVariable<?>) t;
+      if (tv.getGenericDeclaration() == WildcardCapturer.class) {
+        Type[] bounds = tv.getBounds();
+        if (bounds.length > 0) {
+          return resolveWildcardsInType(bounds[0]);
+        }
+        return Object.class;
+      }
+    }
+    if (t instanceof ParameterizedType) {
+      ParameterizedType pt = (ParameterizedType) t;
+      Type[] args = pt.getActualTypeArguments();
+      Type[] resolvedArgs = new Type[args.length];
+      boolean changed = false;
+      for (int i = 0; i < args.length; i++) {
+        resolvedArgs[i] = resolveWildcardsInType(args[i]);
+        if (resolvedArgs[i] != args[i]) {
+          changed = true;
+        }
+      }
+      if (!changed) {
+        return t;
+      }
+      return new ParameterizedTypeImpl(pt.getOwnerType(), pt.getRawType(), resolvedArgs);
+    }
+    if (t instanceof GenericArrayType) {
+      Type componentType = ((GenericArrayType) t).getGenericComponentType();
+      Type resolvedComponent = resolveWildcardsInType(componentType);
+      if (resolvedComponent == componentType) {
+        return t;
+      }
+      return newArrayType(resolvedComponent);
+    }
+    return t;
+  }
+
   /**
    * Returns true if this type is known to be an array type, such as {@code int[]}, {@code T[]},
    * {@code <? extends Map<String, Integer>[]>} etc.
@@ -184,6 +448,9 @@ public class TypeRef<T> {
    * {@code <? extends Map<String, Integer>[]>} etc.), or else {@code null} is returned.
    */
   public TypeRef<?> getComponentType() {
+    if (componentType != null) {
+      return componentType;
+    }
     return of(getComponentType(type));
   }
 
