@@ -23,12 +23,14 @@ import java.lang.reflect.Array;
 import java.util.Arrays;
 import org.apache.fory.Fory;
 import org.apache.fory.config.CompatibleMode;
+import org.apache.fory.config.LongEncoding;
 import org.apache.fory.memory.MemoryBuffer;
 import org.apache.fory.memory.Platform;
-import org.apache.fory.resolver.ClassInfo;
-import org.apache.fory.resolver.ClassInfoHolder;
 import org.apache.fory.resolver.ClassResolver;
 import org.apache.fory.resolver.RefResolver;
+import org.apache.fory.resolver.TypeInfo;
+import org.apache.fory.resolver.TypeInfoHolder;
+import org.apache.fory.resolver.TypeResolver;
 import org.apache.fory.serializer.collection.CollectionFlags;
 import org.apache.fory.serializer.collection.ForyArrayAsListSerializer;
 import org.apache.fory.type.GenericType;
@@ -43,7 +45,7 @@ public class ArraySerializers {
   public static final class ObjectArraySerializer<T> extends Serializer<T[]> {
     private final Class<T> innerType;
     private final Serializer componentTypeSerializer;
-    private final ClassInfoHolder classInfoHolder;
+    private final TypeInfoHolder classInfoHolder;
     private final int[] stubDims;
     private final GenericType componentGenericType;
 
@@ -71,11 +73,11 @@ public class ArraySerializers {
           this.componentTypeSerializer = fory.getClassResolver().getSerializer(componentType);
         }
       } else {
-        // TODO add ClassInfo cache for non-final component type.
+        // TODO add TypeInfo cache for non-final component type.
         this.componentTypeSerializer = null;
       }
       this.stubDims = new int[dimension];
-      classInfoHolder = fory.getClassResolver().nilClassInfoHolder();
+      classInfoHolder = fory.getClassResolver().nilTypeInfoHolder();
     }
 
     @Override
@@ -94,16 +96,16 @@ public class ArraySerializers {
       } else {
         Fory fory = this.fory;
         ClassResolver classResolver = fory.getClassResolver();
-        ClassInfo classInfo = null;
+        TypeInfo typeInfo = null;
         Class<?> elemClass = null;
         for (T t : arr) {
           if (!refResolver.writeRefOrNull(buffer, t)) {
             Class<?> clz = t.getClass();
             if (clz != elemClass) {
               elemClass = clz;
-              classInfo = classResolver.getClassInfo(clz);
+              typeInfo = classResolver.getTypeInfo(clz);
             }
-            fory.writeNonRef(buffer, t, classInfo);
+            fory.writeNonRef(buffer, t, typeInfo);
           }
         }
       }
@@ -166,7 +168,7 @@ public class ArraySerializers {
         }
       } else {
         Fory fory = this.fory;
-        ClassInfoHolder classInfoHolder = this.classInfoHolder;
+        TypeInfoHolder classInfoHolder = this.classInfoHolder;
         for (int i = 0; i < numElements; i++) {
           int nextReadRefId = refResolver.tryPreserveRefId(buffer);
           Object o;
@@ -356,7 +358,11 @@ public class ArraySerializers {
     public void write(MemoryBuffer buffer, char[] value) {
       if (fory.getBufferCallback() == null) {
         int size = Math.multiplyExact(value.length, 2);
-        buffer.writePrimitiveArrayWithSize(value, Platform.CHAR_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.writePrimitiveArrayWithSize(value, Platform.CHAR_ARRAY_OFFSET, size);
+        } else {
+          writeCharBySwapEndian(buffer, value);
+        }
       } else {
         fory.writeBufferObject(
             buffer,
@@ -364,6 +370,17 @@ public class ArraySerializers {
       }
     }
 
+    private void writeCharBySwapEndian(MemoryBuffer buffer, char[] value) {
+      int idx = buffer.writerIndex();
+      int length = value.length;
+      buffer.ensure(idx + 5 + length * 2);
+      idx += buffer._unsafeWriteVarUint32(length * 2);
+      for (int i = 0; i < length; i++) {
+        buffer._unsafePutInt16(idx + i * 2, (short) value[i]);
+      }
+      buffer._unsafeWriterIndex(idx + length * 2);
+    }
+
     @Override
     public char[] copy(char[] originArray) {
       return Arrays.copyOf(originArray, originArray.length);
@@ -376,17 +393,35 @@ public class ArraySerializers {
         int size = buf.remaining();
         int numElements = size / 2;
         char[] values = new char[numElements];
-        buf.copyToUnsafe(0, values, Platform.CHAR_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buf.copyToUnsafe(0, values, Platform.CHAR_ARRAY_OFFSET, size);
+        } else {
+          readCharBySwapEndian(buf, values, numElements);
+        }
         return values;
       } else {
         int size = buffer.readVarUint32Small7();
         int numElements = size / 2;
         char[] values = new char[numElements];
-        buffer.readToUnsafe(values, Platform.CHAR_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.readToUnsafe(values, Platform.CHAR_ARRAY_OFFSET, size);
+        } else {
+          readCharBySwapEndian(buffer, values, numElements);
+        }
         return values;
       }
     }
 
+    private void readCharBySwapEndian(MemoryBuffer buffer, char[] values, int numElements) {
+      int idx = buffer.readerIndex();
+      int size = numElements * 2;
+      buffer.checkReadableBytes(size);
+      for (int i = 0; i < numElements; i++) {
+        values[i] = (char) buffer._unsafeGetInt16(idx + i * 2);
+      }
+      buffer._increaseReaderIndexUnsafe(size);
+    }
+
     @Override
     public void xwrite(MemoryBuffer buffer, char[] value) {
       throw new UnsupportedOperationException();
@@ -408,7 +443,11 @@ public class ArraySerializers {
     public void write(MemoryBuffer buffer, short[] value) {
       if (fory.getBufferCallback() == null) {
         int size = Math.multiplyExact(value.length, 2);
-        buffer.writePrimitiveArrayWithSize(value, Platform.SHORT_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.writePrimitiveArrayWithSize(value, Platform.SHORT_ARRAY_OFFSET, size);
+        } else {
+          writeInt16BySwapEndian(buffer, value);
+        }
       } else {
         fory.writeBufferObject(
             buffer,
@@ -416,6 +455,17 @@ public class ArraySerializers {
       }
     }
 
+    private void writeInt16BySwapEndian(MemoryBuffer buffer, short[] value) {
+      int idx = buffer.writerIndex();
+      int length = value.length;
+      buffer.ensure(idx + 5 + length * 2);
+      idx += buffer._unsafeWriteVarUint32(length * 2);
+      for (int i = 0; i < length; i++) {
+        buffer._unsafePutInt16(idx + i * 2, value[i]);
+      }
+      buffer._unsafeWriterIndex(idx + length * 2);
+    }
+
     @Override
     public short[] copy(short[] originArray) {
       return Arrays.copyOf(originArray, originArray.length);
@@ -428,16 +478,34 @@ public class ArraySerializers {
         int size = buf.remaining();
         int numElements = size / 2;
         short[] values = new short[numElements];
-        buf.copyToUnsafe(0, values, Platform.SHORT_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buf.copyToUnsafe(0, values, Platform.SHORT_ARRAY_OFFSET, size);
+        } else {
+          readInt16BySwapEndian(buf, values, numElements);
+        }
         return values;
       } else {
         int size = buffer.readVarUint32Small7();
         int numElements = size / 2;
         short[] values = new short[numElements];
-        buffer.readToUnsafe(values, Platform.SHORT_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.readToUnsafe(values, Platform.SHORT_ARRAY_OFFSET, size);
+        } else {
+          readInt16BySwapEndian(buffer, values, numElements);
+        }
         return values;
       }
     }
+
+    private void readInt16BySwapEndian(MemoryBuffer buffer, short[] values, int numElements) {
+      int idx = buffer.readerIndex();
+      int size = numElements * 2;
+      buffer.checkReadableBytes(size);
+      for (int i = 0; i < numElements; i++) {
+        values[i] = buffer._unsafeGetInt16(idx + i * 2);
+      }
+      buffer._increaseReaderIndexUnsafe(size);
+    }
   }
 
   public static final class IntArraySerializer extends PrimitiveArraySerializer<int[]> {
@@ -449,8 +517,16 @@ public class ArraySerializers {
     @Override
     public void write(MemoryBuffer buffer, int[] value) {
       if (fory.getBufferCallback() == null) {
+        if (fory.getConfig().compressIntArray()) {
+          writeInt32Compressed(buffer, value);
+          return;
+        }
         int size = Math.multiplyExact(value.length, 4);
-        buffer.writePrimitiveArrayWithSize(value, Platform.INT_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.writePrimitiveArrayWithSize(value, Platform.INT_ARRAY_OFFSET, size);
+        } else {
+          writeInt32BySwapEndian(buffer, value);
+        }
       } else {
         fory.writeBufferObject(
             buffer,
@@ -458,6 +534,17 @@ public class ArraySerializers {
       }
     }
 
+    private void writeInt32BySwapEndian(MemoryBuffer buffer, int[] value) {
+      int idx = buffer.writerIndex();
+      int length = value.length;
+      buffer.ensure(idx + 5 + length * 4);
+      idx += buffer._unsafeWriteVarUint32(length * 4);
+      for (int i = 0; i < length; i++) {
+        buffer._unsafePutInt32(idx + i * 4, value[i]);
+      }
+      buffer._unsafeWriterIndex(idx + length * 4);
+    }
+
     @Override
     public int[] copy(int[] originArray) {
       return Arrays.copyOf(originArray, originArray.length);
@@ -471,32 +558,81 @@ public class ArraySerializers {
         int numElements = size / 4;
         int[] values = new int[numElements];
         if (size > 0) {
-          buf.copyToUnsafe(0, values, Platform.INT_ARRAY_OFFSET, size);
+          if (Platform.IS_LITTLE_ENDIAN) {
+            buf.copyToUnsafe(0, values, Platform.INT_ARRAY_OFFSET, size);
+          } else {
+            readInt32BySwapEndian(buf, values, numElements);
+          }
         }
         return values;
       }
-
+      if (fory.getConfig().compressIntArray()) {
+        return readInt32Compressed(buffer);
+      }
       int size = buffer.readVarUint32Small7();
       int numElements = size / 4;
       int[] values = new int[numElements];
       if (size > 0) {
-        buffer.readToUnsafe(values, Platform.INT_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.readToUnsafe(values, Platform.INT_ARRAY_OFFSET, size);
+        } else {
+          readInt32BySwapEndian(buffer, values, numElements);
+        }
+      }
+      return values;
+    }
+
+    private void readInt32BySwapEndian(MemoryBuffer buffer, int[] values, int numElements) {
+      int idx = buffer.readerIndex();
+      int size = numElements * 4;
+      buffer.checkReadableBytes(size);
+      for (int i = 0; i < numElements; i++) {
+        values[i] = buffer._unsafeGetInt32(idx + i * 4);
+      }
+      buffer._increaseReaderIndexUnsafe(size);
+    }
+
+    private void writeInt32Compressed(MemoryBuffer buffer, int[] value) {
+      buffer.writeVarUint32Small7(value.length);
+      for (int i : value) {
+        buffer.writeVarInt32(i);
+      }
+    }
+
+    private int[] readInt32Compressed(MemoryBuffer buffer) {
+      int numElements = buffer.readVarUint32Small7();
+      int[] values = new int[numElements];
+
+      for (int i = 0; i < numElements; i++) {
+        values[i] = buffer.readVarInt32();
       }
       return values;
     }
   }
 
   public static final class LongArraySerializer extends PrimitiveArraySerializer<long[]> {
+    private final boolean compressLongArray;
 
     public LongArraySerializer(Fory fory) {
       super(fory, long[].class);
+      compressLongArray =
+          fory.getConfig().compressLongArray()
+              && fory.getConfig().longEncoding() != LongEncoding.FIXED;
     }
 
     @Override
     public void write(MemoryBuffer buffer, long[] value) {
       if (fory.getBufferCallback() == null) {
+        if (compressLongArray) {
+          writeInt64Compressed(buffer, value, fory.getConfig().longEncoding());
+          return;
+        }
         int size = Math.multiplyExact(value.length, 8);
-        buffer.writePrimitiveArrayWithSize(value, Platform.LONG_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.writePrimitiveArrayWithSize(value, Platform.LONG_ARRAY_OFFSET, size);
+        } else {
+          writeInt64BySwapEndian(buffer, value);
+        }
       } else {
         fory.writeBufferObject(
             buffer,
@@ -504,6 +640,17 @@ public class ArraySerializers {
       }
     }
 
+    private void writeInt64BySwapEndian(MemoryBuffer buffer, long[] value) {
+      int idx = buffer.writerIndex();
+      int length = value.length;
+      buffer.ensure(idx + 5 + length * 8);
+      idx += buffer._unsafeWriteVarUint32(length * 8);
+      for (int i = 0; i < length; i++) {
+        buffer._unsafePutInt64(idx + i * 8, value[i]);
+      }
+      buffer._unsafeWriterIndex(idx + length * 8);
+    }
+
     @Override
     public long[] copy(long[] originArray) {
       return Arrays.copyOf(originArray, originArray.length);
@@ -517,16 +664,68 @@ public class ArraySerializers {
         int numElements = size / 8;
         long[] values = new long[numElements];
         if (size > 0) {
-          buf.copyToUnsafe(0, values, Platform.LONG_ARRAY_OFFSET, size);
+          if (Platform.IS_LITTLE_ENDIAN) {
+            buf.copyToUnsafe(0, values, Platform.LONG_ARRAY_OFFSET, size);
+          } else {
+            readInt64BySwapEndian(buf, values, numElements);
+          }
         }
         return values;
       }
-
+      if (compressLongArray) {
+        return readInt64Compressed(buffer, fory.getConfig().longEncoding());
+      }
       int size = buffer.readVarUint32Small7();
       int numElements = size / 8;
       long[] values = new long[numElements];
       if (size > 0) {
-        buffer.readToUnsafe(values, Platform.LONG_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.readToUnsafe(values, Platform.LONG_ARRAY_OFFSET, size);
+        } else {
+          readInt64BySwapEndian(buffer, values, numElements);
+        }
+      }
+      return values;
+    }
+
+    private void readInt64BySwapEndian(MemoryBuffer buffer, long[] values, int numElements) {
+      int idx = buffer.readerIndex();
+      int size = numElements * 8;
+      buffer.checkReadableBytes(size);
+      for (int i = 0; i < numElements; i++) {
+        values[i] = buffer._unsafeGetInt64(idx + i * 8);
+      }
+      buffer._increaseReaderIndexUnsafe(size);
+    }
+
+    private void writeInt64Compressed(
+        MemoryBuffer buffer, long[] value, LongEncoding longEncoding) {
+      int length = value.length;
+      buffer.writeVarUint32Small7(length);
+
+      if (longEncoding == LongEncoding.TAGGED) {
+        for (int i = 0; i < length; i++) {
+          buffer.writeTaggedInt64(value[i]);
+        }
+        return;
+      }
+      for (int i = 0; i < length; i++) {
+        buffer.writeVarInt64(value[i]);
+      }
+    }
+
+    private long[] readInt64Compressed(MemoryBuffer buffer, LongEncoding longEncoding) {
+      int numElements = buffer.readVarUint32Small7();
+      long[] values = new long[numElements];
+
+      if (longEncoding == LongEncoding.TAGGED) {
+        for (int i = 0; i < numElements; i++) {
+          values[i] = buffer.readTaggedInt64();
+        }
+      } else {
+        for (int i = 0; i < numElements; i++) {
+          values[i] = buffer.readVarInt64();
+        }
       }
       return values;
     }
@@ -542,7 +741,11 @@ public class ArraySerializers {
     public void write(MemoryBuffer buffer, float[] value) {
       if (fory.getBufferCallback() == null) {
         int size = Math.multiplyExact(value.length, 4);
-        buffer.writePrimitiveArrayWithSize(value, Platform.FLOAT_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.writePrimitiveArrayWithSize(value, Platform.FLOAT_ARRAY_OFFSET, size);
+        } else {
+          writeFloat32BySwapEndian(buffer, value);
+        }
       } else {
         fory.writeBufferObject(
             buffer,
@@ -550,6 +753,17 @@ public class ArraySerializers {
       }
     }
 
+    private void writeFloat32BySwapEndian(MemoryBuffer buffer, float[] value) {
+      int idx = buffer.writerIndex();
+      int length = value.length;
+      buffer.ensure(idx + 5 + length * 4);
+      idx += buffer._unsafeWriteVarUint32(length * 4);
+      for (int i = 0; i < length; i++) {
+        buffer._unsafePutInt32(idx + i * 4, Float.floatToRawIntBits(value[i]));
+      }
+      buffer._unsafeWriterIndex(idx + length * 4);
+    }
+
     @Override
     public float[] copy(float[] originArray) {
       return Arrays.copyOf(originArray, originArray.length);
@@ -562,16 +776,34 @@ public class ArraySerializers {
         int size = buf.remaining();
         int numElements = size / 4;
         float[] values = new float[numElements];
-        buf.copyToUnsafe(0, values, Platform.FLOAT_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buf.copyToUnsafe(0, values, Platform.FLOAT_ARRAY_OFFSET, size);
+        } else {
+          readFloat32BySwapEndian(buf, values, numElements);
+        }
         return values;
       } else {
         int size = buffer.readVarUint32Small7();
         int numElements = size / 4;
         float[] values = new float[numElements];
-        buffer.readToUnsafe(values, Platform.FLOAT_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.readToUnsafe(values, Platform.FLOAT_ARRAY_OFFSET, size);
+        } else {
+          readFloat32BySwapEndian(buffer, values, numElements);
+        }
         return values;
       }
     }
+
+    private void readFloat32BySwapEndian(MemoryBuffer buffer, float[] values, int numElements) {
+      int idx = buffer.readerIndex();
+      int size = numElements * 4;
+      buffer.checkReadableBytes(size);
+      for (int i = 0; i < numElements; i++) {
+        values[i] = Float.intBitsToFloat(buffer._unsafeGetInt32(idx + i * 4));
+      }
+      buffer._increaseReaderIndexUnsafe(size);
+    }
   }
 
   public static final class DoubleArraySerializer extends PrimitiveArraySerializer<double[]> {
@@ -584,7 +816,11 @@ public class ArraySerializers {
     public void write(MemoryBuffer buffer, double[] value) {
       if (fory.getBufferCallback() == null) {
         int size = Math.multiplyExact(value.length, 8);
-        buffer.writePrimitiveArrayWithSize(value, Platform.DOUBLE_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.writePrimitiveArrayWithSize(value, Platform.DOUBLE_ARRAY_OFFSET, size);
+        } else {
+          writeFloat64BySwapEndian(buffer, value);
+        }
       } else {
         fory.writeBufferObject(
             buffer,
@@ -592,6 +828,17 @@ public class ArraySerializers {
       }
     }
 
+    private void writeFloat64BySwapEndian(MemoryBuffer buffer, double[] value) {
+      int idx = buffer.writerIndex();
+      int length = value.length;
+      buffer.ensure(idx + 5 + length * 8);
+      idx += buffer._unsafeWriteVarUint32(length * 8);
+      for (int i = 0; i < length; i++) {
+        buffer._unsafePutInt64(idx + i * 8, Double.doubleToRawLongBits(value[i]));
+      }
+      buffer._unsafeWriterIndex(idx + length * 8);
+    }
+
     @Override
     public double[] copy(double[] originArray) {
       return Arrays.copyOf(originArray, originArray.length);
@@ -604,16 +851,34 @@ public class ArraySerializers {
         int size = buf.remaining();
         int numElements = size / 8;
         double[] values = new double[numElements];
-        buf.copyToUnsafe(0, values, Platform.DOUBLE_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buf.copyToUnsafe(0, values, Platform.DOUBLE_ARRAY_OFFSET, size);
+        } else {
+          readFloat64BySwapEndian(buf, values, numElements);
+        }
         return values;
       } else {
         int size = buffer.readVarUint32Small7();
         int numElements = size / 8;
         double[] values = new double[numElements];
-        buffer.readToUnsafe(values, Platform.DOUBLE_ARRAY_OFFSET, size);
+        if (Platform.IS_LITTLE_ENDIAN) {
+          buffer.readToUnsafe(values, Platform.DOUBLE_ARRAY_OFFSET, size);
+        } else {
+          readFloat64BySwapEndian(buffer, values, numElements);
+        }
         return values;
       }
     }
+
+    private void readFloat64BySwapEndian(MemoryBuffer buffer, double[] values, int numElements) {
+      int idx = buffer.readerIndex();
+      int size = numElements * 8;
+      buffer.checkReadableBytes(size);
+      for (int i = 0; i < numElements; i++) {
+        values[i] = Double.longBitsToDouble(buffer._unsafeGetInt64(idx + i * 8));
+      }
+      buffer._increaseReaderIndexUnsafe(size);
+    }
   }
 
   public static final class StringArraySerializer extends Serializer<String[]> {
@@ -717,7 +982,7 @@ public class ArraySerializers {
   }
 
   public static void registerDefaultSerializers(Fory fory) {
-    ClassResolver resolver = fory.getClassResolver();
+    TypeResolver resolver = fory.getTypeResolver();
     resolver.registerInternalSerializer(
         Object[].class, new ObjectArraySerializer<>(fory, Object[].class));
     resolver.registerInternalSerializer(
@@ -766,17 +1031,21 @@ public class ArraySerializers {
     return new PrimitiveArrayBufferObject(array, Platform.BYTE_ARRAY_OFFSET, 1, array.length);
   }
 
-  public abstract static class AbstractedNonexistentArrayClassSerializer extends Serializer {
+  public abstract static class AbstractUnknownArraySerializer extends Serializer {
     protected final String className;
     private final int dims;
 
-    public AbstractedNonexistentArrayClassSerializer(
-        Fory fory, String className, Class<?> stubClass) {
+    public AbstractUnknownArraySerializer(Fory fory, String className, Class<?> stubClass) {
       super(fory, stubClass);
       this.className = className;
       this.dims = TypeUtils.getArrayDimensions(stubClass);
     }
 
+    @Override
+    public void write(MemoryBuffer buffer, Object value) {
+      throw new UnsupportedOperationException();
+    }
+
     @Override
     public Object[] read(MemoryBuffer buffer) {
       switch (dims) {
@@ -878,22 +1147,20 @@ public class ArraySerializers {
   }
 
   @SuppressWarnings("rawtypes")
-  public static final class NonexistentArrayClassSerializer
-      extends AbstractedNonexistentArrayClassSerializer {
+  public static final class UnknownArraySerializer extends AbstractUnknownArraySerializer {
     private final Serializer componentSerializer;
 
-    public NonexistentArrayClassSerializer(Fory fory, Class<?> cls) {
+    public UnknownArraySerializer(Fory fory, Class<?> cls) {
       this(fory, "Unknown", cls);
     }
 
-    public NonexistentArrayClassSerializer(Fory fory, String className, Class<?> cls) {
+    public UnknownArraySerializer(Fory fory, String className, Class<?> cls) {
       super(fory, className, cls);
       if (TypeUtils.getArrayComponent(cls).isEnum()) {
-        componentSerializer = new NonexistentClassSerializers.NonexistentEnumClassSerializer(fory);
+        componentSerializer = new UnknownClassSerializers.UnknownEnumSerializer(fory);
       } else {
         if (fory.getConfig().getCompatibleMode() == CompatibleMode.COMPATIBLE) {
-          componentSerializer =
-              new ObjectSerializer<>(fory, NonexistentClass.NonexistentSkip.class);
+          componentSerializer = new ObjectSerializer<>(fory, UnknownClass.UnknownEmptyStruct.class);
         } else {
           componentSerializer = null;
         }
