@@ -25,30 +25,33 @@ import java.util.List;
 import java.util.stream.Collectors;
 import org.apache.fory.Fory;
 import org.apache.fory.builder.MetaSharedCodecBuilder;
-import org.apache.fory.collection.Tuple2;
-import org.apache.fory.collection.Tuple3;
 import org.apache.fory.config.CompatibleMode;
 import org.apache.fory.config.ForyBuilder;
+import org.apache.fory.logging.Logger;
+import org.apache.fory.logging.LoggerFactory;
 import org.apache.fory.memory.MemoryBuffer;
 import org.apache.fory.memory.Platform;
-import org.apache.fory.meta.ClassDef;
+import org.apache.fory.meta.TypeDef;
 import org.apache.fory.reflect.FieldAccessor;
-import org.apache.fory.resolver.ClassInfoHolder;
-import org.apache.fory.resolver.ClassResolver;
+import org.apache.fory.resolver.MapRefResolver;
 import org.apache.fory.resolver.RefResolver;
 import org.apache.fory.resolver.TypeResolver;
+import org.apache.fory.serializer.FieldGroups.SerializationFieldInfo;
 import org.apache.fory.type.Descriptor;
 import org.apache.fory.type.DescriptorGrouper;
 import org.apache.fory.type.Generics;
+import org.apache.fory.type.Types;
 import org.apache.fory.util.DefaultValueUtils;
 import org.apache.fory.util.GraalvmSupport;
 import org.apache.fory.util.Preconditions;
+import org.apache.fory.util.StringUtils;
+import org.apache.fory.util.Utils;
 import org.apache.fory.util.record.RecordInfo;
 import org.apache.fory.util.record.RecordUtils;
 
 /**
- * A meta-shared compatible deserializer builder based on {@link ClassDef}. This serializer will
- * compare fields between {@link ClassDef} and class fields, then create serializer to read and
+ * A meta-shared compatible deserializer builder based on {@link TypeDef}. This serializer will
+ * compare fields between {@link TypeDef} and class fields, then create serializer to read and
  * set/skip corresponding fields to support type forward/backward compatibility. Serializer are
  * forward to {@link ObjectSerializer} for now. We can consolidate fields between peers to create
  * better serializers to serialize common fields between peers for efficiency.
@@ -62,47 +65,53 @@ import org.apache.fory.util.record.RecordUtils;
  * @see MetaSharedCodecBuilder
  * @see ObjectSerializer
  */
-@SuppressWarnings({"unchecked"})
 public class MetaSharedSerializer<T> extends AbstractObjectSerializer<T> {
-  private final ObjectSerializer.FinalTypeField[] finalFields;
+  private static final Logger LOG = LoggerFactory.getLogger(MetaSharedSerializer.class);
 
-  /**
-   * Whether write class def for non-inner final types.
-   *
-   * @see ClassResolver#isMonomorphic(Class)
-   */
-  private final boolean[] isFinal;
-
-  private final ObjectSerializer.GenericTypeField[] otherFields;
-  private final ObjectSerializer.GenericTypeField[] containerFields;
+  private final SerializationFieldInfo[] buildInFields;
+  private final SerializationFieldInfo[] containerFields;
+  private final SerializationFieldInfo[] otherFields;
   private final RecordInfo recordInfo;
   private Serializer<T> serializer;
-  private final ClassInfoHolder classInfoHolder;
   private final SerializationBinding binding;
   private final boolean hasDefaultValues;
   private final DefaultValueUtils.DefaultValueField[] defaultValueFields;
 
-  public MetaSharedSerializer(Fory fory, Class<T> type, ClassDef classDef) {
+  public MetaSharedSerializer(Fory fory, Class<T> type, TypeDef typeDef) {
     super(fory, type);
     Preconditions.checkArgument(
         !fory.getConfig().checkClassVersion(),
         "Class version check should be disabled when compatible mode is enabled.");
     Preconditions.checkArgument(
         fory.getConfig().isMetaShareEnabled(), "Meta share must be enabled.");
-    Collection<Descriptor> descriptors = consolidateFields(fory._getTypeResolver(), type, classDef);
+    if (Utils.DEBUG_OUTPUT_ENABLED) {
+      LOG.info("========== MetaSharedSerializer TypeDef for {} ==========", type.getName());
+      LOG.info("TypeDef fieldsInfo count: {}", typeDef.getFieldCount());
+      for (int i = 0; i < typeDef.getFieldsInfo().size(); i++) {
+        LOG.info("  [{}] {}", i, typeDef.getFieldsInfo().get(i));
+      }
+    }
+    Collection<Descriptor> descriptors = consolidateFields(fory.getTypeResolver(), type, typeDef);
     DescriptorGrouper descriptorGrouper =
-        fory._getTypeResolver().createDescriptorGrouper(descriptors, false);
+        fory.getTypeResolver().createDescriptorGrouper(descriptors, false);
+    if (Utils.DEBUG_OUTPUT_ENABLED) {
+      LOG.info(
+          "========== MetaSharedSerializer sorted descriptors for {} ==========", type.getName());
+      for (Descriptor d : descriptorGrouper.getSortedDescriptors()) {
+        LOG.info(
+            "  {} -> {}, ref {}, nullable {}, type id {}",
+            StringUtils.toSnakeCase(d.getName()),
+            d.getTypeName(),
+            d.isTrackingRef(),
+            d.isNullable(),
+            Types.getDescriptorTypeId(fory, d));
+      }
+    }
     // d.getField() may be null if not exists in this class when meta share enabled.
-    Tuple3<
-            Tuple2<ObjectSerializer.FinalTypeField[], boolean[]>,
-            ObjectSerializer.GenericTypeField[],
-            ObjectSerializer.GenericTypeField[]>
-        infos = AbstractObjectSerializer.buildFieldInfos(fory, descriptorGrouper);
-    finalFields = infos.f0.f0;
-    isFinal = infos.f0.f1;
-    otherFields = infos.f1;
-    containerFields = infos.f2;
-    classInfoHolder = this.classResolver.nilClassInfoHolder();
+    FieldGroups fieldGroups = FieldGroups.buildFieldInfos(fory, descriptorGrouper);
+    buildInFields = fieldGroups.buildInFields;
+    containerFields = fieldGroups.containerFields;
+    otherFields = fieldGroups.userTypeFields;
     if (isRecord) {
       List<String> fieldNames =
           descriptorGrouper.getSortedDescriptors().stream()
@@ -116,15 +125,13 @@ public class MetaSharedSerializer<T> extends AbstractObjectSerializer<T> {
     boolean hasDefaultValues = false;
     DefaultValueUtils.DefaultValueField[] defaultValueFields =
         new DefaultValueUtils.DefaultValueField[0];
-    DefaultValueUtils.DefaultValueSupport defaultValueSupport = null;
+    DefaultValueUtils.DefaultValueSupport defaultValueSupport;
     if (fory.getConfig().isScalaOptimizationEnabled()) {
       defaultValueSupport = DefaultValueUtils.getScalaDefaultValueSupport();
-      if (defaultValueSupport != null) {
-        hasDefaultValues = defaultValueSupport.hasDefaultValues(type);
-        defaultValueFields =
-            defaultValueSupport.buildDefaultValueFields(
-                fory, type, descriptorGrouper.getSortedDescriptors());
-      }
+      hasDefaultValues = defaultValueSupport.hasDefaultValues(type);
+      defaultValueFields =
+          defaultValueSupport.buildDefaultValueFields(
+              fory, type, descriptorGrouper.getSortedDescriptors());
     }
     if (!hasDefaultValues) {
       DefaultValueUtils.DefaultValueSupport kotlinDefaultValueSupport =
@@ -140,108 +147,19 @@ public class MetaSharedSerializer<T> extends AbstractObjectSerializer<T> {
     this.defaultValueFields = defaultValueFields;
   }
 
-  @Override
-  public void write(MemoryBuffer buffer, T value) {
-    if (serializer == null) {
-      serializer =
-          this.classResolver.createSerializerSafe(type, () -> new ObjectSerializer<>(fory, type));
-    }
-    serializer.write(buffer, value);
-  }
-
   @Override
   public void xwrite(MemoryBuffer buffer, T value) {
     write(buffer, value);
   }
 
   @Override
-  public T read(MemoryBuffer buffer) {
-    if (isRecord) {
-      Object[] fieldValues =
-          new Object[finalFields.length + otherFields.length + containerFields.length];
-      readFields(buffer, fieldValues);
-      fieldValues = RecordUtils.remapping(recordInfo, fieldValues);
-      T t = objectCreator.newInstanceWithArguments(fieldValues);
-      Arrays.fill(recordInfo.getRecordComponents(), null);
-      return t;
+  public void write(MemoryBuffer buffer, T value) {
+    if (serializer == null) {
+      // xlang mode will register class and create serializer in advance, it won't go to here.
+      serializer =
+          this.classResolver.createSerializerSafe(type, () -> new ObjectSerializer<>(fory, type));
     }
-    T obj = newInstance();
-    Fory fory = this.fory;
-    RefResolver refResolver = this.refResolver;
-    ClassResolver classResolver = this.classResolver;
-    SerializationBinding binding = this.binding;
-    refResolver.reference(obj);
-    // read order: primitive,boxed,final,other,collection,map
-    ObjectSerializer.FinalTypeField[] finalFields = this.finalFields;
-    for (int i = 0; i < finalFields.length; i++) {
-      ObjectSerializer.FinalTypeField fieldInfo = finalFields[i];
-      boolean isFinal = this.isFinal[i];
-      FieldAccessor fieldAccessor = fieldInfo.fieldAccessor;
-      boolean nullable = fieldInfo.nullable;
-      if (fieldAccessor != null) {
-        short classId = fieldInfo.classId;
-        if (AbstractObjectSerializer.readPrimitiveFieldValueFailed(
-                fory, buffer, obj, fieldAccessor, classId)
-            && (nullable
-                ? AbstractObjectSerializer.readBasicNullableObjectFieldValueFailed(
-                    fory, buffer, obj, fieldAccessor, classId)
-                : AbstractObjectSerializer.readBasicObjectFieldValueFailed(
-                    fory, buffer, obj, fieldAccessor, classId))) {
-          assert fieldInfo.classInfo != null;
-          Object fieldValue =
-              AbstractObjectSerializer.readFinalObjectFieldValue(
-                  binding, refResolver, classResolver, fieldInfo, isFinal, buffer);
-          fieldAccessor.putObject(obj, fieldValue);
-        }
-      } else {
-        if (fieldInfo.fieldConverter == null) {
-          // Skip the field value from buffer since it doesn't exist in current class
-          if (skipPrimitiveFieldValueFailed(fory, fieldInfo.classId, buffer)) {
-            if (fieldInfo.classInfo == null) {
-              // TODO(chaokunyang) support registered serializer in peer with ref tracking disabled.
-              fory.readRef(buffer, classInfoHolder);
-            } else {
-              AbstractObjectSerializer.readFinalObjectFieldValue(
-                  binding, refResolver, classResolver, fieldInfo, isFinal, buffer);
-            }
-          }
-        } else {
-          compatibleRead(buffer, fieldInfo, isFinal, obj);
-        }
-      }
-    }
-    Generics generics = fory.getGenerics();
-    for (ObjectSerializer.GenericTypeField fieldInfo : containerFields) {
-      Object fieldValue =
-          AbstractObjectSerializer.readContainerFieldValue(binding, generics, fieldInfo, buffer);
-      FieldAccessor fieldAccessor = fieldInfo.fieldAccessor;
-      if (fieldAccessor != null) {
-        fieldAccessor.putObject(obj, fieldValue);
-      }
-    }
-    for (ObjectSerializer.GenericTypeField fieldInfo : otherFields) {
-      Object fieldValue = AbstractObjectSerializer.readOtherFieldValue(binding, fieldInfo, buffer);
-      FieldAccessor fieldAccessor = fieldInfo.fieldAccessor;
-      if (fieldAccessor != null) {
-        fieldAccessor.putObject(obj, fieldValue);
-      }
-    }
-    return obj;
-  }
-
-  private void compatibleRead(
-      MemoryBuffer buffer, FinalTypeField fieldInfo, boolean isFinal, Object obj) {
-    Object fieldValue;
-    short classId = fieldInfo.classId;
-    if (classId >= ClassResolver.PRIMITIVE_BOOLEAN_CLASS_ID
-        && classId <= ClassResolver.PRIMITIVE_DOUBLE_CLASS_ID) {
-      fieldValue = Serializers.readPrimitiveValue(fory, buffer, classId);
-    } else {
-      fieldValue =
-          AbstractObjectSerializer.readFinalObjectFieldValue(
-              binding, refResolver, classResolver, fieldInfo, isFinal, buffer);
-    }
-    fieldInfo.fieldConverter.set(obj, fieldValue);
+    serializer.write(buffer, value);
   }
 
   private T newInstance() {
@@ -259,97 +177,125 @@ public class MetaSharedSerializer<T> extends AbstractObjectSerializer<T> {
     return read(buffer);
   }
 
+  @Override
+  public T read(MemoryBuffer buffer) {
+    if (isRecord) {
+      Object[] fieldValues =
+          new Object[buildInFields.length + otherFields.length + containerFields.length];
+      readFields(buffer, fieldValues);
+      fieldValues = RecordUtils.remapping(recordInfo, fieldValues);
+      T t = objectCreator.newInstanceWithArguments(fieldValues);
+      Arrays.fill(recordInfo.getRecordComponents(), null);
+      return t;
+    }
+    T targetObject = newInstance();
+    Fory fory = this.fory;
+    RefResolver refResolver = this.refResolver;
+    SerializationBinding binding = this.binding;
+    if (refResolver instanceof MapRefResolver) {
+      MapRefResolver mapRefResolver = (MapRefResolver) refResolver;
+      if (mapRefResolver.hasPreservedRefId()) {
+        refResolver.reference(targetObject);
+      }
+    } else {
+      refResolver.reference(targetObject);
+    }
+    // read order: primitive,boxed,final,other,collection,map
+    for (SerializationFieldInfo fieldInfo : this.buildInFields) {
+      if (Utils.DEBUG_OUTPUT_VERBOSE) {
+        printFieldDebugInfo(fieldInfo, buffer);
+      }
+      FieldAccessor fieldAccessor = fieldInfo.fieldAccessor;
+      if (fieldAccessor != null) {
+        AbstractObjectSerializer.readBuildInFieldValue(binding, fieldInfo, buffer, targetObject);
+      } else {
+        if (fieldInfo.fieldConverter == null) {
+          // Skip the field value from buffer since it doesn't exist in current class
+          FieldSkipper.skipField(binding, fieldInfo, buffer);
+        } else {
+          compatibleRead(buffer, fieldInfo, targetObject);
+        }
+      }
+    }
+    Generics generics = fory.getGenerics();
+    for (SerializationFieldInfo fieldInfo : containerFields) {
+      if (Utils.DEBUG_OUTPUT_VERBOSE) {
+        printFieldDebugInfo(fieldInfo, buffer);
+      }
+      Object fieldValue =
+          AbstractObjectSerializer.readContainerFieldValue(binding, generics, fieldInfo, buffer);
+      FieldAccessor fieldAccessor = fieldInfo.fieldAccessor;
+      if (fieldAccessor != null) {
+        fieldAccessor.putObject(targetObject, fieldValue);
+      }
+    }
+    for (SerializationFieldInfo fieldInfo : otherFields) {
+      if (Utils.DEBUG_OUTPUT_VERBOSE) {
+        printFieldDebugInfo(fieldInfo, buffer);
+      }
+      Object fieldValue = binding.readField(fieldInfo, buffer);
+      FieldAccessor fieldAccessor = fieldInfo.fieldAccessor;
+      if (fieldAccessor != null) {
+        fieldAccessor.putObject(targetObject, fieldValue);
+      }
+    }
+    return targetObject;
+  }
+
+  private void compatibleRead(MemoryBuffer buffer, SerializationFieldInfo fieldInfo, Object obj) {
+    Object fieldValue = AbstractObjectSerializer.readBuildInFieldValue(binding, fieldInfo, buffer);
+    fieldInfo.fieldConverter.set(obj, fieldValue);
+  }
+
   private void readFields(MemoryBuffer buffer, Object[] fields) {
     int counter = 0;
     Fory fory = this.fory;
-    RefResolver refResolver = this.refResolver;
-    ClassResolver classResolver = this.classResolver;
     SerializationBinding binding = this.binding;
     // read order: primitive,boxed,final,other,collection,map
-    ObjectSerializer.FinalTypeField[] finalFields = this.finalFields;
-    for (int i = 0; i < finalFields.length; i++) {
-      ObjectSerializer.FinalTypeField fieldInfo = finalFields[i];
-      boolean isFinal = this.isFinal[i];
+    for (SerializationFieldInfo fieldInfo : this.buildInFields) {
+      if (Utils.DEBUG_OUTPUT_ENABLED) {
+        printFieldDebugInfo(fieldInfo, buffer);
+      }
       if (fieldInfo.fieldAccessor != null) {
-        assert fieldInfo.classInfo != null;
-        short classId = fieldInfo.classId;
-        // primitive field won't write null flag.
-        if (classId >= ClassResolver.PRIMITIVE_BOOLEAN_CLASS_ID
-            && classId <= ClassResolver.PRIMITIVE_DOUBLE_CLASS_ID) {
-          fields[counter++] = Serializers.readPrimitiveValue(fory, buffer, classId);
-        } else {
-          Object fieldValue =
-              AbstractObjectSerializer.readFinalObjectFieldValue(
-                  binding, refResolver, classResolver, fieldInfo, isFinal, buffer);
-          fields[counter++] = fieldValue;
-        }
+        fields[counter++] =
+            AbstractObjectSerializer.readBuildInFieldValue(binding, fieldInfo, buffer);
       } else {
-        // Skip the field value from buffer since it doesn't exist in current class
-        if (skipPrimitiveFieldValueFailed(fory, fieldInfo.classId, buffer)) {
-          if (fieldInfo.classInfo == null) {
-            // TODO(chaokunyang) support registered serializer in peer with ref tracking disabled.
-            fory.readRef(buffer, classInfoHolder);
-          } else {
-            AbstractObjectSerializer.readFinalObjectFieldValue(
-                binding, refResolver, classResolver, fieldInfo, isFinal, buffer);
-          }
-        }
+        // Skip the field value from buffer since it doesn't exist in current class.
+        // For records, fieldConverter can't be used since records are immutable and
+        // constructed all at once. We just read to advance buffer position.
+        FieldSkipper.skipField(binding, fieldInfo, buffer);
         // remapping will handle those extra fields from peers.
         fields[counter++] = null;
       }
     }
-    for (ObjectSerializer.GenericTypeField fieldInfo : otherFields) {
-      Object fieldValue = AbstractObjectSerializer.readOtherFieldValue(binding, fieldInfo, buffer);
-      fields[counter++] = fieldValue;
-    }
     Generics generics = fory.getGenerics();
-    for (ObjectSerializer.GenericTypeField fieldInfo : containerFields) {
+    for (SerializationFieldInfo fieldInfo : containerFields) {
+      if (Utils.DEBUG_OUTPUT_ENABLED) {
+        printFieldDebugInfo(fieldInfo, buffer);
+      }
       Object fieldValue =
           AbstractObjectSerializer.readContainerFieldValue(binding, generics, fieldInfo, buffer);
       fields[counter++] = fieldValue;
     }
-  }
-
-  /** Skip primitive primitive field value since it doesn't write null flag. */
-  static boolean skipPrimitiveFieldValueFailed(Fory fory, short classId, MemoryBuffer buffer) {
-    switch (classId) {
-      case ClassResolver.PRIMITIVE_BOOLEAN_CLASS_ID:
-        buffer.increaseReaderIndex(1);
-        return false;
-      case ClassResolver.PRIMITIVE_BYTE_CLASS_ID:
-        buffer.increaseReaderIndex(1);
-        return false;
-      case ClassResolver.PRIMITIVE_CHAR_CLASS_ID:
-        buffer.increaseReaderIndex(2);
-        return false;
-      case ClassResolver.PRIMITIVE_SHORT_CLASS_ID:
-        buffer.increaseReaderIndex(2);
-        return false;
-      case ClassResolver.PRIMITIVE_INT_CLASS_ID:
-        if (fory.compressInt()) {
-          buffer.readVarInt32();
-        } else {
-          buffer.increaseReaderIndex(4);
-        }
-        return false;
-      case ClassResolver.PRIMITIVE_FLOAT_CLASS_ID:
-        buffer.increaseReaderIndex(4);
-        return false;
-      case ClassResolver.PRIMITIVE_LONG_CLASS_ID:
-        fory.readInt64(buffer);
-        return false;
-      case ClassResolver.PRIMITIVE_DOUBLE_CLASS_ID:
-        buffer.increaseReaderIndex(8);
-        return false;
-      default:
-        {
-          return true;
-        }
+    for (SerializationFieldInfo fieldInfo : otherFields) {
+      if (Utils.DEBUG_OUTPUT_ENABLED) {
+        printFieldDebugInfo(fieldInfo, buffer);
+      }
+      Object fieldValue = binding.readField(fieldInfo, buffer);
+      fields[counter++] = fieldValue;
     }
   }
 
+  private void printFieldDebugInfo(SerializationFieldInfo fieldInfo, MemoryBuffer buffer) {
+    LOG.info(
+        "[Java] read field {} of type {}, reader index {}",
+        fieldInfo.descriptor.getName(),
+        fieldInfo.typeRef,
+        buffer.readerIndex());
+  }
+
   public static Collection<Descriptor> consolidateFields(
-      TypeResolver resolver, Class<?> cls, ClassDef classDef) {
-    return classDef.getDescriptors(resolver, cls);
+      TypeResolver resolver, Class<?> cls, TypeDef typeDef) {
+    return typeDef.getDescriptors(resolver, cls);
   }
 }
