@@ -32,7 +32,6 @@ import java.io.ObjectStreamField;
 import java.io.Serializable;
 import java.io.UnsupportedEncodingException;
 import java.lang.invoke.MethodHandles;
-import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.util.ArrayList;
@@ -41,23 +40,30 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 import java.util.TreeMap;
-import java.util.concurrent.atomic.AtomicInteger;
 import java.util.function.BiConsumer;
 import java.util.function.Consumer;
 import org.apache.fory.Fory;
 import org.apache.fory.builder.CodecUtils;
 import org.apache.fory.builder.LayerMarkerClassGenerator;
+import org.apache.fory.collection.LongMap;
 import org.apache.fory.collection.ObjectArray;
 import org.apache.fory.collection.ObjectIntMap;
 import org.apache.fory.logging.Logger;
 import org.apache.fory.logging.LoggerFactory;
 import org.apache.fory.memory.MemoryBuffer;
 import org.apache.fory.memory.Platform;
-import org.apache.fory.meta.ClassDef;
+import org.apache.fory.meta.FieldInfo;
+import org.apache.fory.meta.FieldTypes;
+import org.apache.fory.meta.NativeTypeDefEncoder;
+import org.apache.fory.meta.TypeDef;
 import org.apache.fory.reflect.ObjectCreator;
 import org.apache.fory.reflect.ObjectCreators;
 import org.apache.fory.reflect.ReflectionUtils;
+import org.apache.fory.resolver.MetaContext;
+import org.apache.fory.resolver.TypeInfo;
 import org.apache.fory.type.Descriptor;
+import org.apache.fory.type.TypeUtils;
+import org.apache.fory.type.Types;
 import org.apache.fory.util.ExceptionUtils;
 import org.apache.fory.util.GraalvmSupport;
 import org.apache.fory.util.Preconditions;
@@ -80,6 +86,8 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
   private static final Logger LOG = LoggerFactory.getLogger(ObjectStreamSerializer.class);
 
   private final SlotInfo[] slotsInfos;
+  // Instance-level cache: TypeDef ID -> TypeInfo (shared across all slots)
+  private final LongMap<TypeInfo> typeDefIdToTypeInfo = new LongMap<>(4, 0.4f);
 
   /**
    * Interface for slot information used in ObjectStreamSerializer. This allows both full SlotsInfo
@@ -88,10 +96,31 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
   private interface SlotInfo {
     Class<?> getCls();
 
-    StreamClassInfo getStreamClassInfo();
+    StreamTypeInfo getStreamTypeInfo();
 
     MetaSharedLayerSerializerBase getSlotsSerializer();
 
+    /**
+     * Read the layer TypeDef from buffer (if meta share enabled) and return the appropriate
+     * serializer. When meta share is enabled, reads the TypeDef from buffer, caches the serializer
+     * by TypeDef ID, and returns it. When meta share is disabled, returns the default slots
+     * serializer. Also stores the returned serializer for later retrieval via {@link
+     * #getCurrentReadSerializer()}.
+     *
+     * @param fory the Fory instance
+     * @param buffer the memory buffer to read TypeDef from
+     * @return the serializer to use for reading
+     */
+    MetaSharedLayerSerializerBase getReadSerializer(Fory fory, MemoryBuffer buffer);
+
+    /**
+     * Get the current read serializer (last returned by {@link #getReadSerializer}). This is used
+     * by defaultReadObject() to get the serializer without reading from buffer again.
+     *
+     * @return the current read serializer
+     */
+    MetaSharedLayerSerializerBase getCurrentReadSerializer();
+
     ForyObjectOutputStream getObjectOutputStream();
 
     ForyObjectInputStream getObjectInputStream();
@@ -138,12 +167,14 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
       throw new IllegalArgumentException(
           String.format("Class %s should implement %s.", type, Serializable.class));
     }
-    LOG.warn(
-        "{} customized jdk serialization, which is inefficient. "
-            + "Please replace it with a {} or implements {}",
-        type,
-        Serializer.class.getName(),
-        Externalizable.class.getName());
+    if (!Throwable.class.isAssignableFrom(type)) {
+      LOG.warn(
+          "{} customized jdk serialization, which is inefficient. "
+              + "Please replace it with a {} or implements {}",
+          type,
+          Serializer.class.getName(),
+          Externalizable.class.getName());
+    }
     // stream serializer may be data serializer of ReplaceResolver serializer.
     fory.getClassResolver().setSerializerIfAbsent(type, this);
     List<SlotInfo> slotsInfoList = new ArrayList<>();
@@ -182,10 +213,16 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
         // create a classinfo to avoid null class bytes when class id is a
         // replacement id.
         classResolver.writeClassInternal(buffer, slotsInfo.getCls());
-        StreamClassInfo streamClassInfo = slotsInfo.getStreamClassInfo();
-        Method writeObjectMethod = streamClassInfo.writeObjectMethod;
+        // Write layer class meta first (if meta share enabled)
+        MetaSharedLayerSerializerBase serializer = slotsInfo.getSlotsSerializer();
+        if (fory.getConfig().isMetaShareEnabled()) {
+          serializer.writeLayerClassMeta(buffer);
+        }
+        StreamTypeInfo streamTypeInfo = slotsInfo.getStreamTypeInfo();
+        Method writeObjectMethod = streamTypeInfo.writeObjectMethod;
         if (writeObjectMethod == null) {
-          slotsInfo.getSlotsSerializer().write(buffer, value);
+          // No custom writeObject - write fields directly
+          serializer.writeFieldsOnly(buffer, value);
         } else {
           ForyObjectOutputStream objectOutputStream = slotsInfo.getObjectOutputStream();
           Object oldObject = objectOutputStream.targetObject;
@@ -197,8 +234,8 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
             objectOutputStream.buffer = buffer;
             objectOutputStream.curPut = null;
             objectOutputStream.fieldsWritten = false;
-            if (streamClassInfo.writeObjectFunc != null) {
-              streamClassInfo.writeObjectFunc.accept(value, objectOutputStream);
+            if (streamTypeInfo.writeObjectFunc != null) {
+              streamTypeInfo.writeObjectFunc.accept(value, objectOutputStream);
             } else {
               writeObjectMethod.invoke(value, objectOutputStream);
             }
@@ -221,36 +258,67 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
     fory.getRefResolver().reference(obj);
     int numClasses = buffer.readInt16();
     int slotIndex = 0;
+
     try {
       TreeMap<Integer, ObjectInputValidation> callbacks = new TreeMap<>(Collections.reverseOrder());
       for (int i = 0; i < numClasses; i++) {
         Class<?> currentClass = classResolver.readClassInternal(buffer);
-        SlotInfo slotsInfo = slotsInfos[slotIndex++];
-        StreamClassInfo streamClassInfo = slotsInfo.getStreamClassInfo();
-        while (currentClass != slotsInfo.getCls()) {
-          // the receiver's version extends classes that are not extended by the sender's version.
-          Method readObjectNoData = streamClassInfo.readObjectNoData;
-          if (readObjectNoData != null) {
-            if (streamClassInfo.readObjectNoDataFunc != null) {
-              streamClassInfo.readObjectNoDataFunc.accept(obj);
-            } else {
-              readObjectNoData.invoke(obj);
+
+        // Find the matching local slot for sender's class
+        SlotInfo matchedSlot = null;
+        while (slotIndex < slotsInfos.length) {
+          SlotInfo candidateSlot = slotsInfos[slotIndex];
+          if (currentClass == candidateSlot.getCls()) {
+            // Found matching slot
+            matchedSlot = candidateSlot;
+            slotIndex++;
+            break;
+          } else if (currentClass.isAssignableFrom(candidateSlot.getCls())) {
+            // Sender's class is an ancestor of candidate's class but they don't match.
+            // This means sender has a layer (currentClass) that receiver doesn't have.
+            // We'll skip sender's data for this layer below.
+            break;
+          } else {
+            // Receiver has an extra layer that sender doesn't have - call readObjectNoData
+            StreamTypeInfo streamTypeInfo = candidateSlot.getStreamTypeInfo();
+            Method readObjectNoData = streamTypeInfo.readObjectNoData;
+            if (readObjectNoData != null) {
+              if (streamTypeInfo.readObjectNoDataFunc != null) {
+                streamTypeInfo.readObjectNoDataFunc.accept(obj);
+              } else {
+                readObjectNoData.invoke(obj);
+              }
             }
+            slotIndex++;
           }
-          slotsInfo = slotsInfos[slotIndex++];
         }
-        Method readObjectMethod = streamClassInfo.readObjectMethod;
+
+        if (matchedSlot == null) {
+          // Sender has a layer that receiver doesn't have - read TypeDef and skip the data
+          skipUnknownLayerData(buffer, currentClass);
+          continue;
+        }
+
+        // Read data for the matched layer - getReadSerializer reads TypeDef from buffer
+        // This must be called exactly once per layer to read the TypeDef
+        matchedSlot.getReadSerializer(fory, buffer);
+
+        StreamTypeInfo streamTypeInfo = matchedSlot.getStreamTypeInfo();
+        Method readObjectMethod = streamTypeInfo.readObjectMethod;
+
         if (readObjectMethod == null) {
-          slotsInfo.getSlotsSerializer().readAndSetFields(buffer, obj);
+          // For standard field serialization - use getCurrentReadSerializer()
+          matchedSlot.getCurrentReadSerializer().readAndSetFields(buffer, obj);
         } else {
-          ForyObjectInputStream objectInputStream = slotsInfo.getObjectInputStream();
+          // For custom readObject, it handles its own format
+          ForyObjectInputStream objectInputStream = matchedSlot.getObjectInputStream();
           MemoryBuffer oldBuffer = objectInputStream.buffer;
           Object oldObject = objectInputStream.targetObject;
           ForyObjectInputStream.GetFieldImpl oldGetField = objectInputStream.getField;
           ForyObjectInputStream.GetFieldImpl getField =
-              (ForyObjectInputStream.GetFieldImpl) slotsInfo.getFieldPool().popOrNull();
+              (ForyObjectInputStream.GetFieldImpl) matchedSlot.getFieldPool().popOrNull();
           if (getField == null) {
-            getField = new ForyObjectInputStream.GetFieldImpl(slotsInfo);
+            getField = new ForyObjectInputStream.GetFieldImpl(matchedSlot);
           }
           boolean fieldsRead = objectInputStream.fieldsRead;
           try {
@@ -259,8 +327,8 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
             objectInputStream.targetObject = obj;
             objectInputStream.getField = getField;
             objectInputStream.callbacks = callbacks;
-            if (streamClassInfo.readObjectFunc != null) {
-              streamClassInfo.readObjectFunc.accept(obj, objectInputStream);
+            if (streamTypeInfo.readObjectFunc != null) {
+              streamTypeInfo.readObjectFunc.accept(obj, objectInputStream);
             } else {
               readObjectMethod.invoke(obj, objectInputStream);
             }
@@ -269,12 +337,27 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
             objectInputStream.buffer = oldBuffer;
             objectInputStream.targetObject = oldObject;
             objectInputStream.getField = oldGetField;
-            slotsInfo.getFieldPool().add(getField);
+            matchedSlot.getFieldPool().add(getField);
             objectInputStream.callbacks = null;
             Arrays.fill(getField.vals, ForyObjectInputStream.NO_VALUE_STUB);
           }
         }
       }
+
+      // Handle any remaining receiver-only layers at the end
+      while (slotIndex < slotsInfos.length) {
+        SlotInfo remainingSlot = slotsInfos[slotIndex++];
+        StreamTypeInfo streamTypeInfo = remainingSlot.getStreamTypeInfo();
+        Method readObjectNoData = streamTypeInfo.readObjectNoData;
+        if (readObjectNoData != null) {
+          if (streamTypeInfo.readObjectNoDataFunc != null) {
+            streamTypeInfo.readObjectNoDataFunc.accept(obj);
+          } else {
+            readObjectNoData.invoke(obj);
+          }
+        }
+      }
+
       for (ObjectInputValidation validation : callbacks.values()) {
         validation.validateObject();
       }
@@ -284,6 +367,68 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
     return obj;
   }
 
+  /**
+   * Skip data for a layer that exists in sender but not in receiver. This is needed for schema
+   * evolution when sender's class hierarchy has layers that receiver doesn't have.
+   *
+   * @param buffer the memory buffer to read from
+   * @param senderClass the class from sender that receiver doesn't have
+   */
+  private void skipUnknownLayerData(MemoryBuffer buffer, Class<?> senderClass) {
+    // For layers without custom writeObject, we can skip using a serializer created from the
+    // TypeDef. Note: For layers with custom writeObject, the sender would have that class
+    // locally, and we'd have a matching slot. This method is only called when sender has a
+    // layer the receiver doesn't have.
+
+    if (!fory.getConfig().isMetaShareEnabled()) {
+      throw new UnsupportedOperationException(
+          "Cannot skip unknown layer data without meta share enabled for class: "
+              + senderClass.getName()
+              + ". Schema evolution with removed parent classes requires meta share.");
+    }
+
+    // Read TypeInfo from buffer (maintains index alignment with readTypeInfos)
+    MetaContext metaContext = fory.getSerializationContext().getMetaContext();
+    if (metaContext == null) {
+      throw new IllegalStateException("MetaContext is null but meta share is enabled");
+    }
+    int indexMarker = buffer.readVarUint32Small14();
+    boolean isRef = (indexMarker & 1) == 1;
+    int index = indexMarker >>> 1;
+    TypeInfo typeInfo;
+    if (isRef) {
+      // Reference to previously read TypeInfo
+      typeInfo = metaContext.readTypeInfos.get(index);
+    } else {
+      // New TypeDef in stream - read ID first to check cache
+      long typeDefId = buffer.readInt64();
+      typeInfo = typeDefIdToTypeInfo.get(typeDefId);
+      if (typeInfo != null) {
+        // Already cached - skip the TypeDef bytes, reuse existing TypeInfo
+        TypeDef.skipTypeDef(buffer, typeDefId);
+      } else {
+        // Not cached - read full TypeDef and create TypeInfo
+        TypeDef typeDef = TypeDef.readTypeDef(fory, buffer, typeDefId);
+        typeInfo = new TypeInfo(senderClass, typeDef);
+        typeDefIdToTypeInfo.put(typeDefId, typeInfo);
+      }
+      metaContext.readTypeInfos.add(typeInfo);
+    }
+
+    // Get or create serializer from TypeInfo to skip the fields
+    MetaSharedLayerSerializerBase skipSerializer =
+        (MetaSharedLayerSerializerBase) typeInfo.getSerializer();
+    if (skipSerializer == null) {
+      Class<?> layerMarkerClass = LayerMarkerClassGenerator.getOrCreate(fory, senderClass, 0);
+      MetaSharedLayerSerializer<?> newSerializer =
+          new MetaSharedLayerSerializer(fory, senderClass, typeInfo.getTypeDef(), layerMarkerClass);
+      typeInfo.setSerializer(newSerializer);
+      skipSerializer = newSerializer;
+    }
+    SerializationBinding binding = SerializationBinding.createBinding(fory);
+    ((MetaSharedLayerSerializer<?>) skipSerializer).skipFields(buffer, binding);
+  }
+
   private static void throwUnsupportedEncodingException(Class<?> cls)
       throws UnsupportedEncodingException {
     throw new UnsupportedEncodingException(
@@ -309,13 +454,12 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
    * images, it needs to reuse the serializers that were generated at build time.
    *
    * @param fory the Fory instance
-   * @param layerClassDef the ClassDef for this layer
+   * @param layerTypeDef the TypeDef for this layer
    * @param type the target class type
    */
   private static void ensureFieldSerializersGenerated(
-      Fory fory, ClassDef layerClassDef, Class<?> type) {
-    Collection<Descriptor> descriptors =
-        layerClassDef.getDescriptors(fory.getClassResolver(), type);
+      Fory fory, TypeDef layerTypeDef, Class<?> type) {
+    Collection<Descriptor> descriptors = layerTypeDef.getDescriptors(fory.getClassResolver(), type);
     for (Descriptor descriptor : descriptors) {
       Class<?> fieldType = descriptor.getRawType();
       if (fieldType != null && !fieldType.isPrimitive()) {
@@ -330,11 +474,74 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
     }
   }
 
+  /**
+   * Build a list of FieldInfo from ObjectStreamClass fields (serialPersistentFields). This creates
+   * FieldInfo objects that represent the serialization contract defined by the class, which may
+   * differ from the actual class fields when serialPersistentFields is defined.
+   *
+   * @param fory the Fory instance
+   * @param objectStreamClass the ObjectStreamClass for the type
+   * @param type the class type
+   * @return list of FieldInfo representing the serialization fields
+   */
+  private static List<FieldInfo> buildFieldInfoFromObjectStreamClass(
+      Fory fory, ObjectStreamClass objectStreamClass, Class<?> type) {
+    ObjectStreamField[] streamFields = objectStreamClass.getFields();
+    List<FieldInfo> fieldInfos = new ArrayList<>(streamFields.length);
+    String className = type.getName();
+
+    for (ObjectStreamField streamField : streamFields) {
+      Class<?> fieldType = streamField.getType();
+      String fieldName = streamField.getName();
+
+      // Build FieldType based on the field's declared type
+      FieldTypes.FieldType fieldTypeInfo = buildFieldTypeFromClass(fory, fieldType);
+
+      fieldInfos.add(new FieldInfo(className, fieldName, fieldTypeInfo));
+    }
+    return fieldInfos;
+  }
+
+  /**
+   * Build a FieldType from a Class. This is a simplified version of FieldTypes.buildFieldType that
+   * works with Class instead of Field, used for ObjectStreamField which only provides type info.
+   */
+  private static FieldTypes.FieldType buildFieldTypeFromClass(Fory fory, Class<?> fieldType) {
+    // For primitives, use RegisteredFieldType
+    if (fieldType.isPrimitive()) {
+      int typeId = Types.getTypeId(fory, fieldType);
+      return new FieldTypes.RegisteredFieldType(false, false, typeId, -1);
+    }
+
+    // For boxed primitives
+    Class<?> unwrapped = TypeUtils.unwrap(fieldType);
+    if (unwrapped.isPrimitive()) {
+      int typeId = Types.getTypeId(fory, unwrapped);
+      return new FieldTypes.RegisteredFieldType(true, true, typeId, -1);
+    }
+
+    // For registered types
+    if (fory.getClassResolver().isRegisteredById(fieldType)) {
+      int typeId = fory.getClassResolver().getTypeIdForTypeDef(fieldType);
+      int userTypeId = fory.getClassResolver().getUserTypeIdForTypeDef(fieldType);
+      return new FieldTypes.RegisteredFieldType(true, true, typeId, userTypeId);
+    }
+
+    // For enums
+    if (fieldType.isEnum()) {
+      return new FieldTypes.EnumFieldType(true, -1, -1);
+    }
+
+    // For arrays, collections, maps - use ObjectFieldType as a fallback
+    // The actual type handling will be done during serialization
+    return new FieldTypes.ObjectFieldType(-1, true, true);
+  }
+
   /**
    * Information about a class's stream methods (writeObject, readObject, readObjectNoData) and
    * their optimized MethodHandle equivalents for fast invocation.
    */
-  private static class StreamClassInfo {
+  private static class StreamTypeInfo {
     private final Method writeObjectMethod;
     private final Method readObjectMethod;
     private final Method readObjectNoData;
@@ -342,7 +549,7 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
     private final BiConsumer readObjectFunc;
     private final Consumer readObjectNoDataFunc;
 
-    private StreamClassInfo(Class<?> type) {
+    private StreamTypeInfo(Class<?> type) {
       // ObjectStreamClass.lookup has cache inside, invocation cost won't be big.
       ObjectStreamClass objectStreamClass = safeObjectStreamClassLookup(type);
       // In JDK17, set private jdk method accessible will fail by default, use ObjectStreamClass
@@ -385,11 +592,11 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
     }
   }
 
-  private static final ClassValue<StreamClassInfo> STREAM_CLASS_INFO_CACHE =
-      new ClassValue<StreamClassInfo>() {
+  private static final ClassValue<StreamTypeInfo> STREAM_CLASS_INFO_CACHE =
+      new ClassValue<StreamTypeInfo>() {
         @Override
-        protected StreamClassInfo computeValue(Class<?> type) {
-          return new StreamClassInfo(type);
+        protected StreamTypeInfo computeValue(Class<?> type) {
+          return new StreamTypeInfo(type);
         }
       };
 
@@ -398,9 +605,9 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
    * all the details of serializing and deserializing a single class in the class hierarchy using
    * Java's ObjectInputStream/ObjectOutputStream protocol.
    */
-  private static class SlotsInfo implements SlotInfo {
+  private class SlotsInfo implements SlotInfo {
     private final Class<?> cls;
-    private final StreamClassInfo streamClassInfo;
+    private final StreamTypeInfo streamTypeInfo;
     // mark non-final for async-jit to update it to jit-serializer.
     private MetaSharedLayerSerializerBase slotsSerializer;
     private final ObjectIntMap<String> fieldIndexMap;
@@ -409,22 +616,35 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
     private final ForyObjectOutputStream objectOutputStream;
     private final ForyObjectInputStream objectInputStream;
     private final ObjectArray getFieldPool;
+    // Current read serializer (set by getReadSerializer, used by getCurrentReadSerializer)
+    private MetaSharedLayerSerializerBase currentReadSerializer;
 
     public SlotsInfo(Fory fory, Class<?> type) {
       this.cls = type;
       ObjectStreamClass objectStreamClass = safeObjectStreamClassLookup(type);
-      streamClassInfo = STREAM_CLASS_INFO_CACHE.get(type);
+      streamTypeInfo = STREAM_CLASS_INFO_CACHE.get(type);
 
-      // Build single-layer ClassDef (resolveParent=false)
-      ClassDef layerClassDef = fory.getClassResolver().getTypeDef(type, false);
+      // Build TypeDef from ObjectStreamClass fields (handles serialPersistentFields).
+      // This ensures the serializer uses the same field layout as defined by
+      // serialPersistentFields (if present) rather than actual class fields.
+      TypeDef layerTypeDef;
+      if (objectStreamClass != null) {
+        List<FieldInfo> fieldInfos =
+            buildFieldInfoFromObjectStreamClass(fory, objectStreamClass, type);
+        layerTypeDef =
+            NativeTypeDefEncoder.buildTypeDefWithFieldInfos(
+                fory.getClassResolver(), type, fieldInfos, true);
+      } else {
+        // Fallback when ObjectStreamClass is not available (e.g., GraalVM native image)
+        layerTypeDef = fory.getClassResolver().getTypeDef(type, false);
+      }
       // Generate marker class for this layer. Use 0 as layer index since each class
       // has its own SlotsInfo, and the (class, 0) pair is unique for each class.
       Class<?> layerMarkerClass = LayerMarkerClassGenerator.getOrCreate(fory, type, 0);
 
       // Create interpreter-mode serializer first
-      MetaSharedLayerSerializer interpreterSerializer =
-          new MetaSharedLayerSerializer(fory, type, layerClassDef, layerMarkerClass);
-      this.slotsSerializer = interpreterSerializer;
+      this.slotsSerializer =
+          new MetaSharedLayerSerializer(fory, type, layerTypeDef, layerMarkerClass);
 
       // Register JIT callback to replace with JIT serializer when ready
       if (fory.getConfig().isCodeGenEnabled()) {
@@ -434,54 +654,35 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
                 () -> MetaSharedLayerSerializer.class,
                 () ->
                     CodecUtils.loadOrGenMetaSharedLayerCodecClass(
-                        type, fory, layerClassDef, layerMarkerClass),
+                        type, fory, layerTypeDef, layerMarkerClass),
                 c ->
                     thisInfo.slotsSerializer =
-                        (MetaSharedLayerSerializerBase) Serializers.newSerializer(fory, type, c));
+                        (MetaSharedLayerSerializerBase)
+                            Serializers.newSerializer(fory, type, (Class<? extends Serializer>) c));
       }
 
       // In GraalVM, ensure serializers are generated for all field types at build time
       // so they're available when new Fory instances are created at runtime
       if (GraalvmSupport.isGraalBuildtime()) {
-        ensureFieldSerializersGenerated(fory, layerClassDef, type);
+        ensureFieldSerializersGenerated(fory, layerTypeDef, type);
       }
 
+      // Build fieldIndexMap and putFieldTypes from serializer's field order.
+      // This ensures putFields/writeFields API uses the same order as the serializer
+      // (buildIn, container, other groups), not ObjectStreamClass order.
       fieldIndexMap = new ObjectIntMap<>(4, 0.4f);
-      // Build field list from ObjectStreamClass or class fields
-      List<PutFieldInfo> putFieldInfos = new ArrayList<>();
-      if (objectStreamClass != null) {
-        for (ObjectStreamField serialField : objectStreamClass.getFields()) {
-          putFieldInfos.add(new PutFieldInfo(serialField.getName(), serialField.getType(), type));
-        }
-      } else {
-        // Fallback when ObjectStreamClass is not available
-        Collection<Descriptor> descriptors =
-            layerClassDef.getDescriptors(fory.getClassResolver(), type);
-        for (Descriptor descriptor : descriptors) {
-          Field field = descriptor.getField();
-          if (field != null) {
-            putFieldInfos.add(new PutFieldInfo(field.getName(), field.getType(), type));
-          }
-        }
-      }
-
-      if (streamClassInfo != null
-          && (streamClassInfo.writeObjectMethod != null
-              || streamClassInfo.readObjectMethod != null)) {
-        this.numPutFields = putFieldInfos.size();
+      if (streamTypeInfo != null
+          && (streamTypeInfo.writeObjectMethod != null
+              || streamTypeInfo.readObjectMethod != null)) {
+        this.numPutFields = slotsSerializer.getNumFields();
         this.putFieldTypes = new Class<?>[numPutFields];
-        AtomicInteger idx = new AtomicInteger(0);
-        for (PutFieldInfo fieldInfo : putFieldInfos) {
-          int index = idx.getAndIncrement();
-          fieldIndexMap.put(fieldInfo.name, index);
-          putFieldTypes[index] = fieldInfo.type;
-        }
+        slotsSerializer.populateFieldInfo(fieldIndexMap, putFieldTypes);
       } else {
         this.numPutFields = 0;
         this.putFieldTypes = null;
       }
 
-      if (streamClassInfo != null && streamClassInfo.writeObjectMethod != null) {
+      if (streamTypeInfo != null && streamTypeInfo.writeObjectMethod != null) {
         try {
           objectOutputStream = new ForyObjectOutputStream(this);
         } catch (IOException e) {
@@ -491,7 +692,7 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
       } else {
         objectOutputStream = null;
       }
-      if (streamClassInfo != null && streamClassInfo.readObjectMethod != null) {
+      if (streamTypeInfo != null && streamTypeInfo.readObjectMethod != null) {
         try {
           objectInputStream = new ForyObjectInputStream(this);
         } catch (IOException e) {
@@ -510,8 +711,8 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
     }
 
     @Override
-    public StreamClassInfo getStreamClassInfo() {
-      return streamClassInfo;
+    public StreamTypeInfo getStreamTypeInfo() {
+      return streamTypeInfo;
     }
 
     @Override
@@ -549,25 +750,78 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
       return putFieldTypes;
     }
 
+    @Override
+    @SuppressWarnings("unchecked")
+    public MetaSharedLayerSerializerBase getReadSerializer(Fory fory, MemoryBuffer buffer) {
+      MetaSharedLayerSerializerBase result;
+      if (!fory.getConfig().isMetaShareEnabled()) {
+        // Meta share not enabled - use the default slots serializer
+        result = slotsSerializer;
+      } else {
+        // Read TypeInfo from buffer (creates new or returns existing)
+        TypeInfo typeInfo = readLayerTypeInfo(fory, buffer);
+        if (typeInfo == null) {
+          result = slotsSerializer;
+        } else {
+          // Get or create serializer from TypeInfo
+          Serializer<?> serializer = typeInfo.getSerializer();
+          if (serializer != null) {
+            result = (MetaSharedLayerSerializerBase) serializer;
+          } else {
+            // Create a new serializer based on the TypeDef from stream
+            Class<?> layerMarkerClass = LayerMarkerClassGenerator.getOrCreate(fory, cls, 0);
+            MetaSharedLayerSerializer<?> newSerializer =
+                new MetaSharedLayerSerializer(fory, cls, typeInfo.getTypeDef(), layerMarkerClass);
+            typeInfo.setSerializer(newSerializer);
+            result = newSerializer;
+          }
+        }
+      }
+      // Store for getCurrentReadSerializer()
+      this.currentReadSerializer = result;
+      return result;
+    }
+
+    @Override
+    public MetaSharedLayerSerializerBase getCurrentReadSerializer() {
+      return currentReadSerializer;
+    }
+
+    private TypeInfo readLayerTypeInfo(Fory fory, MemoryBuffer buffer) {
+      MetaContext metaContext = fory.getSerializationContext().getMetaContext();
+      if (metaContext == null) {
+        return null;
+      }
+      int indexMarker = buffer.readVarUint32Small14();
+      boolean isRef = (indexMarker & 1) == 1;
+      int index = indexMarker >>> 1;
+      if (isRef) {
+        // Reference to previously read TypeInfo
+        return metaContext.readTypeInfos.get(index);
+      } else {
+        // New TypeDef in stream - read ID first to check cache
+        long typeDefId = buffer.readInt64();
+        TypeInfo typeInfo = typeDefIdToTypeInfo.get(typeDefId);
+        if (typeInfo != null) {
+          // Already cached - skip the TypeDef bytes, reuse existing TypeInfo
+          TypeDef.skipTypeDef(buffer, typeDefId);
+        } else {
+          // Not cached - read full TypeDef and create TypeInfo
+          TypeDef typeDef = TypeDef.readTypeDef(fory, buffer, typeDefId);
+          typeInfo = new TypeInfo(cls, typeDef);
+          typeDefIdToTypeInfo.put(typeDefId, typeInfo);
+        }
+        metaContext.readTypeInfos.add(typeInfo);
+        return typeInfo;
+      }
+    }
+
     @Override
     public String toString() {
       return "SlotsInfo{" + "cls=" + cls + '}';
     }
   }
 
-  /** Simple field info for putFields/writeFields support. */
-  private static class PutFieldInfo {
-    final String name;
-    final Class<?> type;
-    final Class<?> declaringClass;
-
-    PutFieldInfo(String name, Class<?> type, Class<?> declaringClass) {
-      this.name = name;
-      this.type = type;
-      this.declaringClass = declaringClass;
-    }
-  }
-
   /**
    * Implement serialization for object output with `writeObject/readObject` defined by java
    * serialization output spec.
@@ -703,14 +957,8 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
       if (curPut == null) {
         throw new NotActiveException("no current PutField object");
       }
-      // Write field values directly by their types
-      Class<?>[] fieldTypes = slotsInfo.getPutFieldTypes();
-      Object[] vals = curPut.vals;
-      for (int i = 0; i < vals.length; i++) {
-        Class<?> fieldType = fieldTypes[i];
-        Object value = vals[i];
-        writePutFieldValue(buffer, fieldType, value);
-      }
+      // Write field values using MetaShare serialization
+      slotsInfo.getSlotsSerializer().writeFieldValues(buffer, curPut.vals);
       Arrays.fill(curPut.vals, null);
       putFieldsCache.add(curPut);
       this.curPut = null;
@@ -719,7 +967,7 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
 
     private void writePutFieldValue(MemoryBuffer buffer, Class<?> fieldType, Object value) {
       if (fieldType == boolean.class) {
-        buffer.writeBoolean(value == null ? false : (Boolean) value);
+        buffer.writeBoolean(value != null && (Boolean) value);
       } else if (fieldType == byte.class) {
         buffer.writeByte(value == null ? (byte) 0 : (Byte) value);
       } else if (fieldType == char.class) {
@@ -745,28 +993,12 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
     }
 
     @Override
-    public void defaultWriteObject() throws IOException, NotActiveException {
+    public void defaultWriteObject() throws IOException {
       if (fieldsWritten) {
         throw new NotActiveException("not in writeObject invocation or fields already written");
       }
-      // Write field values in the same format as writeFields (for compatibility with readFields)
-      Class<?>[] fieldTypes = slotsInfo.getPutFieldTypes();
-      if (fieldTypes != null && fieldTypes.length > 0) {
-        // Write using putField format (for compatibility with readFields)
-        ObjectIntMap<String> fieldIndexMap = slotsInfo.getFieldIndexMap();
-        MetaSharedLayerSerializerBase slotsSerializer = slotsInfo.getSlotsSerializer();
-        Object[] vals =
-            slotsSerializer.getFieldValuesForPutFields(
-                targetObject, fieldIndexMap, fieldTypes.length);
-        for (int i = 0; i < vals.length; i++) {
-          Class<?> fieldType = fieldTypes[i];
-          Object value = vals[i];
-          writePutFieldValue(buffer, fieldType, value);
-        }
-      } else {
-        // No custom writeObject/readObject, use normal serialization
-        slotsInfo.getSlotsSerializer().write(buffer, targetObject);
-      }
+      // Write fields using MetaShare serialization (layer meta already written by write())
+      slotsInfo.getSlotsSerializer().writeFieldsOnly(buffer, targetObject);
       fieldsWritten = true;
     }
 
@@ -1068,12 +1300,9 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
       if (fieldsRead) {
         throw new NotActiveException("not in readObject invocation or fields already read");
       }
-      // Read field values directly by their types
-      Class<?>[] fieldTypes = slotsInfo.getPutFieldTypes();
-      Object[] vals = getField.vals;
-      for (int i = 0; i < vals.length; i++) {
-        vals[i] = readPutFieldValue(buffer, fieldTypes[i]);
-      }
+      // Read field values using MetaShare serialization
+      Object[] vals = slotsInfo.getCurrentReadSerializer().readFieldValues(buffer);
+      System.arraycopy(vals, 0, getField.vals, 0, vals.length);
       fieldsRead = true;
       return getField;
     }
@@ -1106,26 +1335,13 @@ public class ObjectStreamSerializer extends AbstractObjectSerializer {
     }
 
     @Override
+    @SuppressWarnings("rawtypes")
     public void defaultReadObject() throws IOException, ClassNotFoundException {
       if (fieldsRead) {
         throw new NotActiveException("not in readObject invocation or fields already read");
       }
-      // Read field values in the same format as writeFields, but set to actual class fields
-      Class<?>[] fieldTypes = slotsInfo.getPutFieldTypes();
-      if (fieldTypes != null && fieldTypes.length > 0) {
-        // Read using putField format (for compatibility with writeFields)
-        ObjectIntMap<String> fieldIndexMap = slotsInfo.getFieldIndexMap();
-        Object[] vals = new Object[fieldTypes.length];
-        for (int i = 0; i < vals.length; i++) {
-          vals[i] = readPutFieldValue(buffer, fieldTypes[i]);
-        }
-        // Now set matching fields on the target object
-        MetaSharedLayerSerializerBase slotsSerializer = slotsInfo.getSlotsSerializer();
-        slotsSerializer.setFieldValuesFromPutFields(targetObject, fieldIndexMap, vals);
-      } else {
-        // No custom writeObject/readObject, use normal serialization
-        slotsInfo.getSlotsSerializer().readAndSetFields(buffer, targetObject);
-      }
+      // Read fields using MetaShare serialization (layer meta already read by getReadSerializer())
+      slotsInfo.getCurrentReadSerializer().readAndSetFields(buffer, targetObject);
       fieldsRead = true;
     }
 
