@@ -47,9 +47,9 @@ import org.apache.fory.builder.Generated;
 import org.apache.fory.collection.Tuple2;
 import org.apache.fory.memory.MemoryBuffer;
 import org.apache.fory.memory.Platform;
-import org.apache.fory.meta.ClassDef;
+import org.apache.fory.meta.TypeDef;
 import org.apache.fory.reflect.ReflectionUtils;
-import org.apache.fory.resolver.ClassResolver;
+import org.apache.fory.resolver.TypeResolver;
 import org.apache.fory.util.ExceptionUtils;
 import org.apache.fory.util.GraalvmSupport;
 import org.apache.fory.util.GraalvmSupport.GraalvmSerializerHolder;
@@ -87,8 +87,8 @@ public class Serializers {
         return new ObjectSerializer(fory, type);
       }
       if (serializerClass == MetaSharedSerializer.class) {
-        ClassDef classDef = fory.getClassResolver().getTypeDef(type, true);
-        return new MetaSharedSerializer(fory, type, classDef);
+        TypeDef typeDef = fory.getClassResolver().getTypeDef(type, true);
+        return new MetaSharedSerializer(fory, type, typeDef);
       }
       Tuple2<MethodType, MethodHandle> ctrInfo = CTR_MAP.getIfPresent(serializerClass);
       if (ctrInfo != null) {
@@ -176,32 +176,19 @@ public class Serializers {
     }
   }
 
-  public static Object readPrimitiveValue(Fory fory, MemoryBuffer buffer, short classId) {
-    switch (classId) {
-      case ClassResolver.PRIMITIVE_BOOLEAN_CLASS_ID:
-        return buffer.readBoolean();
-      case ClassResolver.PRIMITIVE_BYTE_CLASS_ID:
-        return buffer.readByte();
-      case ClassResolver.PRIMITIVE_CHAR_CLASS_ID:
-        return buffer.readChar();
-      case ClassResolver.PRIMITIVE_SHORT_CLASS_ID:
-        return buffer.readInt16();
-      case ClassResolver.PRIMITIVE_INT_CLASS_ID:
-        if (fory.compressInt()) {
-          return buffer.readVarInt32();
-        } else {
-          return buffer.readInt32();
-        }
-      case ClassResolver.PRIMITIVE_FLOAT_CLASS_ID:
-        return buffer.readFloat32();
-      case ClassResolver.PRIMITIVE_LONG_CLASS_ID:
-        return fory.readInt64(buffer);
-      case ClassResolver.PRIMITIVE_DOUBLE_CLASS_ID:
-        return buffer.readFloat64();
-      default:
-        {
-          throw new IllegalStateException("unreachable");
-        }
+  public static <T> void write(MemoryBuffer buffer, Serializer<T> serializer, T obj) {
+    if (serializer.isJava) {
+      serializer.write(buffer, obj);
+    } else {
+      serializer.xwrite(buffer, obj);
+    }
+  }
+
+  public static <T> T read(MemoryBuffer buffer, Serializer<T> serializer) {
+    if (serializer.isJava) {
+      return serializer.read(buffer);
+    } else {
+      return serializer.xread(buffer);
     }
   }
 
@@ -216,10 +203,6 @@ public class Serializers {
       super(fory, cls, needToWriteRef, immutable);
     }
 
-    public CrossLanguageCompatibleSerializer(Fory fory, Class<T> cls, boolean needToWriteRef) {
-      super(fory, cls, needToWriteRef);
-    }
-
     @Override
     public void xwrite(MemoryBuffer buffer, T value) {
       write(buffer, value);
@@ -587,10 +570,18 @@ public class Serializers {
     public Object read(MemoryBuffer buffer) {
       return new Object();
     }
+
+    @Override
+    public void xwrite(MemoryBuffer buffer, Object value) {}
+
+    @Override
+    public Object xread(MemoryBuffer buffer) {
+      return new Object();
+    }
   }
 
   public static void registerDefaultSerializers(Fory fory) {
-    ClassResolver resolver = fory.getClassResolver();
+    TypeResolver resolver = fory.getTypeResolver();
     resolver.registerInternalSerializer(Class.class, new ClassSerializer(fory));
     resolver.registerInternalSerializer(StringBuilder.class, new StringBuilderSerializer(fory));
     resolver.registerInternalSerializer(StringBuffer.class, new StringBufferSerializer(fory));
