@@ -39,11 +39,13 @@ import org.apache.fory.Fory;
 import org.apache.fory.builder.LayerMarkerClassGenerator;
 import org.apache.fory.config.CompatibleMode;
 import org.apache.fory.memory.MemoryBuffer;
-import org.apache.fory.meta.ClassDef;
+import org.apache.fory.meta.TypeDef;
 import org.apache.fory.reflect.ReflectionUtils;
 import org.apache.fory.resolver.ClassResolver;
+import org.apache.fory.resolver.MetaContext;
 import org.apache.fory.serializer.AbstractObjectSerializer;
-import org.apache.fory.serializer.AbstractObjectSerializer.InternalFieldInfo;
+import org.apache.fory.serializer.FieldGroups;
+import org.apache.fory.serializer.FieldGroups.SerializationFieldInfo;
 import org.apache.fory.serializer.JavaSerializer;
 import org.apache.fory.serializer.MetaSharedLayerSerializer;
 import org.apache.fory.serializer.ObjectSerializer;
@@ -131,7 +133,7 @@ public class ChildContainerSerializers {
             ArrayList.class, LinkedList.class, ArrayDeque.class, Vector.class, HashSet.class
             // PriorityQueue/TreeSet/ConcurrentSkipListSet need comparator as constructor argument
             );
-    protected InternalFieldInfo[] fieldInfos;
+    protected SerializationFieldInfo[] fieldInfos;
     protected final Serializer[] slotsSerializers;
 
     public ChildCollectionSerializer(Fory fory, Class<T> cls) {
@@ -150,7 +152,7 @@ public class ChildContainerSerializers {
 
     public Collection newCollection(MemoryBuffer buffer) {
       Collection collection = super.newCollection(buffer);
-      readAndSetFields(buffer, collection, slotsSerializers);
+      readAndSetFields(fory, buffer, collection, slotsSerializers);
       return collection;
     }
 
@@ -159,7 +161,7 @@ public class ChildContainerSerializers {
       Collection newCollection = super.newCollection(originCollection);
       if (fieldInfos == null) {
         List<Field> fields = ReflectionUtils.getFieldsWithoutSuperClasses(type, superClasses);
-        fieldInfos = AbstractObjectSerializer.buildFieldsInfo(fory, fields);
+        fieldInfos = FieldGroups.buildFieldsInfo(fory, fields).allFields;
       }
       AbstractObjectSerializer.copyFields(fory, fieldInfos, originCollection, newCollection);
       return newCollection;
@@ -193,7 +195,7 @@ public class ChildContainerSerializers {
             // TreeMap/ConcurrentSkipListMap need comparator as constructor argument
             );
     private final Serializer[] slotsSerializers;
-    private InternalFieldInfo[] fieldInfos;
+    private SerializationFieldInfo[] fieldInfos;
 
     public ChildMapSerializer(Fory fory, Class<T> cls) {
       super(fory, cls);
@@ -212,7 +214,7 @@ public class ChildContainerSerializers {
     @Override
     public Map newMap(MemoryBuffer buffer) {
       Map map = super.newMap(buffer);
-      readAndSetFields(buffer, map, slotsSerializers);
+      readAndSetFields(fory, buffer, map, slotsSerializers);
       return map;
     }
 
@@ -221,7 +223,7 @@ public class ChildContainerSerializers {
       Map newMap = super.newMap(originMap);
       if (fieldInfos == null || fieldInfos.length == 0) {
         List<Field> fields = ReflectionUtils.getFieldsWithoutSuperClasses(type, superClasses);
-        fieldInfos = AbstractObjectSerializer.buildFieldsInfo(fory, fields);
+        fieldInfos = FieldGroups.buildFieldsInfo(fory, fields).allFields;
       }
       AbstractObjectSerializer.copyFields(fory, fieldInfos, originMap, newMap);
       return newMap;
@@ -236,11 +238,11 @@ public class ChildContainerSerializers {
     while (!superClasses.contains(cls)) {
       Serializer slotsSerializer;
       if (fory.getConfig().getCompatibleMode() == CompatibleMode.COMPATIBLE) {
-        ClassDef layerClassDef = fory.getClassResolver().getTypeDef(cls, false);
+        TypeDef layerTypeDef = fory.getClassResolver().getTypeDef(cls, false);
         // Use layer index within class hierarchy (not global counter)
         // This ensures unique marker classes for each layer
         Class<?> layerMarkerClass = LayerMarkerClassGenerator.getOrCreate(fory, cls, layerIndex);
-        slotsSerializer = new MetaSharedLayerSerializer(fory, cls, layerClassDef, layerMarkerClass);
+        slotsSerializer = new MetaSharedLayerSerializer(fory, cls, layerTypeDef, layerMarkerClass);
       } else {
         slotsSerializer = new ObjectSerializer<>(fory, cls, false);
       }
@@ -253,13 +255,42 @@ public class ChildContainerSerializers {
   }
 
   private static void readAndSetFields(
-      MemoryBuffer buffer, Object collection, Serializer[] slotsSerializers) {
+      Fory fory, MemoryBuffer buffer, Object collection, Serializer[] slotsSerializers) {
     for (Serializer slotsSerializer : slotsSerializers) {
       if (slotsSerializer instanceof MetaSharedLayerSerializer) {
-        ((MetaSharedLayerSerializer) slotsSerializer).readAndSetFields(buffer, collection);
+        MetaSharedLayerSerializer metaSerializer = (MetaSharedLayerSerializer) slotsSerializer;
+        // Read layer class meta first if meta share is enabled
+        // This corresponds to writeLayerClassMeta() in MetaSharedLayerSerializer.write()
+        if (fory.getConfig().isMetaShareEnabled()) {
+          readAndSkipLayerClassMeta(fory, buffer);
+        }
+        metaSerializer.readAndSetFields(buffer, collection);
       } else {
         ((ObjectSerializer) slotsSerializer).readAndSetFields(buffer, collection);
       }
     }
   }
+
+  /**
+   * Read and skip the layer class meta from buffer. This is used to skip over the class definition
+   * that was written by MetaSharedLayerSerializer.writeLayerClassMeta(). For
+   * ChildContainerSerializers, we use the same serializer on both write and read sides, so we just
+   * need to skip the meta without actually parsing it.
+   */
+  private static void readAndSkipLayerClassMeta(Fory fory, MemoryBuffer buffer) {
+    MetaContext metaContext = fory.getSerializationContext().getMetaContext();
+    if (metaContext == null) {
+      return;
+    }
+    int indexMarker = buffer.readVarUint32Small14();
+    boolean isRef = (indexMarker & 1) == 1;
+    int index = indexMarker >>> 1;
+    if (isRef) {
+      // Reference to previously read type - nothing more to read
+      return;
+    }
+    // New type - need to read and skip the TypeDef bytes
+    long id = buffer.readInt64();
+    TypeDef.skipTypeDef(buffer, id);
+  }
 }
