@@ -37,8 +37,9 @@ import org.apache.fory.collection.MapSnapshot;
 import org.apache.fory.memory.MemoryBuffer;
 import org.apache.fory.memory.Platform;
 import org.apache.fory.reflect.ReflectionUtils;
-import org.apache.fory.resolver.ClassInfo;
 import org.apache.fory.resolver.ClassResolver;
+import org.apache.fory.resolver.TypeInfo;
+import org.apache.fory.resolver.TypeResolver;
 import org.apache.fory.serializer.ReplaceResolveSerializer;
 import org.apache.fory.serializer.Serializer;
 import org.apache.fory.serializer.Serializers;
@@ -326,7 +327,7 @@ public class MapSerializers {
     @Override
     public EnumMap newMap(MemoryBuffer buffer) {
       setNumElements(buffer.readVarUint32Small7());
-      Class<?> keyType = fory.getClassResolver().readClassInfo(buffer).getCls();
+      Class<?> keyType = fory.getClassResolver().readTypeInfo(buffer).getCls();
       return new EnumMap(keyType);
     }
 
@@ -348,10 +349,9 @@ public class MapSerializers {
       for (Entry<K, V> entry : originMap.entrySet()) {
         V value = entry.getValue();
         if (value != null) {
-          ClassInfo classInfo =
-              classResolver.getClassInfo(value.getClass(), valueClassInfoWriteCache);
-          if (!classInfo.getSerializer().isImmutable()) {
-            value = fory.copyObject(value, classInfo.getClassId());
+          TypeInfo typeInfo = classResolver.getTypeInfo(value.getClass(), valueTypeInfoWriteCache);
+          if (!typeInfo.getSerializer().isImmutable()) {
+            value = fory.copyObject(value, typeInfo.getTypeId());
           }
         }
         newMap.put(entry.getKey(), value);
@@ -494,10 +494,9 @@ public class MapSerializers {
 
   // TODO(chaokunyang) support ConcurrentSkipListMap.SubMap mo efficiently.
   public static void registerDefaultSerializers(Fory fory) {
-    ClassResolver resolver = fory.getClassResolver();
+    TypeResolver resolver = fory.getTypeResolver();
     resolver.registerInternalSerializer(HashMap.class, new HashMapSerializer(fory));
-    fory.getClassResolver()
-        .registerInternalSerializer(LinkedHashMap.class, new LinkedHashMapSerializer(fory));
+    resolver.registerInternalSerializer(LinkedHashMap.class, new LinkedHashMapSerializer(fory));
     resolver.registerInternalSerializer(
         TreeMap.class, new SortedMapSerializer<>(fory, TreeMap.class));
     resolver.registerInternalSerializer(
