@@ -39,7 +39,7 @@ import org.apache.fory.logging.Logger;
 import org.apache.fory.logging.LoggerFactory;
 import org.apache.fory.memory.MemoryBuffer;
 import org.apache.fory.memory.Platform;
-import org.apache.fory.resolver.ClassResolver;
+import org.apache.fory.resolver.TypeResolver;
 import org.apache.fory.serializer.Serializer;
 import org.apache.fory.util.ExceptionUtils;
 import org.apache.fory.util.Preconditions;
@@ -210,7 +210,7 @@ public class UnmodifiableSerializers {
    */
   public static void registerSerializers(Fory fory) {
     try {
-      ClassResolver resolver = fory.getClassResolver();
+      TypeResolver resolver = fory.getTypeResolver();
       for (Tuple2<Class<?>, Function> factory : unmodifiableFactories()) {
         resolver.registerInternalSerializer(factory.f0, createSerializer(fory, factory));
       }
