@@ -19,8 +19,6 @@
 
 package org.apache.fory.type;
 
-import static org.apache.fory.type.TypeUtils.getSizeOfPrimitiveType;
-
 import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -34,122 +32,48 @@ import org.apache.fory.util.record.RecordUtils;
 
 /**
  * A utility class to group class fields into groups.
- * <li>primitive fields
- * <li>boxed primitive fields
- * <li>final fields
- * <li>collection fields
- * <li>map fields
- * <li>other fields
+ *
+ * <ul>
+ *   <li>primitive fields
+ *   <li>boxed primitive fields
+ *   <li>final fields
+ *   <li>collection fields
+ *   <li>map fields
+ *   <li>other fields
+ * </ul>
+ *
+ * <p><b>IMPORTANT:</b> Resorting fields is mandatory in cross-language (xlang) serialization. The
+ * Fory protocol specification requires that both serialization peers (e.g., Java, Rust, Go, Python)
+ * use exactly the same sorting algorithm to determine field order. The in-flight byte order of
+ * fields is not guaranteed to match any particular peer's original declaration order. Instead, each
+ * peer must independently sort fields using the same algorithm to ensure consistent
+ * serialization/deserialization.
+ *
+ * <p>The sorting groups fields by type category (primitives, boxed, collections, maps, etc.) and
+ * then sorts by field name within each category. Both reader and writer must apply this sorting to
+ * produce identical field ordering.
  */
 public class DescriptorGrouper {
-  static final Comparator<Descriptor> COMPARATOR_BY_PRIMITIVE_TYPE_ID =
-      (d1, d2) -> {
-        int c =
-            Types.getPrimitiveTypeId(TypeUtils.unwrap(d2.getRawType()))
-                - Types.getPrimitiveTypeId(TypeUtils.unwrap(d1.getRawType()));
-        if (c == 0) {
-          c = d1.getSnakeCaseName().compareTo(d2.getSnakeCaseName());
-          if (c == 0) {
-            // Field name duplicate in super/child classes.
-            c = d1.getDeclaringClass().compareTo(d2.getDeclaringClass());
-          }
-        }
-        return c;
-      };
+
   private final Collection<Descriptor> descriptors;
-  private final Predicate<Class<?>> isMonomorphic;
+  private final Predicate<Descriptor> isBuildIn;
   private final Function<Descriptor, Descriptor> descriptorUpdater;
   private final boolean descriptorsGroupedOrdered;
   private boolean sorted = false;
 
-  /**
-   * When compress disabled, sort primitive descriptors from largest to smallest, if size is the
-   * same, sort by field name to fix order.
-   *
-   * <p>When compress enabled, sort primitive descriptors from largest to smallest but let compress
-   * fields ends in tail. if size is the same, sort by field name to fix order.
-   */
-  public static Comparator<Descriptor> getPrimitiveComparator(
-      boolean compressInt, boolean compressLong) {
-    if (!compressInt && !compressLong) {
-      // sort primitive descriptors from largest to smallest, if size is the same,
-      // sort by field name to fix order.
-      return (d1, d2) -> {
-        int c =
-            getSizeOfPrimitiveType(TypeUtils.unwrap(d2.getRawType()))
-                - getSizeOfPrimitiveType(TypeUtils.unwrap(d1.getRawType()));
-        if (c == 0) {
-          c = COMPARATOR_BY_PRIMITIVE_TYPE_ID.compare(d1, d2);
-        }
-        return c;
-      };
-    }
-    return (d1, d2) -> {
-      Class<?> t1 = TypeUtils.unwrap(d1.getRawType());
-      Class<?> t2 = TypeUtils.unwrap(d2.getRawType());
-      boolean t1Compress = isCompressedType(t1, compressInt, compressLong);
-      boolean t2Compress = isCompressedType(t2, compressInt, compressLong);
-      if ((t1Compress && t2Compress) || (!t1Compress && !t2Compress)) {
-        int c = getSizeOfPrimitiveType(t2) - getSizeOfPrimitiveType(t1);
-        if (c == 0) {
-          c = COMPARATOR_BY_PRIMITIVE_TYPE_ID.compare(d1, d2);
-        }
-        return c;
-      }
-      if (t1Compress) {
-        return 1;
-      }
-      // t2 compress
-      return -1;
-    };
-  }
-
-  private static boolean isCompressedType(Class<?> cls, boolean compressInt, boolean compressLong) {
-    cls = TypeUtils.unwrap(cls);
-    if (cls == int.class) {
-      return compressInt;
-    }
-    if (cls == long.class) {
-      return compressLong;
-    }
-    return false;
-  }
-
-  /** Comparator based on field type, name and declaring class. */
-  public static final Comparator<Descriptor> COMPARATOR_BY_TYPE_AND_NAME =
-      (d1, d2) -> {
-        // sort by type so that we can hit class info cache more possibly.
-        // sort by field name to fix order if type is same.
-        int c =
-            d1
-                // Use type name instead of generic type so that fields with type ref
-                // constructed in ClassDef which take pojo as non-final Object type
-                // will have consistent order between processes if the fields doesn't exist in peer.
-                .getTypeName()
-                .compareTo(d2.getTypeName());
-        if (c == 0) {
-          c = d1.getName().compareTo(d2.getName());
-          if (c == 0) {
-            // Field name duplicate in super/child classes.
-            c = d1.getDeclaringClass().compareTo(d2.getDeclaringClass());
-          }
-        }
-        return c;
-      };
-
   private final Collection<Descriptor> primitiveDescriptors;
   private final Collection<Descriptor> boxedDescriptors;
   // The element type should be final.
   private final Collection<Descriptor> collectionDescriptors;
   // The key/value type should be final.
   private final Collection<Descriptor> mapDescriptors;
-  private final Collection<Descriptor> finalDescriptors;
+  private final Collection<Descriptor> buildInDescriptors;
   private Collection<Descriptor> otherDescriptors;
 
   /**
    * Create a descriptor grouper.
    *
-   * @param isMonomorphic whether the class is monomorphic.
+   * @param isBuildIn whether the class is build-in types.
    * @param descriptors descriptors may have field with same name.
    * @param descriptorsGroupedOrdered whether the descriptors are grouped and ordered.
    * @param descriptorUpdater create a new descriptor from original one.
@@ -157,14 +81,14 @@ public class DescriptorGrouper {
    * @param comparator comparator for non-primitive fields.
    */
   private DescriptorGrouper(
-      Predicate<Class<?>> isMonomorphic,
+      Predicate<Descriptor> isBuildIn,
       Collection<Descriptor> descriptors,
       boolean descriptorsGroupedOrdered,
       Function<Descriptor, Descriptor> descriptorUpdater,
       Comparator<Descriptor> primitiveComparator,
       Comparator<Descriptor> comparator) {
     this.descriptors = descriptors;
-    this.isMonomorphic = isMonomorphic;
+    this.isBuildIn = isBuildIn;
     this.descriptorUpdater = descriptorUpdater;
     this.descriptorsGroupedOrdered = descriptorsGroupedOrdered;
     this.primitiveDescriptors =
@@ -174,7 +98,7 @@ public class DescriptorGrouper {
     this.collectionDescriptors =
         descriptorsGroupedOrdered ? new ArrayList<>() : new TreeSet<>(comparator);
     this.mapDescriptors = descriptorsGroupedOrdered ? new ArrayList<>() : new TreeSet<>(comparator);
-    this.finalDescriptors =
+    this.buildInDescriptors =
         descriptorsGroupedOrdered ? new ArrayList<>() : new TreeSet<>(comparator);
     this.otherDescriptors =
         descriptorsGroupedOrdered ? new ArrayList<>() : new TreeSet<>(comparator);
@@ -193,15 +117,23 @@ public class DescriptorGrouper {
     }
     for (Descriptor descriptor : descriptors) {
       if (TypeUtils.isPrimitive(descriptor.getRawType())) {
-        primitiveDescriptors.add(descriptorUpdater.apply(descriptor));
+        if (!descriptor.isNullable()) {
+          primitiveDescriptors.add(descriptorUpdater.apply(descriptor));
+        } else {
+          boxedDescriptors.add(descriptorUpdater.apply(descriptor));
+        }
       } else if (TypeUtils.isBoxed(descriptor.getRawType())) {
-        boxedDescriptors.add(descriptorUpdater.apply(descriptor));
+        if (!descriptor.isNullable()) {
+          primitiveDescriptors.add(descriptorUpdater.apply(descriptor));
+        } else {
+          boxedDescriptors.add(descriptorUpdater.apply(descriptor));
+        }
       } else if (TypeUtils.isCollection(descriptor.getRawType())) {
         collectionDescriptors.add(descriptorUpdater.apply(descriptor));
       } else if (TypeUtils.isMap(descriptor.getRawType())) {
         mapDescriptors.add(descriptorUpdater.apply(descriptor));
-      } else if (isMonomorphic.test(descriptor.getRawType())) {
-        finalDescriptors.add(descriptorUpdater.apply(descriptor));
+      } else if (isBuildIn.test(descriptor)) {
+        buildInDescriptors.add(descriptorUpdater.apply(descriptor));
       } else {
         otherDescriptors.add(descriptorUpdater.apply(descriptor));
       }
@@ -215,7 +147,7 @@ public class DescriptorGrouper {
     List<Descriptor> descriptors = new ArrayList<>(getNumDescriptors());
     descriptors.addAll(getPrimitiveDescriptors());
     descriptors.addAll(getBoxedDescriptors());
-    descriptors.addAll(getFinalDescriptors());
+    descriptors.addAll(getBuildInDescriptors());
     descriptors.addAll(getCollectionDescriptors());
     descriptors.addAll(getMapDescriptors());
     descriptors.addAll(getOtherDescriptors());
@@ -242,9 +174,9 @@ public class DescriptorGrouper {
     return mapDescriptors;
   }
 
-  public Collection<Descriptor> getFinalDescriptors() {
+  public Collection<Descriptor> getBuildInDescriptors() {
     Preconditions.checkArgument(sorted);
-    return finalDescriptors;
+    return buildInDescriptors;
   }
 
   public Collection<Descriptor> getOtherDescriptors() {
@@ -265,19 +197,18 @@ public class DescriptorGrouper {
   }
 
   public static DescriptorGrouper createDescriptorGrouper(
-      Predicate<Class<?>> isMonomorphic,
+      Predicate<Descriptor> isBuildIn,
       Collection<Descriptor> descriptors,
       boolean descriptorsGroupedOrdered,
       Function<Descriptor, Descriptor> descriptorUpdator,
-      boolean compressInt,
-      boolean compressLong,
+      Comparator<Descriptor> primitiveComparator,
       Comparator<Descriptor> comparator) {
     return new DescriptorGrouper(
-        isMonomorphic,
+        isBuildIn,
         descriptors,
         descriptorsGroupedOrdered,
         descriptorUpdator == null ? DescriptorGrouper::createDescriptor : descriptorUpdator,
-        getPrimitiveComparator(compressInt, compressLong),
+        primitiveComparator,
         comparator);
   }
 
@@ -287,7 +218,7 @@ public class DescriptorGrouper {
         + boxedDescriptors.size()
         + collectionDescriptors.size()
         + mapDescriptors.size()
-        + finalDescriptors.size()
+        + buildInDescriptors.size()
         + otherDescriptors.size();
   }
 }
