@@ -19,6 +19,8 @@
 
 package org.apache.fory.type;
 
+import java.lang.reflect.AnnotatedParameterizedType;
+import java.lang.reflect.AnnotatedType;
 import java.lang.reflect.Array;
 import java.lang.reflect.Field;
 import java.lang.reflect.GenericArrayType;
@@ -67,12 +69,25 @@ import java.util.Set;
 import java.util.TimeZone;
 import java.util.WeakHashMap;
 import java.util.stream.Collectors;
+import org.apache.fory.annotation.Ref;
+import org.apache.fory.collection.BoolList;
+import org.apache.fory.collection.Float32List;
+import org.apache.fory.collection.Float64List;
 import org.apache.fory.collection.IdentityMap;
+import org.apache.fory.collection.Int16List;
+import org.apache.fory.collection.Int32List;
+import org.apache.fory.collection.Int64List;
+import org.apache.fory.collection.Int8List;
 import org.apache.fory.collection.Tuple2;
+import org.apache.fory.collection.Uint16List;
+import org.apache.fory.collection.Uint32List;
+import org.apache.fory.collection.Uint64List;
+import org.apache.fory.collection.Uint8List;
+import org.apache.fory.meta.TypeExtMeta;
 import org.apache.fory.reflect.ReflectionUtils;
 import org.apache.fory.reflect.TypeParameter;
 import org.apache.fory.reflect.TypeRef;
-import org.apache.fory.serializer.NonexistentClass;
+import org.apache.fory.serializer.UnknownClass;
 import org.apache.fory.util.Preconditions;
 import org.apache.fory.util.StringUtils;
 import org.apache.fory.util.record.RecordUtils;
@@ -264,6 +279,9 @@ public class TypeUtils {
   }
 
   public static Class<?> unwrap(Class<?> clz) {
+    if (clz == null) {
+      return null;
+    }
     if (clz.isPrimitive()) {
       return clz;
     }
@@ -271,6 +289,9 @@ public class TypeUtils {
   }
 
   public static Class<?> boxedType(Class<?> clz) {
+    if (clz == null) {
+      return null;
+    }
     if (!clz.isPrimitive()) {
       return clz;
     }
@@ -486,6 +507,15 @@ public class TypeUtils {
 
   /** Returns element type of iterable. */
   public static TypeRef<?> getElementType(TypeRef<?> typeRef) {
+    if (typeRef.hasTypeExtMeta() && typeRef.hasExplicitTypeArguments()) {
+      List<TypeRef<?>> typeArguments = typeRef.getTypeArguments();
+      if (typeArguments.size() == 1) {
+        Class<?> rawType = getRawType(typeRef);
+        if (Iterable.class.isAssignableFrom(rawType) && rawType.getTypeParameters().length == 1) {
+          return typeArguments.get(0);
+        }
+      }
+    }
     Type type = typeRef.getType();
     if (type instanceof ParameterizedType) {
       ParameterizedType parameterizedType = (ParameterizedType) type;
@@ -513,6 +543,15 @@ public class TypeUtils {
 
   /** Returns key/value type of map. */
   public static Tuple2<TypeRef<?>, TypeRef<?>> getMapKeyValueType(TypeRef<?> typeRef) {
+    if (typeRef.hasTypeExtMeta() && typeRef.hasExplicitTypeArguments()) {
+      List<TypeRef<?>> typeArguments = typeRef.getTypeArguments();
+      if (typeArguments.size() == 2) {
+        Class<?> rawType = getRawType(typeRef);
+        if (Map.class.isAssignableFrom(rawType) && rawType.getTypeParameters().length == 2) {
+          return Tuple2.of(typeArguments.get(0), typeArguments.get(1));
+        }
+      }
+    }
     Type type = typeRef.getType();
     if (type instanceof ParameterizedType) {
       ParameterizedType parameterizedType = (ParameterizedType) type;
@@ -536,6 +575,26 @@ public class TypeUtils {
     return Tuple2.of(keyType, valueType);
   }
 
+  public static void applyRefTrackingOverride(
+      GenericType genericType, AnnotatedType annotatedType, boolean globalTrackingRef) {
+    if (genericType == null || annotatedType == null) {
+      return;
+    }
+    Ref ref = annotatedType.getAnnotation(Ref.class);
+    if (ref != null) {
+      genericType.setTrackingRefOverride(ref.enable() && globalTrackingRef);
+    }
+    if (annotatedType instanceof AnnotatedParameterizedType) {
+      AnnotatedType[] annotatedArgs =
+          ((AnnotatedParameterizedType) annotatedType).getAnnotatedActualTypeArguments();
+      GenericType[] typeParameters = genericType.getTypeParameters();
+      int len = Math.min(annotatedArgs.length, typeParameters.length);
+      for (int i = 0; i < len; i++) {
+        applyRefTrackingOverride(typeParameters[i], annotatedArgs[i], globalTrackingRef);
+      }
+    }
+  }
+
   public static <E> TypeRef<ArrayList<E>> arrayListOf(Class<E> elemType) {
     return new TypeRef<ArrayList<E>>() {}.where(new TypeParameter<E>() {}, elemType);
   }
@@ -552,10 +611,16 @@ public class TypeUtils {
     return new TypeRef<Collection<E>>() {}.where(new TypeParameter<E>() {}, elemType);
   }
 
-  public static <E> TypeRef<Collection<E>> collectionOf(TypeRef<E> elemType, Object extMeta) {
+  public static <E> TypeRef<Collection<E>> collectionOf(TypeRef<E> elemType, TypeExtMeta extMeta) {
     return new TypeRef<Collection<E>>(extMeta) {}.where(new TypeParameter<E>() {}, elemType);
   }
 
+  public static <E> TypeRef<? extends Collection<E>> collectionOf(
+      Class<?> collectionType, TypeRef<E> elemType, TypeExtMeta extMeta) {
+    return new TypeRef<Collection<E>>(extMeta) {}.where(new TypeParameter<E>() {}, elemType)
+        .getSubtype(collectionType);
+  }
+
   public static <K, V> TypeRef<Map<K, V>> mapOf(Class<K> keyType, Class<V> valueType) {
     return mapOf(TypeRef.of(keyType), TypeRef.of(valueType));
   }
@@ -566,11 +631,19 @@ public class TypeUtils {
   }
 
   public static <K, V> TypeRef<Map<K, V>> mapOf(
-      TypeRef<K> keyType, TypeRef<V> valueType, Object extMeta) {
+      TypeRef<K> keyType, TypeRef<V> valueType, TypeExtMeta extMeta) {
     return new TypeRef<Map<K, V>>(extMeta) {}.where(new TypeParameter<K>() {}, keyType)
         .where(new TypeParameter<V>() {}, valueType);
   }
 
+  public static <K, V> TypeRef<? extends Map<K, V>> mapOf(
+      Class<?> mapType, TypeRef<K> keyType, TypeRef<V> valueType, TypeExtMeta extMeta) {
+    TypeRef<Map<K, V>> mapTypeRef =
+        new TypeRef<Map<K, V>>(extMeta) {}.where(new TypeParameter<K>() {}, keyType)
+            .where(new TypeParameter<V>() {}, valueType);
+    return mapTypeRef.getSubtype(mapType);
+  }
+
   public static <K, V> TypeRef<? extends Map<K, V>> mapOf(
       Class<?> mapType, TypeRef<K> keyType, TypeRef<V> valueType) {
     TypeRef<Map<K, V>> mapTypeRef = mapOf(keyType, valueType);
@@ -593,11 +666,28 @@ public class TypeUtils {
   }
 
   public static boolean isCollection(Class<?> cls) {
+    if (isPrimitiveListClass(cls)) {
+      return false;
+    }
     return cls == ArrayList.class || Collection.class.isAssignableFrom(cls);
   }
 
+  public static boolean isPrimitiveListClass(Class<?> cls) {
+    return cls == BoolList.class
+        || cls == Int8List.class
+        || cls == Int16List.class
+        || cls == Int32List.class
+        || cls == Int64List.class
+        || cls == Uint8List.class
+        || cls == Uint16List.class
+        || cls == Uint32List.class
+        || cls == Uint64List.class
+        || cls == Float32List.class
+        || cls == Float64List.class;
+  }
+
   public static boolean isMap(Class<?> cls) {
-    if (cls == NonexistentClass.NonexistentMetaShared.class) {
+    if (cls == UnknownClass.UnknownStruct.class) {
       return false;
     }
     return cls == HashMap.class || Map.class.isAssignableFrom(cls);
@@ -822,6 +912,9 @@ public class TypeUtils {
 
   /** Returns generic type arguments of <code>typeToken</code>. */
   public static List<TypeRef<?>> getTypeArguments(TypeRef<?> typeRef) {
+    if (typeRef.hasExplicitTypeArguments()) {
+      return typeRef.getTypeArguments();
+    }
     if (typeRef.getType() instanceof ParameterizedType) {
       ParameterizedType parameterizedType = (ParameterizedType) typeRef.getType();
       return Arrays.stream(parameterizedType.getActualTypeArguments())
