@@ -24,7 +24,6 @@ import static org.testng.Assert.assertNull;
 import static org.testng.Assert.assertSame;
 import static org.testng.Assert.assertTrue;
 
-import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.primitives.Primitives;
 import java.nio.charset.StandardCharsets;
@@ -64,6 +63,7 @@ import org.apache.fory.serializer.collection.CollectionSerializers;
 import org.apache.fory.serializer.collection.MapSerializers;
 import org.apache.fory.test.bean.BeanB;
 import org.apache.fory.type.TypeUtils;
+import org.apache.fory.type.Types;
 import org.testng.Assert;
 import org.testng.annotations.Test;
 
@@ -74,19 +74,21 @@ public class ClassResolverTest extends ForyTestBase {
   public void testPrimitivesClassId() {
     Fory fory = Fory.builder().withLanguage(Language.JAVA).requireClassRegistration(false).build();
     ClassResolver classResolver = fory.getClassResolver();
-    for (List<Class<?>> classes :
-        ImmutableList.of(
-            TypeUtils.getSortedPrimitiveClasses(), TypeUtils.getSortedBoxedClasses())) {
-      for (int i = 0; i < classes.size() - 1; i++) {
-        assertEquals(
-            classResolver.getRegisteredClassId(classes.get(i)) + 1,
-            classResolver.getRegisteredClassId(classes.get(i + 1)).shortValue());
-        assertTrue(classResolver.getRegisteredClassId(classes.get(i)) > 0);
-      }
+    // Test that primitive types have consecutive IDs
+    List<Class<?>> primitiveClasses = TypeUtils.getSortedPrimitiveClasses();
+    for (int i = 0; i < primitiveClasses.size() - 1; i++) {
       assertEquals(
-          classResolver.getRegisteredClassId(classes.get(classes.size() - 2)) + 1,
-          classResolver.getRegisteredClassId(classes.get(classes.size() - 1)).shortValue());
-      assertTrue(classResolver.getRegisteredClassId(classes.get(classes.size() - 1)) > 0);
+          classResolver.getRegisteredClassId(primitiveClasses.get(i)) + 1,
+          classResolver.getRegisteredClassId(primitiveClasses.get(i + 1)).shortValue());
+      assertTrue(classResolver.getRegisteredClassId(primitiveClasses.get(i)) > 0);
+    }
+    assertTrue(
+        classResolver.getRegisteredClassId(primitiveClasses.get(primitiveClasses.size() - 1)) > 0);
+    // Test that boxed types all have valid positive IDs
+    // Note: boxed types are no longer consecutive due to unsigned type IDs being added
+    List<Class<?>> boxedClasses = TypeUtils.getSortedBoxedClasses();
+    for (Class<?> boxedClass : boxedClasses) {
+      assertTrue(classResolver.getRegisteredClassId(boxedClass) > 0);
     }
   }
 
@@ -102,10 +104,12 @@ public class ClassResolverTest extends ForyTestBase {
     classResolver.register(C2.class, "", "C2");
     classResolver.register(Foo.class, "ns", "Foo");
 
-    Assert.assertTrue(fory.serialize(C1.class).length < 12);
+    byte[] serialized = fory.serialize(C1.class);
+    Assert.assertTrue(serialized.length < 13, "Serialize length " + serialized.length);
     serDeCheck(fory, C1.class);
 
-    Assert.assertTrue(fory.serialize(C2.class).length < 12);
+    serialized = fory.serialize(C2.class);
+    Assert.assertTrue(serialized.length < 13, "Serialize length " + serialized.length);
     serDeCheck(fory, C2.class);
 
     Foo foo = new Foo();
@@ -120,7 +124,7 @@ public class ClassResolverTest extends ForyTestBase {
 
   @Test
   public void testRegisterClassWithUserIds() {
-    // Test that user IDs 0 and 1 work correctly (mapped to internal IDs 256 and 257)
+    // Test that user IDs 0 and 1 work correctly with separated user type IDs.
     Fory fory = Fory.builder().withLanguage(Language.JAVA).requireClassRegistration(true).build();
     ClassResolver classResolver = fory.getClassResolver();
 
@@ -129,12 +133,17 @@ public class ClassResolverTest extends ForyTestBase {
     // Register with user ID 1
     classResolver.register(Bar.class, 1);
 
-    // Verify internal IDs are offset by USER_ID_BASE (256)
-    assertEquals(
-        classResolver.getRegisteredClassId(Foo.class).shortValue(), ClassResolver.USER_ID_BASE);
-    assertEquals(
-        classResolver.getRegisteredClassId(Bar.class).shortValue(),
-        (short) (ClassResolver.USER_ID_BASE + 1));
+    // Verify registered IDs are user IDs and type IDs are internal types only.
+    assertEquals(classResolver.getRegisteredClassId(Foo.class).shortValue(), (short) 0);
+    assertEquals(classResolver.getRegisteredClassId(Bar.class).shortValue(), (short) 1);
+    TypeInfo fooTypeInfo = classResolver.getTypeInfo(Foo.class);
+    TypeInfo barTypeInfo = classResolver.getTypeInfo(Bar.class);
+    assertEquals(fooTypeInfo.getUserTypeId(), 0);
+    assertEquals(barTypeInfo.getUserTypeId(), 1);
+    int fooTypeId = fooTypeInfo.getTypeId();
+    int barTypeId = barTypeInfo.getTypeId();
+    assertTrue(fooTypeId == Types.STRUCT || fooTypeId == Types.COMPATIBLE_STRUCT);
+    assertTrue(barTypeId == Types.STRUCT || barTypeId == Types.COMPATIBLE_STRUCT);
 
     // Verify serialization/deserialization works
     Foo foo = new Foo();
@@ -270,8 +279,8 @@ public class ClassResolverTest extends ForyTestBase {
       MemoryBuffer buffer = MemoryUtils.buffer(32);
       classResolver.writeClassAndUpdateCache(buffer, getClass());
       classResolver.writeClassAndUpdateCache(buffer, getClass());
-      Assert.assertSame(classResolver.readClassInfo(buffer).getCls(), getClass());
-      Assert.assertSame(classResolver.readClassInfo(buffer).getCls(), getClass());
+      Assert.assertSame(classResolver.readTypeInfo(buffer).getCls(), getClass());
+      Assert.assertSame(classResolver.readTypeInfo(buffer).getCls(), getClass());
       classResolver.reset();
       buffer.writerIndex(0);
       buffer.readerIndex(0);
@@ -329,7 +338,7 @@ public class ClassResolverTest extends ForyTestBase {
     ClassResolver classResolver = fory.getClassResolver();
     Assert.assertFalse(
         classResolver.needToWriteRef(TypeRef.of(TestNeedToWriteReferenceClass.class)));
-    assertNull(classResolver.getClassInfo(TestNeedToWriteReferenceClass.class, false));
+    assertNull(classResolver.getTypeInfo(TestNeedToWriteReferenceClass.class, false));
   }
 
   @Test
@@ -343,23 +352,23 @@ public class ClassResolverTest extends ForyTestBase {
     ClassResolver classResolver = fory.getClassResolver();
     {
       classResolver.setSerializer(Foo.class, new ObjectSerializer<>(fory, Foo.class));
-      ClassInfo classInfo = classResolver.getClassInfo(Foo.class);
-      assertSame(classInfo.getSerializer().getClass(), ObjectSerializer.class);
+      TypeInfo typeInfo = classResolver.getTypeInfo(Foo.class);
+      assertSame(typeInfo.getSerializer().getClass(), ObjectSerializer.class);
       // Create another ObjectSerializer to test setSerializer updates the existing classInfo
       classResolver.setSerializer(Foo.class, new ObjectSerializer<>(fory, Foo.class, true));
-      Assert.assertSame(classResolver.getClassInfo(Foo.class), classInfo);
-      assertSame(classInfo.getSerializer().getClass(), ObjectSerializer.class);
+      Assert.assertSame(classResolver.getTypeInfo(Foo.class), typeInfo);
+      assertSame(typeInfo.getSerializer().getClass(), ObjectSerializer.class);
     }
     {
       classResolver.registerInternal(Bar.class);
-      ClassInfo classInfo = classResolver.getClassInfo(Bar.class);
+      TypeInfo typeInfo = classResolver.getTypeInfo(Bar.class);
       classResolver.setSerializer(Bar.class, new ObjectSerializer<>(fory, Bar.class));
-      Assert.assertSame(classResolver.getClassInfo(Bar.class), classInfo);
-      assertSame(classInfo.getSerializer().getClass(), ObjectSerializer.class);
+      Assert.assertSame(classResolver.getTypeInfo(Bar.class), typeInfo);
+      assertSame(typeInfo.getSerializer().getClass(), ObjectSerializer.class);
       // Create another ObjectSerializer to test setSerializer updates the existing classInfo
       classResolver.setSerializer(Bar.class, new ObjectSerializer<>(fory, Bar.class, true));
-      Assert.assertSame(classResolver.getClassInfo(Bar.class), classInfo);
-      assertSame(classInfo.getSerializer().getClass(), ObjectSerializer.class);
+      Assert.assertSame(classResolver.getTypeInfo(Bar.class), typeInfo);
+      assertSame(typeInfo.getSerializer().getClass(), ObjectSerializer.class);
     }
   }
 
@@ -564,4 +573,106 @@ public class ClassResolverTest extends ForyTestBase {
         fory.getClassResolver().getSerializer(abs2Test.getClass()).getClass(),
         AbstractCustomSerializer.class);
   }
+
+  // Test enum with abstract methods (which makes the enum class abstract)
+  enum AbstractEnum {
+    VALUE1 {
+      @Override
+      public int getValue() {
+        return 1;
+      }
+    },
+    VALUE2 {
+      @Override
+      public int getValue() {
+        return 2;
+      }
+    };
+
+    public abstract int getValue();
+  }
+
+  @Test
+  public void testAbstractEnumIsSerializable() {
+    Fory fory = Fory.builder().withLanguage(Language.JAVA).requireClassRegistration(false).build();
+    ClassResolver classResolver = fory.getClassResolver();
+    // Abstract enums should be serializable
+    Assert.assertTrue(classResolver.isSerializable(AbstractEnum.class));
+    // The concrete enum value classes should also be serializable
+    Assert.assertTrue(classResolver.isSerializable(AbstractEnum.VALUE1.getClass()));
+    Assert.assertTrue(classResolver.isSerializable(AbstractEnum.VALUE2.getClass()));
+  }
+
+  @Test
+  public void testAbstractEnumSerialization() {
+    Fory fory = Fory.builder().withLanguage(Language.JAVA).requireClassRegistration(false).build();
+    // Serialize and deserialize abstract enum values
+    Assert.assertEquals(AbstractEnum.VALUE1, serDe(fory, AbstractEnum.VALUE1));
+    Assert.assertEquals(AbstractEnum.VALUE2, serDe(fory, AbstractEnum.VALUE2));
+    Assert.assertEquals(1, ((AbstractEnum) serDe(fory, AbstractEnum.VALUE1)).getValue());
+    Assert.assertEquals(2, ((AbstractEnum) serDe(fory, AbstractEnum.VALUE2)).getValue());
+  }
+
+  @Test
+  public void testAbstractObjectArraySerialization() {
+    Fory fory = Fory.builder().withLanguage(Language.JAVA).requireClassRegistration(false).build();
+    // Create an array of abstract type with concrete instances
+    AbsTest[] array = new AbsTest[2];
+    SubAbsTest item1 = new SubAbsTest();
+    item1.setF1(10);
+    item1.f2 = 100L;
+    Sub2AbsTest item2 = new Sub2AbsTest();
+    item2.setF1(20);
+    item2.f2 = 200L;
+    item2.f3 = "test";
+    array[0] = item1;
+    array[1] = item2;
+
+    AbsTest[] result = serDe(fory, array);
+    Assert.assertEquals(result.length, 2);
+    Assert.assertEquals(result[0].getF1(), 10);
+    Assert.assertEquals(((SubAbsTest) result[0]).f2, 100L);
+    Assert.assertEquals(result[1].getF1(), 20);
+    Assert.assertEquals(((Sub2AbsTest) result[1]).f2, 200L);
+    Assert.assertEquals(((Sub2AbsTest) result[1]).f3, "test");
+  }
+
+  @Test
+  public void testAbstractObjectArrayWithRegistration() {
+    Fory fory = Fory.builder().withLanguage(Language.JAVA).requireClassRegistration(true).build();
+    // Register the concrete types but not the abstract type
+    fory.register(SubAbsTest.class);
+    fory.register(Sub2AbsTest.class);
+    fory.register(AbsTest[].class);
+
+    AbsTest[] array = new AbsTest[2];
+    SubAbsTest item1 = new SubAbsTest();
+    item1.setF1(10);
+    item1.f2 = 100L;
+    Sub2AbsTest item2 = new Sub2AbsTest();
+    item2.setF1(20);
+    item2.f2 = 200L;
+    item2.f3 = "test";
+    array[0] = item1;
+    array[1] = item2;
+
+    AbsTest[] result = serDe(fory, array);
+    Assert.assertEquals(result.length, 2);
+    Assert.assertEquals(result[0].getF1(), 10);
+    Assert.assertEquals(result[1].getF1(), 20);
+  }
+
+  @Test
+  public void testAbstractEnumArraySerialization() {
+    Fory fory = Fory.builder().withLanguage(Language.JAVA).requireClassRegistration(false).build();
+    // Create an array of abstract enum type
+    AbstractEnum[] array = new AbstractEnum[] {AbstractEnum.VALUE1, AbstractEnum.VALUE2};
+
+    AbstractEnum[] result = serDe(fory, array);
+    Assert.assertEquals(result.length, 2);
+    Assert.assertEquals(result[0], AbstractEnum.VALUE1);
+    Assert.assertEquals(result[1], AbstractEnum.VALUE2);
+    Assert.assertEquals(result[0].getValue(), 1);
+    Assert.assertEquals(result[1].getValue(), 2);
+  }
 }
