@@ -31,145 +31,180 @@ export const TypeId = {
   // a 32-bit signed integer.
   INT32: 4,
   // a 32-bit signed integer which uses fory var_int32 encoding.
-  VAR_INT32: 5,
+  VARINT32: 5,
   // a 64-bit signed integer.
   INT64: 6,
   // a 64-bit signed integer which uses fory PVL encoding.
-  VAR_INT64: 7,
-  // a 64-bit signed integer which uses fory SLI encoding.
-  SLI_INT64: 8,
+  VARINT64: 7,
+  // a 64-bit signed integer which uses fory hybrid encoding.
+  TAGGED_INT64: 8,
+  // an 8-bit unsigned integer.
+  UINT8: 9,
+  // a 16-bit unsigned integer.
+  UINT16: 10,
+  // a 32-bit unsigned integer.
+  UINT32: 11,
+  // a 32-bit unsigned integer which uses fory var_uint32 encoding.
+  VAR_UINT32: 12,
+  // a 64-bit unsigned integer.
+  UINT64: 13,
+  // a 64-bit unsigned integer which uses fory var_uint64 encoding.
+  VAR_UINT64: 14,
+  // a 64-bit unsigned integer which uses fory hybrid encoding.
+  TAGGED_UINT64: 15,
+  // an 8-bit floating point number.
+  FLOAT8: 16,
   // a 16-bit floating point number.
-  FLOAT16: 9,
+  FLOAT16: 17,
+  // a 16-bit brain floating point number.
+  BFLOAT16: 18,
   // a 32-bit floating point number.
-  FLOAT32: 10,
+  FLOAT32: 19,
   // a 64-bit floating point number including NaN and Infinity.
-  FLOAT64: 11,
+  FLOAT64: 20,
   // a text string encoded using Latin1/UTF16/UTF-8 encoding.
-  STRING: 12,
-  // a data type consisting of a set of named values. Rust enum with non-predefined field values are not supported as
-  // an enum.
-  ENUM: 13,
-  // an enum whose value will be serialized as the registered name.
-  NAMED_ENUM: 14,
-  // a morphic(final) type serialized by Fory Struct serializer. i.e., it doesn't have subclasses. Suppose we're
-  // deserializing `List[SomeClass]`, we can save dynamic serializer dispatch since `SomeClass` is morphic(final).
-  STRUCT: 15,
-  // a morphic(final) type serialized by Fory compatible Struct serializer.
-  COMPATIBLE_STRUCT: 16,
-  // a `struct` whose type mapping will be encoded as a name.
-  NAMED_STRUCT: 17,
-  // a `compatible_struct` whose type mapping will be encoded as a name.
-  NAMED_COMPATIBLE_STRUCT: 18,
-  // a type which will be serialized by a customized serializer.
-  EXT: 19,
-  // an `ext` type whose type mapping will be encoded as a name.
-  NAMED_EXT: 20,
+  STRING: 21,
   // a sequence of objects.
-  LIST: 21,
+  LIST: 22,
   // an unordered set of unique elements.
-  SET: 22,
-  // a map of key-value pairs. Mutable types such as `list/map/set/array/tensor/arrow` are not allowed as key of map.
-  MAP: 23,
+  SET: 23,
+  // a map of key-value pairs.
+  MAP: 24,
+  // a data type consisting of a set of named values.
+  ENUM: 25,
+  // an enum whose value will be serialized as the registered name.
+  NAMED_ENUM: 26,
+  // a morphic(final) type serialized by Fory Struct serializer.
+  STRUCT: 27,
+  // a morphic(final) type serialized by Fory compatible Struct serializer.
+  COMPATIBLE_STRUCT: 28,
+  // a `struct` whose type mapping will be encoded as a name.
+  NAMED_STRUCT: 29,
+  // a `compatible_struct` whose type mapping will be encoded as a name.
+  NAMED_COMPATIBLE_STRUCT: 30,
+  // a type which will be serialized by a customized serializer.
+  EXT: 31,
+  // an `ext` type whose type mapping will be encoded as a name.
+  NAMED_EXT: 32,
+  // a tagged union type that can hold one of several alternative types.
+  UNION: 33,
+  // a union value with embedded numeric union type ID.
+  TYPED_UNION: 34,
+  // a union value with embedded union type name/TypeDef.
+  NAMED_UNION: 35,
+  // represents an empty/unit value with no data.
+  NONE: 36,
   // an absolute length of time, independent of any calendar/timezone, as a count of nanoseconds.
-  DURATION: 24,
-  // a point in time, independent of any calendar/timezone, as a count of nanoseconds. The count is relative
-  // to an epoch at UTC midnight on January 1, 1970.
-  TIMESTAMP: 25,
-  // a naive date without timezone. The count is days relative to an epoch at UTC midnight on Jan 1, 1970.
-  LOCAL_DATE: 26,
+  DURATION: 37,
+  // a point in time, independent of any calendar/timezone, as a count of nanoseconds.
+  TIMESTAMP: 38,
+  // a naive date without timezone.
+  DATE: 39,
   // exact decimal value represented as an integer value in two's complement.
-  DECIMAL: 27,
+  DECIMAL: 40,
   // a variable-length array of bytes.
-  BINARY: 28,
+  BINARY: 41,
   // a multidimensional array which every sub-array can have different sizes but all have the same type.
-  // only allow numeric components. Other arrays will be taken as List. The implementation should support the
-  // interoperability between array and list.
-  ARRAY: 29,
+  ARRAY: 42,
   // one dimensional bool array.
-  BOOL_ARRAY: 30,
+  BOOL_ARRAY: 43,
   // one dimensional int8 array.
-  INT8_ARRAY: 31,
+  INT8_ARRAY: 44,
   // one dimensional int16 array.
-  INT16_ARRAY: 32,
+  INT16_ARRAY: 45,
   // one dimensional int32 array.
-  INT32_ARRAY: 33,
+  INT32_ARRAY: 46,
   // one dimensional int64 array.
-  INT64_ARRAY: 34,
-  // one dimensional half_float_16 array.
-  FLOAT16_ARRAY: 35,
+  INT64_ARRAY: 47,
+  // one dimensional uint8 array.
+  UINT8_ARRAY: 48,
+  // one dimensional uint16 array.
+  UINT16_ARRAY: 49,
+  // one dimensional uint32 array.
+  UINT32_ARRAY: 50,
+  // one dimensional uint64 array.
+  UINT64_ARRAY: 51,
+  // one dimensional float8 array.
+  FLOAT8_ARRAY: 52,
+  // one dimensional float16 array.
+  FLOAT16_ARRAY: 53,
+  // one dimensional bfloat16 array.
+  BFLOAT16_ARRAY: 54,
   // one dimensional float32 array.
-  FLOAT32_ARRAY: 36,
+  FLOAT32_ARRAY: 55,
   // one dimensional float64 array.
-  FLOAT64_ARRAY: 37,
-  // an arrow [record batch](https://arrow.apache.org/docs/cpp/tables.html#record-batches) object.
-  ARROW_RECORD_BATCH: 38,
-  // an arrow [table](https://arrow.apache.org/docs/cpp/tables.html#tables) object.
-  ARROW_TABLE: 39,
+  FLOAT64_ARRAY: 56,
 
   // BOUND id remains at 64
   BOUND: 64,
 
-  IS_NAMED_TYPE(id: number) {
-    return [TypeId.NAMED_COMPATIBLE_STRUCT, TypeId.NAMED_ENUM, TypeId.NAMED_EXT, TypeId.NAMED_STRUCT].includes(id);
+  isNamedType(id: number) {
+    return [
+      TypeId.NAMED_COMPATIBLE_STRUCT,
+      TypeId.NAMED_ENUM,
+      TypeId.NAMED_EXT,
+      TypeId.NAMED_STRUCT,
+      TypeId.NAMED_UNION,
+    ].includes(id as any);
   },
-
-};
-
-export enum InternalSerializerType {
-  // primitive type
-  BOOL,
-  INT8,
-  INT16,
-  INT32,
-  VAR_INT32,
-  INT64,
-  VAR_INT64,
-  SLI_INT64,
-  FLOAT16,
-  FLOAT32,
-  FLOAT64,
-  STRING,
-  ENUM,
-  LIST,
-  SET,
-  MAP,
-  DURATION,
-  TIMESTAMP,
-  DECIMAL,
-  BINARY,
-  ARRAY,
-  BOOL_ARRAY,
-  INT8_ARRAY,
-  INT16_ARRAY,
-  INT32_ARRAY,
-  INT64_ARRAY,
-  FLOAT16_ARRAY,
-  FLOAT32_ARRAY,
-  FLOAT64_ARRAY,
-  STRUCT,
-
-  // alias type, only use by javascript
-  ANY,
-  ONEOF,
-  TUPLE,
-}
+  polymorphicType(id: number) {
+    return [TypeId.STRUCT, TypeId.NAMED_STRUCT, TypeId.COMPATIBLE_STRUCT, TypeId.NAMED_COMPATIBLE_STRUCT, TypeId.EXT, TypeId.NAMED_EXT].includes(id as any);
+  },
+  structType(id: number) {
+    return [TypeId.STRUCT, TypeId.NAMED_STRUCT, TypeId.COMPATIBLE_STRUCT, TypeId.NAMED_COMPATIBLE_STRUCT].includes(id as any);
+  },
+  extType(id: number) {
+    return [TypeId.EXT, TypeId.NAMED_EXT].includes(id as any);
+  },
+  enumType(id: number) {
+    return [TypeId.ENUM, TypeId.NAMED_ENUM].includes(id as any);
+  },
+  userDefinedType(id: number) {
+    return this.structType(id)
+      || this.extType(id)
+      || this.enumType(id)
+      || id == TypeId.TYPED_UNION
+      || id == TypeId.NAMED_UNION;
+  },
+  isBuiltin(id: number) {
+    return !this.userDefinedType(id) && id !== TypeId.UNKNOWN;
+  },
+  needsUserTypeId(id: number) {
+    return [
+      TypeId.ENUM,
+      TypeId.STRUCT,
+      TypeId.COMPATIBLE_STRUCT,
+      TypeId.EXT,
+      TypeId.TYPED_UNION,
+    ].includes(id as any);
+  },
+} as const;
 
 export enum ConfigFlags {
   isNullFlag = 1 << 0,
-  isLittleEndianFlag = 2,
-  isCrossLanguageFlag = 4,
-  isOutOfBandFlag = 8,
+  isCrossLanguageFlag = 1 << 1,
+  isOutOfBandFlag = 1 << 2,
 }
 
 // read, write
-export type Serializer<T = any, T2 = any> = {
-  read: () => T2;
-  write: (v: T2) => T;
-  readInner: (refValue?: boolean) => T2;
-  writeInner: (v: T2) => T;
+export type Serializer<T = any> = {
   fixedSize: number;
   needToWriteRef: () => boolean;
   getTypeId: () => number;
+  getUserTypeId: () => number;
+  getHash: () => number;
+
+  // for writing
+  write: (v: T) => void;
+  writeRef: (v: T) => void;
+  writeNoRef: (v: T) => void;
+  writeRefOrNull: (v: T) => boolean;
+  writeTypeInfo: (v: T) => void;
+
+  read: (fromRef: boolean) => T;
+  readRef: () => T;
+  readNoRef: (fromRef: boolean) => T;
+  readTypeInfo: () => void;
 };
 
 export enum RefFlags {
@@ -191,8 +226,8 @@ export const HalfMaxInt32 = MaxInt32 / 2;
 export const HalfMinInt32 = MinInt32 / 2;
 
 export const LATIN1 = 0;
-export const UTF8 = 1;
-export const UTF16 = 2;
+export const UTF8 = 2;
+export const UTF16 = 1;
 export interface Hps {
   serializeString: (str: string, dist: Uint8Array, offset: number) => number;
 }
@@ -204,8 +239,7 @@ export enum Mode {
 
 export interface Config {
   hps?: Hps;
-  constructClass: boolean;
-  refTracking: boolean;
+  refTracking: boolean | null;
   useSliceString: boolean;
   hooks: {
     afterCodeGenerated?: (code: string) => string;
@@ -213,19 +247,6 @@ export interface Config {
   mode: Mode;
 }
 
-export enum Language {
-  XLANG = 0,
-  JAVA = 1,
-  PYTHON = 2,
-  CPP = 3,
-  GO = 4,
-  JAVASCRIPT = 5,
-  RUST = 6,
-  DART = 7,
-}
-
-export const MAGIC_NUMBER = 0x62D4;
-
 export interface WithForyClsInfo {
   structTypeInfo: StructTypeInfo;
 }
