@@ -17,29 +17,32 @@
  * under the License.
  */
 
-import { TypeMeta } from "./meta/TypeMeta";
-import { ForyTypeInfoSymbol, InternalSerializerType, WithForyClsInfo, TypeId } from "./type";
+import Fory from "./fory";
+import { ForyTypeInfoSymbol, TypeId, Mode } from "./type";
 
 const initMeta = (target: new () => any, typeInfo: TypeInfo) => {
   if (!target.prototype) {
     target.prototype = {};
   }
-  target.prototype[ForyTypeInfoSymbol] = {
-    structTypeInfo: Type.struct({
-      ...(
-        TypeId.IS_NAMED_TYPE(typeInfo.typeId)
-          ? {
-              namespace: typeInfo.namespace,
-              typeName: typeInfo.typeName,
-            }
-          : {
-              typeId: typeInfo.typeId,
-            }
-      ),
-    }, targetFields.get(target) || {}, {
-      withConstructor: true,
-    }),
-  } as WithForyClsInfo;
+  if (!typeInfo.options) {
+    typeInfo.options = {};
+  }
+  typeInfo.options.withConstructor = true;
+  typeInfo.options.constructor = target;
+  Object.assign(typeInfo.options.props, targetFields.get(target) || {})
+  Object.defineProperties(target.prototype, {
+    [ForyTypeInfoSymbol]: {
+      get() {
+        return {
+          structTypeInfo: typeInfo
+        };
+      },
+      enumerable: false,
+      set(_) {
+        throw new Error("fory type info is readonly")
+      },
+    },
+  })
 };
 
 const targetFields = new WeakMap<new () => any, { [key: string]: TypeInfo }>();
@@ -65,13 +68,24 @@ class ExtensibleFunction extends Function {
 // eslint-disable-next-line
 export class TypeInfo<T = unknown> extends ExtensibleFunction {
   dynamicTypeId = -1;
-  hash = BigInt(0);
   named = "";
   namespace = "";
   typeName = "";
+  // Stored as unsigned 32-bit; -1 (0xffffffff) means "unset".
+  userTypeId = -1;
   options?: any;
+  dynamic: "TRUE" | "FALSE" | "AUTO" = "AUTO";
+  static fory: WeakRef<Fory> | null = null;
 
-  private constructor(public type: InternalSerializerType, public typeId: number) {
+  static attach(fory: Fory) {
+    TypeInfo.fory = new WeakRef(fory);
+  }
+  
+  static detach() {
+    TypeInfo.fory = null;
+  }
+
+  private constructor(private _typeId: number, userTypeId = -1) {
     super(function (target: any, key?: string | { name?: string }) {
       if (key === undefined) {
         initMeta(target, that as unknown as StructTypeInfo);
@@ -85,12 +99,68 @@ export class TypeInfo<T = unknown> extends ExtensibleFunction {
     });
     // eslint-disable-next-line
     const that = this;
+    if (userTypeId !== -1) {
+      if (!Number.isInteger(userTypeId) || userTypeId < 0 || userTypeId > 0xFFFFFFFE) {
+        throw new Error("userTypeId must be in range [0, 0xfffffffe]");
+      }
+    }
+    this.userTypeId = userTypeId;
   }
 
-  static fromNonParam<T extends InternalSerializerType>(type: T, typeId: number) {
+  computeTypeId(fory?: Fory) {
+    const internalTypeId = this._typeId;
+    if (internalTypeId !== TypeId.STRUCT && internalTypeId !== TypeId.NAMED_STRUCT) {
+      return this._typeId;
+    }
+    if (!fory) {
+      throw new Error("fory is not attached")
+    }
+    if (internalTypeId === TypeId.NAMED_STRUCT && fory.config.mode === Mode.Compatible) {
+      return TypeId.NAMED_COMPATIBLE_STRUCT;
+    }
+    if (internalTypeId === TypeId.STRUCT && fory.config.mode === Mode.Compatible) {
+      return TypeId.COMPATIBLE_STRUCT;
+    }
+    return this._typeId;
+  }
+
+  get typeId() {
+    return this.computeTypeId(TypeInfo.fory?.deref());
+  }
+
+  isMonomorphic() {
+    switch (this.dynamic) {
+      case "TRUE":
+        return false;
+      case "FALSE":
+        return true;
+      default:
+        if (TypeId.structType(this._typeId)) {
+          return false;
+        }
+        if (TypeId.enumType(this._typeId)) {
+          return true;
+        }
+        const internalTypeId = this._typeId;
+        const fory = TypeInfo.fory?.deref();
+        if (!fory) {
+          throw new Error("fory is not attached")
+        }
+        if (fory.isCompatible()) {
+          return !TypeId.userDefinedType(this._typeId) && internalTypeId != TypeId.UNKNOWN;
+        }
+        return internalTypeId != TypeId.UNKNOWN;
+    }
+  }
+
+  isNamedType() {
+    return TypeId.isNamedType(this._typeId);
+  }
+
+  static fromNonParam<T>(typeId: number) {
     return new TypeInfo<{
       type: T;
-    }>(type, typeId);
+    }>(typeId);
   }
 
   static fromStruct<T = any>(nameInfo: {
@@ -99,8 +169,10 @@ export class TypeInfo<T = unknown> extends ExtensibleFunction {
     typeName?: string;
   } | string | number, props?: Record<string, TypeInfo>, {
     withConstructor = false,
+    fieldInfo = {},
   }: {
     withConstructor?: boolean;
+    fieldInfo?: Record<string, StructFieldInfo>
   } = {}) {
     let typeId: number | undefined;
     let namespace: string | undefined;
@@ -129,24 +201,31 @@ export class TypeInfo<T = unknown> extends ExtensibleFunction {
         typeName = splits.slice(1).join(".");
       }
     }
-    const finalTypeId = typeId !== undefined ? ((typeId << 8) | TypeId.STRUCT) : TypeId.NAMED_STRUCT;
-    const typeInfo = new TypeInfo<T>(InternalSerializerType.STRUCT, finalTypeId).cast<StructTypeInfo>();
+    let finalTypeId = 0;
+    let userTypeId = -1;
+    if (typeId !== undefined) {
+      finalTypeId = TypeId.STRUCT;
+      userTypeId = typeId;
+    } else {
+      finalTypeId = TypeId.NAMED_STRUCT;
+    }
+    const typeInfo = new TypeInfo<T>(finalTypeId, userTypeId).cast<StructTypeInfo>();
     typeInfo.options = {
       props: props || {},
       withConstructor,
+      fieldInfo
     };
     typeInfo.namespace = namespace || "";
     typeInfo.typeName = typeId !== undefined ? "" : typeName!;
-    typeInfo.hash = TypeMeta.fromTypeInfo(typeInfo).getHash();
     typeInfo.named = `${typeInfo.namespace}$${typeInfo.typeName}`;
     return typeInfo as TypeInfo<T>;
   }
 
-  static fromWithOptions<T extends InternalSerializerType, T2>(type: T, typeId: number, options: T2) {
+  static fromWithOptions<T, T2>(typeId: number, options: T2) {
     const typeInfo = new TypeInfo<{
       type: T;
       options: T2;
-    }>(type, typeId);
+    }>(typeId);
     typeInfo.options = options;
     return typeInfo;
   }
@@ -183,8 +262,9 @@ export class TypeInfo<T = unknown> extends ExtensibleFunction {
         typeName = splits.slice(1).join(".");
       }
     }
-    const finalTypeId = typeId !== undefined ? ((typeId << 8) | TypeId.ENUM) : TypeId.NAMED_ENUM;
-    const typeInfo = new TypeInfo<T>(InternalSerializerType.ENUM, finalTypeId);
+    const finalTypeId = typeId !== undefined ? TypeId.ENUM : TypeId.NAMED_ENUM;
+    const userTypeId = typeId !== undefined ? typeId : -1;
+    const typeInfo = new TypeInfo<T>(finalTypeId, userTypeId);
     typeInfo.cast<EnumTypeInfo>().options = {
       inner: props,
     };
@@ -203,10 +283,13 @@ export class TypeInfo<T = unknown> extends ExtensibleFunction {
   }
 }
 
+type StructFieldInfo = {nullable?: boolean, trackingRef?: boolean}
 export interface StructTypeInfo extends TypeInfo {
   options: {
     props?: { [key: string]: TypeInfo };
+    fieldInfo?: {[key: string]: StructFieldInfo};
     withConstructor?: boolean;
+    constructor?: Function;
   };
 }
 
@@ -216,24 +299,12 @@ export interface EnumTypeInfo extends TypeInfo {
   };
 }
 
-export interface OneofTypeInfo extends TypeInfo {
-  options: {
-    inner: { [key: string]: TypeInfo };
-  };
-}
-
 export interface ArrayTypeInfo extends TypeInfo {
   options: {
     inner: TypeInfo;
   };
 }
 
-export interface TupleTypeInfo extends TypeInfo {
-  options: {
-    inner: TypeInfo[];
-  };
-}
-
 export interface SetTypeInfo extends TypeInfo {
   options: {
     key: TypeInfo;
@@ -253,8 +324,8 @@ type Props<T> = T extends {
   };
 }
   ? {
-      [P in keyof T2]?: (InputType<T2[P]> | null);
-    }
+    [P in keyof T2]?: (InputType<T2[P]> | null);
+  }
   : unknown;
 
 type InnerProps<T> = T extends {
@@ -274,13 +345,6 @@ type MapProps<T> = T extends {
   ? Map<InputType<T2>, InputType<T3> | null>
   : unknown;
 
-type TupleProps<T> = T extends {
-  options: {
-    inner: infer T2 extends readonly [...TypeInfo[]];
-  };
-}
-  ? { [K in keyof T2]: InputType<T2[K]> }
-  : unknown;
 
 type Value<T> = T extends { [s: string]: infer T2 } ? T2 : unknown;
 
@@ -292,24 +356,6 @@ type EnumProps<T> = T extends {
   ? Value<T2>
   : unknown;
 
-type OneofProps<T> = T extends {
-  options: {
-    inner?: infer T2 extends { [key: string]: any };
-  };
-}
-  ? {
-      [P in keyof T2]?: (InputType<T2[P]> | null);
-    }
-  : unknown;
-
-type OneofResult<T> = T extends {
-  options: {
-    inner?: infer T2;
-  };
-}
-  ? ResultType<Value<T2>>
-  : unknown;
-
 type SetProps<T> = T extends {
   options: {
     key: infer T2 extends TypeInfo;
@@ -320,172 +366,170 @@ type SetProps<T> = T extends {
 
 export type InputType<T> = T extends TypeInfo<infer M> ? HintInput<M> : unknown;
 
-export type HintInput<T> = T extends unknown ? any : T extends {
-  type: InternalSerializerType.STRUCT;
+
+export type HintInput<T> = T extends {
+  type: typeof TypeId.STRUCT;
 }
   ? Props<T>
   : T extends {
-    type: InternalSerializerType.STRING;
+    type: typeof TypeId.STRING;
   }
-    ? string
-    : T extends {
-      type: InternalSerializerType.TUPLE;
-    }
-      ? TupleProps<T>
-      : T extends {
-        type:
-          | InternalSerializerType.INT8
-          | InternalSerializerType.INT16
-          | InternalSerializerType.INT32
-          | InternalSerializerType.VAR_INT32
-          | InternalSerializerType.FLOAT16
-          | InternalSerializerType.FLOAT32
-          | InternalSerializerType.FLOAT64;
-      }
-        ? number
+  ? string
+  : T extends {
+    type:
+    | typeof TypeId["INT8"]
+    | typeof TypeId.INT16
+    | typeof TypeId.INT32
+    | typeof TypeId.VARINT32
+    | typeof TypeId.UINT8
+    | typeof TypeId.UINT16
+    | typeof TypeId.UINT32
+    | typeof TypeId.UINT64
+    | typeof TypeId.VAR_UINT64
+    | typeof TypeId.VAR_UINT32
+    | typeof TypeId.FLOAT8
+    | typeof TypeId.FLOAT16
+    | typeof TypeId.BFLOAT16
+    | typeof TypeId.FLOAT32
+    | typeof TypeId.FLOAT64;
+  }
+  ? number
 
-        : T extends {
-          type: InternalSerializerType.VAR_INT64
-            | InternalSerializerType.SLI_INT64
-            | InternalSerializerType.INT64;
-        }
-          ? bigint
-          : T extends {
-            type: InternalSerializerType.MAP;
-          }
-            ? MapProps<T>
-            : T extends {
-              type: InternalSerializerType.SET;
-            }
-              ? SetProps<T>
-              : T extends {
-                type: InternalSerializerType.ARRAY;
-              }
-                ? InnerProps<T>
-                : T extends {
-                  type: InternalSerializerType.BOOL;
-                }
-                  ? boolean
-                  : T extends {
-                    type: InternalSerializerType.DURATION;
-                  }
-                    ? Date
-                    : T extends {
-                      type: InternalSerializerType.TIMESTAMP;
-                    }
-                      ? number
-                      : T extends {
-                        type: InternalSerializerType.ANY;
-                      }
-                        ? any
-                        : T extends { type: InternalSerializerType.BINARY;
-                        }
-                          ? Uint8Array
-                          : T extends {
-                            type: InternalSerializerType.ENUM;
-                          }
-                            ? EnumProps<T> : T extends {
-                              type: InternalSerializerType.ONEOF;
-                            } ? OneofProps<T> : unknown;
+  : T extends {
+    type: typeof TypeId.VARINT64
+    | typeof TypeId.TAGGED_INT64
+    | typeof TypeId.INT64
+    | typeof TypeId.UINT64
+    | typeof TypeId.VAR_UINT64
+    | typeof TypeId.TAGGED_UINT64;
+  }
+  ? bigint
+  : T extends {
+    type: typeof TypeId.MAP;
+  }
+  ? MapProps<T>
+  : T extends {
+    type: typeof TypeId.SET;
+  }
+  ? SetProps<T>
+  : T extends {
+    type: typeof TypeId.LIST;
+  }
+  ? InnerProps<T>
+  : T extends {
+    type: typeof TypeId.BOOL;
+  }
+  ? boolean
+  : T extends {
+    type: typeof TypeId.DURATION;
+  }
+  ? Date
+  : T extends {
+    type: typeof TypeId.TIMESTAMP;
+  }
+  ? number
+  : T extends {
+    type: typeof TypeId.BINARY;
+  }
+  ? Uint8Array
+  : T extends {
+    type: typeof TypeId.ENUM;
+  }
+  ? EnumProps<T> : any;
 
 export type ResultType<T> = T extends TypeInfo<infer M> ? HintResult<M> : HintResult<T>;
 
 export type HintResult<T> = T extends never ? any : T extends {
-  type: InternalSerializerType.STRUCT;
+  type: typeof TypeId.STRUCT;
 }
   ? Props<T>
   : T extends {
-    type: InternalSerializerType.STRING;
+    type: typeof TypeId.STRING;
   }
-    ? string
-    : T extends {
-      type: InternalSerializerType.TUPLE;
-    }
-      ? TupleProps<T>
-      : T extends {
-        type:
-          | InternalSerializerType.INT8
-          | InternalSerializerType.INT16
-          | InternalSerializerType.INT32
-          | InternalSerializerType.VAR_INT32
-          | InternalSerializerType.FLOAT16
-          | InternalSerializerType.FLOAT32
-          | InternalSerializerType.FLOAT64;
-      }
-        ? number
+  ? string
+  : T extends {
+    type:
+    | typeof TypeId.INT8
+    | typeof TypeId.INT16
+    | typeof TypeId.INT32
+    | typeof TypeId.VARINT32
+    | typeof TypeId.UINT8
+    | typeof TypeId.UINT16
+    | typeof TypeId.UINT32
+    | typeof TypeId.VAR_UINT32
+    | typeof TypeId.FLOAT8
+    | typeof TypeId.FLOAT16
+    | typeof TypeId.BFLOAT16
+    | typeof TypeId.FLOAT32
+    | typeof TypeId.FLOAT64;
+  }
+  ? number
 
-        : T extends {
-          type: InternalSerializerType.SLI_INT64
-            | InternalSerializerType.INT64;
-        }
-          ? bigint
-          : T extends {
-            type: InternalSerializerType.MAP;
-          }
-            ? MapProps<T>
-            : T extends {
-              type: InternalSerializerType.SET;
-            }
-              ? SetProps<T>
-              : T extends {
-                type: InternalSerializerType.ARRAY;
-              }
-                ? InnerProps<T>
-                : T extends {
-                  type: InternalSerializerType.BOOL;
-                }
-                  ? boolean
-                  : T extends {
-                    type: InternalSerializerType.DURATION;
-                  }
-                    ? Date
-                    : T extends {
-                      type: InternalSerializerType.TIMESTAMP;
-                    }
-                      ? number
-                      : T extends { type: InternalSerializerType.BINARY;
-                      }
-                        ? Uint8Array : T extends {
-                          type: InternalSerializerType.ANY;
-                        }
-                          ? any
-                          : T extends {
-                            type: InternalSerializerType.ENUM;
-                          }
-                            ? EnumProps<T> : T extends {
-                              type: InternalSerializerType.ONEOF;
-                            } ? OneofResult<T> : unknown;
+  : T extends {
+    type: typeof TypeId.TAGGED_INT64
+    | typeof TypeId.INT64
+    | typeof TypeId.UINT64
+    | typeof TypeId.VAR_UINT64
+    | typeof TypeId.TAGGED_UINT64;
+  }
+  ? bigint
+  : T extends {
+    type: typeof TypeId.MAP;
+  }
+  ? MapProps<T>
+  : T extends {
+    type: typeof TypeId.SET;
+  }
+  ? SetProps<T>
+  : T extends {
+    type: typeof TypeId.LIST;
+  }
+  ? InnerProps<T>
+  : T extends {
+    type: typeof TypeId.BOOL;
+  }
+  ? boolean
+  : T extends {
+    type: typeof TypeId.DURATION;
+  }
+  ? Date
+  : T extends {
+    type: typeof TypeId.TIMESTAMP;
+  }
+  ? number
+  : T extends {
+    type: typeof TypeId.BINARY;
+  }
+  ? Uint8Array : T extends {
+    type: typeof TypeId.ENUM;
+  }
+  ? EnumProps<T>: unknown;
 
 export const Type = {
   any() {
-    return TypeInfo.fromNonParam(InternalSerializerType.ANY, TypeId.STRUCT);
-  },
-  oneof<T extends { [key: string]: TypeInfo }>(inner?: T) {
-    return TypeInfo.fromWithOptions(InternalSerializerType.ONEOF as const, TypeId.STRUCT, {
-      inner,
-    });
+    return TypeInfo.fromNonParam<typeof TypeId.UNKNOWN>(TypeId.UNKNOWN);
   },
   array<T extends TypeInfo>(inner: T) {
-    return TypeInfo.fromWithOptions(InternalSerializerType.ARRAY as const, TypeId.ARRAY, {
+    return TypeInfo.fromWithOptions<typeof TypeId.LIST, { inner: T }>(TypeId.LIST, {
       inner,
     });
   },
-  tuple<T1 extends readonly [...readonly TypeInfo[]]>(t1: T1) {
-    return TypeInfo.fromWithOptions(InternalSerializerType.TUPLE as const, TypeId.LIST, {
-      inner: t1,
-    });
-  },
   map<T1 extends TypeInfo, T2 extends TypeInfo>(
     key: T1,
     value: T2
   ) {
-    return TypeInfo.fromWithOptions(InternalSerializerType.MAP as const, TypeId.MAP, {
+    return TypeInfo.fromWithOptions<typeof TypeId.MAP, {
+      key: T1,
+      value: T2
+    }>(TypeId.MAP, {
       key,
       value,
     });
   },
   set<T extends TypeInfo>(key: T) {
-    return TypeInfo.fromWithOptions(InternalSerializerType.SET as const, TypeId.SET, {
+    return TypeInfo.fromWithOptions<typeof TypeId.SET, {
+      key: T
+    }>(TypeId.SET, {
       key,
     });
   },
@@ -495,7 +539,7 @@ export const Type = {
     typeName?: string;
   } | string | number, t1: T1) {
     return TypeInfo.fromEnum<{
-      type: InternalSerializerType.ENUM;
+      type: typeof TypeId.ENUM;
       options: {
         inner: T1;
       };
@@ -507,165 +551,208 @@ export const Type = {
     typeName?: string;
   } | string | number, props?: T, {
     withConstructor = false,
+    fieldInfo,
   }: {
     withConstructor?: boolean;
+    fieldInfo?: Record<string, StructFieldInfo>
   } = {}) {
     return TypeInfo.fromStruct<{
-      type: InternalSerializerType.STRUCT;
+      type: typeof TypeId.STRUCT;
       options: {
         props: T;
       };
     }>(nameInfo, props, {
       withConstructor,
+      fieldInfo
     });
   },
   string() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.STRING as const,
+    return TypeInfo.fromNonParam<typeof TypeId.STRING>(
       (TypeId.STRING),
     );
   },
   bool() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.BOOL as const,
+    return TypeInfo.fromNonParam<typeof TypeId.BOOL>(
       (TypeId.BOOL),
     );
   },
   int8() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.INT8 as const,
+    return TypeInfo.fromNonParam<typeof TypeId.INT8>(
       (TypeId.INT8),
     );
   },
   int16() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.INT16 as const,
+    return TypeInfo.fromNonParam<typeof TypeId.INT16>(
       (TypeId.INT16),
 
     );
   },
   int32() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.INT32 as const,
+    return TypeInfo.fromNonParam<typeof TypeId.INT32>(
       (TypeId.INT32),
 
     );
   },
   varInt32() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.VAR_INT32 as const,
-      (TypeId.VAR_INT32),
+    return TypeInfo.fromNonParam<typeof TypeId.VARINT32>(
+      (TypeId.VARINT32),
 
     );
   },
   int64() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.INT64 as const,
+    return TypeInfo.fromNonParam<typeof TypeId.INT64>(
       (TypeId.INT64),
 
     );
   },
   sliInt64() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.SLI_INT64 as const,
-      (TypeId.SLI_INT64),
+    return TypeInfo.fromNonParam<typeof TypeId.TAGGED_INT64>(
+      (TypeId.TAGGED_INT64),
 
     );
   },
   float16() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.FLOAT16 as const,
+    return TypeInfo.fromNonParam<typeof TypeId.FLOAT16>(
       (TypeId.FLOAT16),
 
     );
   },
   float32() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.FLOAT32 as const,
+    return TypeInfo.fromNonParam<typeof TypeId.FLOAT32>(
       (TypeId.FLOAT32),
 
     );
   },
   float64() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.FLOAT64 as const,
+    return TypeInfo.fromNonParam<typeof TypeId.FLOAT64>(
       (TypeId.FLOAT64),
 
     );
   },
+  uint8() {
+    return TypeInfo.fromNonParam<typeof TypeId.UINT8>(
+      (TypeId.UINT8),
+    );
+  },
+  uint16() {
+    return TypeInfo.fromNonParam<typeof TypeId.UINT16>(
+      (TypeId.UINT16),
+    );
+  },
+  uint32() {
+    return TypeInfo.fromNonParam<typeof TypeId.UINT32>(
+      (TypeId.UINT32),
+    );
+  },
+  varUInt32() {
+    return TypeInfo.fromNonParam<typeof TypeId.VAR_UINT32>(
+      (TypeId.VAR_UINT32),
+    );
+  },
+  uint64() {
+    return TypeInfo.fromNonParam<typeof TypeId.UINT64>(
+      (TypeId.UINT64),
+    );
+  },
+  varUInt64() {
+    return TypeInfo.fromNonParam<typeof TypeId.VAR_UINT64>(
+      (TypeId.VAR_UINT64),
+    );
+  },
+  varInt64() {
+    return TypeInfo.fromNonParam<typeof TypeId.VARINT64>(
+      (TypeId.VARINT64),
+    );
+  },
+  taggedUInt64() {
+    return TypeInfo.fromNonParam<typeof TypeId.TAGGED_UINT64>(
+      (TypeId.TAGGED_UINT64),
+    );
+  },
   binary() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.BINARY as const,
+    return TypeInfo.fromNonParam<typeof TypeId.BINARY>(
       (TypeId.BINARY),
 
     );
   },
   duration() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.DURATION as const,
+    return TypeInfo.fromNonParam<typeof TypeId.DURATION>(
       (TypeId.DURATION),
-
     );
   },
   timestamp() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.TIMESTAMP as const,
+    return TypeInfo.fromNonParam<typeof TypeId.TIMESTAMP>(
       (TypeId.TIMESTAMP),
-
     );
   },
   boolArray() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.BOOL_ARRAY as const,
+    return TypeInfo.fromNonParam<typeof TypeId.BOOL_ARRAY>(
       (TypeId.BOOL_ARRAY),
 
     );
   },
   int8Array() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.INT8_ARRAY as const,
+    return TypeInfo.fromNonParam<typeof TypeId.INT8_ARRAY>(
       (TypeId.INT8_ARRAY),
 
     );
   },
   int16Array() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.INT16_ARRAY as const,
+    return TypeInfo.fromNonParam<typeof TypeId.INT16_ARRAY>(
       (TypeId.INT16_ARRAY),
 
     );
   },
   int32Array() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.INT32_ARRAY as const,
+    return TypeInfo.fromNonParam<typeof TypeId.INT32_ARRAY>(
       (TypeId.INT32_ARRAY),
 
     );
   },
   int64Array() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.INT64_ARRAY as const,
+    return TypeInfo.fromNonParam<typeof TypeId.INT64_ARRAY>(
+      (TypeId.INT64_ARRAY),
+
+    );
+  },
+  uint8Array() {
+    return TypeInfo.fromNonParam<typeof TypeId.UINT8_ARRAY>(
+      (TypeId.INT8_ARRAY),
+
+    );
+  },
+  uint16Array() {
+    return TypeInfo.fromNonParam<typeof TypeId.UINT16_ARRAY>(
+      (TypeId.INT16_ARRAY),
+
+    );
+  },
+  uint32Array() {
+    return TypeInfo.fromNonParam<typeof TypeId.UINT32_ARRAY>(
+      (TypeId.UINT32_ARRAY),
+
+    );
+  },
+  uint64Array() {
+    return TypeInfo.fromNonParam<typeof TypeId.UINT64_ARRAY>(
       (TypeId.INT64_ARRAY),
 
     );
   },
   float16Array() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.FLOAT16_ARRAY as const,
+    return TypeInfo.fromNonParam<typeof TypeId.FLOAT16_ARRAY>(
       (TypeId.FLOAT16_ARRAY),
 
     );
   },
   float32Array() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.FLOAT32_ARRAY as const,
+    return TypeInfo.fromNonParam<typeof TypeId.FLOAT32_ARRAY>(
       (TypeId.FLOAT32_ARRAY),
 
     );
   },
   float64Array() {
-    return TypeInfo.fromNonParam(
-      InternalSerializerType.FLOAT64_ARRAY as const,
+    return TypeInfo.fromNonParam<typeof TypeId.FLOAT64_ARRAY>(
       (TypeId.FLOAT64_ARRAY)
     );
   },
