@@ -15,7 +15,6 @@
 # specific language governing permissions and limitations
 # under the License.
 
-import enum
 import logging
 import os
 from abc import ABC, abstractmethod
@@ -28,8 +27,8 @@ from pyfory.resolver import (
     NULL_FLAG,
     NOT_NULL_VALUE_FLAG,
 )
-from pyfory.util import is_little_endian, set_bit, get_bit, clear_bit
-from pyfory.type import TypeId
+from pyfory.utils import set_bit, get_bit, clear_bit
+from pyfory.types import TypeId
 from pyfory.policy import DeserializationPolicy, DEFAULT_POLICY
 
 try:
@@ -41,14 +40,15 @@ except ImportError:
 logger = logging.getLogger(__name__)
 
 
-MAGIC_NUMBER = 0x62D4
 DEFAULT_DYNAMIC_WRITE_META_STR_ID = -1
 DYNAMIC_TYPE_ID = -1
 USE_TYPE_NAME = 0
 USE_TYPE_ID = 1
 # preserve 0 as flag for type id not set in TypeInfo`
 NO_TYPE_ID = 0
-INT64_TYPE_ID = TypeId.INT64
+# 0xffffffff means "unset" for user type id.
+NO_USER_TYPE_ID = 0xFFFFFFFF
+INT64_TYPE_ID = TypeId.VARINT64
 FLOAT64_TYPE_ID = TypeId.FLOAT64
 BOOL_TYPE_ID = TypeId.BOOL
 STRING_TYPE_ID = TypeId.STRING
@@ -60,17 +60,6 @@ NOT_NULL_STRING_FLAG = NOT_NULL_VALUE_FLAG & 0b11111111 | (STRING_TYPE_ID << 8)
 SMALL_STRING_THRESHOLD = 16
 
 
-class Language(enum.Enum):
-    XLANG = 0
-    JAVA = 1
-    PYTHON = 2
-    CPP = 3
-    GO = 4
-    JAVA_SCRIPT = 5
-    RUST = 6
-    DART = 7
-
-
 class BufferObject(ABC):
     """
     Fory binary representation of an object.
@@ -146,21 +135,20 @@ class Fory:
     """
 
     __slots__ = (
-        "language",
-        "is_py",
+        "xlang",
         "compatible",
-        "ref_tracking",
+        "track_ref",
         "ref_resolver",
         "type_resolver",
         "serialization_context",
         "strict",
         "buffer",
-        "_buffer_callback",
+        "buffer_callback",
         "_buffers",
         "metastring_resolver",
         "_unsupported_callback",
         "_unsupported_objects",
-        "_peer_language",
+        "is_peer_out_of_band_enabled",
         "max_depth",
         "depth",
         "field_nullable",
@@ -176,7 +164,7 @@ class Fory:
         max_depth: int = 50,
         policy: DeserializationPolicy = None,
         field_nullable: bool = False,
-        **kwargs,
+        meta_compressor=None,
     ):
         """
         Initialize a Fory serialization instance.
@@ -223,37 +211,30 @@ class Fory:
             >>> # Cross-language mode with schema evolution
             >>> fory = Fory(xlang=True, compatible=True)
         """
-        self.language = Language.XLANG if xlang else Language.PYTHON
-        if kwargs.get("language") is not None:
-            self.language = kwargs.get("language")
-        self.is_py = self.language == Language.PYTHON
-        if kwargs.get("ref_tracking") is not None:
-            ref = kwargs.get("ref_tracking")
-        self.ref_tracking = ref
-        if self.ref_tracking:
+        self.xlang = xlang
+        self.track_ref = ref
+        if self.track_ref:
             self.ref_resolver = MapRefResolver()
         else:
             self.ref_resolver = NoRefResolver()
-        if kwargs.get("require_type_registration") is not None:
-            strict = kwargs.get("require_type_registration")
         self.strict = _ENABLE_TYPE_REGISTRATION_FORCIBLY or strict
         self.policy = policy or DEFAULT_POLICY
         self.compatible = compatible
-        self.field_nullable = field_nullable if self.is_py else False
+        self.field_nullable = field_nullable if not self.xlang else False
         from pyfory.serialization import MetaStringResolver, SerializationContext
-        from pyfory._registry import TypeResolver
+        from pyfory.registry import TypeResolver
 
         self.metastring_resolver = MetaStringResolver()
-        self.type_resolver = TypeResolver(self, meta_share=compatible)
+        self.type_resolver = TypeResolver(self, meta_share=compatible, meta_compressor=meta_compressor)
         self.serialization_context = SerializationContext(fory=self, scoped_meta_share_enabled=compatible)
         self.type_resolver.initialize()
 
         self.buffer = Buffer.allocate(32)
-        self._buffer_callback = None
+        self.buffer_callback = None
         self._buffers = None
         self._unsupported_callback = None
         self._unsupported_objects = None
-        self._peer_language = None
+        self.is_peer_out_of_band_enabled = False
         self.max_depth = max_depth
         self.depth = 0
 
@@ -359,6 +340,26 @@ class Fory:
             serializer=serializer,
         )
 
+    def register_union(
+        self,
+        cls: Union[type, TypeVar],
+        *,
+        type_id: int = None,
+        namespace: str = None,
+        typename: str = None,
+        serializer=None,
+    ):
+        """
+        Register a union type with a generated serializer.
+        """
+        return self.type_resolver.register_union(
+            cls,
+            type_id=type_id,
+            namespace=namespace,
+            typename=typename,
+            serializer=serializer,
+        )
+
     def register_serializer(self, cls: type, serializer):
         """
         Register a custom serializer for a type.
@@ -447,62 +448,40 @@ class Fory:
     ) -> Union[Buffer, bytes]:
         assert self.depth == 0, "Nested serialization should use write_ref/write_no_ref/xwrite_ref/xwrite_no_ref."
         self.depth += 1
-        self._buffer_callback = buffer_callback
+        self.buffer_callback = buffer_callback
         self._unsupported_callback = unsupported_callback
         if buffer is None:
-            self.buffer.writer_index = 0
+            self.buffer.set_writer_index(0)
             buffer = self.buffer
-        if self.language == Language.XLANG:
-            buffer.write_int16(MAGIC_NUMBER)
-        mask_index = buffer.writer_index
+        mask_index = buffer.get_writer_index()
         # 1byte used for bit mask
         buffer.grow(1)
-        buffer.writer_index = mask_index + 1
+        buffer.set_writer_index(mask_index + 1)
         if obj is None:
             set_bit(buffer, mask_index, 0)
         else:
             clear_bit(buffer, mask_index, 0)
-        # set endian
-        if is_little_endian:
+
+        if self.xlang:
+            # set reader as x_lang.
             set_bit(buffer, mask_index, 1)
-        else:
-            clear_bit(buffer, mask_index, 1)
-
-        if self.language == Language.XLANG:
-            # set reader as x_lang.
-            set_bit(buffer, mask_index, 2)
-            # set writer language.
-            buffer.write_int8(Language.PYTHON.value)
         else:
             # set reader as native.
-            clear_bit(buffer, mask_index, 2)
-        if self._buffer_callback is not None:
-            set_bit(buffer, mask_index, 3)
+            clear_bit(buffer, mask_index, 1)
+        if self.buffer_callback is not None:
+            set_bit(buffer, mask_index, 2)
         else:
-            clear_bit(buffer, mask_index, 3)
-        # Reserve space for type definitions offset, similar to Java implementation
-        type_defs_offset_pos = None
-        if self.serialization_context.scoped_meta_share_enabled:
-            type_defs_offset_pos = buffer.writer_index
-            buffer.write_int32(-1)  # Reserve 4 bytes for type definitions offset
+            clear_bit(buffer, mask_index, 2)
+        # Type definitions are now written inline (streaming) instead of deferred to end
 
-        if self.language == Language.PYTHON:
+        if not self.xlang:
             self.write_ref(buffer, obj)
         else:
             self.xwrite_ref(buffer, obj)
-
-        # Write type definitions at the end, similar to Java implementation
-        if self.serialization_context.scoped_meta_share_enabled:
-            meta_context = self.serialization_context.meta_context
-            if meta_context is not None and len(meta_context.get_writing_type_defs()) > 0:
-                # Update the offset to point to current position
-                current_pos = buffer.writer_index
-                buffer.put_int32(type_defs_offset_pos, current_pos - type_defs_offset_pos - 4)
-                self.type_resolver.write_type_defs(buffer)
         if buffer is not self.buffer:
             return buffer
         else:
-            return buffer.to_bytes(0, buffer.writer_index)
+            return buffer.to_bytes(0, buffer.get_writer_index())
 
     def write_ref(self, buffer, obj, typeinfo=None):
         cls = type(obj)
@@ -521,27 +500,27 @@ class Fory:
         if self.ref_resolver.write_ref_or_null(buffer, obj):
             return
         if typeinfo is None:
-            typeinfo = self.type_resolver.get_typeinfo(cls)
-        self.type_resolver.write_typeinfo(buffer, typeinfo)
+            typeinfo = self.type_resolver.get_type_info(cls)
+        self.type_resolver.write_type_info(buffer, typeinfo)
         typeinfo.serializer.write(buffer, obj)
 
     def write_no_ref(self, buffer, obj):
         cls = type(obj)
         if cls is str:
-            buffer.write_varuint32(STRING_TYPE_ID)
+            buffer.write_uint8(STRING_TYPE_ID)
             buffer.write_string(obj)
             return
         elif cls is int:
-            buffer.write_varuint32(INT64_TYPE_ID)
+            buffer.write_uint8(INT64_TYPE_ID)
             buffer.write_varint64(obj)
             return
         elif cls is bool:
-            buffer.write_varuint32(BOOL_TYPE_ID)
+            buffer.write_uint8(BOOL_TYPE_ID)
             buffer.write_bool(obj)
             return
         else:
-            typeinfo = self.type_resolver.get_typeinfo(cls)
-            self.type_resolver.write_typeinfo(buffer, typeinfo)
+            typeinfo = self.type_resolver.get_type_info(cls)
+            self.type_resolver.write_type_info(buffer, typeinfo)
             typeinfo.serializer.write(buffer, obj)
 
     def xwrite_ref(self, buffer, obj, serializer=None):
@@ -560,8 +539,8 @@ class Fory:
             serializer.xwrite(buffer, obj)
             return
         cls = type(obj)
-        typeinfo = self.type_resolver.get_typeinfo(cls)
-        self.type_resolver.write_typeinfo(buffer, typeinfo)
+        typeinfo = self.type_resolver.get_type_info(cls)
+        self.type_resolver.write_type_info(buffer, typeinfo)
         typeinfo.serializer.xwrite(buffer, obj)
 
     def deserialize(
@@ -609,47 +588,25 @@ class Fory:
             buffer = Buffer(buffer)
         if unsupported_objects is not None:
             self._unsupported_objects = iter(unsupported_objects)
-        if self.language == Language.XLANG:
-            magic_numer = buffer.read_int16()
-            assert magic_numer == MAGIC_NUMBER, (
-                f"The fory xlang serialization must start with magic number {hex(MAGIC_NUMBER)}. "
-                "Please check whether the serialization is based on the xlang protocol and the data didn't corrupt."
-            )
-        reader_index = buffer.reader_index
-        buffer.reader_index = reader_index + 1
+        reader_index = buffer.get_reader_index()
+        buffer.set_reader_index(reader_index + 1)
         if get_bit(buffer, reader_index, 0):
             return None
-        is_little_endian_ = get_bit(buffer, reader_index, 1)
-        assert is_little_endian_, "Big endian is not supported for now, please ensure peer machine is little endian."
-        is_target_x_lang = get_bit(buffer, reader_index, 2)
-        if is_target_x_lang:
-            self._peer_language = Language(buffer.read_int8())
-        else:
-            self._peer_language = Language.PYTHON
-        is_out_of_band_serialization_enabled = get_bit(buffer, reader_index, 3)
-        if is_out_of_band_serialization_enabled:
+        is_target_x_lang = get_bit(buffer, reader_index, 1)
+        self.is_peer_out_of_band_enabled = get_bit(buffer, reader_index, 2)
+        if self.is_peer_out_of_band_enabled:
             assert buffers is not None, "buffers shouldn't be null when the serialized stream is produced with buffer_callback not null."
             self._buffers = iter(buffers)
         else:
             assert buffers is None, "buffers should be null when the serialized stream is produced with buffer_callback null."
 
-        # Read type definitions at the start, similar to Java implementation
-        if self.serialization_context.scoped_meta_share_enabled:
-            relative_type_defs_offset = buffer.read_int32()
-            if relative_type_defs_offset != -1:
-                # Save current reader position
-                current_reader_index = buffer.reader_index
-                # Jump to type definitions
-                buffer.reader_index = current_reader_index + relative_type_defs_offset
-                # Read type definitions
-                self.type_resolver.read_type_defs(buffer)
-                # Jump back to continue with object deserialization
-                buffer.reader_index = current_reader_index
+        # Type definitions are now read inline (streaming) instead of at the end
 
         if is_target_x_lang:
             obj = self.xread_ref(buffer)
         else:
             obj = self.read_ref(buffer)
+
         return obj
 
     def read_ref(self, buffer):
@@ -657,7 +614,7 @@ class Fory:
         ref_id = ref_resolver.try_preserve_ref_id(buffer)
         # indicates that the object is first read.
         if ref_id >= NOT_NULL_VALUE_FLAG:
-            typeinfo = self.type_resolver.read_typeinfo(buffer)
+            typeinfo = self.type_resolver.read_type_info(buffer)
             self.inc_depth()
             o = typeinfo.serializer.read(buffer)
             self.dec_depth()
@@ -668,7 +625,7 @@ class Fory:
 
     def read_no_ref(self, buffer):
         """Deserialize not-null and non-reference object from buffer."""
-        typeinfo = self.type_resolver.read_typeinfo(buffer)
+        typeinfo = self.type_resolver.read_type_info(buffer)
         self.inc_depth()
         o = typeinfo.serializer.read(buffer)
         self.dec_depth()
@@ -680,7 +637,8 @@ class Fory:
             ref_id = ref_resolver.try_preserve_ref_id(buffer)
             # indicates that the object is first read.
             if ref_id >= NOT_NULL_VALUE_FLAG:
-                o = self.xread_no_ref(buffer, serializer=serializer)
+                # Don't push -1 here - try_preserve_ref_id already pushed ref_id
+                o = self._xread_no_ref_internal(buffer, serializer)
                 ref_resolver.set_read_object(ref_id, o)
                 return o
             else:
@@ -692,36 +650,62 @@ class Fory:
 
     def xread_no_ref(self, buffer, serializer=None):
         if serializer is None:
-            serializer = self.type_resolver.read_typeinfo(buffer).serializer
+            serializer = self.type_resolver.read_type_info(buffer).serializer
+        # Push -1 to read_ref_ids so reference() can pop it and skip reference tracking
+        # This handles the case where xread_no_ref is called directly without xread_ref
+        if self.track_ref:
+            self.ref_resolver.read_ref_ids.append(-1)
+        return self._xread_no_ref_internal(buffer, serializer)
+
+    def _xread_no_ref_internal(self, buffer, serializer):
+        """Internal method to read without pushing to read_ref_ids."""
+        if serializer is None:
+            serializer = self.type_resolver.read_type_info(buffer).serializer
         self.inc_depth()
         o = serializer.xread(buffer)
         self.dec_depth()
         return o
 
     def write_buffer_object(self, buffer, buffer_object: BufferObject):
-        if self._buffer_callback is None or self._buffer_callback(buffer_object):
+        if self.buffer_callback is None:
+            size = buffer_object.total_bytes()
+            # writer length.
+            buffer.write_var_uint32(size)
+            writer_index = buffer.get_writer_index()
+            buffer.ensure(writer_index + size)
+            buf = buffer.slice(writer_index, size)
+            buffer_object.write_to(buf)
+            buffer.set_writer_index(writer_index + size)
+            return
+        if self.buffer_callback(buffer_object):
             buffer.write_bool(True)
             size = buffer_object.total_bytes()
             # writer length.
-            buffer.write_varuint32(size)
-            writer_index = buffer.writer_index
+            buffer.write_var_uint32(size)
+            writer_index = buffer.get_writer_index()
             buffer.ensure(writer_index + size)
-            buf = buffer.slice(buffer.writer_index, size)
+            buf = buffer.slice(writer_index, size)
             buffer_object.write_to(buf)
-            buffer.writer_index += size
+            buffer.set_writer_index(writer_index + size)
         else:
             buffer.write_bool(False)
 
     def read_buffer_object(self, buffer) -> Buffer:
-        in_band = buffer.read_bool()
-        if in_band:
-            size = buffer.read_varuint32()
-            buf = buffer.slice(buffer.reader_index, size)
-            buffer.reader_index += size
+        if not self.is_peer_out_of_band_enabled:
+            size = buffer.read_var_uint32()
+            reader_index = buffer.get_reader_index()
+            buf = buffer.slice(reader_index, size)
+            buffer.set_reader_index(reader_index + size)
             return buf
-        else:
+        in_band = buffer.read_bool()
+        if not in_band:
             assert self._buffers is not None
             return next(self._buffers)
+        size = buffer.read_var_uint32()
+        reader_index = buffer.get_reader_index()
+        buf = buffer.slice(reader_index, size)
+        buffer.set_reader_index(reader_index + size)
+        return buf
 
     def handle_unsupported_write(self, buffer, obj):
         if self._unsupported_callback is None or self._unsupported_callback(obj):
@@ -735,8 +719,8 @@ class Fory:
         if self.ref_resolver.write_ref_or_null(buffer, value):
             return
         if typeinfo is None:
-            typeinfo = self.type_resolver.get_typeinfo(type(value))
-        self.type_resolver.write_typeinfo(buffer, typeinfo)
+            typeinfo = self.type_resolver.get_type_info(type(value))
+        self.type_resolver.write_type_info(buffer, typeinfo)
         typeinfo.serializer.write(buffer, value)
 
     def read_ref_pyobject(self, buffer):
@@ -754,7 +738,7 @@ class Fory:
         self.type_resolver.reset_write()
         self.serialization_context.reset_write()
         self.metastring_resolver.reset_write()
-        self._buffer_callback = None
+        self.buffer_callback = None
         self._unsupported_callback = None
 
     def reset_read(self):
@@ -771,6 +755,7 @@ class Fory:
         self.metastring_resolver.reset_write()
         self._buffers = None
         self._unsupported_objects = None
+        self.is_peer_out_of_band_enabled = False
 
     def reset(self):
         """
@@ -856,14 +841,15 @@ class ThreadSafeFory:
         - Both Python and Cython modes are supported automatically
     """
 
-    def __init__(self, **kwargs):
+    def __init__(self, fory_factory=None, **kwargs):
         import threading
 
         self._config = kwargs
+        self._fory_factory = fory_factory
         self._callbacks = []
         self._lock = threading.Lock()
         self._pool = []
-        self._fory_class = self._get_fory_class()
+        self._fory_class = None if fory_factory is not None else self._get_fory_class()
         self._instances_created = False
 
     def _get_fory_class(self):
@@ -883,7 +869,10 @@ class ThreadSafeFory:
             if self._pool:
                 return self._pool.pop()
             self._instances_created = True
-            fory = self._fory_class(**self._config)
+            if self._fory_factory is not None:
+                fory = self._fory_factory()
+            else:
+                fory = self._fory_class(**self._config)
             for callback in self._callbacks:
                 callback(fory)
             return fory
@@ -922,6 +911,17 @@ class ThreadSafeFory:
     ):
         self._register_callback(lambda f: f.register_type(cls, type_id=type_id, namespace=namespace, typename=typename, serializer=serializer))
 
+    def register_union(
+        self,
+        cls: Union[type, TypeVar],
+        *,
+        type_id: int = None,
+        namespace: str = None,
+        typename: str = None,
+        serializer=None,
+    ):
+        self._register_callback(lambda f: f.register_union(cls, type_id=type_id, namespace=namespace, typename=typename, serializer=serializer))
+
     def register_serializer(self, cls: type, serializer):
         self._register_callback(lambda f: f.register_serializer(cls, serializer))
 
