@@ -24,18 +24,24 @@ import os
 import typing
 
 import pyfory
-import pytest
+
+try:
+    import pytest
+except ImportError:
+    pytest = None
 from dataclasses import dataclass
-from pyfory.util import lazy_import
+from pyfory.utils import lazy_import
 from typing import List, Dict, Any
 
 
+import numpy as np
+
 pa = lazy_import("pyarrow")
 
 
 def debug_print(*params):
     """print params if debug is needed."""
-    # print(*params)
+    print(*params)
 
 
 def to_dict(obj):
@@ -52,10 +58,11 @@ def to_dict(obj):
 
 def cross_language_test(test_func):
     env_key = "ENABLE_CROSS_LANGUAGE_TESTS"
-    test_func = pytest.mark.skipif(
-        env_key not in os.environ,
-        reason=f"Pass {env_key} to enable cross-language tests",
-    )(test_func)
+    if pytest is not None:
+        test_func = pytest.mark.skipif(
+            env_key not in os.environ,
+            reason=f"Pass {env_key} to enable cross-language tests",
+        )(test_func)
     return test_func
 
 
@@ -263,7 +270,7 @@ def test_buffer(data_file_path):
         assert buffer.read_int64() == 2**63 - 1
         assert math.isclose(buffer.read_float(), -1.1, rel_tol=1e-03)
         assert math.isclose(buffer.read_double(), -1.1, rel_tol=1e-03)
-        assert buffer.read_varuint32() == 100
+        assert buffer.read_var_uint32() == 100
         binary = b"ab"
         assert buffer.read_bytes(buffer.read_int32()) == binary
         buffer.write_bool(True)
@@ -273,11 +280,11 @@ def test_buffer(data_file_path):
         buffer.write_int64(2**63 - 1)
         buffer.write_float(-1.1)
         buffer.write_double(-1.1)
-        buffer.write_varuint32(100)
+        buffer.write_var_uint32(100)
         buffer.write_int32(len(binary))
         buffer.write_bytes(binary)
     with open(data_file_path, "wb+") as f:
-        f.write(buffer.get_bytes(0, buffer.writer_index))
+        f.write(buffer.get_bytes(0, buffer.get_writer_index()))
 
 
 @cross_language_test
@@ -325,8 +332,6 @@ def test_cross_language_serializer(data_file_path):
         assert _deserialize_and_append(fory, buffer, objects) == set_
 
         # test primitive arrays
-        import numpy as np
-
         np.testing.assert_array_equal(
             _deserialize_and_append(fory, buffer, objects),
             np.array([True, False], dtype=np.bool_),
@@ -355,7 +360,7 @@ def test_cross_language_serializer(data_file_path):
         for obj in objects:
             fory.serialize(obj, buffer=new_buf)
     with open(data_file_path, "wb+") as f:
-        f.write(new_buf.get_bytes(0, new_buf.writer_index))
+        f.write(new_buf.get_bytes(0, new_buf.get_writer_index()))
 
 
 @cross_language_test
@@ -386,7 +391,7 @@ def test_cross_language_reference(data_file_path):
         new_buf = pyfory.Buffer.allocate(32)
         fory.serialize(new_list, buffer=new_buf)
     with open(data_file_path, "wb+") as f:
-        f.write(new_buf.get_bytes(0, new_buf.writer_index))
+        f.write(new_buf.get_bytes(0, new_buf.get_writer_index()))
 
 
 @dataclass
@@ -507,7 +512,7 @@ def test_struct_hash(data_file_path):
     fory = pyfory.Fory(xlang=True, ref=True)
     fory.register_type(ComplexObject1, typename="ComplexObject1")
     serializer = fory.type_resolver.get_serializer(ComplexObject1)._replace()
-    from pyfory._struct import compute_struct_meta
+    from pyfory.struct import compute_struct_meta
 
     v = compute_struct_meta(
         fory.type_resolver,
@@ -543,18 +548,65 @@ def test_serialize_complex_struct(data_file_path):
     struct_round_back(data_file_path, fory, obj1)
 
 
+def _deep_equal(left, right):
+    if np is not None:
+        if isinstance(left, np.ndarray) or isinstance(right, np.ndarray):
+            if isinstance(left, np.ndarray) and isinstance(right, np.ndarray):
+                return np.array_equal(left, right)
+            if isinstance(left, np.ndarray) and isinstance(right, array.array):
+                return np.array_equal(left, np.array(right, dtype=left.dtype))
+            if isinstance(right, np.ndarray) and isinstance(left, array.array):
+                return np.array_equal(right, np.array(left, dtype=right.dtype))
+    if isinstance(left, array.array) and isinstance(right, array.array):
+        return left == right
+    if hasattr(left, "__dataclass_fields__") and hasattr(right, "__dataclass_fields__"):
+        if left.__class__ is not right.__class__:
+            return False
+        for field in left.__dataclass_fields__.keys():
+            if not _deep_equal(getattr(left, field), getattr(right, field)):
+                return False
+        return True
+    if isinstance(left, list) and isinstance(right, list):
+        if len(left) != len(right):
+            return False
+        for l_item, r_item in zip(left, right):
+            if not _deep_equal(l_item, r_item):
+                return False
+        return True
+    if isinstance(left, tuple) and isinstance(right, tuple):
+        if len(left) != len(right):
+            return False
+        for l_item, r_item in zip(left, right):
+            if not _deep_equal(l_item, r_item):
+                return False
+        return True
+    if isinstance(left, dict) and isinstance(right, dict):
+        if left.keys() != right.keys():
+            return False
+        for key in left.keys():
+            if not _deep_equal(left[key], right[key]):
+                return False
+        return True
+    return left == right
+
+
+def _assert_deep_equal(left, right, msg_prefix=""):
+    assert _deep_equal(left, right), f"{msg_prefix}left {left}\n right {right}"
+
+
 def struct_round_back(data_file_path, fory, obj1):
     # new_buf = fory.serialize(obj1)
     # assert fory.deserialize(new_buf) == obj1
     with open(data_file_path, "rb") as f:
         data_bytes = f.read()
     debug_print(f"len {len(data_bytes)}")
+    debug_print(f"first 100 bytes: {data_bytes[:100].hex()}")
     new_obj = fory.deserialize(data_bytes)
     debug_print(new_obj)
-    assert new_obj == obj1, f"new_obj {new_obj}\n expected {obj1}"
+    _assert_deep_equal(new_obj, obj1, "struct_round_back mismatch: ")
     new_buf = fory.serialize(new_obj)
     debug_print(f"new_buf size {len(new_buf)}")
-    assert fory.deserialize(new_buf) == new_obj
+    _assert_deep_equal(fory.deserialize(new_buf), new_obj, "round-trip mismatch: ")
     with open(data_file_path, "wb+") as f:
         f.write(new_buf)
 
@@ -604,12 +656,12 @@ def test_register_serializer(data_file_path):
     fory.serialize(new_obj, buffer=new_buf)
     bytes1 = fory.serialize(new_obj)
     assert len(bytes1) == len(data_bytes), (bytes1, data_bytes)
-    # header can be different to embed writer info like language
+    # header can differ across writers
     assert bytes1[8:] == data_bytes[8:]
     assert fory.deserialize(fory.serialize(new_obj)) == new_obj, new_obj
     print(f"test_register_serializer: {new_obj}")
     with open(data_file_path, "wb+") as f:
-        f.write(new_buf.get_bytes(0, new_buf.writer_index))
+        f.write(new_buf.get_bytes(0, new_buf.get_writer_index()))
 
 
 @cross_language_test
@@ -623,9 +675,9 @@ def test_oob_buffer(in_band_file_path, out_of_band_file_path):
     buffers = []
     for i in range(n_buffers):
         length = out_of_band_buffer.read_int32()
-        reader_index = out_of_band_buffer.reader_index
+        reader_index = out_of_band_buffer.get_reader_index()
         buffers.append(out_of_band_buffer.slice(reader_index, length))
-        out_of_band_buffer.reader_index += length
+        out_of_band_buffer.set_reader_index(out_of_band_buffer.get_reader_index() + length)
     new_obj = fory.deserialize(in_band_bytes, buffers)
     obj = [bytes(bytearray([0, 1])) for _ in range(10)]
     assert new_obj == obj, (obj, new_obj)
@@ -655,7 +707,7 @@ def test_oob_buffer(in_band_file_path, out_of_band_file_path):
         out_of_band_buffer.write_int32(buffer_object.total_bytes())
         buffer_object.write_to(out_of_band_buffer)
     with open(out_of_band_file_path, "wb+") as f:
-        f.write(out_of_band_buffer.to_bytes(0, out_of_band_buffer.writer_index))
+        f.write(out_of_band_buffer.to_bytes(0, out_of_band_buffer.get_writer_index()))
 
 
 @cross_language_test
@@ -690,7 +742,7 @@ def test_cross_language_meta_share(data_file_path):
 
     # Verify round-trip
     round_trip_obj = fory.deserialize(new_serialized)
-    assert round_trip_obj == obj
+    _assert_deep_equal(round_trip_obj, obj, "meta share round-trip mismatch: ")
 
     # Write back for Java to verify
     with open(data_file_path, "wb") as f:
@@ -747,7 +799,7 @@ def test_cross_language_meta_share_complex(data_file_path):
 
     # Verify round-trip
     round_trip_obj = fory.deserialize(new_serialized)
-    assert round_trip_obj == obj
+    _assert_deep_equal(round_trip_obj, obj, "meta share complex round-trip mismatch: ")
 
     # Write back for Java to verify
     with open(data_file_path, "wb") as f:
