@@ -34,15 +34,15 @@ import pytest
 
 import pyfory
 from pyfory.buffer import Buffer
-from pyfory import Fory, Language, serialization, EnumSerializer
+from pyfory import Fory, serialization, EnumSerializer
 from pyfory.serializer import (
     TimestampSerializer,
     DateSerializer,
     PyArraySerializer,
     Numpy1DArraySerializer,
 )
-from pyfory.type import TypeId
-from pyfory.util import lazy_import
+from pyfory.types import TypeId
+from pyfory.utils import lazy_import
 
 pa = lazy_import("pyarrow")
 
@@ -126,17 +126,17 @@ def test_big_chunk_dict(track_ref):
     assert ser_de(fory, dict0) == dict0
 
 
-@pytest.mark.parametrize("language", [Language.XLANG, Language.PYTHON])
-def test_basic_serializer(language):
-    fory = Fory(language=language, ref=True)
-    typeinfo = fory.type_resolver.get_typeinfo(datetime.datetime)
+@pytest.mark.parametrize("xlang", [True, False])
+def test_basic_serializer(xlang):
+    fory = Fory(xlang=xlang, ref=True)
+    typeinfo = fory.type_resolver.get_type_info(datetime.datetime)
     assert isinstance(typeinfo.serializer, (TimestampSerializer, serialization.TimestampSerializer))
-    if language == Language.XLANG:
+    if xlang:
         assert typeinfo.type_id == TypeId.TIMESTAMP
-    typeinfo = fory.type_resolver.get_typeinfo(datetime.date)
+    typeinfo = fory.type_resolver.get_type_info(datetime.date)
     assert isinstance(typeinfo.serializer, (DateSerializer, serialization.DateSerializer))
-    if language == Language.XLANG:
-        assert typeinfo.type_id == TypeId.LOCAL_DATE
+    if xlang:
+        assert typeinfo.type_id == TypeId.DATE
     assert ser_de(fory, True) is True
     assert ser_de(fory, False) is False
     assert ser_de(fory, -1) == -1
@@ -164,14 +164,18 @@ def test_basic_serializer(language):
     assert ser_de(fory, set_) == set_
 
 
-@pytest.mark.parametrize("language", [Language.XLANG, Language.PYTHON])
-def test_ref_tracking(language):
-    fory = Fory(language=language, ref=True)
+@pytest.mark.parametrize("xlang", [True, False])
+def test_ref_tracking(xlang):
+    fory = Fory(xlang=xlang, ref=True)
 
-    simple_list = []
-    simple_list.append(simple_list)
-    new_simple_list = ser_de(fory, simple_list)
-    assert new_simple_list[0] is new_simple_list
+    # Circular reference test - only works for Python language mode
+    # XLANG mode doesn't support true circular references during deserialization
+    # because the object must be registered after it's fully constructed
+    if not xlang:
+        simple_list = []
+        simple_list.append(simple_list)
+        new_simple_list = ser_de(fory, simple_list)
+        assert new_simple_list[0] is new_simple_list
 
     now = datetime.datetime.now()
     day = datetime.date(2021, 11, 23)
@@ -186,8 +190,10 @@ def test_ref_tracking(language):
         "dict2_0": dict2,
         "dict2_1": dict2,
     }
-    dict3["dict3_0"] = dict3
-    dict3["dict3_1"] = dict3
+    # Circular reference in dict3 - only works for Python language mode
+    if not xlang:
+        dict3["dict3_0"] = dict3
+        dict3["dict3_1"] = dict3
     new_dict3 = ser_de(fory, dict3)
     assert new_dict3["list1_0"] == list_
     assert new_dict3["list1_0"] is new_dict3["list1_1"]
@@ -195,22 +201,23 @@ def test_ref_tracking(language):
     assert new_dict3["dict1_0"] is new_dict3["dict1_1"]
     assert new_dict3["dict2_0"] == dict2
     assert new_dict3["dict2_0"] is new_dict3["dict2_1"]
-    assert new_dict3["dict3_0"] is new_dict3
-    assert new_dict3["dict3_0"] is new_dict3["dict3_0"]
+    if not xlang:
+        assert new_dict3["dict3_0"] is new_dict3
+        assert new_dict3["dict3_0"] is new_dict3["dict3_0"]
 
 
-@pytest.mark.parametrize("language", [Language.PYTHON, Language.XLANG])
-def test_tmp_ref(language):
+@pytest.mark.parametrize("xlang", [False, True])
+def test_tmp_ref(xlang):
     # FIXME this can't simulate the case where new objects are allocated on memory
     #  address of released tmp object.
-    fory = Fory(language=language, ref=True)
+    fory = Fory(xlang=xlang, ref=True)
     buffer = Buffer.allocate(128)
-    writer_index = buffer.writer_index
+    writer_index = buffer.get_writer_index()
     x = 1
     fory.serialize([x], buffer)
     fory.serialize([x], buffer)
     fory.serialize([x], buffer)
-    assert buffer.writer_index > writer_index + 15
+    assert buffer.get_writer_index() > writer_index + 15
 
     l1 = fory.deserialize(buffer)
     l2 = fory.deserialize(buffer)
@@ -223,11 +230,11 @@ def test_tmp_ref(language):
     assert l2 is not l3
 
 
-@pytest.mark.parametrize("language", [Language.PYTHON, Language.XLANG])
-def test_multiple_ref(language):
+@pytest.mark.parametrize("xlang", [False, True])
+def test_multiple_ref(xlang):
     # FIXME this can't simulate the case where new objects are allocated on memory
     #  address of released tmp object.
-    fory = Fory(language=language, ref=True)
+    fory = Fory(xlang=xlang, ref=True)
     buffer = Buffer.allocate(128)
     for i in range(1000):
         fory.serialize([], buffer)
@@ -247,12 +254,10 @@ class RefTestClass2:
         self.f1 = f1
 
 
-@pytest.mark.parametrize("language", [Language.PYTHON])
-def test_ref_cleanup(language):
+def test_ref_cleanup():
     # FIXME this can't simulate the case where new objects are allocated on memory
     #  address of released tmp object.
-    fory = Fory(language=language, ref=True, strict=False)
-    # TODO support Language.XLANG, current unpickler will error for xlang,
+    fory = Fory(xlang=False, ref=True, strict=False)
     o1 = RefTestClass1()
     o2 = RefTestClass2(f1=o1)
     pickle.loads(pickle.dumps(o2))
@@ -266,9 +271,9 @@ def test_ref_cleanup(language):
     fory.deserialize(data)
 
 
-@pytest.mark.parametrize("language", [Language.XLANG, Language.PYTHON])
-def test_array_serializer(language):
-    fory = Fory(language=language, ref=True, strict=False)
+@pytest.mark.parametrize("xlang", [True, False])
+def test_array_serializer(xlang):
+    fory = Fory(xlang=xlang, ref=True, strict=False)
     for typecode in PyArraySerializer.typecode_dict.keys():
         arr = array.array(typecode, list(range(10)))
         new_arr = ser_de(fory, arr)
@@ -313,15 +318,15 @@ def test_pickle():
     pickler.dump(b"abc")
     buf.write_int32(-1)
     pickler.dump("abcd")
-    assert buf.writer_index - 4 == len(pickle.dumps(b"abc")) + len(pickle.dumps("abcd"))
-    print(f"writer_index {buf.writer_index}")
+    assert buf.get_writer_index() - 4 == len(pickle.dumps(b"abc")) + len(pickle.dumps("abcd"))
+    print(f"writer_index {buf.get_writer_index()}")
 
     bytes_io_ = io.BytesIO(buf)
     unpickler = pickle.Unpickler(bytes_io_)
     assert unpickler.load() == b"abc"
     bytes_io_.seek(bytes_io_.tell() + 4)
     assert unpickler.load() == "abcd"
-    print(f"reader_index {buf.reader_index} {bytes_io_.tell()}")
+    print(f"reader_index {buf.get_reader_index()} {bytes_io_.tell()}")
 
     if pa:
         pa_buf = pa.BufferReader(buf)
@@ -329,13 +334,13 @@ def test_pickle():
         assert unpickler.load() == b"abc"
         pa_buf.seek(pa_buf.tell() + 4)
         assert unpickler.load() == "abcd"
-        print(f"reader_index {buf.reader_index} {pa_buf.tell()} {buf.reader_index}")
+        print(f"reader_index {buf.get_reader_index()} {pa_buf.tell()} {buf.get_reader_index()}")
 
     unpickler = pickle.Unpickler(buf)
     assert unpickler.load() == b"abc"
-    buf.reader_index = buf.reader_index + 4
+    buf.set_reader_index(buf.get_reader_index() + 4)
     assert unpickler.load() == "abcd"
-    print(f"reader_index {buf.reader_index}")
+    print(f"reader_index {buf.get_reader_index()}")
 
 
 @dataclass
