@@ -29,11 +29,25 @@
 
 use std::borrow::Cow;
 
+use crate::types::format_type_id;
 use thiserror::Error;
 
 /// Global flag to check if FORY_PANIC_ON_ERROR environment variable is set at compile time.
-/// Set FORY_PANIC_ON_ERROR=1 at compile time to enable panic on error.
-pub const PANIC_ON_ERROR: bool = option_env!("FORY_PANIC_ON_ERROR").is_some();
+/// Set FORY_PANIC_ON_ERROR=1 (or true) at compile time to enable panic on error.
+const fn is_truthy_env(value: &str) -> bool {
+    let bytes = value.as_bytes();
+    (bytes.len() == 1 && bytes[0] == b'1')
+        || (bytes.len() == 4
+            && (bytes[0] | 0x20) == b't'
+            && (bytes[1] | 0x20) == b'r'
+            && (bytes[2] | 0x20) == b'u'
+            && (bytes[3] | 0x20) == b'e')
+}
+
+pub const PANIC_ON_ERROR: bool = match option_env!("FORY_PANIC_ON_ERROR") {
+    Some(value) => is_truthy_env(value),
+    None => false,
+};
 
 /// Check if FORY_PANIC_ON_ERROR environment variable is set.
 #[inline(always)]
@@ -108,8 +122,8 @@ pub enum Error {
     /// Type mismatch between local and remote type IDs.
     ///
     /// Do not construct this variant directly; use [`Error::type_mismatch`] instead.
-    #[error("Type mismatch: type_a = {0}, type_b = {1}")]
-    TypeMismatch(u32, u32),
+    #[error("{0}")]
+    TypeMismatch(Cow<'static, str>),
 
     /// Buffer boundary violation during read/write operations.
     ///
@@ -187,6 +201,9 @@ pub enum Error {
 impl Error {
     /// Creates a new [`Error::TypeMismatch`] with the given type IDs.
     ///
+    /// The error message will display both the original registered ID and the internal type name
+    /// for user-registered types, making debugging easier.
+    ///
     /// If `FORY_PANIC_ON_ERROR` environment variable is set, this will panic with the error message.
     ///
     /// # Example
@@ -199,7 +216,12 @@ impl Error {
     #[cold]
     #[track_caller]
     pub fn type_mismatch(type_a: u32, type_b: u32) -> Self {
-        let err = Error::TypeMismatch(type_a, type_b);
+        let msg = format!(
+            "Type mismatch: local {} vs remote {}",
+            format_type_id(type_a),
+            format_type_id(type_b)
+        );
+        let err = Error::TypeMismatch(Cow::Owned(msg));
         if PANIC_ON_ERROR {
             panic!("FORY_PANIC_ON_ERROR: {}", err);
         }
