@@ -23,11 +23,8 @@ use crate::resolver::context::{ContextCache, ReadContext, WriteContext};
 use crate::resolver::type_resolver::TypeResolver;
 use crate::serializer::ForyDefault;
 use crate::serializer::{Serializer, StructSerializer};
-use crate::types::config_flags::IS_NULL_FLAG;
-use crate::types::{
-    config_flags::{IS_CROSS_LANGUAGE_FLAG, IS_LITTLE_ENDIAN_FLAG},
-    Language, MAGIC_NUMBER, SIZE_OF_REF_AND_TYPE,
-};
+use crate::types::config_flags::{IS_CROSS_LANGUAGE_FLAG, IS_NULL_FLAG};
+use crate::types::{RefMode, SIZE_OF_REF_AND_TYPE};
 use std::cell::UnsafeCell;
 use std::mem;
 use std::sync::atomic::{AtomicU64, Ordering};
@@ -153,9 +150,8 @@ impl Fory {
     ///
     /// # Arguments
     ///
-    /// * `xlang` - If `true`, uses the cross-language serialization format that includes
-    ///   language metadata and magic numbers for compatibility with Fory implementations
-    ///   in other languages (Java, Python, C++, etc.). If `false`, uses a Rust-only
+    /// * `xlang` - If `true`, uses the cross-language serialization format compatible with
+    ///   other Fory implementations (Java, Python, C++, etc.). If `false`, uses a Rust-only
     ///   optimized format.
     ///
     /// # Returns
@@ -260,6 +256,34 @@ impl Fory {
         self
     }
 
+    /// Enables or disables reference tracking for shared and circular references.
+    ///
+    /// # Arguments
+    ///
+    /// * `track_ref` - If `true`, enables reference tracking which allows
+    ///   preserving shared object references and circular references during
+    ///   serialization/deserialization.
+    ///
+    /// # Returns
+    ///
+    /// Returns `self` for method chaining.
+    ///
+    /// # Default
+    ///
+    /// The default value is `false`.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use fory_core::Fory;
+    ///
+    /// let fory = Fory::default().track_ref(true);
+    /// ```
+    pub fn track_ref(mut self, track_ref: bool) -> Self {
+        self.config.track_ref = track_ref;
+        self
+    }
+
     /// Sets the maximum depth for nested dynamic object serialization.
     ///
     /// # Arguments
@@ -585,15 +609,17 @@ impl Fory {
     ) -> Result<(), Error> {
         let is_none = record.fory_is_none();
         self.write_head::<T>(is_none, &mut context.writer);
-        let meta_start_offset = context.writer.len();
         if !is_none {
-            if context.is_compatible() {
-                context.writer.write_i32(-1);
+            // Use RefMode based on config:
+            // - If track_ref is enabled, use RefMode::Tracking for the root object
+            // - Otherwise, use RefMode::NullOnly which writes NOT_NULL_VALUE_FLAG
+            let ref_mode = if self.config.track_ref {
+                RefMode::Tracking
+            } else {
+                RefMode::NullOnly
             };
-            <T as Serializer>::fory_write(record, context, true, true, false)?;
-            if context.is_compatible() && !context.empty() {
-                context.write_meta(meta_start_offset);
-            }
+            // TypeMeta is written inline during serialization (streaming protocol)
+            <T as Serializer>::fory_write(record, context, ref_mode, true, false)?;
         }
         Ok(())
     }
@@ -632,6 +658,16 @@ impl Fory {
         self.type_resolver.register_by_id::<T>(id)
     }
 
+    /// Register a union type with a numeric type ID.
+    ///
+    /// This is intended for union-compatible enums generated by the compiler.
+    pub fn register_union<T: 'static + StructSerializer + Serializer + ForyDefault>(
+        &mut self,
+        id: u32,
+    ) -> Result<(), Error> {
+        self.type_resolver.register_union_by_id::<T>(id)
+    }
+
     /// Registers a struct type with a namespace and type name for cross-language serialization.
     ///
     /// # Type Parameters
@@ -671,6 +707,18 @@ impl Fory {
             .register_by_namespace::<T>(namespace, type_name)
     }
 
+    /// Register a union type with namespace and type name.
+    ///
+    /// This is intended for union-compatible enums generated by the compiler.
+    pub fn register_union_by_namespace<T: 'static + StructSerializer + Serializer + ForyDefault>(
+        &mut self,
+        namespace: &str,
+        type_name: &str,
+    ) -> Result<(), Error> {
+        self.type_resolver
+            .register_union_by_namespace::<T>(namespace, type_name)
+    }
+
     /// Registers a struct type with a type name (using the default namespace).
     ///
     /// # Type Parameters
@@ -704,6 +752,14 @@ impl Fory {
         self.register_by_namespace::<T>("", type_name)
     }
 
+    /// Register a union type with type name only (no namespace).
+    pub fn register_union_by_name<T: 'static + StructSerializer + Serializer + ForyDefault>(
+        &mut self,
+        type_name: &str,
+    ) -> Result<(), Error> {
+        self.register_union_by_namespace::<T>("", type_name)
+    }
+
     /// Registers a custom serializer type with a numeric type ID.
     ///
     /// # Type Parameters
@@ -797,13 +853,7 @@ impl Fory {
     pub fn write_head<T: Serializer>(&self, is_none: bool, writer: &mut Writer) {
         const HEAD_SIZE: usize = 10;
         writer.reserve(T::fory_reserved_space() + SIZE_OF_REF_AND_TYPE + HEAD_SIZE);
-        if self.config.xlang {
-            writer.write_u16(MAGIC_NUMBER);
-        }
-        #[cfg(target_endian = "big")]
-        let mut bitmap = 0;
-        #[cfg(target_endian = "little")]
-        let mut bitmap = IS_LITTLE_ENDIAN_FLAG;
+        let mut bitmap: u8 = 0;
         if self.config.xlang {
             bitmap |= IS_CROSS_LANGUAGE_FLAG;
         }
@@ -811,12 +861,6 @@ impl Fory {
             bitmap |= IS_NULL_FLAG;
         }
         writer.write_u8(bitmap);
-        if is_none {
-            return;
-        }
-        if self.config.xlang {
-            writer.write_u8(Language::Rust as u8);
-        }
     }
 
     /// Deserializes data from a byte slice into a value of type `T`.
@@ -970,55 +1014,32 @@ impl Fory {
         if is_none {
             return Ok(T::fory_default());
         }
-        let mut bytes_to_skip = 0;
-        if context.is_compatible() {
-            let meta_offset = context.reader.read_i32()?;
-            if meta_offset != -1 {
-                bytes_to_skip = context.load_type_meta(meta_offset as usize)?;
-            }
-        }
-        let result = <T as Serializer>::fory_read(context, true, true);
-        if bytes_to_skip > 0 {
-            context.reader.skip(bytes_to_skip)?;
-        }
+        // Use RefMode based on config:
+        // - If track_ref is enabled, use RefMode::Tracking for the root object
+        // - Otherwise, use RefMode::NullOnly
+        let ref_mode = if self.config.track_ref {
+            RefMode::Tracking
+        } else {
+            RefMode::NullOnly
+        };
+        // TypeMeta is read inline during deserialization (streaming protocol)
+        let result = <T as Serializer>::fory_read(context, ref_mode, true);
         context.ref_reader.resolve_callbacks();
         result
     }
 
     #[inline(always)]
     fn read_head(&self, reader: &mut Reader) -> Result<bool, Error> {
-        if self.config.xlang {
-            let magic_numer = reader.read_u16()?;
-            ensure!(
-                magic_numer == MAGIC_NUMBER,
-                Error::invalid_data(format!(
-                    "The fory xlang serialization must start with magic number {:X}. \
-                    Please check whether the serialization is based on the xlang protocol \
-                    and the data didn't corrupt.",
-                    MAGIC_NUMBER
-                ))
-            )
-        }
         let bitmap = reader.read_u8()?;
         let peer_is_xlang = (bitmap & IS_CROSS_LANGUAGE_FLAG) != 0;
         ensure!(
             self.config.xlang == peer_is_xlang,
             Error::invalid_data("header bitmap mismatch at xlang bit")
         );
-        let is_little_endian = (bitmap & IS_LITTLE_ENDIAN_FLAG) != 0;
-        ensure!(
-            is_little_endian,
-            Error::invalid_data(
-                "Big endian is not supported for now, please ensure peer machine is little endian."
-            )
-        );
         let is_none = (bitmap & IS_NULL_FLAG) != 0;
         if is_none {
             return Ok(true);
         }
-        if peer_is_xlang {
-            let _peer_lang = reader.read_u8()?;
-        }
         Ok(false)
     }
 }
