@@ -198,4 +198,4 @@ pub use crate::resolver::context::{ReadContext, WriteContext};
 pub use crate::resolver::type_resolver::{TypeInfo, TypeResolver};
 pub use crate::serializer::weak::{ArcWeak, RcWeak};
 pub use crate::serializer::{read_data, write_data, ForyDefault, Serializer, StructSerializer};
-pub use crate::types::{RefFlag, TypeId};
+pub use crate::types::{RefFlag, RefMode, TypeId};
