@@ -95,7 +95,7 @@ impl MetaString {
     }
 
     pub fn write_to(&self, writer: &mut crate::buffer::Writer) {
-        writer.write_varuint32(self.bytes.len() as u32);
+        writer.write_var_uint32(self.bytes.len() as u32);
         writer.write_bytes(&self.bytes);
     }
 
