@@ -27,6 +27,7 @@ use crate::resolver::meta_string_resolver::{MetaStringReaderResolver, MetaString
 use crate::resolver::ref_resolver::{RefReader, RefWriter};
 use crate::resolver::type_resolver::{TypeInfo, TypeResolver};
 use crate::types;
+use crate::TypeId;
 use std::rc::Rc;
 
 /// Thread-local context cache with fast path for single Fory instance.
@@ -120,6 +121,7 @@ pub struct WriteContext<'a> {
     compress_string: bool,
     xlang: bool,
     check_struct_version: bool,
+    track_ref: bool,
 
     // Context-specific fields
     default_writer: Option<Writer<'a>>,
@@ -139,6 +141,7 @@ impl<'a> WriteContext<'a> {
             compress_string: config.compress_string,
             xlang: config.xlang,
             check_struct_version: config.check_struct_version,
+            track_ref: config.track_ref,
             default_writer: None,
             writer: Writer::from_buffer(Self::get_leak_buffer()),
             meta_resolver: MetaWriterResolver::default(),
@@ -205,31 +208,27 @@ impl<'a> WriteContext<'a> {
         self.check_struct_version
     }
 
+    /// Check if reference tracking is enabled
     #[inline(always)]
-    pub fn empty(&mut self) -> bool {
-        self.meta_resolver.empty()
+    pub fn is_track_ref(&self) -> bool {
+        self.track_ref
     }
 
+    /// Write type meta inline using streaming protocol.
+    /// Writes index marker with LSB indicating new type or reference.
     #[inline(always)]
-    pub fn push_meta(&mut self, type_id: std::any::TypeId) -> Result<usize, Error> {
-        self.meta_resolver.push(type_id, &self.type_resolver)
+    pub fn write_type_meta(&mut self, type_id: std::any::TypeId) -> Result<(), Error> {
+        self.meta_resolver
+            .write_type_meta(&mut self.writer, type_id, &self.type_resolver)
     }
 
-    #[inline(always)]
-    pub fn write_meta(&mut self, offset: usize) {
-        let len = self.writer.len();
-        self.writer
-            .set_bytes(offset, &((len - offset - 4) as u32).to_le_bytes());
-        self.meta_resolver.to_bytes(&mut self.writer);
-    }
-
-    pub fn write_any_typeinfo(
+    pub fn write_any_type_info(
         &mut self,
         fory_type_id: u32,
         concrete_type_id: std::any::TypeId,
     ) -> Result<Rc<TypeInfo>, Error> {
         if types::is_internal_type(fory_type_id) {
-            self.writer.write_varuint32(fory_type_id);
+            self.writer.write_u8(fory_type_id as u8);
             return self
                 .type_resolver
                 .get_type_info_by_id(fory_type_id)
@@ -239,22 +238,29 @@ impl<'a> WriteContext<'a> {
         let fory_type_id = type_info.get_type_id();
         let namespace = type_info.get_namespace();
         let type_name = type_info.get_type_name();
-        self.writer.write_varuint32(fory_type_id);
+        self.writer.write_u8(fory_type_id as u8);
         // should be compiled to jump table generation
-        match fory_type_id & 0xff {
-            types::NAMED_COMPATIBLE_STRUCT | types::COMPATIBLE_STRUCT => {
-                let meta_index =
-                    self.meta_resolver
-                        .push(concrete_type_id, &self.type_resolver)? as u32;
-                self.writer.write_varuint32(meta_index);
+        match fory_type_id {
+            TypeId::ENUM | TypeId::STRUCT | TypeId::EXT | TypeId::TYPED_UNION => {
+                let user_type_id = type_info.get_user_type_id();
+                self.writer.write_var_uint32(user_type_id);
             }
-            types::NAMED_ENUM | types::NAMED_EXT | types::NAMED_STRUCT => {
+            TypeId::COMPATIBLE_STRUCT | TypeId::NAMED_COMPATIBLE_STRUCT => {
+                // Write type meta inline using streaming protocol
+                self.meta_resolver.write_type_meta(
+                    &mut self.writer,
+                    concrete_type_id,
+                    &self.type_resolver,
+                )?;
+            }
+            TypeId::NAMED_ENUM | TypeId::NAMED_EXT | TypeId::NAMED_STRUCT | TypeId::NAMED_UNION => {
                 if self.is_share_meta() {
-                    let meta_index = self
-                        .meta_resolver
-                        .push(concrete_type_id, &self.type_resolver)?
-                        as u32;
-                    self.writer.write_varuint32(meta_index);
+                    // Write type meta inline using streaming protocol
+                    self.meta_resolver.write_type_meta(
+                        &mut self.writer,
+                        concrete_type_id,
+                        &self.type_resolver,
+                    )?;
                 } else {
                     self.write_meta_string_bytes(namespace)?;
                     self.write_meta_string_bytes(type_name)?;
@@ -402,36 +408,51 @@ impl<'a> ReadContext<'a> {
         self.get_type_info_by_index(type_index)
     }
 
+    /// Read type meta inline using streaming protocol.
+    /// Returns the TypeInfo for this type.
     #[inline(always)]
-    pub fn load_type_meta(&mut self, offset: usize) -> Result<usize, Error> {
-        self.meta_resolver.load(
-            &self.type_resolver,
-            &mut Reader::new(&self.reader.slice_after_cursor()[offset..]),
-        )
+    pub fn read_type_meta(&mut self) -> Result<Rc<TypeInfo>, Error> {
+        self.meta_resolver
+            .read_type_meta(&mut self.reader, &self.type_resolver)
     }
 
-    pub fn read_any_typeinfo(&mut self) -> Result<Rc<TypeInfo>, Error> {
-        let fory_type_id = self.reader.read_varuint32()?;
+    pub fn read_any_type_info(&mut self) -> Result<Rc<TypeInfo>, Error> {
+        let fory_type_id = self.reader.read_u8()? as u32;
         // should be compiled to jump table generation
-        match fory_type_id & 0xff {
-            types::NAMED_COMPATIBLE_STRUCT | types::COMPATIBLE_STRUCT => {
-                let meta_index = self.reader.read_varuint32()? as usize;
-                let type_info = self.get_type_info_by_index(meta_index)?.clone();
-                Ok(type_info)
+        match fory_type_id {
+            types::ENUM | types::STRUCT | types::EXT | types::TYPED_UNION => {
+                let user_type_id = self.reader.read_varuint32()?;
+                self.type_resolver
+                    .get_user_type_info_by_id(user_type_id)
+                    .ok_or_else(|| Error::type_error("ID harness not found"))
             }
-            types::NAMED_ENUM | types::NAMED_EXT | types::NAMED_STRUCT => {
+            types::COMPATIBLE_STRUCT | types::NAMED_COMPATIBLE_STRUCT => {
+                // Read type meta inline using streaming protocol
+                self.read_type_meta()
+            }
+            types::NAMED_ENUM | types::NAMED_EXT | types::NAMED_STRUCT | types::NAMED_UNION => {
                 if self.is_share_meta() {
-                    let meta_index = self.reader.read_varuint32()? as usize;
-                    let type_info = self.get_type_info_by_index(meta_index)?.clone();
-                    Ok(type_info)
+                    // Read type meta inline using streaming protocol
+                    self.read_type_meta()
                 } else {
                     let namespace = self.read_meta_string()?.to_owned();
                     let type_name = self.read_meta_string()?.to_owned();
-                    let rc_namespace = Rc::from(namespace);
-                    let rc_type_name = Rc::from(type_name);
+                    let rc_namespace = Rc::from(namespace.clone());
+                    let rc_type_name = Rc::from(type_name.clone());
                     self.type_resolver
                         .get_type_info_by_meta_string_name(rc_namespace, rc_type_name)
-                        .ok_or_else(|| Error::type_error("Name harness not found"))
+                        .or_else(|| {
+                            self.type_resolver.get_type_info_by_name(
+                                namespace.original.as_str(),
+                                type_name.original.as_str(),
+                            )
+                        })
+                        .ok_or_else(|| {
+                            Error::type_error(format!(
+                                "Name harness not found: namespace='{}', type='{}'",
+                                namespace.original, type_name.original
+                            ))
+                        })
                 }
             }
             _ => self
