@@ -21,20 +21,28 @@ use crate::resolver::context::{ReadContext, WriteContext};
 use crate::resolver::type_resolver::{TypeInfo, TypeResolver};
 use crate::serializer::util::write_dyn_data_generic;
 use crate::serializer::{ForyDefault, Serializer};
-use crate::types::RefFlag;
-use crate::types::TypeId;
-use crate::types::{LIST, MAP, SET};
+use crate::types::{RefFlag, RefMode, TypeId};
 use std::any::Any;
 use std::rc::Rc;
 use std::sync::Arc;
 
+#[inline(always)]
+fn resolve_registered_type_id(
+    type_resolver: &TypeResolver,
+    concrete_type_id: std::any::TypeId,
+) -> Result<TypeId, Error> {
+    type_resolver
+        .get_fory_type_id(concrete_type_id)
+        .ok_or_else(|| Error::type_error("Type not registered"))
+}
+
 /// Check if the type info represents a generic container type (LIST, SET, MAP).
 /// These types cannot be deserialized polymorphically via `Box<dyn Any>` because
 /// different generic instantiations (e.g., `Vec<A>`, `Vec<B>`) share the same type ID.
 #[inline]
 fn check_generic_container_type(type_info: &TypeInfo) -> Result<(), Error> {
     let type_id = type_info.get_type_id();
-    if type_id == LIST || type_id == SET || type_id == MAP {
+    if type_id == TypeId::LIST || type_id == TypeId::SET || type_id == TypeId::MAP {
         return Err(Error::type_error(
             "Cannot deserialize generic container types (Vec, HashSet, HashMap) polymorphically \
             via Box/Rc/Arc/Weak<dyn Any>. The serialization protocol does not preserve the element type \
@@ -53,7 +61,7 @@ pub fn deserialize_any_box(context: &mut ReadContext) -> Result<Box<dyn Any>, Er
     if ref_flag != RefFlag::NotNullValue as i8 {
         return Err(Error::invalid_ref("Expected NotNullValue for Box<dyn Any>"));
     }
-    let typeinfo = context.read_any_typeinfo()?;
+    let typeinfo = context.read_any_type_info()?;
     // Check for generic container types which cannot be deserialized polymorphically
     check_generic_container_type(&typeinfo)?;
     let result = typeinfo
@@ -73,15 +81,15 @@ impl Serializer for Box<dyn Any> {
     fn fory_write(
         &self,
         context: &mut WriteContext,
-        write_ref_info: bool,
-        write_typeinfo: bool,
+        ref_mode: RefMode,
+        write_type_info: bool,
         has_generics: bool,
     ) -> Result<(), Error> {
         write_box_any(
             self.as_ref(),
             context,
-            write_ref_info,
-            write_typeinfo,
+            ref_mode,
+            write_type_info,
             has_generics,
         )
     }
@@ -103,21 +111,21 @@ impl Serializer for Box<dyn Any> {
 
     fn fory_read(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         read_type_info: bool,
     ) -> Result<Self, Error> {
-        read_box_any(context, read_ref_info, read_type_info, None)
+        read_box_any(context, ref_mode, read_type_info, None)
     }
 
     fn fory_read_with_type_info(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         type_info: Rc<TypeInfo>,
     ) -> Result<Self, Error>
     where
         Self: Sized + ForyDefault,
     {
-        read_box_any(context, read_ref_info, false, Some(type_info))
+        read_box_any(context, ref_mode, false, Some(type_info))
     }
 
     fn fory_read_data(_: &mut ReadContext) -> Result<Self, Error> {
@@ -126,17 +134,14 @@ impl Serializer for Box<dyn Any> {
         ))
     }
 
-    fn fory_get_type_id(_: &TypeResolver) -> Result<u32, Error> {
+    fn fory_get_type_id(_: &TypeResolver) -> Result<TypeId, Error> {
         Err(Error::type_error(
             "Box<dyn Any> has no static type ID - use fory_type_id_dyn",
         ))
     }
 
-    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<u32, Error> {
-        let concrete_type_id = (**self).type_id();
-        type_resolver
-            .get_fory_type_id(concrete_type_id)
-            .ok_or_else(|| Error::type_error("Type not registered"))
+    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<TypeId, Error> {
+        resolve_registered_type_id(type_resolver, (**self).type_id())
     }
 
     fn fory_concrete_type_id(&self) -> std::any::TypeId {
@@ -173,16 +178,16 @@ impl Serializer for Box<dyn Any> {
 pub fn write_box_any(
     value: &dyn Any,
     context: &mut WriteContext,
-    write_ref_info: bool,
-    write_typeinfo: bool,
+    ref_mode: RefMode,
+    write_type_info: bool,
     has_generics: bool,
 ) -> Result<(), Error> {
-    if write_ref_info {
+    if ref_mode != RefMode::None {
         context.writer.write_i8(RefFlag::NotNullValue as i8);
     }
     let concrete_type_id = value.type_id();
-    let typeinfo = if write_typeinfo {
-        context.write_any_typeinfo(TypeId::UNKNOWN as u32, concrete_type_id)?
+    let typeinfo = if write_type_info {
+        context.write_any_type_info(TypeId::UNKNOWN as u32, concrete_type_id)?
     } else {
         context.get_type_info(&concrete_type_id)?
     };
@@ -192,12 +197,12 @@ pub fn write_box_any(
 
 pub fn read_box_any(
     context: &mut ReadContext,
-    read_ref_info: bool,
+    ref_mode: RefMode,
     read_type_info: bool,
     type_info: Option<Rc<TypeInfo>>,
 ) -> Result<Box<dyn Any>, Error> {
     context.inc_depth()?;
-    let ref_flag = if read_ref_info {
+    let ref_flag = if ref_mode != RefMode::None {
         context.reader.read_i8()?
     } else {
         RefFlag::NotNullValue as i8
@@ -214,7 +219,7 @@ pub fn read_box_any(
             read_type_info,
             Error::invalid_data("Type info must be read for Box<dyn Any>")
         );
-        context.read_any_typeinfo()?
+        context.read_any_type_info()?
     };
     // Check for generic container types which cannot be deserialized polymorphically
     check_generic_container_type(&typeinfo)?;
@@ -235,11 +240,11 @@ impl Serializer for Rc<dyn Any> {
     fn fory_write(
         &self,
         context: &mut WriteContext,
-        write_ref_info: bool,
+        ref_mode: RefMode,
         write_type_info: bool,
         has_generics: bool,
     ) -> Result<(), Error> {
-        if !write_ref_info
+        if ref_mode == RefMode::None
             || !context
                 .ref_writer
                 .try_write_rc_ref(&mut context.writer, self)
@@ -247,7 +252,7 @@ impl Serializer for Rc<dyn Any> {
             let concrete_type_id: std::any::TypeId = (**self).type_id();
             let write_data_fn = if write_type_info {
                 let typeinfo =
-                    context.write_any_typeinfo(TypeId::UNKNOWN as u32, concrete_type_id)?;
+                    context.write_any_type_info(TypeId::UNKNOWN as u32, concrete_type_id)?;
                 typeinfo.get_harness().get_write_data_fn()
             } else {
                 context
@@ -274,21 +279,21 @@ impl Serializer for Rc<dyn Any> {
 
     fn fory_read(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         read_type_info: bool,
     ) -> Result<Self, Error> {
-        read_rc_any(context, read_ref_info, read_type_info, None)
+        read_rc_any(context, ref_mode, read_type_info, None)
     }
 
     fn fory_read_with_type_info(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         type_info: Rc<TypeInfo>,
     ) -> Result<Self, Error>
     where
         Self: Sized + ForyDefault,
     {
-        read_rc_any(context, read_ref_info, false, Some(type_info))
+        read_rc_any(context, ref_mode, false, Some(type_info))
     }
 
     fn fory_read_data(_: &mut ReadContext) -> Result<Self, Error> {
@@ -298,17 +303,14 @@ impl Serializer for Rc<dyn Any> {
         )))
     }
 
-    fn fory_get_type_id(_: &TypeResolver) -> Result<u32, Error> {
+    fn fory_get_type_id(_: &TypeResolver) -> Result<TypeId, Error> {
         Err(Error::type_error(
             "Rc<dyn Any> has no static type ID - use fory_type_id_dyn",
         ))
     }
 
-    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<u32, Error> {
-        let concrete_type_id = (**self).type_id();
-        type_resolver
-            .get_fory_type_id(concrete_type_id)
-            .ok_or_else(|| Error::type_error("Type not registered"))
+    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<TypeId, Error> {
+        resolve_registered_type_id(type_resolver, (**self).type_id())
     }
 
     fn fory_concrete_type_id(&self) -> std::any::TypeId {
@@ -344,11 +346,11 @@ impl Serializer for Rc<dyn Any> {
 
 pub fn read_rc_any(
     context: &mut ReadContext,
-    read_ref_info: bool,
+    ref_mode: RefMode,
     read_type_info: bool,
     type_info: Option<Rc<TypeInfo>>,
 ) -> Result<Rc<dyn Any>, Error> {
-    let ref_flag = if read_ref_info {
+    let ref_flag = if ref_mode != RefMode::None {
         context.ref_reader.read_ref_flag(&mut context.reader)?
     } else {
         RefFlag::NotNullValue
@@ -367,7 +369,7 @@ pub fn read_rc_any(
         RefFlag::NotNullValue => {
             context.inc_depth()?;
             let typeinfo = if read_type_info {
-                context.read_any_typeinfo()?
+                context.read_any_type_info()?
             } else {
                 type_info.ok_or_else(|| Error::type_error("No type info found for read"))?
             };
@@ -382,7 +384,7 @@ pub fn read_rc_any(
         RefFlag::RefValue => {
             context.inc_depth()?;
             let typeinfo = if read_type_info {
-                context.read_any_typeinfo()?
+                context.read_any_type_info()?
             } else {
                 type_info.ok_or_else(|| Error::type_error("No type info found for read"))?
             };
@@ -409,11 +411,11 @@ impl Serializer for Arc<dyn Any> {
     fn fory_write(
         &self,
         context: &mut WriteContext,
-        write_ref_info: bool,
+        ref_mode: RefMode,
         write_type_info: bool,
         has_generics: bool,
     ) -> Result<(), Error> {
-        if !write_ref_info
+        if ref_mode == RefMode::None
             || !context
                 .ref_writer
                 .try_write_arc_ref(&mut context.writer, self)
@@ -421,7 +423,7 @@ impl Serializer for Arc<dyn Any> {
             let concrete_type_id: std::any::TypeId = (**self).type_id();
             if write_type_info {
                 let typeinfo =
-                    context.write_any_typeinfo(TypeId::UNKNOWN as u32, concrete_type_id)?;
+                    context.write_any_type_info(TypeId::UNKNOWN as u32, concrete_type_id)?;
                 let serializer_fn = typeinfo.get_harness().get_write_data_fn();
                 serializer_fn(&**self, context, has_generics)?;
             } else {
@@ -449,21 +451,21 @@ impl Serializer for Arc<dyn Any> {
 
     fn fory_read(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         read_type_info: bool,
     ) -> Result<Self, Error> {
-        read_arc_any(context, read_ref_info, read_type_info, None)
+        read_arc_any(context, ref_mode, read_type_info, None)
     }
 
     fn fory_read_with_type_info(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         type_info: Rc<TypeInfo>,
     ) -> Result<Self, Error>
     where
         Self: Sized + ForyDefault,
     {
-        read_arc_any(context, read_ref_info, false, Some(type_info))
+        read_arc_any(context, ref_mode, false, Some(type_info))
     }
 
     fn fory_read_data(_: &mut ReadContext) -> Result<Self, Error> {
@@ -473,17 +475,14 @@ impl Serializer for Arc<dyn Any> {
         )))
     }
 
-    fn fory_get_type_id(_type_resolver: &TypeResolver) -> Result<u32, Error> {
+    fn fory_get_type_id(_type_resolver: &TypeResolver) -> Result<TypeId, Error> {
         Err(Error::type_error(
             "Arc<dyn Any> has no static type ID - use fory_type_id_dyn",
         ))
     }
 
-    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<u32, Error> {
-        let concrete_type_id = (**self).type_id();
-        type_resolver
-            .get_fory_type_id(concrete_type_id)
-            .ok_or_else(|| Error::type_error("Type not registered"))
+    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<TypeId, Error> {
+        resolve_registered_type_id(type_resolver, (**self).type_id())
     }
 
     fn fory_concrete_type_id(&self) -> std::any::TypeId {
@@ -519,11 +518,11 @@ impl Serializer for Arc<dyn Any> {
 
 pub fn read_arc_any(
     context: &mut ReadContext,
-    read_ref_info: bool,
+    ref_mode: RefMode,
     read_type_info: bool,
     type_info: Option<Rc<TypeInfo>>,
 ) -> Result<Arc<dyn Any>, Error> {
-    let ref_flag = if read_ref_info {
+    let ref_flag = if ref_mode != RefMode::None {
         context.ref_reader.read_ref_flag(&mut context.reader)?
     } else {
         RefFlag::NotNullValue
@@ -542,7 +541,7 @@ pub fn read_arc_any(
         RefFlag::NotNullValue => {
             context.inc_depth()?;
             let typeinfo = if read_type_info {
-                context.read_any_typeinfo()?
+                context.read_any_type_info()?
             } else {
                 type_info
                     .ok_or_else(|| Error::type_error("No type info found for read Arc<dyn Any>"))?
@@ -558,7 +557,7 @@ pub fn read_arc_any(
         RefFlag::RefValue => {
             context.inc_depth()?;
             let typeinfo = if read_type_info {
-                context.read_any_typeinfo()?
+                context.read_any_type_info()?
             } else {
                 type_info
                     .ok_or_else(|| Error::type_error("No type info found for read Arc<dyn Any>"))?
