@@ -20,7 +20,7 @@ use crate::error::Error;
 use crate::resolver::context::ReadContext;
 use crate::resolver::context::WriteContext;
 use crate::serializer::{ForyDefault, Serializer};
-use crate::types::{need_to_write_type_for_field, RefFlag, PRIMITIVE_ARRAY_TYPES};
+use crate::types::{need_to_write_type_for_field, RefFlag, RefMode, PRIMITIVE_ARRAY_TYPES};
 
 const TRACKING_REF: u8 = 0b1;
 
@@ -36,7 +36,7 @@ pub fn write_collection_type_info(
     context: &mut WriteContext,
     collection_type_id: u32,
 ) -> Result<(), Error> {
-    context.writer.write_varuint32(collection_type_id);
+    context.writer.write_u8(collection_type_id as u8);
     Ok(())
 }
 
@@ -52,7 +52,7 @@ where
 {
     let iter = iter.into_iter();
     let len = iter.len();
-    context.writer.write_varuint32(len as u32);
+    context.writer.write_var_uint32(len as u32);
     if len == 0 {
         return Ok(());
     }
@@ -164,49 +164,38 @@ where
                     "Unable to determine concrete type for polymorphic collection elements",
                 )
             })?;
-            context.write_any_typeinfo(T::fory_static_type_id() as u32, type_id)?;
+            context.write_any_type_info(T::fory_static_type_id() as u32, type_id)?;
         } else {
             T::fory_write_type_info(context)?;
         }
     }
     // Write elements data
+    // Compute RefMode from flags
+    let elem_ref_mode = if elem_is_shared_ref {
+        RefMode::Tracking
+    } else if has_null {
+        RefMode::NullOnly
+    } else {
+        RefMode::None
+    };
+
     if is_same_type {
         // All elements are same type
-        if !has_null {
-            // No null elements
-            if elem_is_shared_ref {
-                for item in iter {
-                    item.fory_write(context, true, false, has_generics)?;
-                }
-            } else {
-                for item in iter {
-                    item.fory_write_data_generic(context, has_generics)?;
-                }
+        if elem_ref_mode == RefMode::None {
+            // No null elements, no tracking
+            for item in iter {
+                item.fory_write_data_generic(context, has_generics)?;
             }
         } else {
-            // Has null elements
+            // Has null or tracking
             for item in iter {
-                item.fory_write(context, true, false, has_generics)?;
+                item.fory_write(context, elem_ref_mode, false, has_generics)?;
             }
         }
     } else {
         // Different types (polymorphic elements with different types)
-        if !has_null {
-            // No null elements
-            if elem_is_shared_ref {
-                for item in iter {
-                    item.fory_write(context, true, true, has_generics)?;
-                }
-            } else {
-                for item in iter {
-                    item.fory_write(context, false, true, has_generics)?;
-                }
-            }
-        } else {
-            // Has null elements
-            for item in iter {
-                item.fory_write(context, true, true, has_generics)?;
-            }
+        for item in iter {
+            item.fory_write(context, elem_ref_mode, true, has_generics)?;
         }
     }
 
@@ -217,7 +206,7 @@ pub fn read_collection_type_info(
     context: &mut ReadContext,
     collection_type_id: u32,
 ) -> Result<(), Error> {
-    let remote_collection_type_id = context.reader.read_varuint32()?;
+    let remote_collection_type_id = context.reader.read_u8()? as u32;
     if PRIMITIVE_ARRAY_TYPES.contains(&remote_collection_type_id) {
         return Err(Error::type_error(
             "Vec<number> belongs to the `number_array` type, \
@@ -247,7 +236,7 @@ where
     let header = context.reader.read_u8()?;
     let declared = (header & DECL_ELEMENT_TYPE) != 0;
     if !declared {
-        // context.read_any_typeinfo();
+        // context.read_any_type_info();
         // TODO check whether type info consistent with T
         T::fory_read_type_info(context)?;
     }
@@ -273,6 +262,97 @@ where
     }
 }
 
+#[inline(always)]
+pub fn read_vec_data<T>(context: &mut ReadContext) -> Result<Vec<T>, Error>
+where
+    T: Serializer + ForyDefault,
+{
+    let len = context.reader.read_varuint32()?;
+    if len == 0 {
+        return Ok(Vec::new());
+    }
+    if T::fory_is_polymorphic() || T::fory_is_shared_ref() {
+        return read_vec_data_dyn_ref(context, len);
+    }
+    let header = context.reader.read_u8()?;
+    let declared = (header & DECL_ELEMENT_TYPE) != 0;
+    if !declared {
+        T::fory_read_type_info(context)?;
+    }
+    let has_null = (header & HAS_NULL) != 0;
+    ensure!(
+        (header & IS_SAME_TYPE) != 0,
+        Error::type_error("Type inconsistent, target type is not polymorphic")
+    );
+    let mut vec = Vec::with_capacity(len as usize);
+    if !has_null {
+        for _ in 0..len {
+            vec.push(T::fory_read_data(context)?);
+        }
+    } else {
+        for _ in 0..len {
+            let flag = context.reader.read_i8()?;
+            if flag == RefFlag::Null as i8 {
+                vec.push(T::fory_default());
+            } else {
+                vec.push(T::fory_read_data(context)?);
+            }
+        }
+    }
+    Ok(vec)
+}
+
+#[inline(always)]
+fn read_vec_data_dyn_ref<T>(context: &mut ReadContext, len: u32) -> Result<Vec<T>, Error>
+where
+    T: Serializer + ForyDefault,
+{
+    let header = context.reader.read_u8()?;
+    let is_track_ref = (header & TRACKING_REF) != 0;
+    let is_same_type = (header & IS_SAME_TYPE) != 0;
+    let has_null = (header & HAS_NULL) != 0;
+    let is_declared = (header & DECL_ELEMENT_TYPE) != 0;
+
+    let elem_ref_mode = if is_track_ref {
+        RefMode::Tracking
+    } else if has_null {
+        RefMode::NullOnly
+    } else {
+        RefMode::None
+    };
+
+    let mut vec = Vec::with_capacity(len as usize);
+    if is_same_type {
+        let type_info = if !is_declared {
+            context.read_any_type_info()?
+        } else {
+            T::fory_get_type_info(context.get_type_resolver())?
+        };
+        if elem_ref_mode == RefMode::None {
+            for _ in 0..len {
+                vec.push(T::fory_read_with_type_info(
+                    context,
+                    RefMode::None,
+                    type_info.clone(),
+                )?);
+            }
+        } else {
+            for _ in 0..len {
+                vec.push(T::fory_read_with_type_info(
+                    context,
+                    elem_ref_mode,
+                    type_info.clone(),
+                )?);
+            }
+        }
+    } else {
+        for _ in 0..len {
+            vec.push(T::fory_read(context, elem_ref_mode, true)?);
+        }
+    }
+    Ok(vec)
+}
+
 /// Slow but versatile collection deserialization for dynamic trait object and shared/circular reference.
 pub fn read_collection_data_dyn_ref<C, T>(context: &mut ReadContext, len: u32) -> Result<C, Error>
 where
@@ -285,40 +365,38 @@ where
     let is_same_type = (header & IS_SAME_TYPE) != 0;
     let has_null = (header & HAS_NULL) != 0;
     let is_declared = (header & DECL_ELEMENT_TYPE) != 0;
+
+    // Compute RefMode from flags
+    let elem_ref_mode = if is_track_ref {
+        RefMode::Tracking
+    } else if has_null {
+        RefMode::NullOnly
+    } else {
+        RefMode::None
+    };
+
     // Read elements
     if is_same_type {
         let type_info = if !is_declared {
-            context.read_any_typeinfo()?
+            context.read_any_type_info()?
         } else {
-            let rs_type_id = std::any::TypeId::of::<T>();
-            context.get_type_resolver().get_type_info(&rs_type_id)?
+            T::fory_get_type_info(context.get_type_resolver())?
         };
         // All elements are same type
-        if is_track_ref {
+        if elem_ref_mode == RefMode::None {
+            // No null elements, no tracking
             (0..len)
-                .map(|_| T::fory_read_with_type_info(context, true, type_info.clone()))
-                .collect::<Result<C, Error>>()
-        } else if !has_null {
-            // No null elements
-            (0..len)
-                .map(|_| T::fory_read_with_type_info(context, false, type_info.clone()))
+                .map(|_| T::fory_read_with_type_info(context, RefMode::None, type_info.clone()))
                 .collect::<Result<C, Error>>()
         } else {
-            // Has null elements
+            // Has null or tracking - use ref mode
             (0..len)
-                .map(|_| {
-                    let flag = context.reader.read_i8()?;
-                    if flag == RefFlag::Null as i8 {
-                        Ok(T::fory_default())
-                    } else {
-                        T::fory_read_with_type_info(context, false, type_info.clone())
-                    }
-                })
+                .map(|_| T::fory_read_with_type_info(context, elem_ref_mode, type_info.clone()))
                 .collect::<Result<C, Error>>()
         }
     } else {
         (0..len)
-            .map(|_| T::fory_read(context, is_track_ref, true))
+            .map(|_| T::fory_read(context, elem_ref_mode, true))
             .collect::<Result<C, Error>>()
     }
 }
