@@ -20,15 +20,15 @@ use crate::error::Error;
 use crate::meta::FieldInfo;
 use crate::resolver::context::{ReadContext, WriteContext};
 use crate::serializer::{ForyDefault, Serializer};
-use crate::types::{RefFlag, TypeId};
+use crate::types::{RefFlag, RefMode, TypeId};
 use crate::TypeResolver;
 
 #[inline(always)]
-pub fn actual_type_id(type_id: u32, register_by_name: bool, _compatible: bool) -> u32 {
+pub fn actual_type_id(_type_id: u32, register_by_name: bool, _compatible: bool) -> u32 {
     if register_by_name {
         TypeId::NAMED_ENUM as u32
     } else {
-        (type_id << 8) + TypeId::ENUM as u32
+        TypeId::ENUM as u32
     }
 }
 
@@ -36,10 +36,10 @@ pub fn actual_type_id(type_id: u32, register_by_name: bool, _compatible: bool) -
 pub fn write<T: Serializer>(
     this: &T,
     context: &mut WriteContext,
-    write_ref_info: bool,
+    ref_mode: RefMode,
     write_type_info: bool,
 ) -> Result<(), Error> {
-    if write_ref_info {
+    if ref_mode != RefMode::None {
         context.writer.write_i8(RefFlag::NotNullValue as i8);
     }
     if write_type_info {
@@ -51,15 +51,18 @@ pub fn write<T: Serializer>(
 #[inline(always)]
 pub fn write_type_info<T: Serializer>(context: &mut WriteContext) -> Result<(), Error> {
     let type_id = T::fory_get_type_id(context.get_type_resolver())?;
-    context.writer.write_varuint32(type_id);
-    let is_named_enum = type_id & 0xff == TypeId::NAMED_ENUM as u32;
-    if !is_named_enum {
+    context.writer.write_u8(type_id as u8);
+    let rs_type_id = std::any::TypeId::of::<T>();
+    if type_id == TypeId::ENUM {
+        let type_info = context.get_type_resolver().get_type_info(&rs_type_id)?;
+        context
+            .writer
+            .write_var_uint32(type_info.get_user_type_id());
         return Ok(());
     }
-    let rs_type_id = std::any::TypeId::of::<T>();
     if context.is_share_meta() {
-        let meta_index = context.push_meta(rs_type_id)? as u32;
-        context.writer.write_varuint32(meta_index);
+        // Write type meta inline using streaming protocol
+        context.write_type_meta(rs_type_id)?;
     } else {
         let type_info = context.get_type_resolver().get_type_info(&rs_type_id)?;
         let namespace = type_info.get_namespace();
@@ -73,10 +76,10 @@ pub fn write_type_info<T: Serializer>(context: &mut WriteContext) -> Result<(),
 #[inline(always)]
 pub fn read<T: Serializer + ForyDefault>(
     context: &mut ReadContext,
-    read_ref_info: bool,
+    ref_mode: RefMode,
     read_type_info: bool,
 ) -> Result<T, Error> {
-    let ref_flag = if read_ref_info {
+    let ref_flag = if ref_mode != RefMode::None {
         context.reader.read_i8()?
     } else {
         RefFlag::NotNullValue as i8
@@ -101,20 +104,21 @@ pub fn read<T: Serializer + ForyDefault>(
 #[inline(always)]
 pub fn read_type_info<T: Serializer>(context: &mut ReadContext) -> Result<(), Error> {
     let local_type_id = T::fory_get_type_id(context.get_type_resolver())?;
-    let remote_type_id = context.reader.read_varuint32()?;
+    let remote_type_id = context.reader.read_u8()?;
     ensure!(
-        local_type_id == remote_type_id,
-        Error::type_mismatch(local_type_id, remote_type_id)
+        local_type_id as u8 == remote_type_id,
+        Error::type_mismatch(local_type_id as u32, remote_type_id as u32)
     );
-    let is_named_enum = local_type_id & 0xff == TypeId::NAMED_ENUM as u32;
-    if !is_named_enum {
-        return Ok(());
-    }
-    if context.is_share_meta() {
-        let _meta_index = context.reader.read_varuint32()?;
+    if remote_type_id == TypeId::NAMED_ENUM as u8 {
+        if context.is_share_meta() {
+            // Read type meta inline using streaming protocol
+            let _type_info = context.read_type_meta()?;
+        } else {
+            let _namespace_msb = context.read_meta_string()?;
+            let _type_name_msb = context.read_meta_string()?;
+        }
     } else {
-        let _namespace_msb = context.read_meta_string()?;
-        let _type_name_msb = context.read_meta_string()?;
+        context.reader.read_varuint32()?;
     }
     Ok(())
 }
