@@ -21,7 +21,7 @@ use crate::resolver::context::{ReadContext, WriteContext};
 use crate::resolver::type_resolver::{TypeInfo, TypeResolver};
 use crate::serializer::util::read_basic_type_info;
 use crate::serializer::{ForyDefault, Serializer};
-use crate::types::{need_to_write_type_for_field, TypeId, SIZE_OF_REF_AND_TYPE};
+use crate::types::{need_to_write_type_for_field, RefMode, TypeId, SIZE_OF_REF_AND_TYPE};
 use std::collections::{BTreeMap, HashMap};
 use std::rc::Rc;
 
@@ -49,7 +49,7 @@ where
     V: Serializer,
     I: Iterator<Item = (&'a K, &'a V)>,
 {
-    context.writer.write_varuint32(length as u32);
+    context.writer.write_var_uint32(length as u32);
     if length == 0 {
         return Ok(());
     }
@@ -170,8 +170,8 @@ where
     let val_is_shared_ref = V::fory_is_shared_ref();
 
     // Track the current chunk's key and value types (for polymorphic types)
-    let mut current_key_type_id: Option<u32> = None;
-    let mut current_val_type_id: Option<u32> = None;
+    let mut current_key_type_id: Option<std::any::TypeId> = None;
+    let mut current_val_type_id: Option<std::any::TypeId> = None;
 
     for (key, value) in iter {
         // Handle null key/value entries (write as separate single-entry chunks)
@@ -197,7 +197,7 @@ where
                 } else {
                     context.writer.write_u8(chunk_header);
                     if key_is_polymorphic {
-                        context.write_any_typeinfo(
+                        context.write_any_type_info(
                             K::fory_static_type_id() as u32,
                             key.fory_concrete_type_id(),
                         )?;
@@ -206,7 +206,7 @@ where
                     }
                 }
                 if key_is_shared_ref {
-                    key.fory_write(context, true, false, has_generics)?;
+                    key.fory_write(context, RefMode::Tracking, false, has_generics)?;
                 } else {
                     key.fory_write_data_generic(context, has_generics)?;
                 }
@@ -223,7 +223,7 @@ where
                 } else {
                     context.writer.write_u8(chunk_header);
                     if val_is_polymorphic {
-                        context.write_any_typeinfo(
+                        context.write_any_type_info(
                             V::fory_static_type_id() as u32,
                             value.fory_concrete_type_id(),
                         )?;
@@ -232,7 +232,7 @@ where
                     }
                 }
                 if val_is_shared_ref {
-                    value.fory_write(context, true, false, has_generics)?;
+                    value.fory_write(context, RefMode::Tracking, false, has_generics)?;
                 } else {
                     value.fory_write_data_generic(context, has_generics)?;
                 }
@@ -242,12 +242,12 @@ where
 
         // Get type IDs for polymorphic types
         let key_type_id = if key_is_polymorphic {
-            Some(key.fory_type_id_dyn(context.get_type_resolver())?)
+            Some(key.fory_concrete_type_id())
         } else {
             None
         };
         let val_type_id = if val_is_polymorphic {
-            Some(value.fory_type_id_dyn(context.get_type_resolver())?)
+            Some(value.fory_concrete_type_id())
         } else {
             None
         };
@@ -281,7 +281,7 @@ where
             } else {
                 // Write type info for key
                 if key_is_polymorphic {
-                    context.write_any_typeinfo(
+                    context.write_any_type_info(
                         K::fory_static_type_id() as u32,
                         key.fory_concrete_type_id(),
                     )?;
@@ -299,7 +299,7 @@ where
             } else {
                 // Write type info for value
                 if val_is_polymorphic {
-                    context.write_any_typeinfo(
+                    context.write_any_type_info(
                         V::fory_static_type_id() as u32,
                         value.fory_concrete_type_id(),
                     )?;
@@ -316,12 +316,12 @@ where
 
         // Write key-value pair
         if key_is_shared_ref {
-            key.fory_write(context, true, false, has_generics)?;
+            key.fory_write(context, RefMode::Tracking, false, has_generics)?;
         } else {
             key.fory_write_data_generic(context, has_generics)?;
         }
         if val_is_shared_ref {
-            value.fory_write(context, true, false, has_generics)?;
+            value.fory_write(context, RefMode::Tracking, false, has_generics)?;
         } else {
             value.fory_write_data_generic(context, has_generics)?;
         }
@@ -358,9 +358,6 @@ macro_rules! impl_read_map_dyn_ref {
         {
             let key_is_polymorphic = K::fory_is_polymorphic();
             let val_is_polymorphic = V::fory_is_polymorphic();
-            let key_is_shared_ref = K::fory_is_shared_ref();
-            let val_is_shared_ref = V::fory_is_shared_ref();
-
             let mut len_counter = 0u32;
 
             while len_counter < length {
@@ -382,7 +379,7 @@ macro_rules! impl_read_map_dyn_ref {
                     // Determine value type info (if any)
                     let value_type_info: Option<Rc<TypeInfo>> = if !value_declared {
                         if val_is_polymorphic {
-                            Some(context.read_any_typeinfo()?)
+                            Some(context.read_any_type_info()?)
                         } else {
                             V::fory_read_type_info(context)?;
                             None
@@ -392,11 +389,15 @@ macro_rules! impl_read_map_dyn_ref {
                     };
 
                     // Read value payload
-                    let read_ref = val_is_shared_ref || track_value_ref;
+                    let ref_mode = if track_value_ref {
+                        RefMode::Tracking
+                    } else {
+                        RefMode::None
+                    };
                     let value = if let Some(type_info) = value_type_info {
-                        V::fory_read_with_type_info(context, read_ref, type_info)?
-                    } else if read_ref {
-                        V::fory_read(context, read_ref, false)?
+                        V::fory_read_with_type_info(context, ref_mode, type_info)?
+                    } else if track_value_ref {
+                        V::fory_read(context, ref_mode, false)?
                     } else {
                         V::fory_read_data(context)?
                     };
@@ -413,7 +414,7 @@ macro_rules! impl_read_map_dyn_ref {
 
                     let key_type_info: Option<Rc<TypeInfo>> = if !key_declared {
                         if key_is_polymorphic {
-                            Some(context.read_any_typeinfo()?)
+                            Some(context.read_any_type_info()?)
                         } else {
                             K::fory_read_type_info(context)?;
                             None
@@ -422,11 +423,15 @@ macro_rules! impl_read_map_dyn_ref {
                         None
                     };
 
-                    let read_ref = key_is_shared_ref || track_key_ref;
+                    let ref_mode = if track_key_ref {
+                        RefMode::Tracking
+                    } else {
+                        RefMode::None
+                    };
                     let key = if let Some(type_info) = key_type_info {
-                        K::fory_read_with_type_info(context, read_ref, type_info)?
-                    } else if read_ref {
-                        K::fory_read(context, read_ref, false)?
+                        K::fory_read_with_type_info(context, ref_mode, type_info)?
+                    } else if track_key_ref {
+                        K::fory_read(context, ref_mode, false)?
                     } else {
                         K::fory_read_data(context)?
                     };
@@ -445,7 +450,7 @@ macro_rules! impl_read_map_dyn_ref {
 
                 let key_type_info: Option<Rc<TypeInfo>> = if !key_declared {
                     if key_is_polymorphic {
-                        Some(context.read_any_typeinfo()?)
+                        Some(context.read_any_type_info()?)
                     } else {
                         K::fory_read_type_info(context)?;
                         None
@@ -455,7 +460,7 @@ macro_rules! impl_read_map_dyn_ref {
                 };
                 let value_type_info: Option<Rc<TypeInfo>> = if !value_declared {
                     if val_is_polymorphic {
-                        Some(context.read_any_typeinfo()?)
+                        Some(context.read_any_type_info()?)
                     } else {
                         V::fory_read_type_info(context)?;
                         None
@@ -473,21 +478,29 @@ macro_rules! impl_read_map_dyn_ref {
                 );
 
                 // Read chunk_size pairs of key-value
-                let key_read_ref = key_is_shared_ref || track_key_ref;
-                let val_read_ref = val_is_shared_ref || track_value_ref;
+                let key_ref_mode = if track_key_ref {
+                    RefMode::Tracking
+                } else {
+                    RefMode::None
+                };
+                let val_ref_mode = if track_value_ref {
+                    RefMode::Tracking
+                } else {
+                    RefMode::None
+                };
                 for _ in 0..chunk_size {
                     let key = if let Some(type_info) = key_type_info.as_ref() {
-                        K::fory_read_with_type_info(context, key_read_ref, type_info.clone())?
-                    } else if key_read_ref {
-                        K::fory_read(context, key_read_ref, false)?
+                        K::fory_read_with_type_info(context, key_ref_mode, type_info.clone())?
+                    } else if track_key_ref {
+                        K::fory_read(context, key_ref_mode, false)?
                     } else {
                         K::fory_read_data(context)?
                     };
 
                     let value = if let Some(type_info) = value_type_info.as_ref() {
-                        V::fory_read_with_type_info(context, val_read_ref, type_info.clone())?
-                    } else if val_read_ref {
-                        V::fory_read(context, val_read_ref, false)?
+                        V::fory_read_with_type_info(context, val_ref_mode, type_info.clone())?
+                    } else if track_value_ref {
+                        V::fory_read(context, val_ref_mode, false)?
                     } else {
                         V::fory_read_data(context)?
                     };
@@ -562,13 +575,25 @@ impl<K: Serializer + ForyDefault + Eq + std::hash::Hash, V: Serializer + ForyDef
             let track_key_ref = (header & TRACKING_KEY_REF) != 0;
             let track_value_ref = (header & TRACKING_VALUE_REF) != 0;
             if header & KEY_NULL != 0 {
-                let value = V::fory_read(context, track_value_ref, !value_declared)?;
+                // Null case is handled by KEY_NULL flag, so use None (no ref flag) unless tracking
+                let ref_mode = if track_value_ref {
+                    RefMode::Tracking
+                } else {
+                    RefMode::None
+                };
+                let value = V::fory_read(context, ref_mode, !value_declared)?;
                 map.insert(K::fory_default(), value);
                 len_counter += 1;
                 continue;
             }
             if header & VALUE_NULL != 0 {
-                let key = K::fory_read(context, track_key_ref, !key_declared)?;
+                // Null case is handled by VALUE_NULL flag, so use None (no ref flag) unless tracking
+                let ref_mode = if track_key_ref {
+                    RefMode::Tracking
+                } else {
+                    RefMode::None
+                };
+                let key = K::fory_read(context, ref_mode, !key_declared)?;
                 map.insert(key, V::fory_default());
                 len_counter += 1;
                 continue;
@@ -595,9 +620,19 @@ impl<K: Serializer + ForyDefault + Eq + std::hash::Hash, V: Serializer + ForyDef
                     map.insert(key, value);
                 }
             } else {
+                let key_ref_mode = if track_key_ref {
+                    RefMode::Tracking
+                } else {
+                    RefMode::None
+                };
+                let val_ref_mode = if track_value_ref {
+                    RefMode::Tracking
+                } else {
+                    RefMode::None
+                };
                 for _ in 0..chunk_size {
-                    let key = K::fory_read(context, track_key_ref, false)?;
-                    let value = V::fory_read(context, track_value_ref, false)?;
+                    let key = K::fory_read(context, key_ref_mode, false)?;
+                    let value = V::fory_read(context, val_ref_mode, false)?;
                     map.insert(key, value);
                 }
             }
@@ -611,12 +646,12 @@ impl<K: Serializer + ForyDefault + Eq + std::hash::Hash, V: Serializer + ForyDef
         size_of::<i32>()
     }
 
-    fn fory_get_type_id(_: &TypeResolver) -> Result<u32, Error> {
-        Ok(TypeId::MAP as u32)
+    fn fory_get_type_id(_: &TypeResolver) -> Result<TypeId, Error> {
+        Ok(TypeId::MAP)
     }
 
-    fn fory_type_id_dyn(&self, _: &TypeResolver) -> Result<u32, Error> {
-        Ok(TypeId::MAP as u32)
+    fn fory_type_id_dyn(&self, _: &TypeResolver) -> Result<TypeId, Error> {
+        Ok(TypeId::MAP)
     }
 
     fn fory_static_type_id() -> TypeId
@@ -631,7 +666,7 @@ impl<K: Serializer + ForyDefault + Eq + std::hash::Hash, V: Serializer + ForyDef
     }
 
     fn fory_write_type_info(context: &mut WriteContext) -> Result<(), Error> {
-        context.writer.write_varuint32(TypeId::MAP as u32);
+        context.writer.write_u8(TypeId::MAP as u8);
         Ok(())
     }
 
@@ -691,13 +726,25 @@ impl<K: Serializer + ForyDefault + Ord + std::hash::Hash, V: Serializer + ForyDe
             let track_key_ref = (header & TRACKING_KEY_REF) != 0;
             let track_value_ref = (header & TRACKING_VALUE_REF) != 0;
             if header & KEY_NULL != 0 {
-                let value = V::fory_read(context, track_value_ref, !value_declared)?;
+                // Null case is handled by KEY_NULL flag, so use None (no ref flag) unless tracking
+                let ref_mode = if track_value_ref {
+                    RefMode::Tracking
+                } else {
+                    RefMode::None
+                };
+                let value = V::fory_read(context, ref_mode, !value_declared)?;
                 map.insert(K::fory_default(), value);
                 len_counter += 1;
                 continue;
             }
             if header & VALUE_NULL != 0 {
-                let key = K::fory_read(context, track_key_ref, !key_declared)?;
+                // Null case is handled by VALUE_NULL flag, so use None (no ref flag) unless tracking
+                let ref_mode = if track_key_ref {
+                    RefMode::Tracking
+                } else {
+                    RefMode::None
+                };
+                let key = K::fory_read(context, ref_mode, !key_declared)?;
                 map.insert(key, V::fory_default());
                 len_counter += 1;
                 continue;
@@ -724,9 +771,19 @@ impl<K: Serializer + ForyDefault + Ord + std::hash::Hash, V: Serializer + ForyDe
                     map.insert(key, value);
                 }
             } else {
+                let key_ref_mode = if track_key_ref {
+                    RefMode::Tracking
+                } else {
+                    RefMode::None
+                };
+                let val_ref_mode = if track_value_ref {
+                    RefMode::Tracking
+                } else {
+                    RefMode::None
+                };
                 for _ in 0..chunk_size {
-                    let key = K::fory_read(context, track_key_ref, false)?;
-                    let value = V::fory_read(context, track_value_ref, false)?;
+                    let key = K::fory_read(context, key_ref_mode, false)?;
+                    let value = V::fory_read(context, val_ref_mode, false)?;
                     map.insert(key, value);
                 }
             }
@@ -739,12 +796,12 @@ impl<K: Serializer + ForyDefault + Ord + std::hash::Hash, V: Serializer + ForyDe
         size_of::<i32>()
     }
 
-    fn fory_get_type_id(_: &TypeResolver) -> Result<u32, Error> {
-        Ok(TypeId::MAP as u32)
+    fn fory_get_type_id(_: &TypeResolver) -> Result<TypeId, Error> {
+        Ok(TypeId::MAP)
     }
 
-    fn fory_type_id_dyn(&self, _: &TypeResolver) -> Result<u32, Error> {
-        Ok(TypeId::MAP as u32)
+    fn fory_type_id_dyn(&self, _: &TypeResolver) -> Result<TypeId, Error> {
+        Ok(TypeId::MAP)
     }
 
     fn fory_static_type_id() -> TypeId
@@ -759,7 +816,7 @@ impl<K: Serializer + ForyDefault + Ord + std::hash::Hash, V: Serializer + ForyDe
     }
 
     fn fory_write_type_info(context: &mut WriteContext) -> Result<(), Error> {
-        context.writer.write_varuint32(TypeId::MAP as u32);
+        context.writer.write_u8(TypeId::MAP as u8);
         Ok(())
     }
 
