@@ -34,6 +34,14 @@ impl<T: Serializer + ForyDefault + Eq + std::hash::Hash> Serializer for HashSet<
         write_collection_data(self, context, false)
     }
 
+    fn fory_write_data_generic(
+        &self,
+        context: &mut WriteContext,
+        has_generics: bool,
+    ) -> Result<(), Error> {
+        write_collection_data(self, context, has_generics)
+    }
+
     fn fory_write_type_info(context: &mut WriteContext) -> Result<(), Error> {
         write_collection_type_info(context, TypeId::SET as u32)
     }
@@ -50,12 +58,12 @@ impl<T: Serializer + ForyDefault + Eq + std::hash::Hash> Serializer for HashSet<
         mem::size_of::<i32>()
     }
 
-    fn fory_get_type_id(_: &TypeResolver) -> Result<u32, Error> {
-        Ok(TypeId::SET as u32)
+    fn fory_get_type_id(_: &TypeResolver) -> Result<TypeId, Error> {
+        Ok(TypeId::SET)
     }
 
-    fn fory_type_id_dyn(&self, _: &TypeResolver) -> Result<u32, Error> {
-        Ok(TypeId::SET as u32)
+    fn fory_type_id_dyn(&self, _: &TypeResolver) -> Result<TypeId, Error> {
+        Ok(TypeId::SET)
     }
 
     fn fory_static_type_id() -> TypeId
@@ -81,6 +89,14 @@ impl<T: Serializer + ForyDefault + Ord> Serializer for BTreeSet<T> {
         write_collection_data(self, context, false)
     }
 
+    fn fory_write_data_generic(
+        &self,
+        context: &mut WriteContext,
+        has_generics: bool,
+    ) -> Result<(), Error> {
+        write_collection_data(self, context, has_generics)
+    }
+
     fn fory_write_type_info(context: &mut WriteContext) -> Result<(), Error> {
         write_collection_type_info(context, TypeId::SET as u32)
     }
@@ -97,12 +113,12 @@ impl<T: Serializer + ForyDefault + Ord> Serializer for BTreeSet<T> {
         mem::size_of::<i32>()
     }
 
-    fn fory_get_type_id(_: &TypeResolver) -> Result<u32, Error> {
-        Ok(TypeId::SET as u32)
+    fn fory_get_type_id(_: &TypeResolver) -> Result<TypeId, Error> {
+        Ok(TypeId::SET)
     }
 
-    fn fory_type_id_dyn(&self, _: &TypeResolver) -> Result<u32, Error> {
-        Ok(TypeId::SET as u32)
+    fn fory_type_id_dyn(&self, _: &TypeResolver) -> Result<TypeId, Error> {
+        Ok(TypeId::SET)
     }
 
     fn fory_static_type_id() -> TypeId
