@@ -35,7 +35,7 @@ impl Serializer for String {
     #[inline(always)]
     fn fory_write_data(&self, context: &mut WriteContext) -> Result<(), Error> {
         let bitor = (self.len() as i32 as u64) << 2 | StrEncoding::Utf8 as u64;
-        context.writer.write_varuint36_small(bitor);
+        context.writer.write_var_uint36_small(bitor);
         context.writer.write_utf8_string(self);
         Ok(())
     }
@@ -66,13 +66,13 @@ impl Serializer for String {
     }
 
     #[inline(always)]
-    fn fory_get_type_id(_: &TypeResolver) -> Result<u32, Error> {
-        Ok(TypeId::STRING as u32)
+    fn fory_get_type_id(_: &TypeResolver) -> Result<TypeId, Error> {
+        Ok(TypeId::STRING)
     }
 
     #[inline(always)]
-    fn fory_type_id_dyn(&self, _: &TypeResolver) -> Result<u32, Error> {
-        Ok(TypeId::STRING as u32)
+    fn fory_type_id_dyn(&self, _: &TypeResolver) -> Result<TypeId, Error> {
+        Ok(TypeId::STRING)
     }
 
     #[inline(always)]
@@ -90,7 +90,7 @@ impl Serializer for String {
 
     #[inline(always)]
     fn fory_write_type_info(context: &mut WriteContext) -> Result<(), Error> {
-        context.writer.write_varuint32(TypeId::STRING as u32);
+        context.writer.write_u8(TypeId::STRING as u8);
         Ok(())
     }
 
