@@ -18,72 +18,66 @@
 use crate::ensure;
 use crate::error::Error;
 use crate::resolver::context::{ReadContext, WriteContext};
-use crate::serializer::Serializer;
-use crate::types::{RefFlag, TypeId};
+use crate::serializer::{Serializer, StructSerializer};
+use crate::types::{RefFlag, RefMode, TypeId};
 use crate::util::ENABLE_FORY_DEBUG_OUTPUT;
 use std::any::Any;
 
 #[inline(always)]
-pub fn actual_type_id(type_id: u32, register_by_name: bool, compatible: bool) -> u32 {
+pub fn actual_type_id(_type_id: u32, register_by_name: bool, compatible: bool) -> u32 {
     if compatible {
         if register_by_name {
             TypeId::NAMED_COMPATIBLE_STRUCT as u32
         } else {
-            (type_id << 8) + TypeId::COMPATIBLE_STRUCT as u32
+            TypeId::COMPATIBLE_STRUCT as u32
         }
     } else if register_by_name {
         TypeId::NAMED_STRUCT as u32
     } else {
-        (type_id << 8) + TypeId::STRUCT as u32
+        TypeId::STRUCT as u32
     }
 }
 
 #[inline(always)]
 pub fn write_type_info<T: Serializer>(context: &mut WriteContext) -> Result<(), Error> {
-    let type_id = T::fory_get_type_id(context.get_type_resolver())?;
-    context.writer.write_varuint32(type_id);
     let rs_type_id = std::any::TypeId::of::<T>();
-
-    if type_id & 0xff == TypeId::NAMED_STRUCT as u32 {
-        if context.is_share_meta() {
-            let meta_index = context.push_meta(rs_type_id)? as u32;
-            context.writer.write_varuint32(meta_index);
-        } else {
-            let type_info = context.get_type_resolver().get_type_info(&rs_type_id)?;
-            let namespace = type_info.get_namespace();
-            let type_name = type_info.get_type_name();
-            context.write_meta_string_bytes(namespace)?;
-            context.write_meta_string_bytes(type_name)?;
-        }
-    } else if type_id & 0xff == TypeId::NAMED_COMPATIBLE_STRUCT as u32
-        || type_id & 0xff == TypeId::COMPATIBLE_STRUCT as u32
-    {
-        let meta_index = context.push_meta(rs_type_id)? as u32;
-        context.writer.write_varuint32(meta_index);
-    }
+    let type_id = T::fory_get_type_id(context.get_type_resolver())?;
+    context.write_any_type_info(type_id as u32, rs_type_id)?;
     Ok(())
 }
 
 #[inline(always)]
 pub fn read_type_info<T: Serializer>(context: &mut ReadContext) -> Result<(), Error> {
-    let remote_type_id = context.reader.read_varuint32()?;
-    let local_type_id = T::fory_get_type_id(context.get_type_resolver())?;
-    ensure!(
-        local_type_id == remote_type_id,
-        Error::type_mismatch(local_type_id, remote_type_id)
-    );
+    context.read_any_type_info()?;
+    Ok(())
+}
 
-    if local_type_id & 0xff == TypeId::NAMED_STRUCT as u32 {
-        if context.is_share_meta() {
-            let _meta_index = context.reader.read_varuint32()?;
-        } else {
-            let _namespace_msb = context.read_meta_string()?;
-            let _type_name_msb = context.read_meta_string()?;
-        }
-    } else if local_type_id & 0xff == TypeId::NAMED_COMPATIBLE_STRUCT as u32
-        || local_type_id & 0xff == TypeId::COMPATIBLE_STRUCT as u32
-    {
-        let _meta_index = context.reader.read_varuint32();
+#[inline(always)]
+pub fn read_type_info_fast<T: StructSerializer>(context: &mut ReadContext) -> Result<(), Error> {
+    if context.is_compatible() || context.is_xlang() {
+        return read_type_info::<T>(context);
+    }
+    let local_type_id = context
+        .get_type_resolver()
+        .get_type_id_by_index(T::fory_type_index())?;
+    let local_type_id_u32 = local_type_id as u32;
+    if !crate::types::needs_user_type_id(local_type_id_u32) {
+        return read_type_info::<T>(context);
+    }
+    let remote_type_id = context.reader.read_u8()? as u32;
+    ensure!(
+        local_type_id_u32 == remote_type_id,
+        Error::type_mismatch(local_type_id_u32, remote_type_id)
+    );
+    let remote_user_type_id = context.reader.read_varuint32()?;
+    let local_user_type_id = context
+        .get_type_resolver()
+        .get_user_type_id_by_index(&std::any::TypeId::of::<T>(), T::fory_type_index())?;
+    if remote_user_type_id != local_user_type_id {
+        return Err(Error::type_error(format!(
+            "User type id mismatch: local {} vs remote {}",
+            local_user_type_id, remote_user_type_id
+        )));
     }
     Ok(())
 }
@@ -92,11 +86,20 @@ pub fn read_type_info<T: Serializer>(context: &mut ReadContext) -> Result<(), Er
 pub fn write<T: Serializer>(
     this: &T,
     context: &mut WriteContext,
-    write_ref_info: bool,
+    ref_mode: RefMode,
     write_type_info: bool,
 ) -> Result<(), Error> {
-    if write_ref_info {
-        context.writer.write_i8(RefFlag::NotNullValue as i8);
+    match ref_mode {
+        RefMode::None => {}
+        RefMode::NullOnly => {
+            context.writer.write_i8(RefFlag::NotNullValue as i8);
+        }
+        RefMode::Tracking => {
+            // For ref tracking mode, write RefValue flag and reserve a ref_id
+            // so this struct participates in reference tracking.
+            context.writer.write_i8(RefFlag::RefValue as i8);
+            context.ref_writer.reserve_ref_id();
+        }
     }
     if write_type_info {
         T::fory_write_type_info(context)?;
