@@ -23,7 +23,9 @@ use crate::error::Error;
 use crate::resolver::context::{ReadContext, WriteContext};
 use crate::resolver::type_resolver::{TypeInfo, TypeResolver};
 use crate::serializer::{ForyDefault, Serializer};
+use crate::types::RefMode;
 use crate::RefFlag;
+use crate::TypeId;
 use std::rc::Rc;
 
 /// Helper macro for common type resolution and downcasting pattern
@@ -42,24 +44,6 @@ macro_rules! downcast_and_serialize {
     }};
 }
 
-/// Helper macro for common type resolution and deserialization pattern
-#[macro_export]
-macro_rules! resolve_and_deserialize {
-    ($fory_type_id:expr, $context:expr, $constructor:expr, $trait_name:ident, $($impl_type:ty),+) => {{
-        $(
-            if let Some(registered_type_id) = $context.get_type_resolver().get_fory_type_id(std::any::TypeId::of::<$impl_type>()) {
-                if $fory_type_id == registered_type_id {
-                    let concrete_obj = <$impl_type as fory_core::Serializer>::fory_read_data($context)?;
-                    return Ok($constructor(concrete_obj));
-                }
-            }
-        )*
-        Err(fory_core::Error::type_error(
-            format!("Type ID {} not registered for trait {}", $fory_type_id, stringify!($trait_name))
-        ))
-    }};
-}
-
 /// Macro to register trait object conversions for custom traits.
 ///
 /// This macro automatically generates serializers for `Box<dyn Trait>` trait objects.
@@ -154,25 +138,25 @@ macro_rules! register_trait_type {
         // 4. Serializer implementation for Box<dyn Trait> (existing functionality)
         impl fory_core::Serializer for Box<dyn $trait_name> {
             #[inline(always)]
-            fn fory_write(&self, context: &mut fory_core::WriteContext, write_ref_info: bool, write_type_info: bool, has_generics: bool) -> Result<(), fory_core::Error> {
+            fn fory_write(&self, context: &mut fory_core::WriteContext, ref_mode: fory_core::RefMode, write_type_info: bool, has_generics: bool) -> Result<(), fory_core::Error> {
                 let any_ref = <dyn $trait_name as fory_core::Serializer>::as_any(&**self);
-                fory_core::serializer::write_box_any(any_ref, context, write_ref_info, write_type_info, has_generics)
+                fory_core::serializer::write_box_any(any_ref, context, ref_mode, write_type_info, has_generics)
             }
 
             #[inline(always)]
             fn fory_write_data(&self, context: &mut fory_core::WriteContext) -> Result<(), fory_core::Error> {
                 let any_ref = <dyn $trait_name as fory_core::Serializer>::as_any(&**self);
-                fory_core::serializer::write_box_any(any_ref, context, false, false, false)
+                fory_core::serializer::write_box_any(any_ref, context, fory_core::RefMode::None, false, false)
             }
 
             #[inline(always)]
             fn fory_write_data_generic(&self, context: &mut fory_core::WriteContext, has_generics: bool) -> Result<(), fory_core::Error> {
                 let any_ref = <dyn $trait_name as fory_core::Serializer>::as_any(&**self);
-                fory_core::serializer::write_box_any(any_ref, context, false, false, has_generics)
+                fory_core::serializer::write_box_any(any_ref, context, fory_core::RefMode::None, false, has_generics)
             }
 
             #[inline(always)]
-            fn fory_type_id_dyn(&self, type_resolver: &fory_core::TypeResolver) -> Result<u32, fory_core::Error> {
+            fn fory_type_id_dyn(&self, type_resolver: &fory_core::TypeResolver) -> Result<fory_core::TypeId, fory_core::Error> {
                 let any_ref = <dyn $trait_name as fory_core::Serializer>::as_any(&**self);
                 let concrete_type_id = any_ref.type_id();
                 type_resolver
@@ -193,8 +177,8 @@ macro_rules! register_trait_type {
                 $crate::not_allowed!("fory_read_type_info should not be called directly on polymorphic Box<dyn {}> trait object", stringify!($trait_name))
             }
 
-            fn fory_read(context: &mut fory_core::ReadContext, read_ref_info: bool, read_type_info: bool) -> Result<Self, fory_core::Error> {
-                let boxed_any = fory_core::serializer::read_box_any(context, read_ref_info, read_type_info, None)?;
+            fn fory_read(context: &mut fory_core::ReadContext, ref_mode: fory_core::RefMode, read_type_info: bool) -> Result<Self, fory_core::Error> {
+                let boxed_any = fory_core::serializer::read_box_any(context, ref_mode, read_type_info, None)?;
                 $(
                     if boxed_any.is::<$impl_type>() {
                         let concrete = boxed_any.downcast::<$impl_type>()
@@ -210,13 +194,13 @@ macro_rules! register_trait_type {
 
             fn fory_read_with_type_info(
                 context: &mut fory_core::ReadContext,
-                read_ref_info: bool,
+                ref_mode: fory_core::RefMode,
                 type_info: std::rc::Rc<fory_core::TypeInfo>,
             ) -> Result<Self, fory_core::Error>
             where
                 Self: Sized + fory_core::ForyDefault,
             {
-                let boxed_any = fory_core::serializer::read_box_any(context, read_ref_info, false, Some(type_info))?;
+                let boxed_any = fory_core::serializer::read_box_any(context, ref_mode, false, Some(type_info))?;
                 $(
                     if boxed_any.is::<$impl_type>() {
                         let concrete = boxed_any.downcast::<$impl_type>()
@@ -236,7 +220,7 @@ macro_rules! register_trait_type {
                 $crate::not_allowed!("fory_read_data should not be called directly on polymorphic Box<dyn {}> trait object", stringify!($trait_name))
             }
 
-            fn fory_get_type_id(_type_resolver: &fory_core::TypeResolver) -> Result<u32, fory_core::Error> {
+            fn fory_get_type_id(_type_resolver: &fory_core::TypeResolver) -> Result<fory_core::TypeId, fory_core::Error> {
                 $crate::not_allowed!("fory_get_type_id should not be called directly on polymorphic Box<dyn {}> trait object", stringify!($trait_name))
             }
 
@@ -358,8 +342,8 @@ macro_rules! generate_smart_pointer_wrapper {
 /// This macro handles ref tracking and directly constructs the trait object from concrete types
 #[macro_export]
 macro_rules! read_ptr_trait_object {
-    ($context:expr, $read_ref_info:expr, $read_type_info:expr, $type_info:expr, $pointer_type:ty, $trait_name:ident, $constructor_expr:expr, $get_ref:ident, $store_ref:ident, $($impl_type:ty),+) => {{
-        let ref_flag = if $read_ref_info {
+    ($context:expr, $ref_mode:expr, $read_type_info:expr, $type_info:expr, $pointer_type:ty, $trait_name:ident, $constructor_expr:expr, $get_ref:ident, $store_ref:ident, $($impl_type:ty),+) => {{
+        let ref_flag = if $ref_mode != fory_core::RefMode::None {
             $context.ref_reader.read_ref_flag(&mut $context.reader)?
         } else {
             fory_core::RefFlag::NotNullValue
@@ -375,14 +359,29 @@ macro_rules! read_ptr_trait_object {
             fory_core::RefFlag::NotNullValue => {
                 $context.inc_depth()?;
                 let typeinfo = if $read_type_info {
-                    $context.read_any_typeinfo()?
+                    $context.read_any_type_info()?
                 } else {
                     $type_info.ok_or_else(|| fory_core::Error::type_error("No type info found for read"))?
                 };
                 let fory_type_id = typeinfo.get_type_id();
+                let user_type_id = typeinfo.get_user_type_id();
+                let registered_by_name = typeinfo.is_registered_by_name();
+                let namespace = typeinfo.get_namespace();
+                let type_name = typeinfo.get_type_name();
+                let matches_type = |local_info: &fory_core::TypeInfo| -> bool {
+                    if registered_by_name {
+                        local_info.is_registered_by_name()
+                            && local_info.get_namespace().original == namespace.original
+                            && local_info.get_type_name().original == type_name.original
+                    } else if user_type_id != u32::MAX {
+                        local_info.get_user_type_id() == user_type_id
+                    } else {
+                        local_info.get_type_id() == fory_type_id
+                    }
+                };
                 $(
-                    if let Some(registered_type_id) = $context.get_type_resolver().get_fory_type_id(std::any::TypeId::of::<$impl_type>()) {
-                        if fory_type_id == registered_type_id {
+                    if let Ok(local_info) = $context.get_type_resolver().get_type_info(&std::any::TypeId::of::<$impl_type>()) {
+                        if matches_type(&local_info) {
                             let concrete_obj = <$impl_type as fory_core::Serializer>::fory_read_data($context)?;
                             $context.dec_depth();
                             let ptr = $constructor_expr(concrete_obj) as $pointer_type;
@@ -391,21 +390,38 @@ macro_rules! read_ptr_trait_object {
                     }
                 )*
                 $context.dec_depth();
-                Err(fory_core::Error::type_error(
-                    format!("Type ID {} not registered for trait {}", fory_type_id, stringify!($trait_name))
-                ))
+                Err(fory_core::Error::type_error(format!(
+                    "Type ID {} not registered for trait {}",
+                    fory_type_id as u32,
+                    stringify!($trait_name)
+                )))
             }
             fory_core::RefFlag::RefValue => {
                 $context.inc_depth()?;
                 let typeinfo = if $read_type_info {
-                    $context.read_any_typeinfo()?
+                    $context.read_any_type_info()?
                 } else {
                     $type_info.ok_or_else(|| fory_core::Error::type_error("No type info found for read"))?
                 };
                 let fory_type_id = typeinfo.get_type_id();
+                let user_type_id = typeinfo.get_user_type_id();
+                let registered_by_name = typeinfo.is_registered_by_name();
+                let namespace = typeinfo.get_namespace();
+                let type_name = typeinfo.get_type_name();
+                let matches_type = |local_info: &fory_core::TypeInfo| -> bool {
+                    if registered_by_name {
+                        local_info.is_registered_by_name()
+                            && local_info.get_namespace().original == namespace.original
+                            && local_info.get_type_name().original == type_name.original
+                    } else if user_type_id != u32::MAX {
+                        local_info.get_user_type_id() == user_type_id
+                    } else {
+                        local_info.get_type_id() == fory_type_id
+                    }
+                };
                 $(
-                    if let Some(registered_type_id) = $context.get_type_resolver().get_fory_type_id(std::any::TypeId::of::<$impl_type>()) {
-                        if fory_type_id == registered_type_id {
+                    if let Ok(local_info) = $context.get_type_resolver().get_type_info(&std::any::TypeId::of::<$impl_type>()) {
+                        if matches_type(&local_info) {
                             let concrete_obj = <$impl_type as fory_core::Serializer>::fory_read_data($context)?;
                             $context.dec_depth();
                             let ptr = $constructor_expr(concrete_obj) as $pointer_type;
@@ -416,9 +432,11 @@ macro_rules! read_ptr_trait_object {
                     }
                 )*
                 $context.dec_depth();
-                Err(fory_core::Error::type_error(
-                    format!("Type ID {} not registered for trait {}", fory_type_id, stringify!($trait_name))
-                ))
+                Err(fory_core::Error::type_error(format!(
+                    "Type ID {} not registered for trait {}",
+                    fory_type_id as u32,
+                    stringify!($trait_name)
+                )))
             }
         }
     }};
@@ -429,12 +447,12 @@ macro_rules! read_ptr_trait_object {
 macro_rules! impl_smart_pointer_serializer {
     ($wrapper_name:ident, $pointer_type:ty, $constructor_expr:expr, $trait_name:ident, $try_write_ref:ident, $get_ref:ident, $store_ref:ident, $($impl_type:ty),+) => {
         impl fory_core::Serializer for $wrapper_name {
-            fn fory_write(&self, context: &mut fory_core::WriteContext, write_ref_info: bool, write_type_info: bool, has_generics: bool) -> Result<(), fory_core::Error> {
-                if !write_ref_info || !context.ref_writer.$try_write_ref(&mut context.writer, &self.0) {
+            fn fory_write(&self, context: &mut fory_core::WriteContext, ref_mode: fory_core::RefMode, write_type_info: bool, has_generics: bool) -> Result<(), fory_core::Error> {
+                if ref_mode == fory_core::RefMode::None || !context.ref_writer.$try_write_ref(&mut context.writer, &self.0) {
                     let any_obj = <dyn $trait_name as fory_core::Serializer>::as_any(&*self.0);
                     let concrete_type_id = any_obj.type_id();
                     let typeinfo = if write_type_info {
-                         context.write_any_typeinfo(fory_core::TypeId::UNKNOWN as u32, concrete_type_id)?
+                         context.write_any_type_info(fory_core::TypeId::UNKNOWN as u32, concrete_type_id)?
                     } else {
                         context.get_type_info(&concrete_type_id)?
                     };
@@ -449,10 +467,10 @@ macro_rules! impl_smart_pointer_serializer {
                 $crate::downcast_and_serialize!(any_obj, context, $trait_name, $($impl_type),+)
             }
 
-            fn fory_read(context: &mut fory_core::ReadContext, read_ref_info: bool, read_type_info: bool) -> Result<Self, fory_core::Error> {
+            fn fory_read(context: &mut fory_core::ReadContext, ref_mode: fory_core::RefMode, read_type_info: bool) -> Result<Self, fory_core::Error> {
                 $crate::read_ptr_trait_object!(
                     context,
-                    read_ref_info,
+                    ref_mode,
                     read_type_info,
                     None,
                     $pointer_type,
@@ -464,10 +482,10 @@ macro_rules! impl_smart_pointer_serializer {
                 )
             }
 
-            fn fory_read_with_type_info(context: &mut fory_core::ReadContext, read_ref_info: bool, type_info: std::rc::Rc<fory_core::TypeInfo>) -> Result<Self, fory_core::Error> {
+            fn fory_read_with_type_info(context: &mut fory_core::ReadContext, ref_mode: fory_core::RefMode, type_info: std::rc::Rc<fory_core::TypeInfo>) -> Result<Self, fory_core::Error> {
                 $crate::read_ptr_trait_object!(
                     context,
-                    read_ref_info,
+                    ref_mode,
                     false,
                     Some(type_info),
                     $pointer_type,
@@ -484,8 +502,8 @@ macro_rules! impl_smart_pointer_serializer {
             }
 
             #[inline(always)]
-            fn fory_get_type_id(_type_resolver: &fory_core::TypeResolver) -> Result<u32, fory_core::Error> {
-                Ok(fory_core::TypeId::STRUCT as u32)
+            fn fory_get_type_id(_type_resolver: &fory_core::TypeResolver) -> Result<fory_core::TypeId, fory_core::Error> {
+                Ok(fory_core::TypeId::STRUCT)
             }
 
             #[inline(always)]
@@ -514,7 +532,7 @@ macro_rules! impl_smart_pointer_serializer {
             }
 
             #[inline(always)]
-            fn fory_type_id_dyn(&self, type_resolver: &fory_core::TypeResolver) -> Result<u32, fory_core::Error> {
+            fn fory_type_id_dyn(&self, type_resolver: &fory_core::TypeResolver) -> Result<fory_core::TypeId, fory_core::Error> {
                 let any_obj = <dyn $trait_name as fory_core::Serializer>::as_any(&*self.0);
                 let concrete_type_id = any_obj.type_id();
                 type_resolver
@@ -597,17 +615,17 @@ impl Serializer for Box<dyn Serializer> {
     fn fory_write(
         &self,
         context: &mut WriteContext,
-        write_ref_info: bool,
+        ref_mode: RefMode,
         write_type_info: bool,
         has_generics: bool,
     ) -> Result<(), Error> {
-        if write_ref_info {
+        if ref_mode != RefMode::None {
             context.writer.write_i8(RefFlag::NotNullValue as i8);
         }
         let fory_type_id_dyn = self.fory_type_id_dyn(context.get_type_resolver())?;
         let concrete_type_id = (**self).fory_concrete_type_id();
         if write_type_info {
-            context.write_any_typeinfo(fory_type_id_dyn, concrete_type_id)?;
+            context.write_any_type_info(fory_type_id_dyn as u32, concrete_type_id)?;
         };
         self.fory_write_data_generic(context, has_generics)
     }
@@ -627,7 +645,7 @@ impl Serializer for Box<dyn Serializer> {
     }
 
     #[inline(always)]
-    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<u32, Error> {
+    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<TypeId, Error> {
         (**self).fory_type_id_dyn(type_resolver)
     }
 
@@ -649,29 +667,29 @@ impl Serializer for Box<dyn Serializer> {
 
     #[inline(always)]
     fn fory_read_type_info(context: &mut ReadContext) -> Result<(), Error> {
-        context.read_any_typeinfo()?;
+        context.read_any_type_info()?;
         Ok(())
     }
 
     #[inline(always)]
     fn fory_read(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         read_type_info: bool,
     ) -> Result<Self, Error> {
-        read_box_seralizer(context, read_ref_info, read_type_info, None)
+        read_box_seralizer(context, ref_mode, read_type_info, None)
     }
 
     #[inline(always)]
     fn fory_read_with_type_info(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         type_info: Rc<TypeInfo>,
     ) -> Result<Self, Error>
     where
         Self: Sized + ForyDefault,
     {
-        read_box_seralizer(context, read_ref_info, false, Some(type_info))
+        read_box_seralizer(context, ref_mode, false, Some(type_info))
     }
 
     fn fory_read_data(_context: &mut ReadContext) -> Result<Self, Error> {
@@ -683,12 +701,12 @@ impl Serializer for Box<dyn Serializer> {
 
 fn read_box_seralizer(
     context: &mut ReadContext,
-    read_ref_info: bool,
+    ref_mode: RefMode,
     read_type_info: bool,
     type_info: Option<Rc<TypeInfo>>,
 ) -> Result<Box<dyn Serializer>, Error> {
     context.inc_depth()?;
-    let ref_flag = if read_ref_info {
+    let ref_flag = if ref_mode != RefMode::None {
         context.reader.read_i8()?
     } else {
         RefFlag::NotNullValue as i8
@@ -705,7 +723,7 @@ fn read_box_seralizer(
             read_type_info,
             Error::invalid_data("Type info must be read for Box<dyn Serializer>")
         );
-        context.read_any_typeinfo()?
+        context.read_any_type_info()?
     };
     let harness = typeinfo.get_harness();
     let boxed_any = harness.get_read_data_fn()(context)?;
