@@ -121,13 +121,11 @@
 //! - During deserialization, unresolved references will be patched up by `RefReader::add_callback`
 //!   once the strong pointer becomes available.
 
-use crate::ensure;
 use crate::error::Error;
 use crate::resolver::context::{ReadContext, WriteContext};
 use crate::resolver::type_resolver::{TypeInfo, TypeResolver};
 use crate::serializer::{ForyDefault, Serializer};
-use crate::types::RefFlag;
-use crate::types::TypeId;
+use crate::types::{RefFlag, RefMode, TypeId};
 use std::cell::UnsafeCell;
 use std::rc::Rc;
 use std::sync::Arc;
@@ -199,6 +197,14 @@ impl<T: ?Sized> RcWeak<T> {
     }
 }
 
+impl<T: ?Sized> PartialEq for RcWeak<T> {
+    fn eq(&self, other: &Self) -> bool {
+        self.ptr_eq(other)
+    }
+}
+
+impl<T: ?Sized> Eq for RcWeak<T> {}
+
 impl<T> Default for RcWeak<T> {
     fn default() -> Self {
         Self::new()
@@ -285,6 +291,14 @@ impl<T: ?Sized> ArcWeak<T> {
     }
 }
 
+impl<T: ?Sized> PartialEq for ArcWeak<T> {
+    fn eq(&self, other: &Self) -> bool {
+        self.ptr_eq(other)
+    }
+}
+
+impl<T: ?Sized> Eq for ArcWeak<T> {}
+
 impl<T> Default for ArcWeak<T> {
     fn default() -> Self {
         Self::new()
@@ -317,15 +331,26 @@ impl<T: Serializer + ForyDefault + 'static> Serializer for RcWeak<T> {
     fn fory_write(
         &self,
         context: &mut WriteContext,
-        write_ref_info: bool,
+        ref_mode: RefMode,
         write_type_info: bool,
         has_generics: bool,
     ) -> Result<(), Error> {
+        // Weak pointers require track_ref to be enabled on the Fory instance
+        if !context.is_track_ref() {
+            return Err(Error::invalid_ref(
+                "RcWeak requires track_ref to be enabled. Use Fory::default().track_ref(true)",
+            ));
+        }
+        // Weak MUST use ref tracking - otherwise read value will be lost
+        if ref_mode != RefMode::Tracking {
+            return Err(Error::invalid_ref(
+                "RcWeak requires RefMode::Tracking for serialization",
+            ));
+        }
         if let Some(rc) = self.upgrade() {
-            if !write_ref_info
-                || !context
-                    .ref_writer
-                    .try_write_rc_ref(&mut context.writer, &rc)
+            if !context
+                .ref_writer
+                .try_write_rc_ref(&mut context.writer, &rc)
             {
                 if write_type_info {
                     T::fory_write_type_info(context)?;
@@ -333,8 +358,6 @@ impl<T: Serializer + ForyDefault + 'static> Serializer for RcWeak<T> {
                 T::fory_write_data_generic(&*rc, context, has_generics)?;
             }
         } else {
-            ensure!(write_ref_info, "Value pointed by RcWeak<T> is null");
-
             context.writer.write_i8(RefFlag::Null as i8);
         }
         Ok(())
@@ -358,18 +381,27 @@ impl<T: Serializer + ForyDefault + 'static> Serializer for RcWeak<T> {
 
     fn fory_read(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         read_type_info: bool,
     ) -> Result<Self, Error> {
-        read_rc_weak::<T>(context, read_ref_info, read_type_info, None)
+        // Weak MUST track refs
+        debug_assert!(
+            ref_mode == RefMode::Tracking,
+            "RcWeak requires RefMode::Tracking"
+        );
+        read_rc_weak::<T>(context, read_type_info, None)
     }
 
     fn fory_read_with_type_info(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         typeinfo: Rc<TypeInfo>,
     ) -> Result<Self, Error> {
-        read_rc_weak::<T>(context, read_ref_info, false, Some(typeinfo))
+        debug_assert!(
+            ref_mode == RefMode::Tracking,
+            "RcWeak requires RefMode::Tracking"
+        );
+        read_rc_weak::<T>(context, false, Some(typeinfo))
     }
 
     fn fory_read_data(_: &mut ReadContext) -> Result<Self, Error> {
@@ -385,11 +417,18 @@ impl<T: Serializer + ForyDefault + 'static> Serializer for RcWeak<T> {
         4
     }
 
-    fn fory_get_type_id(type_resolver: &TypeResolver) -> Result<u32, Error> {
+    fn fory_get_type_id(type_resolver: &TypeResolver) -> Result<TypeId, Error> {
         T::fory_get_type_id(type_resolver)
     }
 
-    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<u32, Error> {
+    fn fory_get_type_info(type_resolver: &TypeResolver) -> Result<Rc<TypeInfo>, Error> {
+        match type_resolver.get_type_info(&std::any::TypeId::of::<T>()) {
+            Ok(info) => Ok(info),
+            Err(e) => Err(Error::enhance_type_error::<T>(e)),
+        }
+    }
+
+    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<TypeId, Error> {
         if let Some(rc) = self.upgrade() {
             (*rc).fory_type_id_dyn(type_resolver)
         } else {
@@ -408,24 +447,20 @@ impl<T: Serializer + ForyDefault + 'static> Serializer for RcWeak<T> {
 
 fn read_rc_weak<T: Serializer + ForyDefault + 'static>(
     context: &mut ReadContext,
-    read_ref_info: bool,
     read_type_info: bool,
     type_info: Option<Rc<TypeInfo>>,
 ) -> Result<RcWeak<T>, Error> {
-    let ref_flag = if read_ref_info {
-        context.ref_reader.read_ref_flag(&mut context.reader)?
-    } else {
-        RefFlag::NotNullValue
-    };
+    // Always read ref flag - Weak requires tracking
+    let ref_flag = context.ref_reader.read_ref_flag(&mut context.reader)?;
     match ref_flag {
         RefFlag::Null => Ok(RcWeak::new()),
         RefFlag::RefValue => {
             context.inc_depth()?;
             let data = if let Some(type_info) = type_info {
-                T::fory_read_with_type_info(context, false, type_info)?
+                T::fory_read_with_type_info(context, RefMode::None, type_info)?
             } else {
                 if read_type_info {
-                    context.read_any_typeinfo()?;
+                    context.read_any_type_info()?;
                 }
                 T::fory_read_data(context)?
             };
@@ -451,15 +486,7 @@ fn read_rc_weak<T: Serializer + ForyDefault + 'static>(
                 Ok(result_weak)
             }
         }
-        RefFlag::NotNullValue => {
-            // let inner = if let Some(typeinfo) = type_info {
-            //     T::fory_read_with_type_info(context, false, typeinfo)?
-            // } else {
-            //     T::fory_read_data(context)?
-            // };
-            // Ok(RcWeak::from(&Rc::new(inner)))
-            Err(Error::invalid_ref("RcWeak can't hold a strong ref value"))
-        }
+        RefFlag::NotNullValue => Err(Error::invalid_ref("RcWeak can't hold a strong ref value")),
     }
 }
 
@@ -477,15 +504,26 @@ impl<T: Serializer + ForyDefault + Send + Sync + 'static> Serializer for ArcWeak
     fn fory_write(
         &self,
         context: &mut WriteContext,
-        write_ref_info: bool,
+        ref_mode: RefMode,
         write_type_info: bool,
         has_generics: bool,
     ) -> Result<(), Error> {
+        // Weak pointers require track_ref to be enabled on the Fory instance
+        if !context.is_track_ref() {
+            return Err(Error::invalid_ref(
+                "ArcWeak requires track_ref to be enabled. Use Fory::default().track_ref(true)",
+            ));
+        }
+        // Weak MUST use ref tracking - otherwise read value will be lost
+        if ref_mode != RefMode::Tracking {
+            return Err(Error::invalid_ref(
+                "ArcWeak requires RefMode::Tracking for serialization",
+            ));
+        }
         if let Some(arc) = self.upgrade() {
-            if !write_ref_info
-                || !context
-                    .ref_writer
-                    .try_write_arc_ref(&mut context.writer, &arc)
+            if !context
+                .ref_writer
+                .try_write_arc_ref(&mut context.writer, &arc)
             {
                 if write_type_info {
                     T::fory_write_type_info(context)?;
@@ -493,7 +531,6 @@ impl<T: Serializer + ForyDefault + Send + Sync + 'static> Serializer for ArcWeak
                 T::fory_write_data_generic(&*arc, context, has_generics)?;
             }
         } else {
-            ensure!(write_ref_info, "Value pointed by ArcWeak<T> is null");
             context.writer.write_i8(RefFlag::Null as i8);
         }
         Ok(())
@@ -517,18 +554,27 @@ impl<T: Serializer + ForyDefault + Send + Sync + 'static> Serializer for ArcWeak
 
     fn fory_read(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         read_type_info: bool,
     ) -> Result<Self, Error> {
-        read_arc_weak(context, read_ref_info, read_type_info, None)
+        // Weak MUST track refs
+        debug_assert!(
+            ref_mode == RefMode::Tracking,
+            "ArcWeak requires RefMode::Tracking"
+        );
+        read_arc_weak(context, read_type_info, None)
     }
 
     fn fory_read_with_type_info(
         context: &mut ReadContext,
-        read_ref_info: bool,
+        ref_mode: RefMode,
         typeinfo: Rc<TypeInfo>,
     ) -> Result<Self, Error> {
-        read_arc_weak::<T>(context, read_ref_info, false, Some(typeinfo))
+        debug_assert!(
+            ref_mode == RefMode::Tracking,
+            "ArcWeak requires RefMode::Tracking"
+        );
+        read_arc_weak::<T>(context, false, Some(typeinfo))
     }
 
     fn fory_read_data(_: &mut ReadContext) -> Result<Self, Error> {
@@ -544,11 +590,18 @@ impl<T: Serializer + ForyDefault + Send + Sync + 'static> Serializer for ArcWeak
         4
     }
 
-    fn fory_get_type_id(type_resolver: &TypeResolver) -> Result<u32, Error> {
+    fn fory_get_type_id(type_resolver: &TypeResolver) -> Result<TypeId, Error> {
         T::fory_get_type_id(type_resolver)
     }
 
-    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<u32, Error> {
+    fn fory_get_type_info(type_resolver: &TypeResolver) -> Result<Rc<TypeInfo>, Error> {
+        match type_resolver.get_type_info(&std::any::TypeId::of::<T>()) {
+            Ok(info) => Ok(info),
+            Err(e) => Err(Error::enhance_type_error::<T>(e)),
+        }
+    }
+
+    fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<TypeId, Error> {
         if let Some(arc) = self.upgrade() {
             (*arc).fory_type_id_dyn(type_resolver)
         } else {
@@ -567,24 +620,20 @@ impl<T: Serializer + ForyDefault + Send + Sync + 'static> Serializer for ArcWeak
 
 fn read_arc_weak<T: Serializer + ForyDefault + 'static>(
     context: &mut ReadContext,
-    read_ref_info: bool,
     read_type_info: bool,
     type_info: Option<Rc<TypeInfo>>,
 ) -> Result<ArcWeak<T>, Error> {
-    let ref_flag = if read_ref_info {
-        context.ref_reader.read_ref_flag(&mut context.reader)?
-    } else {
-        RefFlag::NotNullValue
-    };
+    // Always read ref flag - Weak requires tracking
+    let ref_flag = context.ref_reader.read_ref_flag(&mut context.reader)?;
     match ref_flag {
         RefFlag::Null => Ok(ArcWeak::new()),
         RefFlag::RefValue => {
             context.inc_depth()?;
             let data = if let Some(type_info) = type_info {
-                T::fory_read_with_type_info(context, false, type_info)?
+                T::fory_read_with_type_info(context, RefMode::None, type_info)?
             } else {
                 if read_type_info {
-                    context.read_any_typeinfo()?;
+                    context.read_any_type_info()?;
                 }
                 T::fory_read_data(context)?
             };
@@ -614,15 +663,7 @@ fn read_arc_weak<T: Serializer + ForyDefault + 'static>(
             }
             Ok(weak)
         }
-        RefFlag::NotNullValue => {
-            // let inner = if let Some(typeinfo) = type_info {
-            //     T::fory_read_with_type_info(context, false, typeinfo)?
-            // } else {
-            //     T::fory_read_data(context)?
-            // };
-            // Ok(ArcWeak::from(&Arc::new(inner)))
-            Err(Error::invalid_ref("ArcWeak can't hold a strong ref value"))
-        }
+        RefFlag::NotNullValue => Err(Error::invalid_ref("ArcWeak can't hold a strong ref value")),
     }
 }
 
