@@ -15,12 +15,12 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use super::util::{is_default_value_variant, is_skip_enum_variant};
+use super::util::{enum_variant_id, is_default_value_variant, is_skip_enum_variant};
 use crate::object::misc;
 use crate::object::read::gen_read_field;
 use crate::object::util::{get_filtered_fields_iter, get_sorted_field_names};
 use crate::object::write::gen_write_field;
-use crate::util::sorted_fields;
+use crate::util::{extract_fields, source_fields};
 use proc_macro2::{Ident, TokenStream};
 use quote::quote;
 use syn::{DataEnum, Fields};
@@ -28,9 +28,32 @@ fn temp_var_name(i: usize) -> String {
     format!("f{}", i)
 }
 
-pub fn gen_actual_type_id() -> TokenStream {
-    quote! {
-       fory_core::serializer::enum_::actual_type_id(type_id, register_by_name, compatible)
+/// For Union-compatible enums with data variants, return UNION TypeId in xlang mode.
+pub fn gen_actual_type_id(data_enum: &DataEnum) -> TokenStream {
+    let is_union_compatible = is_union_compatible_enum(data_enum);
+    let has_data_variants = data_enum
+        .variants
+        .iter()
+        .any(|v| !matches!(v.fields, Fields::Unit));
+
+    if is_union_compatible && has_data_variants {
+        // Union-compatible enum: use typed/named union IDs in xlang mode
+        quote! {
+            if xlang {
+                if register_by_name {
+                    fory_core::types::TypeId::NAMED_UNION as u32
+                } else {
+                    fory_core::types::TypeId::TYPED_UNION as u32
+                }
+            } else {
+                fory_core::serializer::enum_::actual_type_id(type_id, register_by_name, compatible)
+            }
+        }
+    } else {
+        quote! {
+            let _ = xlang;
+            fory_core::serializer::enum_::actual_type_id(type_id, register_by_name, compatible)
+        }
     }
 }
 
@@ -119,8 +142,9 @@ pub(crate) fn gen_named_variant_meta_type_impl_with_enum_name(
     fields: &syn::FieldsNamed,
 ) -> TokenStream {
     let fields_clone = syn::Fields::Named(fields.clone());
-    let sorted_fields_slice = sorted_fields(&fields_clone);
-    let filtered_fields: Vec<_> = get_filtered_fields_iter(&sorted_fields_slice).collect();
+    let source_fields = source_fields(&fields_clone);
+    let fields_slice = extract_fields(&source_fields);
+    let filtered_fields: Vec<_> = get_filtered_fields_iter(&fields_slice).collect();
     let sorted_field_names_vec = get_sorted_field_names(&filtered_fields);
 
     // Generate individual field name literals
@@ -131,7 +155,7 @@ pub(crate) fn gen_named_variant_meta_type_impl_with_enum_name(
         })
         .collect();
 
-    let fields_info_ts = misc::gen_field_fields_info(&sorted_fields_slice);
+    let fields_info_ts = misc::gen_field_fields_info(&source_fields);
 
     // Include enum name to make meta type unique
     let meta_type_ident = Ident::new(
@@ -156,41 +180,82 @@ pub(crate) fn gen_named_variant_meta_type_impl_with_enum_name(
 
 pub fn gen_write(_data_enum: &DataEnum) -> TokenStream {
     quote! {
-        fory_core::serializer::enum_::write::<Self>(self, context, write_ref_info, write_type_info)
+        fory_core::serializer::enum_::write::<Self>(self, context, ref_mode, write_type_info)
     }
 }
 
 fn xlang_variant_branches(data_enum: &DataEnum, default_variant_value: u32) -> Vec<TokenStream> {
+    let is_union_compatible = is_union_compatible_enum(data_enum);
+
     data_enum
         .variants
         .iter()
         .enumerate()
         .map(|(idx, v)| {
             let ident = &v.ident;
-            let mut tag_value = idx as u32;
+            let mut tag_value = if is_union_compatible {
+                enum_variant_id(v).unwrap_or(idx as u32)
+            } else {
+                idx as u32
+            };
             if is_skip_enum_variant(v) {
                 tag_value = default_variant_value;
             }
 
             match &v.fields {
                 Fields::Unit => {
-                    quote! {
-                        Self::#ident => {
-                            context.writer.write_varuint32(#tag_value);
+                    if is_union_compatible {
+                        // Union-compatible: write tag + null flag (matches Java/C++ Union with null value)
+                        quote! {
+                            Self::#ident => {
+                                context.writer.write_var_uint32(#tag_value);
+                                // Write null flag for unit variant (no value)
+                                context.writer.write_i8(fory_core::types::RefFlag::Null as i8);
+                            }
+                        }
+                    } else {
+                        quote! {
+                            Self::#ident => {
+                                context.writer.write_var_uint32(#tag_value);
+                            }
                         }
                     }
                 }
-                Fields::Unnamed(_) => {
-                    quote! {
-                        Self::#ident(..) => {
-                            context.writer.write_varuint32(#tag_value);
+                Fields::Unnamed(fields_unnamed) => {
+                    if is_union_compatible && fields_unnamed.unnamed.len() == 1 {
+                        // Union-compatible single field: write tag + value with type info (like xwriteRef)
+                        quote! {
+                            Self::#ident(ref value) => {
+                                context.writer.write_var_uint32(#tag_value);
+                                use fory_core::serializer::Serializer;
+                                value.fory_write(context, fory_core::types::RefMode::Tracking, true, false)?;
+                            }
+                        }
+                    } else {
+                        quote! {
+                            Self::#ident(..) => {
+                                context.writer.write_var_uint32(#tag_value);
+                            }
                         }
                     }
                 }
-                Fields::Named(_) => {
-                    quote! {
-                        Self::#ident { .. } => {
-                            context.writer.write_varuint32(#tag_value);
+                Fields::Named(fields_named) => {
+                    if is_union_compatible && fields_named.named.len() == 1 {
+                        // Union-compatible single field: write tag + value with type info (like xwriteRef)
+                        let field_ident =
+                            fields_named.named.first().unwrap().ident.as_ref().unwrap();
+                        quote! {
+                            Self::#ident { ref #field_ident } => {
+                                context.writer.write_var_uint32(#tag_value);
+                                use fory_core::serializer::Serializer;
+                                #field_ident.fory_write(context, fory_core::types::RefMode::Tracking, true, false)?;
+                            }
+                        }
+                    } else {
+                        quote! {
+                            Self::#ident { .. } => {
+                                context.writer.write_var_uint32(#tag_value);
+                            }
                         }
                     }
                 }
@@ -215,7 +280,7 @@ fn rust_variant_branches(data_enum: &DataEnum, default_variant_value: u32) -> Ve
                 Fields::Unit => {
                     quote! {
                         Self::#ident => {
-                            context.writer.write_varuint32(#tag_value);
+                            context.writer.write_var_uint32(#tag_value);
                         }
                     }
                 }
@@ -233,31 +298,31 @@ fn rust_variant_branches(data_enum: &DataEnum, default_variant_value: u32) -> Ve
 
                     quote! {
                         Self::#ident( #(#field_idents),* ) => {
-                            context.writer.write_varuint32(#tag_value);
+                            context.writer.write_var_uint32(#tag_value);
                             #(#write_fields)*
                         }
                     }
                 }
                 Fields::Named(fields_named) => {
-                    use crate::util::sorted_fields;
+                    use crate::util::source_fields;
 
                     let fields_clone = syn::Fields::Named(fields_named.clone());
-                    let sorted_fields = sorted_fields(&fields_clone);
+                    let source_fields = source_fields(&fields_clone);
 
-                    let field_idents: Vec<_> = sorted_fields
+                    let field_idents: Vec<_> = source_fields
                         .iter()
-                        .map(|f| f.ident.as_ref().unwrap())
+                        .map(|sf| sf.field.ident.as_ref().unwrap())
                         .collect();
 
-                    let write_fields: Vec<_> = sorted_fields
+                    let write_fields: Vec<_> = source_fields
                         .iter()
                         .zip(field_idents.iter())
-                        .map(|(f, ident)| gen_write_field(f, ident, false))
+                        .map(|(sf, ident)| gen_write_field(sf.field, ident, false))
                         .collect();
 
                     quote! {
                         Self::#ident { #(#field_idents),* } => {
-                            context.writer.write_varuint32(#tag_value) ;
+                            context.writer.write_var_uint32(#tag_value) ;
                             #(#write_fields)*
                         }
                     }
@@ -289,7 +354,7 @@ fn rust_compatible_variant_write_branches(
                 Fields::Unit => {
                     quote! {
                         Self::#ident => {
-                            context.writer.write_varuint32((#tag_value << 2) | 0b0);
+                            context.writer.write_var_uint32((#tag_value << 2) | 0b0);
                         }
                     }
                 }
@@ -303,14 +368,14 @@ fn rust_compatible_variant_write_branches(
 
                     quote! {
                         Self::#ident( #(ref #field_idents),* ) => {
-                            context.writer.write_varuint32((#tag_value << 2) | 0b1);
+                            context.writer.write_var_uint32((#tag_value << 2) | 0b1);
                             // Write as collection format (same as tuple)
-                            context.writer.write_varuint32(#field_count as u32);
+                            context.writer.write_var_uint32(#field_count as u32);
                             let header = 0u8; // No IS_SAME_TYPE flag
                             context.writer.write_u8(header);
                             use fory_core::serializer::Serializer;
                             #(
-                                #field_idents.fory_write(context, true, true, false)?;
+                                #field_idents.fory_write(context, fory_core::RefMode::NullOnly, true, false)?;
                             )*
                         }
                     }
@@ -322,24 +387,23 @@ fn rust_compatible_variant_write_branches(
                         proc_macro2::Span::call_site()
                     );
                     let fields_clone = syn::Fields::Named(fields_named.clone());
-                    let sorted_fields = sorted_fields(&fields_clone);
-                    let field_idents: Vec<_> = sorted_fields
+                    let source_fields = source_fields(&fields_clone);
+                    let field_idents: Vec<_> = source_fields
                         .iter()
-                        .map(|f| f.ident.as_ref().unwrap())
+                        .map(|sf| sf.field.ident.as_ref().unwrap())
                         .collect();
 
-                    let write_fields: Vec<_> = sorted_fields
+                    let write_fields: Vec<_> = source_fields
                         .iter()
                         .zip(field_idents.iter())
-                        .map(|(f, ident)| gen_write_field(f, ident, false))
+                        .map(|(sf, ident)| gen_write_field(sf.field, ident, false))
                         .collect();
 
                     quote! {
                         Self::#ident { #(#field_idents),* } => {
-                            context.writer.write_varuint32((#tag_value << 2) | 0b10);
-                            // Push named variant meta
-                            let meta_index = context.push_meta(std::any::TypeId::of::<#meta_type_ident>())? as u32;
-                            context.writer.write_varuint32(meta_index);
+                            context.writer.write_var_uint32((#tag_value << 2) | 0b10);
+                            // Write type meta inline using streaming protocol
+                            context.write_type_meta(std::any::TypeId::of::<#meta_type_ident>())?;
                             // Write fields same as struct
                             #(#write_fields)*
                         }
@@ -386,21 +450,74 @@ pub fn gen_write_data(data_enum: &DataEnum) -> TokenStream {
     }
 }
 
-pub fn gen_write_type_info() -> TokenStream {
-    quote! {
-        fory_core::serializer::enum_::write_type_info::<Self>(context)
+pub fn gen_write_type_info(data_enum: &DataEnum) -> TokenStream {
+    let is_union_compatible = is_union_compatible_enum(data_enum);
+    let has_data_variants = data_enum
+        .variants
+        .iter()
+        .any(|v| !matches!(v.fields, Fields::Unit));
+
+    if is_union_compatible && has_data_variants {
+        // Union-compatible with data: write typed/named union type info in xlang mode
+        quote! {
+            if context.is_xlang() {
+                let rs_type_id = std::any::TypeId::of::<Self>();
+                context.write_any_type_info(fory_core::types::UNKNOWN, rs_type_id)?;
+                Ok(())
+            } else {
+                fory_core::serializer::enum_::write_type_info::<Self>(context)
+            }
+        }
+    } else {
+        quote! {
+            fory_core::serializer::enum_::write_type_info::<Self>(context)
+        }
     }
 }
 
 pub fn gen_read(_: &DataEnum) -> TokenStream {
     quote! {
-        fory_core::serializer::enum_::read::<Self>(context, read_ref_info, read_type_info)
+        fory_core::serializer::enum_::read::<Self>(context, ref_mode, read_type_info)
     }
 }
 
 pub fn gen_read_with_type_info(_: &DataEnum) -> TokenStream {
     quote! {
-        fory_core::serializer::enum_::read::<Self>(context, read_ref_info, false)
+        fory_core::serializer::enum_::read::<Self>(context, ref_mode, false)
+    }
+}
+
+/// Check if enum is Union-compatible:
+/// - Must have at least one data-carrying variant (single-field)
+/// - All variants must be either unit or single-field
+fn is_union_compatible_enum(data_enum: &DataEnum) -> bool {
+    let has_data_variant = data_enum
+        .variants
+        .iter()
+        .any(|v| !matches!(v.fields, Fields::Unit));
+    let all_variants_compatible = data_enum.variants.iter().all(|v| match &v.fields {
+        Fields::Unit => true,
+        Fields::Unnamed(f) => f.unnamed.len() == 1,
+        Fields::Named(f) => f.named.len() == 1,
+    });
+
+    has_data_variant && all_variants_compatible
+}
+
+/// Generate the static TypeId for enum.
+/// For Union-compatible enums with data variants, return UNION TypeId
+/// to ensure correct type info handling in xlang mode struct field read/write.
+pub fn gen_static_type_id(data_enum: &DataEnum) -> TokenStream {
+    let is_union_compatible = is_union_compatible_enum(data_enum);
+    let has_data_variants = data_enum
+        .variants
+        .iter()
+        .any(|v| !matches!(v.fields, Fields::Unit));
+
+    if is_union_compatible && has_data_variants {
+        quote! { fory_core::TypeId::UNION }
+    } else {
+        quote! { fory_core::TypeId::ENUM }
     }
 }
 
@@ -408,50 +525,90 @@ fn xlang_variant_read_branches(
     data_enum: &DataEnum,
     default_variant_value: u32,
 ) -> Vec<TokenStream> {
+    let is_union_compatible = is_union_compatible_enum(data_enum);
+
     data_enum
         .variants
         .iter()
         .enumerate()
         .map(|(idx, v)| {
             let ident = &v.ident;
-            let mut tag_value = idx as u32;
+            let mut tag_value = if is_union_compatible {
+                enum_variant_id(v).unwrap_or(idx as u32)
+            } else {
+                idx as u32
+            };
             if is_skip_enum_variant(v) {
                 tag_value = default_variant_value;
             }
 
             match &v.fields {
                 Fields::Unit => {
-                    quote! {
-                        #tag_value => Ok(Self::#ident),
+                    if is_union_compatible {
+                        // Union-compatible: read null flag (matches Java/C++ Union with null value)
+                        quote! {
+                            #tag_value => {
+                                let _ = context.reader.read_i8()?;
+                                Ok(Self::#ident)
+                            }
+                        }
+                    } else {
+                        quote! {
+                            #tag_value => Ok(Self::#ident),
+                        }
                     }
                 }
                 Fields::Unnamed(fields_unnamed) => {
-                    let default_fields: Vec<TokenStream> = fields_unnamed
-                        .unnamed
-                        .iter()
-                        .map(|f| {
-                            let ty = &f.ty;
-                            quote! { <#ty as fory_core::ForyDefault>::fory_default() }
-                        })
-                        .collect();
-
-                    quote! {
-                        #tag_value => Ok(Self::#ident( #(#default_fields),* )),
+                    if is_union_compatible && fields_unnamed.unnamed.len() == 1 {
+                        // Union-compatible single field: read value with ref_info=Tracking, type_info=true
+                        let field_ty = &fields_unnamed.unnamed.first().unwrap().ty;
+                        quote! {
+                            #tag_value => {
+                                use fory_core::serializer::Serializer;
+                                let value = <#field_ty as Serializer>::fory_read(context, fory_core::types::RefMode::Tracking, true)?;
+                                Ok(Self::#ident(value))
+                            }
+                        }
+                    } else {
+                        let default_fields: Vec<TokenStream> = fields_unnamed
+                            .unnamed
+                            .iter()
+                            .map(|f| {
+                                let ty = &f.ty;
+                                quote! { <#ty as fory_core::ForyDefault>::fory_default() }
+                            })
+                            .collect();
+                        quote! {
+                            #tag_value => Ok(Self::#ident( #(#default_fields),* )),
+                        }
                     }
                 }
                 Fields::Named(fields_named) => {
-                    let default_fields: Vec<TokenStream> = fields_named
-                        .named
-                        .iter()
-                        .map(|f| {
-                            let field_ident = f.ident.as_ref().unwrap();
-                            let ty = &f.ty;
-                            quote! { #field_ident: <#ty as fory_core::ForyDefault>::fory_default() }
-                        })
-                        .collect();
-
-                    quote! {
-                        #tag_value => Ok(Self::#ident { #(#default_fields),* }),
+                    if is_union_compatible && fields_named.named.len() == 1 {
+                        // Union-compatible single field: read value with ref_info=Tracking, type_info=true
+                        let field = fields_named.named.first().unwrap();
+                        let field_ident = field.ident.as_ref().unwrap();
+                        let field_ty = &field.ty;
+                        quote! {
+                            #tag_value => {
+                                use fory_core::serializer::Serializer;
+                                let value = <#field_ty as Serializer>::fory_read(context, fory_core::types::RefMode::Tracking, true)?;
+                                Ok(Self::#ident { #field_ident: value })
+                            }
+                        }
+                    } else {
+                        let default_fields: Vec<TokenStream> = fields_named
+                            .named
+                            .iter()
+                            .map(|f| {
+                                let field_ident = f.ident.as_ref().unwrap();
+                                let ty = &f.ty;
+                                quote! { #field_ident: <#ty as fory_core::ForyDefault>::fory_default() }
+                            })
+                            .collect();
+                        quote! {
+                            #tag_value => Ok(Self::#ident { #(#default_fields),* }),
+                        }
                     }
                 }
             }
@@ -488,8 +645,12 @@ fn rust_variant_read_branches(
                     let read_fields: Vec<TokenStream> = fields_unnamed
                         .unnamed
                         .iter()
+                        .enumerate()
                         .zip(field_idents.iter())
-                        .map(|(f, ident)| gen_read_field(f, ident))
+                        .map(|((idx, f), ident)| {
+                            let field_name = idx.to_string();
+                            gen_read_field(f, ident, &field_name)
+                        })
                         .collect();
 
                     quote! {
@@ -501,17 +662,20 @@ fn rust_variant_read_branches(
                 }
                 Fields::Named(fields_named) => {
                     let fields_clone = syn::Fields::Named(fields_named.clone());
-                    let sorted_fields = sorted_fields(&fields_clone);
+                    let source_fields = source_fields(&fields_clone);
 
-                    let field_idents: Vec<_> = sorted_fields
+                    let field_idents: Vec<_> = source_fields
                         .iter()
-                        .map(|f| f.ident.as_ref().unwrap())
+                        .map(|sf| sf.field.ident.as_ref().unwrap())
                         .collect();
 
-                    let read_fields: Vec<_> = sorted_fields
+                    let read_fields: Vec<_> = source_fields
                         .iter()
                         .zip(field_idents.iter())
-                        .map(|(f, ident)| gen_read_field(f, ident))
+                        .map(|(sf, ident)| {
+                            let field_name = ident.to_string();
+                            gen_read_field(sf.field, ident, &field_name)
+                        })
                         .collect();
 
                     quote! {
@@ -577,7 +741,7 @@ fn rust_compatible_variant_read_branches(
                             quote! {
                                 let #field_ident = if #i < len {
                                     use fory_core::serializer::Serializer;
-                                    <#field_ty>::fory_read(context, true, true)?
+                                    <#field_ty>::fory_read(context, fory_core::RefMode::NullOnly, true)?
                                 } else {
                                     <#field_ty as fory_core::ForyDefault>::fory_default()
                                 }
@@ -622,16 +786,16 @@ fn rust_compatible_variant_read_branches(
                     }
                 }
                 Fields::Named(fields_named) => {
-                    use crate::util::sorted_fields;
+                    use crate::util::source_fields;
 
                     // Sort fields to match the meta type generation
                     let fields_clone = syn::Fields::Named(fields_named.clone());
-                    let sorted_fields_slice = sorted_fields(&fields_clone);
+                    let source_fields = source_fields(&fields_clone);
 
                     // Generate compatible read logic using gen_read_compatible_with_construction
                     let compatible_read_body =
                         crate::object::read::gen_read_compatible_with_construction(
-                            &sorted_fields_slice,
+                            &source_fields,
                             Some(ident),
                         );
 
@@ -657,9 +821,8 @@ fn rust_compatible_variant_read_branches(
                                 return Ok(#default_value);
                             }
                             // Named variant should have variant_type == 0b10
-                            // Read named variant meta (peer wrote this because variant_type == 0b10)
-                            let meta_index = context.reader.read_varuint32()? as usize;
-                            let type_info = context.get_meta(meta_index)?.clone();
+                            // Read type meta inline using streaming protocol
+                            let type_info = context.read_type_meta()?;
                             // Use gen_read_compatible logic
                             #compatible_read_body
                         }
@@ -671,6 +834,11 @@ fn rust_compatible_variant_read_branches(
 }
 
 pub fn gen_read_data(data_enum: &DataEnum) -> TokenStream {
+    let is_union_compatible = is_union_compatible_enum(data_enum);
+    let has_data_variants = data_enum
+        .variants
+        .iter()
+        .any(|v| !matches!(v.fields, Fields::Unit));
     let default_variant_value = data_enum
         .variants
         .iter()
@@ -722,19 +890,37 @@ pub fn gen_read_data(data_enum: &DataEnum) -> TokenStream {
         }
     };
 
+    let unknown_xlang_branch = if is_union_compatible && has_data_variants {
+        quote! {
+            _ => {
+                use fory_core::serializer::skip::skip_any_value;
+                skip_any_value(context, true)?;
+                if context.is_compatible() {
+                    Ok(#default_variant_construction)
+                } else {
+                    return Err(fory_core::error::Error::unknown_enum("unknown enum value"));
+                }
+            }
+        }
+    } else {
+        quote! {
+            _ => {
+                // Unknown variant: in compatible mode, return default; otherwise error
+                if context.is_compatible() {
+                    Ok(#default_variant_construction)
+                } else {
+                    return Err(fory_core::error::Error::unknown_enum("unknown enum value"));
+                }
+            }
+        }
+    };
+
     quote! {
         if context.is_xlang() {
             let ordinal = context.reader.read_varuint32()?;
             match ordinal {
                 #(#xlang_variant_branches)*
-                _ => {
-                    // Unknown variant: in compatible mode, return default; otherwise error
-                    if context.is_compatible() {
-                        Ok(#default_variant_construction)
-                    } else {
-                        return Err(fory_core::error::Error::unknown_enum("unknown enum value"));
-                    }
-                }
+                #unknown_xlang_branch
             }
         } else {
             if context.is_compatible() {
@@ -765,8 +951,35 @@ pub fn gen_read_data(data_enum: &DataEnum) -> TokenStream {
     }
 }
 
-pub fn gen_read_type_info() -> TokenStream {
-    quote! {
-        fory_core::serializer::enum_::read_type_info::<Self>(context)
+pub fn gen_read_type_info(data_enum: &DataEnum) -> TokenStream {
+    // Only use UNION TypeId for Union-compatible enums (unit or single-field variants)
+    let is_union_compatible = is_union_compatible_enum(data_enum);
+    let has_data_variants = data_enum
+        .variants
+        .iter()
+        .any(|v| !matches!(v.fields, Fields::Unit));
+
+    if is_union_compatible && has_data_variants {
+        // Union-compatible with data: read typed/named union type info in xlang mode
+        quote! {
+            if context.is_xlang() {
+                let expected_type_id = Self::fory_get_type_id(context.get_type_resolver())?;
+                let type_info = context.read_any_type_info()?;
+                let remote_type_id = type_info.get_type_id();
+                if remote_type_id != expected_type_id {
+                    return Err(fory_core::error::Error::type_mismatch(
+                        expected_type_id as u32,
+                        remote_type_id as u32,
+                    ));
+                }
+                Ok(())
+            } else {
+                fory_core::serializer::enum_::read_type_info::<Self>(context)
+            }
+        }
+    } else {
+        quote! {
+            fory_core::serializer::enum_::read_type_info::<Self>(context)
+        }
     }
 }
