@@ -15,12 +15,16 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use super::field_meta::parse_field_meta;
 use super::util::{
-    classify_trait_object_field, compute_struct_version_hash, create_wrapper_types_arc,
-    create_wrapper_types_rc, extract_type_name, get_filtered_fields_iter,
-    get_primitive_writer_method, get_struct_name, get_type_id_by_type_ast, is_debug_enabled,
-    is_direct_primitive_numeric_type, should_skip_type_info_for_field, skip_ref_flag, StructField,
+    classify_trait_object_field, create_wrapper_types_arc, create_wrapper_types_rc,
+    determine_field_ref_mode, extract_type_name, gen_struct_version_hash_ts, get_field_accessor,
+    get_field_name, get_filtered_source_fields_iter, get_option_inner_primitive_name,
+    get_primitive_writer_method_with_encoding, get_struct_name, get_type_id_by_type_ast,
+    is_debug_enabled, is_direct_primitive_type, is_option_encoding_primitive, FieldRefMode,
+    StructField,
 };
+use crate::util::SourceField;
 use fory_core::types::TypeId;
 use proc_macro2::{Ident, TokenStream};
 use quote::quote;
@@ -114,38 +118,61 @@ pub fn gen_write_type_info() -> TokenStream {
     }
 }
 
+/// Generate write code for a field using index-based access (supports tuple structs)
+pub fn gen_write_field_with_index(field: &Field, index: usize, use_self: bool) -> TokenStream {
+    let value_ts = get_field_accessor(field, index, use_self);
+    let field_name = get_field_name(field, index);
+    gen_write_field_impl(field, &value_ts, &field_name, use_self)
+}
+
 pub fn gen_write_field(field: &Field, ident: &Ident, use_self: bool) -> TokenStream {
-    let ty = &field.ty;
     let value_ts = if use_self {
         quote! { self.#ident }
     } else {
         quote! { #ident }
     };
+    let field_name = ident.to_string();
+    gen_write_field_impl(field, &value_ts, &field_name, use_self)
+}
+
+fn gen_write_field_impl(
+    field: &Field,
+    value_ts: &TokenStream,
+    field_name: &str,
+    use_self: bool,
+) -> TokenStream {
+    let ty = &field.ty;
+    // Determine RefMode from field meta (respects #[fory(ref=false)] attribute)
+    let ref_mode = determine_field_ref_mode(field);
     let base = match classify_trait_object_field(ty) {
         StructField::BoxDyn => {
+            // Box<dyn Trait> - respect field meta for ref mode
             quote! {
-                <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, true, true, false)?;
+                <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, #ref_mode, true, false)?;
             }
         }
         StructField::RcDyn(trait_name) => {
+            // Rc<dyn Trait> - respect field meta for ref mode
             let types = create_wrapper_types_rc(&trait_name);
             let wrapper_ty = types.wrapper_ty;
             let trait_ident = types.trait_ident;
             quote! {
                 let wrapper = #wrapper_ty::from(#value_ts.clone() as std::rc::Rc<dyn #trait_ident>);
-                <#wrapper_ty as fory_core::Serializer>::fory_write(&wrapper, context, true, true, false)?;
+                <#wrapper_ty as fory_core::Serializer>::fory_write(&wrapper, context, #ref_mode, true, false)?;
             }
         }
         StructField::ArcDyn(trait_name) => {
+            // Arc<dyn Trait> - respect field meta for ref mode
             let types = create_wrapper_types_arc(&trait_name);
             let wrapper_ty = types.wrapper_ty;
             let trait_ident = types.trait_ident;
             quote! {
                 let wrapper = #wrapper_ty::from(#value_ts.clone() as std::sync::Arc<dyn #trait_ident>);
-                <#wrapper_ty as fory_core::Serializer>::fory_write(&wrapper, context, true, true, false)?;
+                <#wrapper_ty as fory_core::Serializer>::fory_write(&wrapper, context, #ref_mode, true, false)?;
             }
         }
         StructField::VecRc(trait_name) => {
+            // Vec<Rc<dyn Trait>> - respect field meta for ref mode
             let types = create_wrapper_types_rc(&trait_name);
             let wrapper_ty = types.wrapper_ty;
             let trait_ident = types.trait_ident;
@@ -153,10 +180,11 @@ pub fn gen_write_field(field: &Field, ident: &Ident, use_self: bool) -> TokenStr
                 let wrapper_vec: Vec<#wrapper_ty> = #value_ts.iter()
                     .map(|item| #wrapper_ty::from(item.clone() as std::rc::Rc<dyn #trait_ident>))
                     .collect();
-                <Vec<#wrapper_ty> as fory_core::Serializer>::fory_write(&wrapper_vec, context, true, false, true)?;
+                <Vec<#wrapper_ty> as fory_core::Serializer>::fory_write(&wrapper_vec, context, #ref_mode, false, true)?;
             }
         }
         StructField::VecArc(trait_name) => {
+            // Vec<Arc<dyn Trait>> - respect field meta for ref mode
             let types = create_wrapper_types_arc(&trait_name);
             let wrapper_ty = types.wrapper_ty;
             let trait_ident = types.trait_ident;
@@ -164,22 +192,35 @@ pub fn gen_write_field(field: &Field, ident: &Ident, use_self: bool) -> TokenStr
                 let wrapper_vec: Vec<#wrapper_ty> = #value_ts.iter()
                     .map(|item| #wrapper_ty::from(item.clone() as std::sync::Arc<dyn #trait_ident>))
                     .collect();
-                <Vec<#wrapper_ty> as fory_core::Serializer>::fory_write(&wrapper_vec, context, true, false, true)?;
+                <Vec<#wrapper_ty> as fory_core::Serializer>::fory_write(&wrapper_vec, context, #ref_mode, false, true)?;
             }
         }
         StructField::VecBox(_) => {
-            // Vec<Box<dyn Any>> uses standard Vec serialization with polymorphic elements
-            quote! {
-                <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, true, false, true)?;
+            // Vec<Box<dyn Any>> - respect field meta for ref mode
+            if ref_mode == FieldRefMode::None {
+                quote! {
+                    <#ty as fory_core::Serializer>::fory_write_data_generic(&#value_ts, context, true)?;
+                }
+            } else {
+                quote! {
+                    <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, #ref_mode, false, true)?;
+                }
             }
         }
         StructField::HashMapBox(_, _) => {
-            // HashMap<K, Box<dyn Any>> uses standard HashMap serialization with polymorphic values
-            quote! {
-                <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, true, false, true)?;
+            // HashMap<K, Box<dyn Any>> - respect field meta for ref mode
+            if ref_mode == FieldRefMode::None {
+                quote! {
+                    <#ty as fory_core::Serializer>::fory_write_data_generic(&#value_ts, context, true)?;
+                }
+            } else {
+                quote! {
+                    <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, #ref_mode, false, true)?;
+                }
             }
         }
         StructField::HashMapRc(key_ty, trait_name) => {
+            // HashMap<K, Rc<dyn Trait>> - respect field meta for ref mode
             let types = create_wrapper_types_rc(&trait_name);
             let wrapper_ty = types.wrapper_ty;
             let trait_ident = types.trait_ident;
@@ -187,10 +228,11 @@ pub fn gen_write_field(field: &Field, ident: &Ident, use_self: bool) -> TokenStr
                 let wrapper_map: std::collections::HashMap<#key_ty, #wrapper_ty> = #value_ts.iter()
                     .map(|(k, v)| (k.clone(), #wrapper_ty::from(v.clone() as std::rc::Rc<dyn #trait_ident>)))
                     .collect();
-                <std::collections::HashMap<#key_ty, #wrapper_ty> as fory_core::Serializer>::fory_write(&wrapper_map, context, true, false, true)?;
+                <std::collections::HashMap<#key_ty, #wrapper_ty> as fory_core::Serializer>::fory_write(&wrapper_map, context, #ref_mode, false, true)?;
             }
         }
         StructField::HashMapArc(key_ty, trait_name) => {
+            // HashMap<K, Arc<dyn Trait>> - respect field meta for ref mode
             let types = create_wrapper_types_arc(&trait_name);
             let wrapper_ty = types.wrapper_ty;
             let trait_ident = types.trait_ident;
@@ -198,67 +240,95 @@ pub fn gen_write_field(field: &Field, ident: &Ident, use_self: bool) -> TokenStr
                 let wrapper_map: std::collections::HashMap<#key_ty, #wrapper_ty> = #value_ts.iter()
                     .map(|(k, v)| (k.clone(), #wrapper_ty::from(v.clone() as std::sync::Arc<dyn #trait_ident>)))
                     .collect();
-                <std::collections::HashMap<#key_ty, #wrapper_ty> as fory_core::Serializer>::fory_write(&wrapper_map, context, true, false, true)?;
+                <std::collections::HashMap<#key_ty, #wrapper_ty> as fory_core::Serializer>::fory_write(&wrapper_map, context, #ref_mode, false, true)?;
             }
         }
         StructField::Forward => {
+            // Forward types (trait objects, forward references) - polymorphic, always need type info
             quote! {
-                <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, true, true, false)?;
+                <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, #ref_mode, true, false)?;
             }
         }
         _ => {
-            let skip_ref_flag = skip_ref_flag(ty);
-            let skip_type_info = should_skip_type_info_for_field(ty);
             let type_id = get_type_id_by_type_ast(ty);
+            let meta = parse_field_meta(field).unwrap_or_default();
 
-            // Check if this is a direct primitive numeric type that can use direct writer calls
-            if is_direct_primitive_numeric_type(ty) {
-                let type_name = extract_type_name(ty);
-                let writer_method = get_primitive_writer_method(&type_name);
+            // Check if this is Option<u32> or Option<u64> with encoding attributes
+            // These need special inline handling because the generic Option<T> serializer
+            // doesn't know about field-level encoding attributes.
+            if is_option_encoding_primitive(ty, &meta) {
+                let inner_name = get_option_inner_primitive_name(ty).unwrap();
+                let writer_method = get_primitive_writer_method_with_encoding(inner_name, &meta);
                 let writer_ident = syn::Ident::new(writer_method, proc_macro2::Span::call_site());
-                // For primitives:
-                // - use_self=true: #value_ts is `self.field`, which is T (copy happens automatically)
-                // - use_self=false: #value_ts is `field` from pattern match on &self, which is &T
-                let value_expr = if use_self {
-                    quote! { #value_ts }
-                } else {
-                    quote! { *#value_ts }
-                };
+                // For Option<primitive>, write null flag first, then value if Some
                 quote! {
-                    context.writer.#writer_ident(#value_expr);
+                    if let Some(v) = &#value_ts {
+                        context.writer.write_i8(fory_core::RefFlag::NotNullValue as i8);
+                        context.writer.#writer_ident(*v);
+                    } else {
+                        context.writer.write_i8(fory_core::RefFlag::Null as i8);
+                    }
+                }
+            }
+            // Check if this is a direct primitive type that can use direct writer calls
+            // Only apply when ref_mode is None (no ref tracking needed)
+            else if ref_mode == FieldRefMode::None && is_direct_primitive_type(ty) {
+                let type_name = extract_type_name(ty);
+                if type_name == "String" {
+                    // String: call fory_write_data directly
+                    quote! {
+                        <#ty as fory_core::Serializer>::fory_write_data(&#value_ts, context)?;
+                    }
+                } else {
+                    // Numeric primitives: use direct buffer methods
+                    // For u32/u64, consider encoding attributes
+                    let writer_method =
+                        get_primitive_writer_method_with_encoding(&type_name, &meta);
+                    let writer_ident =
+                        syn::Ident::new(writer_method, proc_macro2::Span::call_site());
+                    // For primitives:
+                    // - use_self=true: #value_ts is `self.field`, which is T (copy happens automatically)
+                    // - use_self=false: #value_ts is `field` from pattern match on &self, which is &T
+                    let value_expr = if use_self {
+                        quote! { #value_ts }
+                    } else {
+                        quote! { *#value_ts }
+                    };
+                    quote! {
+                        context.writer.#writer_ident(#value_expr);
+                    }
                 }
             } else if type_id == TypeId::LIST as u32
                 || type_id == TypeId::SET as u32
                 || type_id == TypeId::MAP as u32
             {
-                quote! {
-                    <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, true, false, true)?;
-                }
-            } else {
-                // Known types (primitives, strings, collections) - skip type info at compile time
-                // For custom types that we can't determine at compile time (like enums),
-                // we need to check at runtime whether to skip type info
-                if skip_type_info {
-                    if skip_ref_flag {
-                        quote! {
-                            <#ty as fory_core::Serializer>::fory_write_data(&#value_ts, context)?;
-                        }
-                    } else {
-                        quote! {
-                            <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, true, false, false)?;
-                        }
-                    }
-                } else if skip_ref_flag {
+                // For collections - respect field meta for ref mode
+                if ref_mode == FieldRefMode::None {
                     quote! {
-                        let need_type_info = fory_core::serializer::util::field_need_write_type_info(<#ty as fory_core::Serializer>::fory_static_type_id());
-                        <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, false, need_type_info, false)?;
+                        <#ty as fory_core::Serializer>::fory_write_data_generic(&#value_ts, context, true)?;
                     }
                 } else {
                     quote! {
-                        let need_type_info = fory_core::serializer::util::field_need_write_type_info(<#ty as fory_core::Serializer>::fory_static_type_id());
-                        <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, true, need_type_info, false)?;
+                        <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, #ref_mode, false, true)?;
                     }
                 }
+            } else {
+                // Custom types (struct/enum/ext) - always need mode-dependent type info logic
+                // Determine write_type_info based on mode:
+                // - compatible=true: use need_to_write_type_for_field (struct types need type info)
+                // - compatible=false: use fory_is_polymorphic
+                // This applies regardless of ref_mode because Java always writes type info
+                // for struct-type fields in compatible mode, even for non-nullable fields.
+                quote! {
+                    let write_type_info = if context.is_compatible() {
+                        fory_core::types::need_to_write_type_for_field(
+                            <#ty as fory_core::Serializer>::fory_static_type_id()
+                        )
+                    } else {
+                        <#ty as fory_core::Serializer>::fory_is_polymorphic()
+                    };
+                    <#ty as fory_core::Serializer>::fory_write(&#value_ts, context, #ref_mode, write_type_info, false)?;
+                }
             }
         }
     };
@@ -266,20 +336,19 @@ pub fn gen_write_field(field: &Field, ident: &Ident, use_self: bool) -> TokenStr
     if is_debug_enabled() && use_self {
         let struct_name = get_struct_name().expect("struct context not set");
         let struct_name_lit = syn::LitStr::new(&struct_name, proc_macro2::Span::call_site());
-        let field_name = field.ident.as_ref().unwrap().to_string();
-        let field_name_lit = syn::LitStr::new(&field_name, proc_macro2::Span::call_site());
+        let field_name_lit = syn::LitStr::new(field_name, proc_macro2::Span::call_site());
         quote! {
             fory_core::serializer::struct_::struct_before_write_field(
                 #struct_name_lit,
                 #field_name_lit,
-                (&self.#ident) as &dyn std::any::Any,
+                (&#value_ts) as &dyn std::any::Any,
                 context,
             );
             #base
             fory_core::serializer::struct_::struct_after_write_field(
                 #struct_name_lit,
                 #field_name_lit,
-                (&self.#ident) as &dyn std::any::Any,
+                (&#value_ts) as &dyn std::any::Any,
                 context,
             );
         }
@@ -288,19 +357,17 @@ pub fn gen_write_field(field: &Field, ident: &Ident, use_self: bool) -> TokenStr
     }
 }
 
-pub fn gen_write_data(fields: &[&Field]) -> TokenStream {
-    let write_fields_ts: Vec<_> = get_filtered_fields_iter(fields)
-        .map(|field| {
-            let ident = field.ident.as_ref().unwrap();
-            gen_write_field(field, ident, true)
-        })
+pub fn gen_write_data(source_fields: &[SourceField<'_>]) -> TokenStream {
+    let fields: Vec<&Field> = source_fields.iter().map(|sf| sf.field).collect();
+    let write_fields_ts: Vec<_> = get_filtered_source_fields_iter(source_fields)
+        .map(|sf| gen_write_field_with_index(sf.field, sf.original_index, true))
         .collect();
 
-    let version_hash = compute_struct_version_hash(fields);
+    let version_hash_ts = gen_struct_version_hash_ts(&fields);
     quote! {
-        // Write version hash when class version checking is enabled
         if context.is_check_struct_version() {
-            context.writer.write_i32(#version_hash);
+            let version_hash: i32 = #version_hash_ts;
+            context.writer.write_i32(version_hash);
         }
         #(#write_fields_ts)*
         Ok(())
@@ -309,6 +376,6 @@ pub fn gen_write_data(fields: &[&Field]) -> TokenStream {
 
 pub fn gen_write() -> TokenStream {
     quote! {
-        fory_core::serializer::struct_::write::<Self>(self, context, write_ref_info, write_type_info)
+        fory_core::serializer::struct_::write::<Self>(self, context, ref_mode, write_type_info)
     }
 }
