@@ -628,7 +628,7 @@ fn test_struct_with_generic() {
 
     impl<T: 'static + Serializer + ForyDefault> Serializer for Wrapper<T> {
         fn fory_write_data(&self, context: &mut WriteContext) -> Result<(), Error> {
-            context.writer.write_varuint32(self.value.len() as u32);
+            context.writer.write_var_uint32(self.value.len() as u32);
             context.writer.write_utf8_string(&self.value);
             self.data.fory_write_data(context)?;
             Ok(())
@@ -645,7 +645,10 @@ fn test_struct_with_generic() {
             })
         }
 
-        fn fory_type_id_dyn(&self, type_resolver: &TypeResolver) -> Result<u32, Error> {
+        fn fory_type_id_dyn(
+            &self,
+            type_resolver: &TypeResolver,
+        ) -> Result<fory_core::TypeId, Error> {
             Self::fory_get_type_id(type_resolver)
         }
 
