@@ -286,3 +286,48 @@ fn test_unregistered_type_error_message() {
         err_str
     );
 }
+
+#[test]
+fn test_type_mismatch_error_shows_type_name() {
+    use fory_core::error::Error;
+    use fory_core::types::{format_type_id, TypeId};
+
+    // Test internal type (BOOL = 1), no registered_id
+    let formatted = format_type_id(TypeId::BOOL as u32);
+    assert_eq!(formatted, "BOOL");
+
+    // Test struct type
+    let struct_type_id = TypeId::STRUCT as u32;
+    let formatted = format_type_id(struct_type_id);
+    assert_eq!(formatted, "STRUCT");
+
+    // Test enum type
+    let enum_type_id = TypeId::ENUM as u32;
+    let formatted = format_type_id(enum_type_id);
+    assert_eq!(formatted, "ENUM");
+
+    // Test EXT type
+    let ext_type_id = TypeId::EXT as u32;
+    let formatted = format_type_id(ext_type_id);
+    assert_eq!(formatted, "EXT");
+
+    // Test error message format
+    let err = Error::type_mismatch(struct_type_id, enum_type_id);
+    let err_str = format!("{}", err);
+    assert!(
+        err_str.contains("STRUCT"),
+        "error should contain STRUCT, got: {}",
+        err_str
+    );
+    assert!(
+        err_str.contains("ENUM"),
+        "error should contain ENUM, got: {}",
+        err_str
+    );
+    // Check the message contains "local" and "remote" for clarity
+    assert!(
+        err_str.contains("local") && err_str.contains("remote"),
+        "error should indicate local vs remote types, got: {}",
+        err_str
+    );
+}
